<?php

App::uses('AppModel', 'Model');
App::uses('SCTarget', 'SimpleComment/Model/Behavior');

/**
 * Courrier model class
 *
 * web-GFC : Gestion de Flux Citoyens (https://adullact.net/projects/webgfc)
 *
 * PHP version 7
 * @author Stéphane Sampaio
 * @copyright Initié par ADULLACT - Développé par ADULLACT Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * @package		app
 * @subpackage		app.Model
 */
class Courrier extends AppModel {

    /**
     * Model name
     *
     * @var string
     * @access public
     */
    public $name = 'Courrier';

    /**
     * Model Behaviors
     *
     * @var mixed
     * @access public
     */
//	public $beforesAs = array('Tree'); //TODO: remove if not used

    public $actsAs = array('SimpleComment.SCTarget');

    /**
     * Validation rules
     *
     * @var mixed
     * @access public
     */
    public $validate = array(
        'name' => array(
            array(
                'rule' => array('notBlank'),
                'allowEmpty' => false,
            ),
        ),
        'etat' => array(
            array(
                'rule' => array('numeric'),
                'allowEmpty' => true,
            ),
        ),
        'typeflux' => array(
            array(
                'rule' => array('numeric'),
                'allowEmpty' => true,
            ),
        ),
        'circuit_id' => array(
            array(
                'rule' => array('numeric'),
                'allowEmpty' => true,
            ),
        ),
        'soustype_id' => array(
            array(
                'rule' => array('numeric'),
                'allowEmpty' => true,
            ),
        ),
        'contact_id' => array(
            array(
                'rule' => array('notBlank'),
                'allowEmpty' => false
            ),
        ),
        'organisme_id' => array(
            array(
                'rule' => array('notBlank'),
                'allowEmpty' => false
            ),
        )
    );

    /**
     *
     * @var type
     */
    public $hasOne = array(
        'Traitement' => array(
            'className' => 'Cakeflow.Traitement',
            'foreignKey' => 'target_id'
        ),
        'Ordreservice' => array(
            'className' => 'Ordreservice',
            'foreignKey' => 'courrier_id'
        ),
        'Pliconsultatif' => array(
            'className' => 'Pliconsultatif',
            'foreignKey' => 'courrier_id'
        )
    );

    /**
     * belongsTo
     *
     * @var mixed
     * @access public
     */
    public $belongsTo = array(
        'Soustype' => array(
            'className' => 'Soustype',
            'foreignKey' => 'soustype_id',
            'type' => null,
            'conditions' => null,
            'fields' => null,
            'order' => null
        ),
        /* 'Contactinfo' => array(
          'className' => 'Contactinfo',
          'foreignKey' => 'contactinfo_id',
          'type' => null,
          'conditions' => null,
          'fields' => null,
          'order' => null
          ), */
        'Affaire' => array(
            'className' => 'Affaire',
            'foreignKey' => 'affaire_id',
            'type' => null,
            'conditions' => null,
            'fields' => null,
            'order' => null
        ),
        'User' => array(
            'className' => 'User',
            'foreignKey' => 'user_creator_id',
            'type' => null,
            'conditions' => null,
            'fields' => null,
            'order' => null
        ),
        'Desktop' => array(
            'className' => 'Desktop',
            'foreignKey' => 'desktop_creator_id',
            'type' => null,
            'conditions' => null,
            'fields' => null,
            'order' => null
        ),
        'Affairesuivie' => array(
            'className' => 'Desktop',
            'foreignKey' => 'affairesuiviepar_id',
            'type' => null,
            'conditions' => null,
            'fields' => null,
            'order' => null
        ),
        'Service' => array(
            'className' => 'Service',
            'foreignKey' => 'service_creator_id',
            'type' => null,
            'conditions' => null,
            'fields' => null,
            'order' => null
        ),
        'Origineflux' => array(
            'className' => 'Origineflux',
            'foreignKey' => 'origineflux_id',
            'type' => null,
            'conditions' => null,
            'fields' => null,
            'order' => null
        ),
        'Organisme' => array(
            'className' => 'Organisme',
            'foreignKey' => 'organisme_id',
            'type' => null,
            'conditions' => null,
            'fields' => null,
            'order' => null
        ),
        'Contact' => array(
            'className' => 'Contact',
            'foreignKey' => 'contact_id',
            'type' => null,
            'conditions' => null,
            'fields' => null,
            'order' => null
        ),
        'Marche' => array(
            'className' => 'Marche',
            'foreignKey' => 'marche_id',
            'type' => 'LEFT OUTER',
            'conditions' => null,
            'fields' => null,
            'order' => null
        ),
        'Consultation' => array(
            'className' => 'Consultation',
            'foreignKey' => 'consultation_id',
            'type' => 'LEFT OUTER',
            'conditions' => null,
            'fields' => null,
            'order' => null
        )
    );

    /**
     * hasMany
     *
     * @var mixed
     * @access public
     */
    public $hasMany = array(
        'Tache' => array(
            'className' => 'Tache',
            'foreignKey' => 'courrier_id',
            'dependent' => false,
            'conditions' => null,
            'fields' => null,
            'order' => null,
            'limit' => null,
            'offset' => null,
            'exclusive' => null,
            'finderQuery' => null,
            'counterQuery' => null
        ),
        'Traitement' => array(
            'className' => 'Cakeflow.Traitement',
            'foreignKey' => 'target_id',
            'dependent' => false,
            'conditions' => null,
            'fields' => null,
            'order' => null,
            'limit' => null,
            'offset' => null,
            'exclusive' => null,
            'finderQuery' => null,
            'counterQuery' => null
        ),
        'Bancontenu' => array(
            'className' => 'Bancontenu',
            'foreignKey' => 'courrier_id',
            'dependent' => false,
            'conditions' => null,
            'fields' => null,
            'order' => null,
            'limit' => null,
            'offset' => null,
            'exclusive' => null,
            'finderQuery' => null,
            'counterQuery' => null
        ),
        'Notifieddesktop' => array(
            'className' => 'Notifieddesktop',
            'foreignKey' => 'courrier_id',
            'dependent' => false,
            'conditions' => null,
            'fields' => null,
            'order' => null,
            'limit' => null,
            'offset' => null,
            'exclusive' => null,
            'finderQuery' => null,
            'counterQuery' => null
        ),
        'Document' => array(
            'className' => 'Document',
            'foreignKey' => 'courrier_id',
            'dependent' => false,
            'conditions' => null,
            'fields' => null,
            'order' => null,
            'limit' => null,
            'offset' => null,
            'exclusive' => null,
            'finderQuery' => null,
            'counterQuery' => null
        ),
        'Ar' => array(
            'className' => 'Ar',
            'foreignKey' => 'courrier_id',
            'dependent' => false,
            'conditions' => null,
            'fields' => null,
            'order' => null,
            'limit' => null,
            'offset' => null,
            'exclusive' => null,
            'finderQuery' => null,
            'counterQuery' => null
        ),
        'Relement' => array(
            'className' => 'Relement',
            'foreignKey' => 'courrier_id',
            'dependent' => false,
            'conditions' => null,
            'fields' => null,
            'order' => null,
            'limit' => null,
            'offset' => null,
            'exclusive' => null,
            'finderQuery' => null,
            'counterQuery' => null
        ),
        'Sendmail' => array(
            'className' => 'Sendmail',
            'foreignKey' => 'courrier_id',
            'dependent' => false,
            'conditions' => null,
            'fields' => null,
            'order' => null,
            'limit' => null,
            'offset' => null,
            'exclusive' => null,
            'finderQuery' => null,
            'counterQuery' => null
        ),
        'Ordreservice' => array(
            'className' => 'Ordreservice',
            'foreignKey' => 'courrier_id',
            'dependent' => false,
            'conditions' => null,
            'fields' => null,
            'order' => null,
            'limit' => null,
            'offset' => null,
            'exclusive' => null,
            'finderQuery' => null,
            'counterQuery' => null
        ),
        'Pliconsultatif' => array(
            'className' => 'Pliconsultatif',
            'foreignKey' => 'courrier_id',
            'dependent' => false,
            'conditions' => null,
            'fields' => null,
            'order' => null,
            'limit' => null,
            'offset' => null,
            'exclusive' => null,
            'finderQuery' => null,
            'counterQuery' => null
        ),
        'Notifquotidien' => array(
            'className' => 'Notifquotidien',
            'foreignKey' => 'courrier_id',
            'dependent' => false,
            'conditions' => null,
            'fields' => null,
            'order' => null,
            'limit' => null,
            'offset' => null,
            'exclusive' => null,
            'finderQuery' => null,
            'counterQuery' => null
        )
    );

    /**
     * hasAndBelongsToMany
     *
     * @var mixed
     * @access public
     */
    public $hasAndBelongsToMany = array(
        'Metadonnee' => array(
            'className' => 'Metadonnee',
            'joinTable' => 'courriers_metadonnees',
            'foreignKey' => 'courrier_id',
            'associationForeignKey' => 'metadonnee_id',
            'unique' => null,
            'conditions' => null,
            'fields' => null,
            'order' => null,
            'limit' => null,
            'offset' => null,
            'finderQuery' => null,
            'deleteQuery' => null,
            'insertQuery' => null,
            'with' => 'CourrierMetadonnee'
        ),
        'Repertoire' => array(
            'className' => 'Repertoire',
            'joinTable' => 'courriers_repertoires',
            'foreignKey' => 'courrier_id',
            'associationForeignKey' => 'repertoire_id',
            'unique' => null,
            'conditions' => null,
            'fields' => null,
            'order' => null,
            'limit' => null,
            'offset' => null,
            'finderQuery' => null,
            'deleteQuery' => null,
            'insertQuery' => null,
            'with' => 'CourrierRepertoire'
        )
    );
    // Variable permettant de définir le dernier numéro de séqauence d'un courrier généré
    protected $_lastSequenceReference = null;
    protected $_lastSequenceDocument = null;

    /**
     *
     * @param type $id
     * @return boolean
     */
    public function isInCircuit($id = null) {
        $return = false;
        if ($id != null) {
            $tmp = $this->Traitement->find('first', array('fields' => array('id'), 'conditions' => array('target_id' => $id), 'recursive' => -1));
            if (!empty($tmp)) {
                $return = true;
            }
        }
        return $return;
    }

    /**
     *
     * @param type $id
     * @return boolean
     */
    public function isRefused($id = null) {
        $return = false;
        if ($id != null) {
            $courriers = $this->find('first', array('fields' => array('id'), 'conditions' => array('ref_ancien' => $id), 'recursive' => -1));
            if (!empty($courriers)) {
                $return = true;
            }
        }
        return $return;
    }

    /**
     *
     * @param type $id
     * @return boolean
     */
    public function isTreated($id = null) {
        $return = false;
        if ($id != null) {
            $traitement = $this->Traitement->find('first', array('fields' => array('treated'), 'conditions' => array('target_id' => $id), 'contain' => false));
            if ($traitement['Traitement']['treated']) {
                $return = true;
            }
        }
        return $return;
    }

    /**
     *
     * @param type $id
     * @return boolean
     */
    public function isDuplicated($id = null) {
        $return = false;
        if ($id != null) {
            $courrier = $this->read($id);
            if (!empty($courrier) && !empty($courrier['Courrier']['ref_ancien'])) {
                $return = true;
            }
        }
        return $return;
    }

    /**
     *
     * @param type $id
     * @return type
     */
    public function getDuplicated($id = null) {
        $return = null;
        if ($id != null) {
            if ($this->isRefused($id)) {
                $return = $this->find('first', array('conditions' => array('ref_ancien' => $id)));
            }
        }
        return $return;
    }

    /**
     *
     * @param type $id
     * @return type
     */
    public function getOriginal($id = null) {
        $return = null;
        if ($id != null) {
            $courrier = $this->read($id);
            $return = $this->find('first', array('conditions' => array('id' => $courrier['Courrier']['ref_ancien'])));
        }
        return $return;
    }

    /**
     * check there is a 'etape' in a 'circuit' correspond a 'flux'
     * @param type $fluxId
     */
    public function isEtape($fluxId) {
        $isetape = false;
        $querydata = array(
            'fields' => array(
                'Soustype.circuit_id'
            ),
            'contain' => false,
            'joins' => array(
                $this->join($this->Soustype->alias)
            ),
            'conditions' => array(
                'Courrier.id' => $fluxId
            )
        );
        $flux = $this->find('first', $querydata);
        $circuit_id = $flux['Soustype']['circuit_id'];
        $etapes = $this->Soustype->Circuit->Etape->find('all', array(
            'fields' => array(
                'Etape.id',
                'Etape.type'
            ),
            'conditions' => array(
                'Etape.circuit_id' => $circuit_id
            )
        ));
        foreach ($etapes as $etape) {
            if ($etape['Etape']['type'] != 1) {
                if (count($etape['Composition']) < 2) {
                    $isetape = false;
                } else {
                    $isetape = true;
                }
            } else {
                if (count($etape['Composition']) < 1) {
                    $isetape = false;
                } else {
                    $isetape = true;
                }
            }
        }
        return $isetape;
    }

    /**
     * Checks if the Flux is outaded or not
     *
     * @param integer $id The id of a Flux
     * @return boolean if true the Flux is outdated
     */
    public function isInTime($id = null) {
        $return = true;
        if ($id != null) {

            $this->id = $id;
            $courrier = $this->find(
                    'first', array(
                'conditions' => array(
                    'Courrier.id' => $id
                ),
                'contain' => false
                    )
            );
            if (!empty($courrier['Courrier']['soustype_id'])) {

                $soustype = $this->Soustype->find(
                        'first', array(
                    'conditions' => array(
                        'Soustype.id' => $courrier['Courrier']['soustype_id']
                    ),
                    'contain' => false
                        )
                );

                if (!empty($soustype['Soustype']['delai_nb']) && !empty($soustype['Soustype']['delai_unite'])) {

                    //recuperation de la date du jour
                    $date_act = mktime();

                    //calcul de la date limite
                    $date_courrier_tmp = explode("-", $courrier['Courrier']['datereception']);
                    $date_courrier = mktime(0, 0, 0, $date_courrier_tmp[1], $date_courrier_tmp[2], $date_courrier_tmp[0]);

                    if ($soustype['Soustype']['delai_unite'] == 0) { // 0 : jour
                        $thePHPDate = getdate($date_courrier);
                        $thePHPDate['mday'] = $thePHPDate['mday'] + $soustype['Soustype']['delai_nb'];
                        $date_valid = mktime($thePHPDate['hours'], $thePHPDate['minutes'], $thePHPDate['seconds'], $thePHPDate['mon'], $thePHPDate['mday'], $thePHPDate['year']); // Convert back to timestamp
                    } else if ($soustype['Soustype']['delai_unite'] == 1) { // 1 : semaine
                        $thePHPDate = getdate($date_courrier);
                        $thePHPDate['mday'] = $thePHPDate['mday'] + ($soustype['Soustype']['delai_nb'] * 7);
                        $date_valid = mktime($thePHPDate['hours'], $thePHPDate['minutes'], $thePHPDate['seconds'], $thePHPDate['mon'], $thePHPDate['mday'], $thePHPDate['year']); // Convert back to timestamp
                    } else if ($soustype['Soustype']['delai_unite'] == 2) { // 2 : mois
                        $thePHPDate = getdate($date_courrier);
                        $thePHPDate['mon'] = $thePHPDate['mon'] + $soustype['Soustype']['delai_nb'];
                        $date_valid = mktime($thePHPDate['hours'], $thePHPDate['minutes'], $thePHPDate['seconds'], $thePHPDate['mon'], $thePHPDate['mday'], $thePHPDate['year']); // Convert back to timestamp
                    }

                    //si la date est depassee
                    if ($date_valid < $date_act) {
                        $return = false;
                    }
                }
            }
        }
        return $return;
    }

    /**
     *
     * @param type $id
     * @return type
     */
    public function isClosed($id = null) {
        $return = $this->Bancontenu->find('list', array('conditions' => array('Bancontenu.courrier_id' => $id, 'Bancontenu.etat' => 2), 'recursive' => -1));
        return count($return) > 0;
    }

    /**
     *
     * @param type $id
     * @return string
     */
    public function getDirection($id) {
        $this->id = $id;
        $directionDef = array(
            0 => "S",
            1 => "E",
            2 => "I"
        );
        $direction = $this->field('direction');
        $return = null;
        if (isset($direction) && in_array($direction, array_keys($directionDef))) {
            $return = $directionDef[$direction];
        }
        return $return;
//		$ref = $this->field('reference');
//		$return = '';
//		if (!empty($ref)) {
//			if (preg_match('#^S.*#', $ref)) {
//				$return = "S";
//			}
//			if (preg_match('#^I.*#', $ref)) {
//				$return = "I";
//			}
//			if (preg_match('#^E.*#', $ref)) {
//				$return = "E";
//			}
//		}
//		return $return;
    }

    /**
     *
     * @param type $id
     * @return type
     */
    public function hasAssociatedFile($id) {
        $associatedFiles = $this->Document->find('first', array(
            'fields' => array('id'),
            'conditions' => array('Document.courrier_id' => $id),
            'recursive' => -1
        ));
        return !empty($associatedFiles);
    }

    /**
     * Checks integrity of tree table before modification
     *
     * @param mixed $data
     * @return boolean
     */
    public function checkParadox($data) {
        if (isset($this->data[$this->alias]['id'])) {
            return $data['parent_id'] != $this->data[$this->alias]['id'];
        }
        return true;
    }

    /**
     * Creates a response to Flux and store reference to original
     *
     * @param type $flux
     * @param type $soustype_id
     * @return type
     */
    public function createResponse($flux, $soustype_id) {
        $newFlux = array(
            'Courrier' => array(
                'name' => 'Réponse à ' . $flux['Courrier']['name'],
//              'contactinfo_id' => isset($flux['Courrier']['contactinfo_id']) ? $flux['Courrier']['contactinfo_id'] : null,
                'organisme_id' => isset($flux['Courrier']['organisme_id']) ? $flux['Courrier']['organisme_id'] : null,
                'user_creator_id' => $flux['Courrier']['user_creator_id'],
                'desktop_creator_id' => $flux['Courrier']['desktop_creator_id'],
                'soustype_id' => $soustype_id,
                'parent_id' => $flux['Courrier']['id'],
                'reference' => $this->Soustype->getRef($soustype_id),
                'affairesuiviepar_id' => isset($flux['Courrier']['affairesuiviepar_id']) ? $flux['Courrier']['affairesuiviepar_id'] : null,
                'contact_id' => isset($flux['Courrier']['contact_id']) ? $flux['Courrier']['contact_id'] : null,
                'organisme_id' => isset($flux['Courrier']['organisme_id']) ? $flux['Courrier']['organisme_id'] : null
            )
        );
        $this->create($newFlux);
        return $this->save();
    }

    /**
     * Creates a copy of a Flux and store reference to refused one
     *
     * @param type $flux
     * @return type
     */
    public function createRefus($flux) {
// 		$success = true;


        $newFlux = $flux;
        $newFlux['Courrier']['id'] = null;
        $newFlux['Courrier']['ref_ancien'] = $flux['Courrier']['id'];
        $newFlux['Courrier']['desktop_creator_id'] = $flux['Courrier']['desktop_creator_id'];
        $newFlux['Courrier']['parapheur_etat'] = 1;
        $newFlux['Courrier']['parapheur_id'] = null;
        $newFlux['Courrier']['reference'] = $this->genReferenceUnique();
// 		$newFlux['Document'] = $flux['Document'];
//        if(!empty($flux['Document']) && isset($flux['Document'])) {
//            $newFlux['Document'] = $flux['Document'];
//            $newFlux['Document']['courrier_id'] = $flux['Document'];
//        }
//        $this->begin();
        $this->create($newFlux);
        $data = $this->save();

        $success = !empty($data);
        if ($success) {
            //On récupère les documents liés au courrier nouvellement créé
            if (isset($flux['Document']) && !empty($flux['Document'])) {
                $newDocument['Document'] = $flux['Document'];

                foreach ($newDocument['Document'] as $i => $document) {
                    $document['id'] = null;
                    $document['courrier_id'] = $this->id;
                    $this->Document->create($document);
                    $success = $this->Document->save() && $success;
                }
            }
        }

//		return $this->save();
        return ( $success ? $data : false );
    }

    /**
     *
     * @param type $id
     * @return type
     */
    /* public function genRef($soustype_id) {
      $refInOut = array(
      0 => "S_",
      1 => "E_",
      2 => "I_"
      );
      $compteur = ClassRegistry::init('Compteur');
      $compteur->setDataSource($this->useDbConfig);
      $this->Soustype->setDataSource($this->useDbConfig);

      $stype = $this->Soustype->find('first', array('conditions' => array('Soustype.id' => $soustype_id)));
      return $refInOut[$stype['Soustype']['entrant']] . $compteur->genereCompteur($stype['Soustype']['compteur_id']);
      } */

    /**
     * Génération auto de la référence unique du courrier
     * @param type $id
     * @return string|null
     */
    public function genReferenceUnique($id = null, $data = null) {
        $referenceunique = null;
        $sql = "SELECT nextval('courriers_reference_seq'); /*" . mktime(true) . " " . $this->_lastSequenceReference . "*/";

        // quel est l'année du dernier courrier référencé ?
        $lastNumReference = $this->lastNumReference();

        if (!empty($id)) {
            $courrier = $this->find('first', array(
                'fields' => array(
                    'Courrier.reference',
                    'Courrier.soustype_id'
                ),
                'conditions' => array(
                    'Courrier.id' => $id
                ),
                'contain' => array(
                    'Soustype'
                )
            ));
            $referenceunique = $courrier['Courrier']['reference'];

            $soustypeId = $courrier['Courrier']['soustype_id'];
            if (!empty($data['Courrier']['soustype_id'])) {
                $soustypeId = $data['Courrier']['soustype_id'];
            }

            if (empty($referenceunique)/* && !empty( $soustypeId ) */) {
                // à chaque début d'année, on remet le compteur de référence à zéro
                // si l'année du dernier courrier est != de l'année en cours, on remet à zéro
                if ($lastNumReference != date('Y')) {
                    $sqlNewYear = "ALTER SEQUENCE courriers_reference_seq RESTART WITH 1 ;";
                    $this->query($sqlNewYear);
                }
                $numSeq = $this->query($sql);

                if ($numSeq === false) {
                    return null;
                }
                $referenceunique = date('Y') . sprintf("%06s", $numSeq[0][0]['nextval']);
            }
        } else {

            // à chaque début d'année, on remet le compteur de référence à zéro
            // si l'année du dernier courrier est != de l'année en cours, on remet à zéro
            if ($lastNumReference != date('Y')) {
                $sqlNewYear = "ALTER SEQUENCE courriers_reference_seq RESTART WITH 1 ;";
                $this->query($sqlNewYear);
            }
            $numSeq = $this->query($sql);

            if ($numSeq === false) {
                return null; // FIXME: Throw exception
            }
            $this->_lastSequenceReference = $numSeq[0][0]['nextval'];
            $referenceunique = date('Y') . sprintf("%06s", $numSeq[0][0]['nextval']);
        }

        return $referenceunique;
    }

    /**
     *
     * @param type $id
     * @return type
     */
    public function genRef2($data) {
        $id = $data['Courrier']['id'];
        $courrier = $this->find('first', array(
            'conditions' => array(
                'Courrier.id' => $id
            ),
            'contain' => false
        ));
        return $this->genReferenceUnique($id, $data);
    }

    /**
     *
     * @param type $targetId
     * @return type
     */
    public function cakeflowIsEnd($targetId) {
        $before_num_traitement = $this->Traitement->field('numero_traitement', array('Traitement.target_id' => $targetId));
        $last_num_traitement_query = 'select max(V.numero_traitement) from wkf_traitements T, wkf_visas V where T.id = V.traitement_id and T.target_id = ' . $targetId . ';';
        $last_num_traitement = $this->Traitement->query($last_num_traitement_query);
        return ($before_num_traitement === $last_num_traitement[0][0]['max']);
    }

    /**
     *
     * @param type $fluxId
     * @return type
     */
    private function _cakeflowGetInfo($fluxId) {
        return array(
            'ref_traitement' => $this->Traitement->field('numero_traitement', array('Traitement.target_id' => $fluxId)),
            'desktops' => $this->Traitement->whoIsNext($fluxId),
            'typeEtape' => $this->Traitement->typeEtape($fluxId),
            'treated' => $this->Traitement->field('treated', array('Traitement.target_id' => $fluxId)),
        );
    }

    /**
     *
     * @param type $fluxId
     * @param type $desktopId
     * @return type
     * @throws NotFoundException
     * @throws InvalidArgumentException
     */
    public function cakeflowInsert($fluxId, $desktopOrigId, $insertAuto = 0) {
        $querydata = array(
            'fields' => array(
                'Courrier.id',
                'Courrier.desktop_creator_id',
                'Courrier.ref_ancien',
                'Courrier.parent_id',
                'Courrier.priorite',
                'Courrier.date',
                'Courrier.datereception',
                'Courrier.motifrefus',
                'Courrier.soustype_id',
                'Soustype.circuit_id',
                'Soustype.name'
            ),
            'contain' => false,
            'joins' => array(
                $this->join($this->Soustype->alias)
            ),
            'conditions' => array(
                'Courrier.id' => $fluxId
            )
        );
        $flux = $this->find('first', $querydata);

//debug($flux);
//die();
        if (empty($flux)) {
            throw new NotFoundException();
        }

        //mise a jour de champ desktop_creator_id (utilisé en cas de refus du flux dans un circuit)
        $flux['Courrier']['desktop_creator_id'] = $desktopOrigId;
        $this->create();
        $this->save($flux);


        if ($insertAuto == 0) {
            if ($this->Traitement->targetExists($fluxId)) {
                throw new InvalidArgumentException();
            }
        }

        $valid = array(
            'bancontenuEnd' => false,
            'bancontenuAdd' => false,
            'notification' => true
        );

        $circuitId = $flux['Soustype']['circuit_id'];

        // Si on est la sAERP et que l'on a un sous-typr Fantôme (cloture du flux auto)
        if (Configure::read('Conf.SAERP')) {
            $soustype = $this->Soustype->find(
                    'first', array(
                'conditions' => array(
                    'Soustype.id' => $flux['Courrier']['soustype_id']
                ),
                'contain' => false
                    )
            );
            $soustypeName = $soustype['Soustype']['name'];
            $soustypeFantome = false;
            if ($soustypeName == Configure::read('Soustype.Fantome')) {
                $soustypeFantome = true;
            }

            if ($soustypeFantome) {
                $valid = array(
                    'bancontenuAdd' => true,
                    'Notification' => true
                );
                $valid['bancontenuEnd'] = $this->Bancontenu->fullend($fluxId);
                return !in_array(false, $valid, true);
            }
        }
        // sinon, si on a définit un "sous-type fantôme", on clos le flux automatiquement
        else if( !empty(Configure::read('Soustype.Fantome')) ){
            $soustype = $this->Soustype->find(
                    'first', array(
                'conditions' => array(
                    'Soustype.id' => $flux['Courrier']['soustype_id']
                ),
                'contain' => false
                    )
            );
            $soustypeName = $soustype['Soustype']['name'];
            $soustypeFantome = false;
            if ($soustypeName == Configure::read('Soustype.Fantome')) {
                $soustypeFantome = true;
            }

            if ($soustypeFantome) {
                $valid = array(
                    'bancontenuAdd' => true,
                    'Notification' => true
                );
                $valid['bancontenuEnd'] = $this->Bancontenu->fullend($fluxId);
                return !in_array(false, $valid, true);
            }
        }


        if ($this->Traitement->Circuit->insertDansCircuit($circuitId, $fluxId)) {
            $after = $this->_cakeflowGetInfo($fluxId);
            if ($insertAuto != 1 && $after['desktops'][0] != -1) {
                $valid['bancontenuEnd'] = $this->Bancontenu->end($desktopOrigId, BAN_DOCS, $fluxId);
            }


            // Si le premier bureau est un parapheur, on exécute le traitement pour verser au parpaheur
            if ($after['desktops'][0] == -1) {
                $traitement = $this->Traitement->find('first', array(
                    'recursive' => -1,
                    'fields' => array('id', 'numero_traitement', 'treated', 'circuit_id'),
                    'conditions' => array(
                        'target_id' => $fluxId,
                        'treated' => '0'
                    )
                ));
                $success = $this->Traitement->delegSiBesoin($fluxId, $traitement);

                if ($success !== true) {
                    $ret = $success;
                    $this->Traitement->delete($traitement['Traitement']['id']);
                    return $ret;
                }

                if ($insertAuto != 1) {
                    $valid['bancontenuEnd'] = $this->Bancontenu->end($desktopOrigId, BAN_DOCS, $fluxId);
                }

                // Ajout dans le bancontenu si parapheur = 1ere etape
                $validBancontenuAdd = array();
                $validBancontenuAdd[] = $this->Bancontenu->add($after['desktops'][0], BAN_FLUX, $fluxId);


//                        if(!empty($flux['Courrier']['parent_id'])) {
                if (!empty($flux['Courrier']['motifrefus'])) {
                    $valid['bancontenuEnd'] = $this->Bancontenu->end($desktopOrigId, BAN_REFUS, $fluxId);
                }
                $valid['bancontenuAdd'] = !in_array(false, $validBancontenuAdd, true);
                //                $valid['bancontenuAdd'] = true;
                $valid['notification'] = true;
            } else {

                $notifyDesktops = array();
                $validBancontenuAdd = array();
//                        for ($i = 0; $i < count($after['desktops']); $i++) {
//                                $validBancontenuAdd[] = $this->Bancontenu->add($after['desktops'][$i], BAN_FLUX, $fluxId);
//                                $notifyDesktops[$after['desktops'][$i]] = array($fluxId);
//                        }
//debug( $after['desktops'] );    
                $desktopsIds = array();
                foreach ($after['desktops'] as $i => $desktopmanager) {
                    $this->Desktopmanager = ClassRegistry::init('Desktopmanager');
                    $desktopsIds[] = $this->Desktopmanager->getDesktops($desktopmanager);
                }


                foreach ($desktopsIds as $i => $desktopId) {
                    for ($i = 0; $i < count($desktopId); $i++) {
//debug($desktopId[$i]);
                        $validBancontenuAdd[] = $this->Bancontenu->add($desktopId[$i], BAN_FLUX, $fluxId);
                        $notifyDesktops[$desktopId[$i]] = array($fluxId);

                        $notifInfos = $this->Notifieddesktop->Notification->setNotification('wkf_new');
                        $valid['notification'] = $this->Notifieddesktop->Notification->add($notifInfos['name'], $notifInfos['description'], $notifInfos['typeNotif'], $desktopId[$i], $notifyDesktops);
                    }
                }
//debug($validNotificationAdd);
                // On termine le flux dans la bannette flux refusé à retraiter

                if (!empty($flux['Courrier']['ref_ancien']) || !empty($flux['Courrier']['parent_id'])) {
                    /* $fluxRef = $this->find(
                      'first',
                      array(
                      'fields' => array(
                      'Courrier.motifrefus'
                      ),
                      'conditions' => array(
                      'Courrier.id' => $flux['Courrier']['ref_ancien']
                      )
                      )
                      ); */
                    if (!empty($flux['Courrier']['motifrefus'])) {
                        $valid['bancontenuEnd'] = $this->Bancontenu->end($desktopOrigId, BAN_REFUS, $fluxId);
                    } else { //FIXME à vérifier
                        $valid['bancontenuEnd'] = $this->Bancontenu->end($desktopOrigId, BAN_REPONSES, $fluxId);
                    }
                }

                $valid['bancontenuAdd'] = !in_array(false, $validBancontenuAdd, true);
            }

//         debug($valid);       
            /*
             * 
             */
            $banettesActuelles = $this->Bancontenu->find(
                    'all', array(
                'conditions' => array(
                    'Bancontenu.courrier_id' => $fluxId,
                    'Bancontenu.etat' => 1,
                    'Bancontenu.bannette_id' => array(1, 5)
                ),
                'contain' => false
                    )
            );
            $oldBannettesDesktopsIds = Hash::extract($banettesActuelles, '{n}.Bancontenu.desktop_id');

            // Dans le cas où le même flux est chez plusieurs utilisateurs, 
            // on l'enlève des bannetes car il a été traité par l'un d'entre eux
            if (!empty($oldBannettesDesktopsIds)) {
                foreach ($oldBannettesDesktopsIds as $j => $otherDesktopId) {

                    $desktops = $this->Desktop->find(
                            'all', array(
                        'conditions' => array(
                            'Desktop.id' => $otherDesktopId
                        ),
                        'contain' => false
                            )
                    );

                    if ($desktops[$j]['Desktop']['group_id'] == DISP_GID) {
                        $BAN = BAN_AIGUILLAGE;
                    } else if ($desktops[$j]['Desktop']['group_id'] == INIT_GID) {
                        $BAN = BAN_DOCS;
                    }

                    $valid['bancontenuEnd'] = $this->Bancontenu->end($otherDesktopId, $BAN, $fluxId);
                    if( !empty($BAN) ) {
                        $valid['bancontenuEnd'] = $this->Bancontenu->end($otherDesktopId, $BAN, $fluxId);
                    }
                }
            }


            // Envoi du mail au destinataire présent
            $destinataires = $this->Traitement->whoIsNext($fluxId);
            foreach ($destinataires as $destinataire_id) {
                $this->Desktopmanager = ClassRegistry::init('Desktopmanager');
                $desktopsIds = $this->Desktopmanager->getDesktops($destinataire_id);
                foreach ($desktopsIds as $deskId) {
                    $user_id = $this->User->userIdByDesktop($deskId);
                    $typeNotif = $this->User->find(
                            'first', array(
                        'fields' => array('User.typeabonnement'),
                        'conditions' => array('User.id' => $user_id),
                        'contain' => false
                            )
                    );
                    $notificationId = $this->Notifieddesktop->Notification->id;
                    if ($typeNotif['User']['typeabonnement'] == 'quotidien') {
                        $this->User->saveNotif($fluxId, $user_id, $notificationId, 'traitement');
                    } else {
                        $this->User->notifier($fluxId, $user_id, $notificationId, 'traitement');
                    }
                }
            }
        }
//debug($valid);
        return !in_array(false, $valid, true);
    }

    /**
     *
     * @param type $code
     * @param type $desktopId
     * @param type $fluxId
     * @param type $options
     * @return type
     */
    public function cakeflowExecuteSend($code, $desktopId, $fluxId, $options) {
        return $this->cakeflowExecute($code, $desktopId, $fluxId, $options);
    }

    /**
     *
     * @param type $desktopId
     * @param type $fluxId
     * @param type $response
     * @param type $stype_id
     * @return type
     */
    public function cakeflowExecuteNext($desktopId, $fluxId, $response = 0, $stype_id = null, $insertAuto = 0) {
        return $this->cakeflowExecute('OK', $desktopId, $fluxId, array(), $response, $stype_id, $insertAuto);
    }

    /**
     *
     * @param type $code
     * @param type $desktopId
     * @param type $fluxId
     * @param type $options
     * @param type $response
     * @param type $stype_id
     * @return type
     * @throws BadMethodCallException
     * @throws InvalidArgumentException
     */
    public function cakeflowExecute($code, $desktopOrigId, $fluxId, $options = array(), $response = 0, $stype_id = null, $insertAuto = 0) {
//		if (!in_array($code, array('IP', 'IL', 'OK', 'KO')) || $fluxId <= 0 || $desktopOrigId <= 0) {
        if (!in_array($code, array('IP', 'IL', 'OK', 'KO')) || $fluxId <= 0 || $desktopOrigId < -1) {
            throw new BadMethodCallException();
        }

        if (!$this->Traitement->targetExists($fluxId)) {
            throw new InvalidArgumentException();
        }

        $valid = array(
            'cakeflow' => false,
            'notification' => false,
            'bancontenu_treated' => false
        );

        $parapheurDesktopId = '';
        if( $desktopOrigId == '-1' ) {
            $parapheurDesktopId = $desktopOrigId;
        }

//		$expDesktopId = CakeSession::read('Auth.User.Courrier.Desktop.id');

        $fluxOrigine = $this->find('first', array(
            'fields' => array(
                'Courrier.id',
                'Courrier.desktop_creator_id',
                'Courrier.reference'
            ),
            'conditions' => array(
                'Courrier.id' => $fluxId
            ),
            'contain' => false,
            'recursive' => -1
        ));
//         debug($fluxOrigine);
        // -- -- -- -- -- -- --
        // Traitement Cakeflow
        // -- -- -- -- -- -- --
        $desktopsManagersIds = $this->Desktop->getDesktopManager($desktopOrigId);
//$this->log($desktopOrigId);
        // Si on est en admin et qu'on tente de pousser un flux à la palce d'un autre compte
        // on prend l'ID de ce bureau au lieu de celui de l'admin
        $secondaryDesktops = CakeSession::read('Auth.User.SecondaryDesktops');
        if( !empty($secondaryDesktops)) {
            $desktopName = Hash::extract($secondaryDesktops, '{n}.Group.name');   
        }
        else {
            $desktopName = array();
        }
        $isAdmin = false;
        $groupName = CakeSession::read('Auth.User.Desktop.Group');
        if (in_array('Admin', $desktopName) || !empty($groupName) && $groupName['name'] == 'Admin') {
            $isAdmin = true;
        }
        if( $isAdmin ) {
             $desktopsManagersIds = $this->Traitement->whoIs($fluxId);
        }

//$this->log($desktopsManagersIds);
        $before = $this->_cakeflowGetInfo($fluxId);
        $cakeflowReturn = $this->Traitement->execute($code, $desktopsManagersIds, $fluxId, $options);
        $after = $this->_cakeflowGetInfo($fluxId);
//debug($cakeflowReturn);
//$this->log($before);
//$this->log($after);
        /*
         *  Ajout suite à modif circuit
         */
        /// FIXME
        $desktopsIds = array();
        if (!empty($after['desktops'])) {
            foreach ($after['desktops'] as $i => $desktopmanager) {
                $this->Desktopmanager = ClassRegistry::init('Desktopmanager');
                $desktopsIds[] = $this->Desktopmanager->getDesktops($desktopmanager);
            }
        } else {
            $desktopsIds[] = $after['desktops'];
        }

        $desktopsExpIds = array();
        if (!empty($before['desktops'])) {
            foreach ($before['desktops'] as $i => $desktops) {
                $this->Desktopmanager = ClassRegistry::init('Desktopmanager');
                $desktopsExpIds[] = $this->Desktopmanager->getDesktops($desktops);
            }
        } else {
            $desktopsExpIds[] = $before['desktops'];
        }
        /*
         * Fin ajout
         */

//debug($desktopsExpIds);
//debug($desktopsIds);
//die();
//        $expDesktopId = @$before['desktops'][0];

        $remainingDesktops = array_intersect($before['desktops'], $after['desktops']);
        $diffDesktops = array_diff($before['desktops'], $after['desktops']);

        if ($before['typeEtape'] == ETAPEET) {
            $valid['cakeflow'] = ($before['ref_traitement'] == $after['ref_traitement']) && !empty($diffDesktops);
        } else {
            $valid['cakeflow'] = ($before['ref_traitement'] != $after['ref_traitement']);
        }
        
        // FIXME: à tester et vérifier, notamment pour le traitement par lots
        if( Configure::read('Conf.SAERP') ) {
            $valid['cakeflow'] = true;
        }

        if (is_bool($cakeflowReturn)) {
            $valid['cakeflow'] = true;
        } else {
            $ret = $cakeflowReturn;
            return $ret;
        }

        // -- -- -- -- -- -- -- -- --
        // Notification et Bannettes
        // -- -- -- -- -- -- -- -- --
        //preparation
        $notifyDesktops = array();
        $notifInfos = array(
            'name' => '',
            'description' => '',
            'typeNotif' => ''
        );
        if ($code == 'KO') {
            $desktop = $this->field('Courrier.desktop_creator_id', array('Courrier.id' => $fluxId));
            $notifyDesktops[$desktop] = array($fluxId);
            $notifInfos = $this->Notifieddesktop->Notification->setNotification('ban_cancel');
            // On envoie un mail pour le refus
//            $destinataires = $this->Traitement->whoIs($fluxId, 'in', array('OK','IN'));
//            foreach ($destinataires as $destinataire_id) {
//                $this->User->notifier($nouvelId, $destinataire_id, 'refus');
//            }
        } else if ($code == 'OK') {
            //--------
            //Etape ET
            //--------
            if ($before['typeEtape'] == ETAPEET) {
//				for ($i = 0; $i < count($after['desktops']); $i++) {
//					$notifyDesktops[$after['desktops'][$i]] = array($fluxId);
//				}
                // FIXME: modification ici
                foreach ($desktopsIds as $i => $desktopId) {
                    for ($i = 0; $i < count($desktopId); $i++) {
                        $notifyDesktops[$desktopId[$i]] = array($fluxId);
                    }
                }

                if (empty($remainingDesktops)) {
                    $notifInfos = $this->Notifieddesktop->Notification->setNotification('wkf_new');
                } else {
                    //si en attente
                    $notifInfos = $this->Notifieddesktop->Notification->setNotification('wkf_wait');
                }
                //--------
                //Etape OU
                //--------
            } else if ($before['typeEtape'] == ETAPEOU) {
//				for ($i = 0; $i < count($after['desktops']); $i++) {
//					$notifyDesktops[$after['desktops'][$i]] = array($fluxId);
//				}
                // FIXME: modification ici
                foreach ($desktopsIds as $i => $desktopId) {
                    for ($i = 0; $i < count($desktopId); $i++) {
                        $notifyDesktops[$desktopId[$i]] = array($fluxId);
                    }
                }
                $notifInfos = $this->Notifieddesktop->Notification->setNotification('wkf_new');
                //------------
                //Etape Simple
                //------------
            } else if ($before['typeEtape'] == ETAPESIMPLE) {//Etape simple
                $notifInfos = $this->Notifieddesktop->Notification->setNotification('wkf_new'); //FIXME parpaheur ?
//				for ($i = 0; $i < count($after['desktops']); $i++) {
//					$notifyDesktops[$after['desktops'][$i]] = array($fluxId);
//				}
                // FIXME: modification ici
                foreach ($desktopsIds as $i => $desktopId) {
                    for ($i = 0; $i < count($desktopId); $i++) {
                        $notifyDesktops[$desktopId[$i]] = array($fluxId);
                    }
                }
            }
            //----------------
            //Rebond et Retour
            //----------------
        } else if ($code == 'IL' || $code == 'IP') {
            $notifInfos = $this->Notifieddesktop->Notification->setNotification('wkf_jmp');
            if ($code == 'IL') {
                $notifInfos = $this->Notifieddesktop->Notification->setNotification('wkf_jbk');
            }
//			for ($i = 0; $i < count($after['desktops']); $i++) {
//				$notifyDesktops[$after['desktops'][$i]] = array($fluxId);
//			}
            // FIXME: modification ici
            foreach ($desktopsIds as $i => $desktopId) {
                for ($i = 0; $i < count($desktopId); $i++) {
                    $notifyDesktops[$desktopId[$i]] = array($fluxId);
                }
            }
        } else {
            throw new BadMethodCallException();
        }

        // -- -- -- --
        // Traitement
        // -- -- -- --
        //si le flux est traité
        if ($after['treated'] && $code == 'OK') {
            $this->Bancontenu->fullend($fluxId); // TODO: problème lors de la finalisation suite à récupération parapheur
            //reponse
            if ($response != 0) {
                $flux = $this->find('first', array('conditions' => array('Courrier.id' => $fluxId)));
                $newFlux = $this->createResponse($flux, $stype_id);
                $valid['response_copy'] = false;
                if (!empty($newFlux)) {
                    $valid['response_copy'] = true;
                    if ($insertAuto == 0) {
                        $valid['bancontenu_treated'] = $valid['bancontenu'] = $this->Bancontenu->add($newFlux['Courrier']['desktop_creator_id'], BAN_REPONSES, $newFlux['Courrier']['id']);
                        $notifInfos = $this->Notifieddesktop->Notification->setNotification('ban_response');
                        $notifyDesktops = array($newFlux['Courrier']['desktop_creator_id'] => array($newFlux['Courrier']['id']));
//                        if( !empty($expDesktopId)) {
                        if (!empty($desktopsExpIds)) {
//                            $valid['notification'] = $this->Notifieddesktop->Notification->add($notifInfos['name'], $notifInfos['description'], $notifInfos['typeNotif'], $expDesktopId, $notifyDesktops);
                            // FIXME
                            foreach ($desktopsExpIds as $k => $deskId) {
                                for ($i = 0; $i < count($deskId); $i++) {

                                    $valid['notification'] = $this->Notifieddesktop->Notification->add($notifInfos['name'], $notifInfos['description'], $notifInfos['typeNotif'], $deskId[$i], $notifyDesktops);
                                }
                            }
                        } else {
                            $valid['notification'] = true;
                        }

                        // Flux à insérer
                        $user_id = $this->User->userIdByDesktop($newFlux['Courrier']['desktop_creator_id']);
                        $typeNotif = $this->User->find(
                                'first', array(
                            'fields' => array('User.typeabonnement'),
                            'conditions' => array('User.id' => $user_id),
                            'contain' => false
                                )
                        );
                        $notificationId = $this->Notifieddesktop->Notification->id;
                        if ($typeNotif['User']['typeabonnement'] == 'quotidien') {
                            $this->User->saveNotif($newFlux['Courrier']['id'], $user_id, $notificationId, 'insertion_reponse');
                        } else {
                            $this->User->notifier($newFlux['Courrier']['id'], $user_id, $notificationId, 'insertion_reponse');
                        }
                    } else {
                        $notifInfos = $this->Notifieddesktop->Notification->setNotification('ban_response');
                        $notifyDesktops = array($newFlux['Courrier']['desktop_creator_id'] => array($newFlux['Courrier']['id']));
                        $valid['bancontenu_treated'] = true;
//                        $this->cakeflowInsert($newFlux['Courrier']['id'], $expDesktopId, $insertAuto);
                        // FIXME
                        if (!empty($desktopsExpIds)) {
                            foreach ($desktopsExpIds as $k => $deskId) {
                                for ($i = 0; $i < count($deskId); $i++) {
                                    $this->cakeflowInsert($newFlux['Courrier']['id'], $deskId[$i], $insertAuto);
                                    $valid['notification'] = $this->Notifieddesktop->Notification->add($notifInfos['name'], $notifInfos['description'], $notifInfos['typeNotif'], $deskId[$i], $notifyDesktops);
                                }
                            }
                        }

                        // Flux insérer directement donc à traiter
                        /* $destinataires = $this->Traitement->whoIsNext($newFlux['Courrier']['id']);
                          //debug( $destinataires);
                          foreach ($destinataires as $destinataire_id) {
                          $user_id = $this->User->userIdByDesktop($destinataire_id);
                          $this->User->notifier($newFlux['Courrier']['id'], $user_id, 'traitement');
                          } */

                        $destinataires = $this->Traitement->whoIsNext($newFlux['Courrier']['id']);
                        foreach ($destinataires as $destinataire_id) {
                            $this->Desktopmanager = ClassRegistry::init('Desktopmanager');
                            $desktopsToSendIds = $this->Desktopmanager->getDesktops($destinataire_id);
                            foreach ($desktopsToSendIds as $deskId) {
                                if ($deskId != -1) {
                                    $user_id = $this->User->userIdByDesktop($deskId);
                                    if (!empty($user_id)) {
                                        $typeNotif = $this->User->find(
                                                'first', array(
                                            'fields' => array('User.typeabonnement'),
                                            'conditions' => array('User.id' => $user_id),
                                            'contain' => false
                                                )
                                        );
                                        $notificationId = $this->Notifieddesktop->Notification->id;
                                        if ($typeNotif['User']['typeabonnement'] == 'quotidien') {
                                            $this->User->saveNotif($fluxId, $user_id, $notificationId, 'traitement');
                                        } else {
                                            $this->User->notifier($fluxId, $user_id, $notificationId, 'traitement');
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            } else {
                $valid['notification'] = true;
                $valid['bancontenu_treated'] = true;
                // Flux traité
            }
        } else {
            //Dans le cas d une etape de type ou, il faut marquer comme lues les notifications de tous les utilisateurs des bureaux de l'etape validée.
            //Il faut aussi changer l'etat de bancontenu de la meme maniere
            if ($before['typeEtape'] == ETAPEOU) {
                $validEtapeOuNotif = array();
                $validEtapeOuBan = array();
                foreach ($before['desktops'] as $prevDesktop) {
                    $desktopsEtapeOuIds = $this->Desktopmanager->getDesktops($prevDesktop);
                    foreach( $desktopsEtapeOuIds as $k => $deskOuId ) {
                        $valid['readnotification'] = $this->Notifieddesktop->Notification->setReadByDesktopCourrier($deskOuId, $fluxId);
                        $valid['bancontenu_treated'] = $this->Bancontenu->end($deskOuId, BAN_FLUX, $fluxId);
                    }
                }
                $valid['readnotification'] = !in_array(false, $validEtapeOuNotif, true);
                $valid['bancontenu_treated'] = !in_array(false, $validEtapeOuBan, true);
            } else { //sinon dans tous les autres cas, on marque comme lue la notification de l'utilisateur ayant validé et on change l'état de bancontenu
//				$valid['readnotification'] = $this->Notifieddesktop->Notification->setReadByDesktopCourrier($expDesktopId, $fluxId);
//				$valid['bancontenu_treated'] = $this->Bancontenu->end($expDesktopId, BAN_FLUX, $fluxId);
//$this->log($before);
//$this->log($after);
//                if( Configure::read('Conf.SAERP') ) {
                    if($before['typeEtape'] == ETAPEET) {
                        $newdiffDesktops = $this->Traitement->whichIsPrevious($fluxId);
                        if( !empty($newdiffDesktops) ){
                            $this->Desktopmanager = ClassRegistry::init('Desktopmanager');
                            $desktopsEtapeEtIds = $this->Desktopmanager->getDesktops($newdiffDesktops);
                            foreach( $desktopsEtapeEtIds as $k => $deskEtId ) {
                                $valid['readnotification'] = $this->Notifieddesktop->Notification->setReadByDesktopCourrier($deskEtId, $fluxId);
                                $valid['bancontenu_treated'] = $this->Bancontenu->end($deskEtId, BAN_FLUX, $fluxId);
                            }
                        }
                    }
                    
                    if($before['typeEtape'] == ETAPESIMPLE) {
                        foreach ($before['desktops'] as $beforeDesktop) {
                            $desktopsEtapeSimpleIds = $this->Desktopmanager->getDesktops($beforeDesktop);
                            foreach( $desktopsEtapeSimpleIds as $k => $deskSimpleId ) {
                                $valid['readnotification'] = $this->Notifieddesktop->Notification->setReadByDesktopCourrier($deskSimpleId, $fluxId);
                                $valid['bancontenu_treated'] = $this->Bancontenu->end($deskSimpleId, BAN_FLUX, $fluxId);
                            }
                        }
                    }
//                }
//                else {
                    if (!empty($desktopsExpIds)) {
                        foreach ($desktopsExpIds as $k => $deskId) {

                            for ($i = 0; $i < count($deskId); $i++) {
                                $deskInBancontenu[$i] = $this->Bancontenu->find(
                                        'first', array(
                                    'conditions' => array(
                                        'Bancontenu.desktop_id' => $deskId[$i],
                                        'Bancontenu.bannette_id' => BAN_FLUX,
                                        'Bancontenu.courrier_id' => $fluxId
                                    ),
                                    'contain' => false
                                        )
                                );

                                if (!empty($deskInBancontenu[$i])) {
                                    $valid['readnotification'] = $this->Notifieddesktop->Notification->setReadByDesktopCourrier($deskId[$i], $fluxId);
                                    $valid['bancontenu_treated'] = $this->Bancontenu->end($deskId[$i], BAN_FLUX, $fluxId);
                                } else {
                                    $valid['readnotification'] = $this->Notifieddesktop->Notification->setReadByDesktopCourrier($deskId[$i], $fluxId);
                                    $valid['bancontenu_treated'] = true;
                                }
                            }
                        }
                    }
//                }
                if( !empty($parapheurDesktopId ) && $parapheurDesktopId == '-1' ) {
                    $deskParapheurInBancontenu = $this->Bancontenu->find(
                        'first',
                        array(
                            'conditions' => array(
                                'Bancontenu.desktop_id' => $parapheurDesktopId,
                                'Bancontenu.bannette_id' => BAN_FLUX,
                                'Bancontenu.courrier_id' => $fluxId
                            ),
                            'contain' => false
                        )
                    );

                    if (!empty($deskParapheurInBancontenu)) {
                        $valid['readnotification'] = true;
                        $valid['bancontenu_treated'] = $this->Bancontenu->end($parapheurDesktopId, BAN_FLUX, $fluxId);
                    } else {
                        $valid['readnotification'] = true;
                        $valid['bancontenu_treated'] = true;
                    }
                }
            }
            //Notification pour les bureaux de l'etape en cours (nouvelle etape, ou etape actuelle dans le cas d une etape de type ET)
//            if( !empty( $expDesktopId )) {
//                if ($this->Notifieddesktop->Notification->add($notifInfos['name'], $notifInfos['description'], $notifInfos['typeNotif'], $expDesktopId, $notifyDesktops)) {
//                        $valid['notification'] = true;
//                }
            if (!empty($desktopsExpIds)) {
                foreach ($desktopsExpIds as $k => $deskId) {
                    for ($i = 0; $i < count($deskId); $i++) {
                        if ($this->Notifieddesktop->Notification->add($notifInfos['name'], $notifInfos['description'], $notifInfos['typeNotif'], $deskId[$i], $notifyDesktops)) {
                            $valid['notification'] = true;
                        }
                    }
                }
            } else {
                $valid['notification'] = true;
            }
            //On met à jour les contenus de bannette pour les bureaux de l etape suivante
            //Si on est à une étape de type ET et que tous les bureaux ont validé ou si etape autre que ET
            if (($before['typeEtape'] == ETAPEET && $before['ref_traitement'] != $after['ref_traitement']) || $before['typeEtape'] != ETAPEET && !Configure::read('Conf.SAERP')) {
                $validEtapeEtBan = array();
                // utiliser ici pour ajouter le flux dans la bannette du bureau ajouté par le module Envoyer à
                foreach ($desktopsIds as $i => $desktopId) {
                    for ($i = 0; $i < count($desktopId); $i++) {
                        if ($desktopId[$i] != -1) {
                            $validEtapeEtBan[] = $this->Bancontenu->add($desktopId[$i], BAN_FLUX, $fluxId);
                        } else {
                            $validEtapeEtBan[] = true;
                            $valid['notification'] = true;
                        }
                    }
                }
                $valid['bancontenu'] = !in_array(false, $validEtapeEtBan, true);
            }
            if ( Configure::read('Conf.SAERP') ) {
                $validEtapeEtBan = array();
                // utiliser ici pour ajouter le flux dans la bannette du bureau ajouté par le module Envoyer à
                foreach ($desktopsIds as $i => $desktopId) {
                    for ($i = 0; $i < count($desktopId); $i++) {
                        if ($desktopId[$i] != -1) {
                            $validEtapeEtBan[] = $this->Bancontenu->add($desktopId[$i], BAN_FLUX, $fluxId);
                        } else {
                            $validEtapeEtBan[] = true;
                            $valid['notification'] = true;
                        }
                    }
                }
                $valid['bancontenu'] = !in_array(false, $validEtapeEtBan, true);
            }

            // Ajout pour les notifs durant le traitement
            // Ajout pour les notifs durant le traitement
            $destinataires = $this->Traitement->whoIsNext($fluxId);
            foreach ($destinataires as $destinataire_id) {
                $this->Desktopmanager = ClassRegistry::init('Desktopmanager');
                $desktopsToSendIds = $this->Desktopmanager->getDesktops($destinataire_id);
                foreach ($desktopsToSendIds as $deskId) {
                    if ($deskId != -1) {
                        $user_id = $this->User->userIdByDesktop($deskId);
                        if (!empty($user_id)) {
                            $typeNotif = $this->User->find(
                                    'first', array(
                                'fields' => array('User.typeabonnement'),
                                'conditions' => array('User.id' => $user_id),
                                'contain' => false
                                    )
                            );
                            $notificationId = $this->Notifieddesktop->Notification->id;
                            if ($typeNotif['User']['typeabonnement'] == 'quotidien') {
                                $this->User->saveNotif($fluxId, $user_id, $notificationId, 'traitement');
                            } else {
                                $this->User->notifier($fluxId, $user_id, $notificationId, 'traitement');
                            }
//                            $this->User->notifier($fluxId, $user_id, 'traitement');
                        }
                    }
                }
            }
            // Ajout pour les notifs durant le traitement
//            $destinataires = $this->Traitement->whoIsNext($fluxId);
//            foreach ($destinataires as $destinataire_id) {
//                if($destinataire_id != -1) {
//                    $user_id = $this->User->userIdByDesktop($destinataire_id);
//                    if( !empty($user_id)) {
//                            $this->User->notifier($fluxId, $user_id, 'traitement');
//                    }
//                }
//            }
        }
//debug($valid);
        // En fin de traitement, on met à jour le bancontenu de l'agent du bureau ayant réelleent validé le flux
        if( $valid['bancontenu_treated'] == true || $valid['bancontenu_treated'] == 1 ) {
            $this->Bancontenu->updateAll(
                array(
                    'Bancontenu.modified' => "'".date( 'Y-m-d H:i:s')."'"
                ),
                array(
                    'Bancontenu.courrier_id' => $fluxId,
                    'Bancontenu.desktop_id' =>  $desktopOrigId
                )
            );
        }
        $this->log(CakeSession::read('Auth.User.username') );
        $this->log($fluxOrigine['Courrier']['id'], LOG_ERROR);
        $this->log($fluxOrigine['Courrier']['reference'], LOG_ERROR);
        return !in_array(false, $valid, true);
    }

    /**
     *
     * @param type $desktopId
     * @param type $fluxId
     * @param type $destId
     * @return type
     */
    public function copy($desktopId, $fluxId, $destId) {

        $notifInfos = $this->Notifieddesktop->Notification->setNotification('ban_copy');
//		$notifyDesktops = array(
//			$destId => array($fluxId)
//        );       
//        $notified = $this->Notifieddesktop->Notification->add($notifInfos['name'], $notifInfos['description'], $notifInfos['typeNotif'], $desktopId, $notifyDesktops);
//
//        $valid = array(
//            'notification' => !empty($notified),
//            'bancontenu' => $this->Bancontenu->add($destId, BAN_COPY, $fluxId),
//        );

        $desktopmanagerId = $destId;
        $desktopsIds[] = $this->Desktop->Desktopmanager->getDesktops($desktopmanagerId);
        foreach ($desktopsIds as $i => $dId) {

            for ($i = 0; $i < count($dId); $i++) {
                $validBancontenuAdd[] = $this->Bancontenu->add($dId[$i], BAN_COPY, $fluxId);
                $notifyDesktops[$dId[$i]] = array($fluxId);
            }
        }

        $valid['bancontenu'] = !in_array(false, $validBancontenuAdd, true);
        $valid['notification'] = $this->Notifieddesktop->Notification->add($notifInfos['name'], $notifInfos['description'], $notifInfos['typeNotif'], $desktopId, $notifyDesktops);
//debug($desktopsIds);
//debug($valid);
        return !in_array(false, $valid, true);
    }

    /**
     *
     * @param type $id
     * @return type
     */
    public function getLightMainDoc($id) {
        $qdMainDoc = array(
            'recursive' => -1,
            'fields' => array(
                'Document.id',
                'Document.courrier_id',
                'Document.name',
                'Document.ext',
//                    'Document.content',
//                    'Document.gedpath',
                'Document.size',
                'Document.mime',
                'Document.main_doc',
                'Document.desktop_creator_id'
            ),
//			'fields' => $this->Document->getLightFields(),
            'conditions' => array(
                'Document.courrier_id' => $id,
                'Document.main_doc' => true
            ),
            'contain' => false
        );
// var_dump( $this->Document->find('first', $qdMainDoc) );
        $document = $this->Document->find('first', $qdMainDoc);
        if (!empty($document)) {
            $document['Document']['isgenerated'] = false;
            if (is_dir(WEBDAV_DIR . DS . $id)) {
                if ($document['Document']['ext'] === 'odt') {
                    $document['Document']['isgenerated'] = true;
                }
            }
        } else {
            $document = array();
        }

//		return $this->Document->find('first', $qdMainDoc);
        return $document;
    }

    /**
     *
     * @param type $id
     * @return type
     */
    public function getMainDoc($id) {
        $qdMainDoc = array(
            'recursive' => -1,
            'fields' => array(
                'Document.id',
                'Document.courrier_id',
                'Document.name',
                'Document.ext',
                'Document.content',
                'Document.gedpath',
                'Document.size',
                'Document.mime',
                'Document.main_doc',
                'Document.desktop_creator_id'
            ),
//			'fields' => $this->Document->getLightFields(),
            'conditions' => array(
                'Document.courrier_id' => $id,
                'Document.main_doc' => true
            ),
            'contain' => false
        );
// var_dump( $this->Document->find('first', $qdMainDoc) );
        $document = $this->Document->find('first', $qdMainDoc);
        if (!empty($document)) {
            $document['Document']['isgenerated'] = false;
            if (is_dir(WEBDAV_DIR . DS . $id)) {
                if ($document['Document']['ext'] === 'odt') {
                    $document['Document']['isgenerated'] = true;
                }
            }
        } else {
            $document = array();
        }
//		return $this->Document->find('first', $qdMainDoc);
        return $document;
    }

    /**
     *
     * @param type $id
     * @return type
     */
    public function getDocs($id) {
        $qdMainDoc = array(
            'recursive' => -1,
            'fields' => $this->Document->getLightFields(),
            'conditions' => array(
                'Document.courrier_id' => $id,
                'Document.main_doc' => false
            )
        );
        return $this->Document->find('all', $qdMainDoc);
    }

    /**
     *
     * @param type $id
     * @return type
     */
    public function getDocsSignes($id) {
        $qdDocSigne = array(
            'recursive' => -1,
            'fields' => $this->Document->getLightFields(),
            'conditions' => array(
                'Document.courrier_id' => $id,
                'Document.main_doc' => true,
                'Document.parapheur_bordereau <>' => null
            )
        );
        return $this->Document->find('first', $qdDocSigne);
    }

    /**
     * Génération d'un document lié (à partir du script lancé à la main)
     *
     * @access private
     * @param base64 $fichier_associe contenu du document lié
     * @param integer $fluxId identifiant du flux
     * @param array $return valeur de retour
     * @return void
     */
    private function _genFileAssoc($fichier_associe, $fluxId, &$return, $main_doc = true) {

        $finfo = new finfo(FILEINFO_MIME_TYPE);

        $fileMimeType = $finfo->buffer($fichier_associe);

//		$filename = "fichier_" . date('Ymd_His'). "_" .mktime(true);

        $sql = "SELECT nextval('documents_name_seq'); /*" . mktime(true) . " " . $this->_lastSequenceDocument . "*/";



        $fileMimeTypeError = false;
        if ($fileMimeType == "application/pdf") {
            $fileExt = "pdf";
        } else if ($fileMimeType == "application/vnd.oasis.opendocument.text") {
            $fileExt = "odt";
        } else if ($fileMimeType == "image/bmp") {
            $fileExt = "bmp";
        } else if ($fileMimeType == "image/tiff") {
            $fileExt = "tif";
        } else if ($fileMimeType == "text/plain") {
            $fileExt = "txt";
        } else if ($fileMimeType == "application/vnd.ms-office") {
            $fileExt = "doc";
        } else if ($fileMimeType == "application/msword") {
            $fileExt = "doc";
        } else if ($fileMimeType == "application/rtf") {
            $fileExt = "rtf";
        } else {
            $fileMimeTypeError = true;
        }


        if ($fileMimeTypeError) {
            $return['CodeRetour'] .= "FILENOK";
            $return['Message'] .= ". Le fichier n'est pas reconnu";
        } else {

            $numSeqDoc = $this->query($sql);
            if ($numSeqDoc === false) {
                return null; // FIXME: Throw exception
            }
            $this->_lastSequenceDocument = $numSeqDoc[0][0]['nextval'];
            $filename = date('Y-m-d') . "_fichier_" . sprintf("%06s", $numSeqDoc[0][0]['nextval']);
            $filename .= "." . $fileExt;

//debug($filename);
//debug($fichier_associe);           
            $doc = array(
                'Document' => array(
                    'name' => $filename,
                    'size' => strlen($fichier_associe),
                    'mime' => $fileMimeType,
                    'content' => $fichier_associe, //TODO: prevoir l utilisation d une GED
                    'ext' => $fileExt,
                    'courrier_id' => $fluxId,
                    'main_doc' => $main_doc
                )
            );
            $this->setDataSource(Configure::read('conn'));
            $this->Document->setDataSource(Configure::read('conn'));
            $this->Document->create($doc);
            $docsaved = $this->Document->save();

            if (empty($docsaved)) {
                $return['CodeRetour'] .= "NOFILE";
                $return['Message'] .= ".Le document joint n'a pas été envoyé";
            } else {
                $return['CodeRetour'] .= "FILEOK";
                $return['Message'] .= ". Le document joint a été envoyé";
            }
        }
    }

    /**
     * Fonctionnalité de numérisation de masse (création de flux)
     *
     * @logical-group WebService
     *
     * @access public
     * @param integer $collectiviteId identifiant de la collectivité
     * @param base64 $fichier_associe contenu du fichier lié
     * @param string $username identifiant de connexion de l'utilisateur cible
     * @return array
     */
    public function remoteCreate($fichier_associe, $username, $dispDesktop = true, $desktopId = null, $filename, $fluxName = null) {
        $return = array(
            'CodeRetour' => 'KO',
            'CourrierId' => -1,
            'Message' => 'Impossible de créer le flux.',
            'Severite' => 'important'
        );

        $noRoleErrMsg = "Le rôle n'existe pas.";

        if (empty($desktopId)) {
            $GID = $dispDesktop ? DISP_GID : INIT_GID;
            $BAN = $dispDesktop ? BAN_AIGUILLAGE : BAN_DOCS;
            $noRoleErrMsg = $dispDesktop ? "L'utilisateur n'a pas de rôle Aiguilleur." : "L'utilisateur n'a pas de rôle Initiateur.";

            $querydata = array(
                'conditions' => array(
                    'User.username' => $username
                ),
                'contain' => array(
                    $this->Bancontenu->Desktop->alias => array(
                        $this->Bancontenu->Desktop->Group->alias
                    ),
                    $this->Bancontenu->SecondaryDesktop->alias => array(
                        $this->Bancontenu->SecondaryDesktop->Group->alias
                    ),
                )
            );
            $user = $this->User->find('first', $querydata);

            if (empty($user)) {
                $return['CodeRetour'] = "KONOUSER";
                $return['Message'] .= " L'utilisateur n'existe pas.";
            } else {
                if ($user['Desktop']['group_id'] == $GID) {
                    $desktopId = $user['Desktop']['id'];
                } else {
                    $found = false;
                    for ($i = 0; $i < count($user['SecondaryDesktop']); $i++) {
                        if ($user['SecondaryDesktop'][$i]['group_id'] == $GID && !$found) {
                            $desktopId = $user['SecondaryDesktop'][$i]['id'];
                            $found = true;
                        }
                    }
                }
            }
        } else {
            $desktop = $this->Bancontenu->Desktop->find('first', array('conditions' => array('Desktop.id' => $desktopId)));
            if ($desktop['Desktop']['group_id'] == DISP_GID) {
                $BAN = BAN_AIGUILLAGE;
            } else if ($desktop['Desktop']['group_id'] == INIT_GID) {
                $BAN = BAN_DOCS;
            }
        }


        if (empty($desktopId)) {
            $return['CodeRetour'] = "KONODESKTOP";
            $return['Message'] .= $noRoleErrMsg;
        } else if (empty($BAN)) {
            $return['CodeRetour'] = "KONOBAN";
            $return['Message'] .= 'Aucun bannette renseignée';
        } else {
            if (!is_array($filename)) {
                $pos_point = strpos($filename, '.');
                $courrierName = substr($filename, 0, $pos_point);
            }
            $referenceUnique = $this->genReferenceUnique();

            if ($fluxName != null) {
                $data = array(
                    'Courrier' => array(
                        'name' => $fluxName . '_' . date('Y-m-d_H:i:s'),
                        'reference' => $referenceUnique,
                        'user_creator_id' => !empty($desktop['User'][0]['id']) ? $desktop['User'][0]['id'] : null
                    )
                );
            } else {
                $data = array(
                    'Courrier' => array(
                        'name' => $courrierName . '_' . date('Y-m-d_H:i:s'),
                        'reference' => $referenceUnique,
                        'user_creator_id' => !empty($desktop['User'][0]['id']) ? $desktop['User'][0]['id'] : null
                    )
                );
            }
            $this->begin();
            $this->create($data);
            if ($this->save()) {
                $courrierId = $this->id;
                $return['CodeRetour'] = "CREATED";
                $return['Message'] = "Le flux a été créé";
                $return['Severite'] = "normal";
                $return['CourrierId'] = $courrierId;

                $this->commit();

                //ajout dans la bannette de l aiguilleur
                $this->Bancontenu->add($desktopId, $BAN, $courrierId);
                if (is_array($fichier_associe)) {
                    for ($i = 0; $i < count($fichier_associe); $i++) {
                        if ($i != 0) {
                            $this->_genFileAssoc($fichier_associe[$i], $courrierId, $return, false);
                        } else {
                            $this->_genFileAssoc($fichier_associe[$i], $courrierId, $return, true);
                        }
                    }

                    $codeRetour = str_split($return['CodeRetour'], 13);
                    $return['CodeRetour'] = $codeRetour[0];
                } else {
                    $this->_genFileAssoc($fichier_associe, $courrierId, $return);
                }
                // Notification par mail de la génération du flux
                $user_id = $this->User->userIdByDesktop($desktopId);
                $typeNotif = $this->User->find(
                        'first', array(
                    'fields' => array('User.typeabonnement'),
                    'conditions' => array('User.id' => $user_id),
                    'contain' => false
                        )
                );
                
                $notifInfos = $this->Notifieddesktop->Notification->setNotification('ban_xnew');
                for ($d = 0; $d < count($desktopId); $d++) {
                    $notifyDesktops[$desktopId[$d]] = array($courrierId);
                    $notified = $this->Notifieddesktop->Notification->add($notifInfos['name'], $notifInfos['description'], $notifInfos['typeNotif'], $desktopId, $notifyDesktops);
                }
                $notificationId = $this->Notifieddesktop->Notification->id;
                if ($typeNotif['User']['typeabonnement'] == 'quotidien') {
                    $this->User->saveNotif($courrierId, $user_id, $notificationId, 'aiguillage');
                } else {
                    $this->User->notifier($courrierId, $user_id, $notificationId, 'aiguillage');
                }
            } else {
                $this->rollback();
            }
        }
        return $return;
    }

    /**
     * Construit le tableau à envoyer au parapheur
     * @param $acte_id
     * @return array
     */
    public function getDocumentsForDelegation($courrier_id) {
        $docs = array(
            'docPrincipale' => $this->getMainDoc($courrier_id),
            'annexes' => array()
        );

        // On envoie le document principal édité via webdav si tel est le cas.
        $path = APP . DS . WEBROOT_DIR . DS . 'files/webdav' . DS . "{$courrier_id}" . DS . "{$docs['docPrincipale']['Document']['name']}";
        if( file_exists( $path) ) {
            $docs['docPrincipale']['Document']['content'] = file_get_contents($path);
        }

        foreach ($this->Document->getPiecesJointes($courrier_id) as $piecejointe) {

            if ($piecejointe['Document']['mime'] != 'application/zip' && $piecejointe['Document']['mime'] != 'application/x-octetstream') {
                $docs['annexes'][] = array(
                    'content' => $piecejointe['Document']['content'],
                    'mimetype' => $piecejointe['Document']['mime'],
                    'filename' => $piecejointe['Document']['name'],
                    'ext' => $piecejointe['Document']['ext']
                );
            }
        }
// debug($docs);
        return $docs;
    }

    public function majTraitementsParapheur($courrier_id) {

        $before = $this->Traitement->whoIsNext($courrier_id, 'current');

        $ret = $this->Traitement->majTraitementsParapheur($courrier_id);

        $msg = 'Traitement en cours Parapheur';
        if ($ret == 'TRAITEMENT_TERMINE_OK') {
            $parent = 0;
            $type_id = null;
            $insertAuto = 0;
            $desktop_id = @$before[0];
            if (!empty($desktop_id)) {
                if ($this->cakeflowExecuteNext($desktop_id, $courrier_id, $parent, $type_id, $insertAuto)) {
                    $msg = 'Flux récupéré et transféré à l\'étape suivante';
                } else {
                    $msg = 'Echec de la transmission à l\'étape suivante';
                }
            } else {
                $msg = 'Aucun rôle destinataire défini';
            }
        }
        return $msg;
    }

    public function createRefusParapheur($fluxRefuse = array(), $desktopId, $message) {
        $this->Traitement->execute('KO', $desktopId, $fluxRefuse['Courrier']['id']);
        $msg = array();

        $traitements = $this->Traitement->find(
                'all', array(
            'conditions' => array(
                'Traitement.target_id' => $fluxRefuse['Courrier']['id']
            ),
            'contain' => false
                )
        );
        foreach ($traitements as $t => $traitement) {
            $this->Traitement->delete($traitement['Traitement']['id']);
        }

        if (empty($message)) {
            $this->updateAll(
                array(
                    'Courrier.motifrefus' => "'" . Sanitize::clean($this->request->data['Notification']['message'], array('encode', false)) . "'",
                    'Courrier.ref_ancien' => $fluxRefuse['Courrier']['id'],
                    'Courrier.parapheur_etat' => null,
                    'Courrier.parapheur_id' => null
                ),
                array('Courrier.id' => $fluxRefuse['Courrier']['id'])
            );
        } else {
            $this->updateAll(
                array(
                    'Courrier.motifrefus' => "'" . Sanitize::clean($message, array('encode', false)) . "'",
                    'Courrier.ref_ancien' => $fluxRefuse['Courrier']['id'],
                    'Courrier.parapheur_etat' => null,
                    'Courrier.parapheur_id' => null
                ),
                array('Courrier.id' => $fluxRefuse['Courrier']['id'])
            );
        }
        //on change l etat du flux dans toutes les bannettes
        $this->Bancontenu->cancel($fluxRefuse['Courrier']['id']);
        //on ajoute le flux dupliqué dans la bannette du rédacteur
        if ($this->Bancontenu->add($fluxRefuse['Courrier']['desktop_creator_id'], BAN_REFUS, $fluxRefuse['Courrier']['id'])) {
            //notification au rédacteur
            $notifyDesktops = array($fluxRefuse['Courrier']['desktop_creator_id'] => array($fluxRefuse['Courrier']['id']));
            if ($this->Notifieddesktop->Notification->add(__d('notification', 'ban_cancel.name'), __d('notification', 'ban_cancel.description'), BAN_REFUS, $fluxRefuse['Courrier']['desktop_creator_id'], $notifyDesktops)) {


                // Ajout pour les notifs
                $destinataires = $this->Traitement->whoIs($fluxRefuse['Courrier']['id'], 'in', array('OK', 'RI', 'IN'));
                $destinataires = Hash::merge($destinataires, $fluxRefuse['Courrier']['desktop_creator_id']);

                foreach ($destinataires as $destinataire_id) {
                    if ($destinataire_id != '-1') {
                        $user_id = $this->User->userIdByDesktop($destinataire_id);
                        $typeNotif = $this->User->find(
                                'first', array(
                            'fields' => array('User.typeabonnement'),
                            'conditions' => array('User.id' => $user_id),
                            'contain' => false
                                )
                        );
                        $notificationId = $this->Notifieddesktop->Notification->id;
                        if ($typeNotif['User']['typeabonnement'] == 'quotidien') {
                            $this->User->saveNotif($fluxRefuse['Courrier']['id'], $user_id, $notificationId, 'refus');
                        } else {
                            $this->User->notifier($fluxRefuse['Courrier']['id'], $user_id, $notificationId, 'refus');
                        }
//                                        $this->User->notifier($fluxRefuse['Courrier']['id'], $user_id, 'refus');
                    }
                }


                if (empty($message)) {
                    $msg['message'] = 'Le flux ' . $fluxRefuse['Courrier']['name'] . ' a été refusé avec le motif suivant : <br />' . $this->request->data['Notification']['message'];
                } else {
                    $msg['message'] = 'Le flux ' . $fluxRefuse['Courrier']['name'] . ' a été refusé avec le motif suivant : <br />' . $message;
                }
                $msg['success'] = true;
            } else {
                $msg['message'] .= '<br />Notification: error';
            }
        } else {
            $msg['message'] .= '<br />Bannette add: error';
        }
        return $msg;
    }

    /**
     * 	Mise à jour du document avec la récupératin de la signature et du bordereau
     */
    public function majDocument($id, $dossier) {

        $document = $this->Document->find('first', array(
            'fields' => array(
                'Document.id',
                'Document.courrier_id',
                'Document.main_doc',
                'Document.size'
            ),
            'conditions' => array(
                'Document.courrier_id' => $id,
                'Document.main_doc' => true
            ),
            'contain' => false,
            'recursive' => -1
        ));

        $this->Document->id = $document['Document']['id'];
        $this->Document->saveField('parapheur_bordereau', base64_decode($dossier['getdossier']['bordereau']));
        $this->Document->saveField('signature', base64_decode($dossier['getdossier']['signature']));
        $this->Document->saveField('signee', true);
//debug($dossier);        
        $success = true;
        // Affichage du bordereau
        $documentBordereau = array(
            'Document' => array(
                'courrier_id' => $id,
                'size' => strlen(base64_decode($dossier['getdossier']['bordereau'])),
                'main_doc' => false,
                'content' => base64_decode($dossier['getdossier']['bordereau']),
                'name' => 'Bordereau_' . $id . '.pdf',
                'ext' => 'pdf',
                'mime' => 'application/pdf'
            )
        );
        $this->Document->create($documentBordereau);
        $success = $this->Document->save() && $success;

        // Affichage de la signature
        $documentSignature = array(
            'Document' => array(
                'courrier_id' => $id,
                'size' => strlen(base64_decode($dossier['getdossier']['signature'])),
                'main_doc' => false,
                'content' => base64_decode($dossier['getdossier']['signature']),
                'name' => 'Signature_Recuperee_' . $id . '.zip',
                'ext' => 'zip',
//                          'mime' => 'application/pdf'
            )
        );
        $this->Document->create($documentSignature);
        $success = $this->Document->save() && $success;
    }

    /**
     * Envoi par mail d'un document lié à un flux 
     *
     * @param integer $courrier_id identifiant du flux
     * @param integer $user_id identifiant de l'utilisateur à notifier
     * @param string $type notification à envoyer
     * @return bool succès de l'envoi
     */
    public function docParMail($mailDest, $doc, $sujet, $message, $mailCCDest, $attachements) {

        App::uses('CakeEmail', 'Network/Email');
        $config_mail = Configure::read('SMTP_USE') ? 'docparmail' : 'default';
        $this->Email = new CakeEmail($config_mail);

        $mailDest = explode('; ', $mailDest);
        foreach ($mailDest as $mails) {
            $this->Email->addTo($mails);
        }
        if (!empty($mailCCDest)) {
            $mailCCDest = explode('; ', $mailCCDest);
            foreach ($mailCCDest as $mailsCC) {
                $this->Email->addCc($mailsCC);
            }
        }
//echo '<pre>'; var_dump($doc);
        if (empty($doc['Document'])) {
            $courrier_id = $doc['Ar']['courrier_id'];
        } else {
            $courrier_id = $doc['Document']['courrier_id'];
        }
        App::uses('Courrier', 'Model');
        $this->Courrier = new Courrier();
        $flux = $this->Courrier->find('first', array(
            'recursive' => -1,
            'conditions' => array('Courrier.id' => $courrier_id),
            'fields' => array(
                'Courrier.id',
                'Courrier.objet',
                'Courrier.reference',
                'Courrier.parent_id',
                'Courrier.name',
                'Courrier.motifrefus',
                'Soustype.circuit_id'
            ),
            'joins' => array(
                $this->Courrier->join('Soustype')
            )
        ));
        if (empty($sujet)) {
            $sujet = "Un nouveau document vous a été envoyé depuis web-GFC";
        }
        $type = "docparmail";

        if (empty($doc['Document'])) {
            $docId = $doc['Ar']['id'];
            $docName = $doc['Ar']['name'] . '.' . $doc['Ar']['ext'];
            $docContent = $doc['Ar']['content'];
        } else {
            $isAR = strpos($doc['Document']['name'], '[AR]');
            if ($isAR !== false && $doc['Document']['mime'] != 'application/pdf') {
                require_once 'XML/RPC2/Client.php';
                $options = array(
                    'uglyStructHack' => true
                );
                $url = 'http://' . Configure::read('CLOUDOOO_HOST') . ':' . Configure::read('CLOUDOOO_PORT');
                $client = XML_RPC2_Client::create($url, $options);
                try {
                    $docContent = $client->convertFile(base64_encode($doc['Document']['content']), 'odt', 'pdf', false, true);
                    return base64_decode($docContent);
                } catch (XML_RPC2_FaultException $e) {
                    $this->log('Exception #' . $e->getFaultCode() . ' : ' . $e->getFaultString(), 'debug');
                    return false;
                }
                //                    $docContent = $this->Conversion->convertirFlux($doc['Ar']['content'], 'odt', 'pdf');
                //                    $doc['Ar']['mime'] = 'application/pdf';
                $docId = $doc['Document']['id'];
                $docName = $doc['Document']['name'];
            } else {
                $docId = $doc['Document']['id'];
                $docName = $doc['Document']['name'];
                $docContent = $doc['Document']['content'];
            }
        }

        // création du répertoire local
        $folder = WWW_ROOT . 'files/attachments/' . $docId . '/';
        if (!file_exists($folder)) {
            mkdir($folder, 0777, true);
        }
        // création du fichier
        $filename = $docName;
        if (!file_exists($folder . $filename)) {
            $fileToSend = fopen($folder . $filename, "x+");
        } else {
            $fileToSend = $folder . $filename;
        }

        $filecontent = $docContent;
        // écriture
        fputs($fileToSend, $filecontent);
        // fermeture
        fclose($fileToSend);
        
        if( !empty($attachements) ) {
            foreach($attachements as $d => $attachement) {
                $nameToSend = $attachement['Document']['name'];
                $docContentToSend = $attachement['Document']['content'];

                if (!file_exists($folder . $nameToSend)) {
                    $fileAttachedToSend = fopen($folder . $nameToSend, "x+");
                } else {
                    $fileAttachedToSend = $folder . $nameToSend;
                }

                $fileAttachedContent = $docContentToSend;
                // écriture
                fputs($fileAttachedToSend, $fileAttachedContent);
                // fermeture
                fclose($fileAttachedToSend);
//                $this->Email->attachments(array(
                $nameAttached[$nameToSend] = array(
                    'file' => $folder . $nameToSend
                );
            }
            
        }

        if( !empty($nameAttached)) {
            $this->Email->attachments(
                array_merge(
                    array($docName => array(
                        'file' => $folder . $docName
                    ) ),
                    $nameAttached
                )
            );
        }
        else {
            $this->Email->attachments(
                array(
                    $docName => array(
                        'file' => $folder . $docName
                    )
                )
            );
        }
        
        $this->Email->subject($sujet);
        $content = $this->_paramMails($type, $flux, $message);
        $success = $this->Email->send($content);
        if ($success) {
            if( !empty($nameAttached ) ) {
                foreach( $nameAttached as $nameToUnlink => $data) {
                    unlink($folder . $nameToUnlink);
                }
            }
            unlink($folder . $docName);
            rmdir($folder);
        }

        return true;
    }

    /**
     * Détermine le contenu du mail à envoyer en fonction du type de mail, le flux et l'utilisateur
     * @param string $type
     * @param array $flux
     * @param array $user
     * @return string
     */
    function _paramMails($type, $flux, $message) {
//            $file = new File(APP . "/Config/emails/$type.txt", false);
//            $content = $file->read();
//            $file->close();
//            $addrTraiter = Configure::read('WEBGFC_URL') . '/courriers/historiqueGlobal/' . $flux['Courrier']['id'];

        $searchReplace = array(
            "#IDENTIFIANT_FLUX#" => $flux['Courrier']['id'],
            "#OBJET_FLUX#" => $flux['Courrier']['name'],
            "#LIBELLE_CIRCUIT#" => $this->Courrier->Traitement->Circuit->getLibelle($flux['Soustype']['circuit_id']),
//                    "#ADRESSE_A_TRAITER#" => $addrTraiter,            
            "#FLUX_ORIGINE#" => isset($flux['Courrier']['parent_id']) ? $flux['Courrier']['parent_id'] : null,
            "#REFERENCE_FLUX#" => isset($flux['Courrier']['reference']) ? $flux['Courrier']['reference'] : null
        );

        return str_replace(array_keys($searchReplace), array_values($searchReplace), $message);
//            return str_replace(array_keys($searchReplace), array_values($searchReplace), $content);
    }

    /**
     * Fonctionnalité de numérisation de masse (création de flux)
     *
     * @logical-group WebService
     *
     * @access public
     * @param integer $collectiviteId identifiant de la collectivité
     * @param base64 $fichier_associe contenu du fichier lié
     * @param string $username identifiant de connexion de l'utilisateur cible
     * @return array
     */
    public function remoteCreateMail($desktopmanagerId, $filename, $collId, $typeId = null, $soustypeId = null) {
        $return = array(
            'CodeRetour' => 'KO',
            'CourrierId' => -1,
            'Message' => 'Impossible de créer le flux.',
            'Severite' => 'important'
        );


        $noRoleErrMsg = "Le rôle n'existe pas.";

        $this->Desktopmanager = ClassRegistry::init('Desktopmanager');
        $desktopsIds[] = $this->Desktopmanager->getDesktops($desktopmanagerId);
        $desktops = array();
        foreach ($desktopsIds as $i => $desktopId) {
//debug($desktopsIds);
//debug($desktopId);
            $desktops = $this->Bancontenu->Desktop->find(
                    'all', array(
                'conditions' => array(
                    'Desktop.id' => $desktopsIds[$i]
                ),
                'contain' => false
                    )
            );

            if ($desktops[$i]['Desktop']['group_id'] == DISP_GID) {
                $BAN = BAN_AIGUILLAGE;
            } else if ($desktops[$i]['Desktop']['group_id'] == INIT_GID) {
                $BAN = BAN_DOCS;
            }
//            $desktops[] = $desktop;
        }
//debug($desktops);
//die();
        /* if (empty($desktopmanagerId)) {
          $GID = $dispDesktop ? DISP_GID : INIT_GID;
          $BAN = $dispDesktop ? BAN_AIGUILLAGE : BAN_DOCS;
          $noRoleErrMsg = $dispDesktop ? "L'utilisateur n'a pas de rôle Aiguilleur." : "L'utilisateur n'a pas de rôle Initiateur.";

          $querydata = array(
          'conditions' => array(
          'User.username' => $username
          ),
          'contain' => array(
          $this->Bancontenu->Desktop->alias => array(
          $this->Bancontenu->Desktop->Group->alias
          ),
          $this->Bancontenu->SecondaryDesktop->alias => array(
          $this->Bancontenu->SecondaryDesktop->Group->alias
          ),
          )
          );
          $user = $this->User->find('first', $querydata);

          if (empty($user)) {
          $return['CodeRetour'] = "KONOUSER";
          $return['Message'] .= " L'utilisateur n'existe pas.";
          } else {
          if ($user['Desktop']['group_id'] == $GID) {
          $desktopId = $user['Desktop']['id'];
          } else {
          $found = false;
          for ($i = 0; $i < count($user['SecondaryDesktop']); $i++) {
          if ($user['SecondaryDesktop'][$i]['group_id'] == $GID && !$found) {
          $desktopId = $user['SecondaryDesktop'][$i]['id'];
          $found = true;
          }
          }
          }
          }
          } else {
          $desktop = $this->Bancontenu->Desktop->find('first', array('conditions' => array('Desktop.id' => $desktopId)));

          if ($desktop['Desktop']['group_id'] == DISP_GID) {
          $BAN = BAN_AIGUILLAGE;
          } else if ($desktop['Desktop']['group_id'] == INIT_GID) {
          $BAN = BAN_DOCS;
          }
          } */


        if (empty($desktopmanagerId)) {
            $return['CodeRetour'] = "KONODESKTOP";
            $return['Message'] .= $noRoleErrMsg;
        } else if (empty($BAN)) {
            $return['CodeRetour'] = "KONOBAN";
            $return['Message'] .= 'Aucun bannette renseignée';
        } else {

            $referenceUnique = $this->genReferenceUnique();
//            if (imap_base64($filename->body)) {
//                $filename->body = base64_decode($filename->body);
//            }
            $pos = strpos($filename->subject, '=?UTF-8?B?');
            if ($pos !== false) {
                $filename->subject = mb_decode_mimeheader($filename->subject);
                $filename->body = mb_decode_mimeheader($filename->body);
            }
            $pos = strpos($filename->subject, '=?utf-8?Q?');
            if ($pos !== false) {
                $filename->subject = mb_decode_mimeheader($filename->subject);
                $filename->body = mb_decode_mimeheader($filename->body);
            }
            mb_internal_encoding('UTF-8');

            $data = array(
                'Courrier' => array(
                    'name' => '[MAIL] ' . preg_replace( "/[:]/", "_", $filename->subject ),
                    'objet' => !empty($filename->body) ? ( $filename->body ) : null,
                    'reference' => $referenceUnique,
                    'type_id' => !empty($typeId) ? ( $typeId ) : null,
                    'soustype_id' => !empty($soustypeId) ? $soustypeId : null
                )
            );

            $this->begin();
            $this->create($data);

            if ($this->save()) {
                $courrierId = $this->id;
                $return['CodeRetour'] = "CREATED";
                $return['Message'] = "Le flux a été créé";
                $return['Severite'] = "normal";
                $return['CourrierId'] = $courrierId;

                $this->commit();

                //ajout dans les bannette + notifications par mail
                foreach ($desktopsIds as $i => $desktopId) {
                    for ($i = 0; $i < count($desktopId); $i++) {
                        //ajout dans la bannette de chacun des aiguilleurs
                        if ($desktops[$i]['Desktop']['group_id'] == DISP_GID) {
                            $this->Bancontenu->add($desktopId[$i], BAN_AIGUILLAGE, $courrierId);
                        } else if ($desktops[$i]['Desktop']['group_id'] == INIT_GID) {
                            $this->Bancontenu->add($desktopId[$i], BAN_DOCS, $courrierId);
                        }

                        // Notification par mail de la génération du flux
                        $user_id = $this->User->userIdByDesktop($desktopId[$i]);
                        $typeNotif = $this->User->find(
                                'first', array(
                            'fields' => array('User.typeabonnement'),
                            'conditions' => array('User.id' => $user_id),
                            'contain' => false
                                )
                        );
                        $notifInfos = $this->Notifieddesktop->Notification->setNotification('ban_xnew');
//                        $desktopAuthId = CakeSession::read('Auth.User.desktop_id');
                        for ($d = 0; $d < count($desktopId); $d++) {
                            $notifyDesktops[$desktopId[$d]] = array($courrierId);
                            $notified = $this->Notifieddesktop->Notification->add($notifInfos['name'], $notifInfos['description'], $notifInfos['typeNotif'], $desktopId[$i], $notifyDesktops);
                        }
                        
                        $notificationId = $this->Notifieddesktop->Notification->id;
                        if ($typeNotif['User']['typeabonnement'] == 'quotidien') {
                            $this->User->saveNotif($courrierId, $user_id, $notificationId, 'aiguillage');
                        } else {
                            $this->User->notifier($courrierId, $user_id, $notificationId, 'aiguillage');
                        }
                    }
                }
//var_dump(isset( $filename->pj) );
//$this->log($filename->body);
//$this->log($filename->subject);

                if (isset($filename->pj) && !empty($filename->pj)) {                    
                    //$files = Hash::extract( $filename->pj, '{n}.is_attachment' );
                    $this->_genFileAssocMail($filename->pj, $courrierId /* , $return */);
                    $this->_genFileAssocMailWithoutpj($filename->subject, $filename->body, $courrierId);
                } else {
                    //on génère un fichier pdf du contenu du mail
                    $this->_genFileAssocMailWithoutpj($filename->subject, $filename->body, $courrierId);
                }
            } else {
                $this->rollback();
            }
        }

        return $return;
    }

    /**
     * Génération d'un document lié (à partir du script lancé à la main)
     *
     * @access private
     * @param base64 $fichier_associe contenu du document lié
     * @param integer $fluxId identifiant du flux
     * @param array $return valeur de retour
     * @return void
     */
    private function _genFileAssocMail($fichiers_associes, $fluxId /* , &$return */) {


        foreach ($fichiers_associes as $i => $pj) {
            if ($pj['is_attachment'] == true) {
                $finfo = new finfo(FILEINFO_MIME_TYPE);

                $sql = "SELECT nextval('documents_name_seq'); /*" . mktime(true) . " " . $this->_lastSequenceDocument . "*/";

                $fileMimeType = $pj['subtype'];

                $fileMimeTypeError = false;
                if ($fileMimeType == "application/pdf" || $fileMimeType == "PDF") {
                    $fileExt = "pdf";
                } else if ($fileMimeType == "application/vnd.oasis.opendocument.text" || $fileMimeType == "VND.OASIS.OPENDOCUMENT.TEXT" || $fileMimeType == "OCTET-STREAM") {
                    $fileExt = "odt";
                } else if ($fileMimeType == "image/bmp") {
                    $fileExt = "bmp";
                } else if ($fileMimeType == "image/tiff") {
                    $fileExt = "tif";
                } else if ($fileMimeType == "text/plain") {
                    $fileExt = "txt";
                } else if ($fileMimeType == "application/vnd.ms-office") {
                    $fileExt = "doc";
                } else if ($fileMimeType == "application/msword") {
                    $fileExt = "doc";
                } else if ($fileMimeType == "application/rtf") {
                    $fileExt = "rtf";
                } else if ($fileMimeType == "image/png" || $fileMimeType == "PNG") {
                    $fileExt = "png";
                } else if ($fileMimeType == "image/jpeg" || $fileMimeType == "JPEG") {
                    $fileExt = "jpg";
                } else if ($fileMimeType == "application/vnd.oasis.opendocument.spreadsheet" || $fileMimeType == "VND.OASIS.OPENDOCUMENT.SPREADSHEET") {
                    $fileExt = "ods";
                } else if ($fileMimeType == '') {
                    $fileMimeType = 'txt';
                    $fileExt = "rtf";
                } else {
                    $fileMimeTypeError = true;
                }




                if ($fileMimeType == "PDF") {
                    $mime = "application/pdf";
                } else if ($fileMimeType == "VND.OASIS.OPENDOCUMENT.TEXT" || $fileMimeType == "OCTET-STREAM") {
                    $mime = "application/vnd.oasis.opendocument.text";
                } else if ($fileMimeType == "application/vnd.oasis.opendocument.spreadsheet" || $fileMimeType == "VND.OASIS.OPENDOCUMENT.SPREADSHEET") {
                    $mime = "application/vnd.oasis.opendocument.spreadsheet";
                } else if ($fileMimeType == "image/png" || $fileMimeType == "PNG") {
                    $mime = "image/png";
                } else if ($fileMimeType == "image/jpeg" || $fileMimeType == "JPEG") {
                    $mime = "image/jpeg";
                }
//debug($fileMimeType);
                if ($fileMimeTypeError) {
//                    $this->out("<error>Le fichier n'est pas reconnu.</error>");
                    echo "La pièce jointe n'est pas reconnue.";
//                                $return['CodeRetour'] .= "FILENOK";
//                                $return['Message'] .= ". Le fichier n'est pas reconnu";
                } else {

                    $numSeqDoc = $this->query($sql);
                    if ($numSeqDoc === false) {
                        return null; // FIXME: Throw exception
                    }
                    $this->_lastSequenceDocument = $numSeqDoc[0][0]['nextval'];
                    $filename = date('Y-m-d') . "_fichier_" . sprintf("%06s", $numSeqDoc[0][0]['nextval']);
                    $filename .= "." . $fileExt;

//debug($pj['subtype']);
//debug($pj['filename']);
//debug($i);

                    $doc[] = array(
                        'Document' => array(
                            'name' => $pj['filename'],
                            'courrier_id' => $fluxId,
                            'size' => $pj['size'],
                            'mime' => !empty($mime) ? $mime : $fileMimeType,
                            'ext' => $fileExt,
                            'main_doc' => false,
                            'content' => $pj['attachment']
                        )
                    );
                }
            }
        }

        $this->setDataSource(Configure::read('conn'));
        $this->Document->setDataSource(Configure::read('conn'));
//        $this->Document->begin();
        $this->Document->create();
//        $success = $this->Document->save($doc);
        if (!empty($doc)) {
            $doc[0]['Document']['main_doc'] = true;
            $success = $this->Document->saveAll($doc, array('validate' => 'first', 'atomic' => false));

            if ($success) {
                echo 'La pièce jointe a été envoyée.';
            } else {
                echo "La pièce jointe n'a pas été envoyée.";
            }
        } else {
            echo "Aucune pièce jointe à insérer";
        }
    }

    public function notifier($courrier = array()) {

        $courrierId = $courrier['Courrier']['id'];
        $success = array();
        // Notification par mail de la génération du flux
        $destinataires = $this->Traitement->whoIsNext($courrierId, 'current');
        if (!empty($destinataires)) {
            foreach ($destinataires as $destinataire_id) {
                if ($destinataire_id != -1) {
                    $this->Desktopmanager = ClassRegistry::init('Desktopmanager');
                    $desktopsIds = $this->Desktopmanager->getDesktops($destinataire_id);
                    foreach ($desktopsIds as $deskId) {
                        $userId = $this->User->userIdByDesktop($deskId);
                        if (!empty($userId)) {
                            $typeNotif = $this->User->find(
                                    'first', array(
                                'fields' => array('User.typeabonnement'),
                                'conditions' => array('User.id' => $userId),
                                'contain' => false
                                    )
                            );

                            if ($typeNotif['User']['typeabonnement'] == 'quotidien') {
                                $success[] = $this->User->saveNotif($courrierId, $userId, null, 'retard_validation') && $success;
                            } else {
                                $success[] = $this->User->notifier($courrierId, $userId, null, 'retard_validation') && $success;
                            }

                            if (in_array(true, $success)) {
                                $this->updateAll(
                                        array(
                                    'Courrier.mail_retard_envoye' => true
                                        ), array(
                                    'Courrier.id' => $courrierId
                                        )
                                );
                            }
                        } else {
                            $success = false;
                        }
                    }
                } else {
                    $success = false;
                }
            }
        }
    }

    /**
     * Fonciton permettant de rechercher les flux de mon historique selon certains critères
     *
     * @param type $criteres
     * @return array()
     */
    public function search($criteres) {
        /// Conditions de base
        $conditions = array();
        // Critères sur le courrier
        if (isset($criteres['Courrier']['reference']) && !empty($criteres['Courrier']['reference'])) {
            $conditions[] = 'Courrier.reference ILIKE \'' . $this->wildcard('%' . $criteres['Courrier']['reference'] . '%') . '\'';
        }

        if (isset($criteres['Courrier']['name']) && !empty($criteres['Courrier']['name'])) {
            $conditions[] = 'NOACCENTS_UPPER(Courrier.name) LIKE \'' . $this->wildcard('%' . noaccents_upper( $criteres['Courrier']['name'] ) . '%') . '\'';
        }

        if (isset($criteres['Courrier']['datereception']) && !empty($criteres['Courrier']['datereception'])) {
            if (isset($criteres['Courrier']['datereceptionfin']) && !empty($criteres['Courrier']['datereceptionfin'])) {
                if( isset($criteres['Courrier']['datereception'])  && !empty($criteres['Courrier']['datereception']) ) {
                    $dateReception = $criteres['Courrier']['datereception'];
                    if( strpos( $dateReception,  '/'  ) != 0 ) {
                        $debut = explode( '/', $dateReception );
                        $day = $debut[0];
                        $month = $debut[1];
                        $year = $debut[2];
                        $dateDebut = $year.'-'.$month.'-'.$day;
                        $criteres['Courrier']['datereception'] = $dateDebut;
                    }
                }
                if( isset($criteres['Courrier']['datereceptionfin'])  && !empty($criteres['Courrier']['datereceptionfin']) ) {
                    $dateReceptionFin = $criteres['Courrier']['datereceptionfin'];
                    if( strpos( $dateReceptionFin,  '/'  ) != 0 ) {
                        $fin = explode( '/', $dateReceptionFin );
                        $day = $fin[0];
                        $month = $fin[1];
                        $year = $fin[2];
                        $dateFin = $year.'-'.$month.'-'.$day;
                        $criteres['Courrier']['datereceptionfin'] = $dateFin;
                    }
                }
                
                $conditions[] = array('AND' => array(array('Courrier.datereception >=' => $criteres['Courrier']['datereception']), array('Courrier.datereception <=' => $criteres['Courrier']['datereceptionfin'])));
            } else {
                $conditions[] = array('Courrier.datereception' => $criteres['Courrier']['datereception']);
            }
        }

        //critères Type
        if (!empty($criteres['Type']['Type'])) {
            $type_conditions = array();
            foreach ($criteres['Type']['Type'] as $type_id) {
                $type_conditions[] = array('Type.id' => $type_id);
            }
            $conditions[] = array('OR' => $type_conditions);
        }

        //critères Soustype
        if (!empty($criteres['Soustype']['Soustype'])) {
            $soustype_conditions = array();
            foreach ($criteres['Soustype']['Soustype'] as $soustype_id) {
                $soustype_conditions[] = array('Courrier.soustype_id' => $soustype_id);
            }
            $conditions[] = array('OR' => $soustype_conditions);
        }

        if ( isset($criteres['Courrier']['etat'] ) && ($criteres['Courrier']['etat'] != '' ) ) {
            $etat = $criteres['Courrier']['etat'];
            $sqBancontenu = $this->Bancontenu->sqDerniereBannette( 'Courrier.id', $etat );
            $conditions[] = array('Bancontenu.etat' => $criteres['Courrier']['etat']);
        }
//        $Desktop = ClassRegistry::init('Desktop');   
//        $sq = $Desktop->sq(
//                array(
//                'fields' => array( 'desktops.id' ),
//                'alias' => 'desktops',
//                'contain' => false
//            )
//        );
//        $conditions[] = array( "Bancontenu.desktop_id IN ( {$sq} )" );
//        $query = array(
//            'joins' => array(
//                $this->join( 'Bancontenu', array( 'type' => 'LEFT OUTER' ) ),
//                $this->Bancontenu->join( 'Desktop', array( 'type' => 'LEFT OUTER' ) ),
//                alias_querydata(
//                    $this->Bancontenu->join( 'Desktop', array( 'type' => 'LEFT OUTER' ) ),
//                    array( 'Desktop' => Inflector::camelize("bannette_" . Inflector::camelize(Inflector::slug($desktopName)) ) )
//                ),
//                $this->join( 'Soustype', array( 'type' => 'LEFT OUTER' ) ),
//                $this->Soustype->join( 'Type', array( 'type' => 'LEFT OUTER' ) ),
//                $this->join( 'Contactinfo', array( 'type' => 'LEFT OUTER' ) )
//            ),
//            'limit' => 200,
//            'fields' => array_merge(
//                $this->fields(),
//                $this->Bancontenu->fields(),
//                $this->Bancontenu->Desktop->fields(),
//                $this->Soustype->fields(),
//                $this->Soustype->Type->fields(),
//                $this->Contactinfo->fields(),
//                $this->_virtualFieldsCourrier(),
//                alias_querydata(
//                    $this->Bancontenu->Desktop->fields(),
//                    array( 'Desktop' => Inflector::camelize("bannette_" . Inflector::camelize(Inflector::slug($desktopName)) ) )
//                )
//            ),
//            'order' => 'Courrier.reference DESC',
//            'conditions' => array(
//                $conditions
//            ),
//            'contain' => false
//        );
//debug($desktopName);
        return $conditions;
    }

    /*
      function _virtualFieldsCourrier() {
      return array(
      'EXISTS( SELECT documents.id FROM documents WHERE documents.courrier_id = "Courrier"."id" ) AS "Courrier__ficassoc"',
      '( CASE WHEN "Courrier"."direction" = \'0\' THEN \'S\' WHEN "Courrier"."direction" = \'1\' THEN \'E\' WHEN "Courrier"."direction" = \'2\' THEN \'I\' ELSE NULL END ) AS "Courrier__direction"',
      '( CASE
      WHEN "Courrier"."soustype_id" IS NULL THEN true
      WHEN "Bancontenu"."etat" IN ( -1, 2 ) THEN true
      WHEN "Courrier"."delai_unite" = \'0\' THEN ( ( "Courrier"."datereception" + ( "Courrier"."delai_nb" * INTERVAL \'1 day\' ) ) > NOW() )
      WHEN "Courrier"."delai_unite" = \'1\' THEN ( ( "Courrier"."datereception" + ( "Courrier"."delai_nb" * INTERVAL \'1 week\' ) ) > NOW() )
      WHEN "Courrier"."delai_unite" = \'2\' THEN ( ( "Courrier"."datereception" + ( "Courrier"."delai_nb" * INTERVAL \'1 month\' ) ) > NOW() )
      ELSE true
      END ) AS "Courrier__is_in_time"'
      );
      }
     */

    /**
     * Vérifie que les paramètres sont bons.
     *
     * @param mixed $checks
     * @param mixed $references
     * @return boolean
     */
    protected function _checkParams($checks, $references) {
        return ( is_array($checks) && !empty($references) );
    }

    /**
     * Compare la date en valeur par-rapport à la date de référence, suivant
     * le comparateur.
     *
     * @param Model $Model
     * @param array $check
     * @param string $reference
     * @param string $comparator Une valeur parmi: >, <, ==, <=, >=
     * @return boolean
     */
    public function compareDates($check, $reference, $comparator) {
        if (!$this->_checkParams($check, $reference)) {
            return false;
        }

        $check_value = strtotime(Hash::get($check, 'datereception'));
        $reference_value = strtotime(Hash::get($this->data, "{$this->alias}.{$reference}"));

        if (empty($reference_value) || empty($check_value)) {
            return true;
        }

        $seconds = $reference_value - $check_value;
        $days = floor($seconds / 3600 / 24);

        if ($days > 0) {
            return in_array($comparator, array('<', '<='), true);
        } else if ($days < 0) {
            return in_array($comparator, array('>', '>='), true);
        }

        return in_array($comparator, array('==', '<=', '>='), true);
    }

    /**
     *
     * @param type $id
     * @return type
     */
    public function getMailsSent($id) {
        $qdMainDoc = array(
//			'recursive' => -1,
            'fields' => array_merge(
                    $this->Sendmail->fields(), $this->Sendmail->Desktop->fields(), array(
                'Document.name'
                    )
            ),
            'conditions' => array(
                'Sendmail.courrier_id' => $id
            ),
            'joins' => array(
                $this->Sendmail->join('Desktop'),
                $this->Sendmail->join('Document')
            ),
            'contain' => false,
            'order' => 'Sendmail.created DESC'
        );

        $sendsmails = $this->Sendmail->find('all', $qdMainDoc);

        return $sendsmails;
    }

    /**
     * Fonctionnalité de numérisation de masse (création de flux)
     *
     * @logical-group WebService
     *
     * @access public
     * @param integer $collectiviteId identifiant de la collectivité
     * @param base64 $fichier_associe contenu du fichier lié
     * @param string $username identifiant de connexion de l'utilisateur cible
     * @return array
     */
    public function remoteCreateDesktopmanager($fichier_associe, $username, $dispDesktop = true, $desktopId = null, $filename, $fluxName = null) {

        $return = array(
            'CodeRetour' => 'KO',
            'CourrierId' => -1,
            'Message' => 'Impossible de créer le flux.',
            'Severite' => 'important'
        );

        $noRoleErrMsg = "Le rôle n'existe pas.";
//debug($desktopId);
        if (empty($desktopId)) {
            $GID = $dispDesktop ? DISP_GID : INIT_GID;
            $BAN = $dispDesktop ? BAN_AIGUILLAGE : BAN_DOCS;
            $noRoleErrMsg = $dispDesktop ? "L'utilisateur n'a pas de rôle Aiguilleur." : "L'utilisateur n'a pas de rôle Initiateur.";
        } else {
            // On utilise ici le desktopmanager pour obtenir la liste des desktops associés
            $this->Desktopmanager = ClassRegistry::init('Desktopmanager');
            $desktopsIds[] = $this->Desktopmanager->getDesktops($desktopId);
            foreach ($desktopsIds as $i => $desktopId) {
                $desktops = $this->Bancontenu->Desktop->find(
                        'all', array(
                    'conditions' => array(
                        'Desktop.id' => $desktopsIds[$i]
                    ),
                    'contain' => false
                        )
                );

                if ($desktops[$i]['Desktop']['group_id'] == DISP_GID) {
                    $BAN = BAN_AIGUILLAGE;
                } else if ($desktops[$i]['Desktop']['group_id'] == INIT_GID) {
                    $BAN = BAN_DOCS;
                }
                $desktop = $this->Bancontenu->Desktop->find(
                        'first', array(
                    'conditions' => array(
                        'Desktop.id' => $desktopsIds[$i]
                    ),
                    'contain' => array(
                        'User'
                    )
                        )
                );
//                if ($desktop['Desktop']['group_id'] == DISP_GID) {
//                    $BAN = BAN_AIGUILLAGE;
//                } else if ($desktop['Desktop']['group_id'] == INIT_GID) {
//                    $BAN = BAN_DOCS;
//                }
            }
        }


        if (empty($desktopId)) {
            $return['CodeRetour'] = "KONODESKTOP";
            $return['Message'] .= $noRoleErrMsg;
        } else if (empty($BAN)) {
            $return['CodeRetour'] = "KONOBAN";
            $return['Message'] .= 'Aucun bannette renseignée';
        } else {
            if (!is_array($filename)) {
                $pos_point = strpos($filename, '.');
                $courrierName = substr($filename, 0, $pos_point);
            }
            $referenceUnique = $this->genReferenceUnique();

            if ($fluxName != null) {
                $data = array(
                    'Courrier' => array(
                        'name' => $fluxName . '_' . date('Y-m-d_H:i:s'),
                        'reference' => $referenceUnique,
                        'user_creator_id' => !empty($desktop['User'][0]['id']) ? $desktop['User'][0]['id'] : null
                    )
                );
            } else {
                $data = array(
                    'Courrier' => array(
                        'name' => $courrierName . '_' . date('Y-m-d_H:i:s'),
                        'reference' => $referenceUnique,
                        'user_creator_id' => !empty($desktop['User'][0]['id']) ? $desktop['User'][0]['id'] : null
                    )
                );
            }
            $this->begin();
            $this->create($data);
            if ($this->save()) {
                $courrierId = $this->id;
                $return['CodeRetour'] = "CREATED";
                $return['Message'] = "Le flux a été créé";
                $return['Severite'] = "normal";
                $return['CourrierId'] = $courrierId;

                $this->commit();

                //ajout dans la bannette de chacun de l'aiguilleur
//                $this->Bancontenu->add($desktopId, $BAN, $courrierId);
//                
                //ajout dans les bannette + notifications par mail
                foreach ($desktopsIds as $i => $desktopId) {
                    for ($i = 0; $i < count($desktopId); $i++) {
                        //ajout dans la bannette de chacun des aiguilleurs
                        if ($desktops[$i]['Desktop']['group_id'] == DISP_GID) {
                            $this->Bancontenu->add($desktopId[$i], BAN_AIGUILLAGE, $courrierId);
                        } else if ($desktops[$i]['Desktop']['group_id'] == INIT_GID) {
                            $this->Bancontenu->add($desktopId[$i], BAN_DOCS, $courrierId);
                        }

                        //ajout dans la bannette de chacun des aiguilleurs
//                        $this->Bancontenu->add($desktopId[$i], $BAN, $courrierId);
                        // Notification par mail de la génération du flux
                        $user_id = $this->User->userIdByDesktop($desktopId[$i]);
                        $typeNotif = $this->User->find(
                                'first', array(
                            'fields' => array('User.typeabonnement'),
                            'conditions' => array('User.id' => $user_id),
                            'contain' => false
                                )
                        );
                        $notifInfos = $this->Notifieddesktop->Notification->setNotification('ban_xnew');
                        for ($d = 0; $d < count($desktopId); $d++) {
                            $notifyDesktops[$desktopId[$d]] = array($courrierId);
                            $notified = $this->Notifieddesktop->Notification->add($notifInfos['name'], $notifInfos['description'], $notifInfos['typeNotif'], $desktopId[$i], $notifyDesktops);
                        }
                        $notificationId = $this->Notifieddesktop->Notification->id;
                        if ($typeNotif['User']['typeabonnement'] == 'quotidien') {
                            $this->User->saveNotif($courrierId, $user_id, $notificationId, 'aiguillage');
                        } else {
                            $this->User->notifier($courrierId, $user_id, $notificationId, 'aiguillage');
                        }
                    }
                }


                // Association des fichiers aux flux générés
                if (is_array($fichier_associe)) {
                    for ($j = 0; $j < count($fichier_associe); $j++) {
                        if ($j != 0) {
                            $this->_genFileAssoc($fichier_associe[$j], $courrierId, $return, false);
                        } else {
                            $this->_genFileAssoc($fichier_associe[$j], $courrierId, $return, true);
                        }
                    }

                    $codeRetour = str_split($return['CodeRetour'], 13);
                    $return['CodeRetour'] = $codeRetour[0];
                } else {
                    $this->_genFileAssoc($fichier_associe, $courrierId, $return);
                }
                /* if(is_array($fichier_associe)){
                  for($i = 0; $i<count($fichier_associe); $i++){
                  if($i != 0){
                  $this->_genFileAssoc($fichier_associe[$i], $courrierId, $return,false);
                  }else{
                  $this->_genFileAssoc($fichier_associe[$i], $courrierId, $return,true);
                  }

                  }

                  $codeRetour = str_split($return['CodeRetour'], 13);
                  $return['CodeRetour'] = $codeRetour[0];
                  }
                  else{
                  $this->_genFileAssoc($fichier_associe, $courrierId, $return);
                  }
                  // Notification par mail de la génération du flux
                  $user_id = $this->User->userIdByDesktop($desktopId);
                  $this->User->notifier($courrierId, $user_id, 'aiguillage'); */
            } else {
                $this->rollback();
            }
        }
        return $return;
    }

    /**
     * Fonction retournant l'année du dernier numéro de référence présent en BDD
     * Ceci nous permet de savoir à quel moment la séquence courriers_reference_seq doit être remise à zéro
     * @return type
     */
    public function lastNumReference() {
        $subquery = $this->sq(
                array(
                    'fields' => array(
                        'courriers.reference'
                    ),
                    'alias' => 'courriers',
                    'conditions' => array(
                        "courriers.reference IS NOT NULL"
                    ),
                    'order' => array("courriers.reference DESC"),
                    'limit' => 1
                )
        );
        // requête pour obtenir le résultat 
        $lastNumReference = $this->query($subquery);
        // on extrait la valeur prise par le numéro de référence
        $valueLastReference = Hash::get($lastNumReference, '0.courriers.reference');
        // on ne conserve que les 4 premiers caractères 2016000001 => 2016
        $valueLastReferenceOnlyYear = substr($valueLastReference, 0, -6);
        // on retourne l'année du dernier flux référencé du coup
        return $valueLastReferenceOnlyYear;
    }

    /**
     * Fonction remontant le service auquel le flux est lié
     * @param type $id
     */
    public function getMyService($id) {


        /* Liste des profils auxquels l'utilisateur est lié
          select id, name from desktops where id in (
          select desktop_id from desktops_users where user_id=5
          union
          select desktop_id from users where id=5
          );
          7, 26, 2
          Liste des services auxquels les bureaux de l'utyilsiateur connecté sont liés
          select service_id from desktops_services where desktop_id in (2,7,26);

          2, 19
         */
    }

    /**
     * Génération d'un document lié (à partir du script lancé à la main)
     *
     * @access private
     * @param base64 $fichier_associe contenu du document lié
     * @param integer $fluxId identifiant du flux
     * @param array $return valeur de retour
     * @return void
     */
    private function _genFileAssocMailWithoutpj($file_name, $file_body, $fluxId) {

        require_once 'XML/RPC2/Client.php';
        // initialisations
        $ret = array();
        $convertorType = Configure::read('CONVERSION_TYPE');

        if (empty($convertorType)) {
            $ret['resultat'] = false;
            $ret['info'] = __('Type du programme de conversion non déclaré dans le fichier de configuration de web-GFC', true);
            return $ret;
        }

        $options = array(
            'uglyStructHack' => true
        );

        $url = 'http://' . Configure::read('CLOUDOOO_HOST') . ':' . Configure::read('CLOUDOOO_PORT');
        $client = XML_RPC2_Client::create($url, $options);
        try {
            $result = $client->convertFile(base64_encode($file_body), 'odt', 'pdf', false, true);
            $pdfMail = base64_decode($result);
        } catch (XML_RPC2_FaultException $e) {
            $this->log('Exception #' . $e->getFaultCode() . ' : ' . $e->getFaultString(), 'debug');
            return false;
        }

        $sql = "SELECT nextval('documents_name_seq'); /*" . mktime(true) . " " . $this->_lastSequenceDocument . "*/";


        $mime = "application/pdf";

        $numSeqDoc = $this->query($sql);
        if ($numSeqDoc === false) {
            return null; // FIXME: Throw exception
        }
        $this->_lastSequenceDocument = $numSeqDoc[0][0]['nextval'];

        $file_name = preg_replace( "/[:']/", "_", $file_name ). '.pdf';
        
        $countDoc = $this->Document->find('count', array('conditions' => array('Document.courrier_id' => $fluxId), 'contain' => false));
        $doc[] = array(
            'Document' => array(
                'name' => $file_name,
                'courrier_id' => $fluxId,
                'size' => strlen($pdfMail),
                'mime' => $mime,
                'ext' => 'pdf',
                'main_doc' => (($countDoc == 0)) ? true : false,
                'content' => $pdfMail
            )
        );

        $this->setDataSource(Configure::read('conn'));
        $this->Document->setDataSource(Configure::read('conn'));
        $this->Document->create();
        if (!empty($doc)) {
            $success = $this->Document->saveAll($doc, array('validate' => 'first', 'atomic' => false));
            if ($success) {
                echo 'Le fichier a été généré.';
            } else {
                echo "Le fichier n'a pas été généré.";
            }
        } else {
            echo "Aucun fichier à insérer";
        }
    }

    /**
     * Récupération du schéma du circuit emprunté par le flux
     *
     * @logical-group Flux
     * @logical-group Context
     * @user-profile User
     *
     * @access public
     * @param integer $id
     * @throws NotFoundException
     * @return void
     */
    public function getCircuitFromCourrier($fluxId) {
        $qdCircuit = array(
            'fields' => array(
                'Circuit.id',
                'Circuit.nom'
            ),
            'conditions' => array(
                'Courrier.id' => $fluxId
            ),
            'contain' => false,
            'joins' => array(
                $this->join($this->Soustype->alias),
                $this->Soustype->join($this->Soustype->Circuit->alias),
            )
        );
        $circuit = $this->find('first', $qdCircuit);
        $etapes = null;
        $visas = null;
        if (!empty($circuit)) {
            //definition de l identifiant du circuit
            $circuitId = $circuit['Circuit']['id'];
            //recuperation de la structure initiale du circuit
            $qdEtapes = array(
                'fields' => array(
                    'Etape.nom',
                    'Etape.type',
                    'Etape.ordre',
                    'Etape.description'
                ),
                'contain' => array(
                    'Composition.type_validation',
                    'Composition.trigger_id',
                    'Composition' => array(
                        CAKEFLOW_TRIGGER_MODEL => array('fields' => CAKEFLOW_TRIGGER_FIELDS)
                    )
                ),
                'conditions' => array(
                    'Etape.circuit_id' => $circuitId
                ),
                'order' => 'Etape.ordre'
            );
            $etapes = $this->Traitement->Circuit->Etape->find('all', $qdEtapes);
            // Ajout du cas où aucune composition mais on alimente la compo par les données stockées en aramétrage // SAERP
            if (Configure::read('Conf.SAERP')) {
                //@TODO: lire les infos associées au type / sous-type SAAERP (cf OPs) !!!!
                $courrier = $this->find(
                        'first', array(
                    'fields' => array_merge(
                            $this->fields(), $this->Soustype->fields(), $this->Soustype->Type->fields(), $this->Soustype->Type->Operation->fields()
                    ),
                    'conditions' => array(
                        'Courrier.id' => $fluxId
                    ),
                    'joins' => array(
                        $this->join('Soustype'),
                        $this->Soustype->join('Type'),
                        $this->Soustype->Type->join('Operation')
                    ),
                    'recursive' => -1
                        )
                );
                if (!empty($courrier['Operation'])) {
                    $sousoperation = $this->Soustype->Type->Operation->Sousoperation->find(
                            'first', array(
                        'conditions' => array(
                            'Sousoperation.soustype_id' => $courrier['Soustype']['id']
                        ),
                        'contain' => false
                            )
                    );
                    if (!empty($sousoperation)) {
                        $sousoperationName = $sousoperation['Sousoperation']['name'];
                        if (!empty($sousoperation) && $sousoperationName != Configure::read('Soustype.Fantome')) {
                            $sousoperationId = $sousoperation['Sousoperation']['id'];


                            $operationsCompos = $this->Soustype->Type->Operation->Sousoperation->DesktopmanagerSousoperation->find(
                                    'all', array(
                                'conditions' => array(
                                    'DesktopmanagerSousoperation.sousoperation_id' => $sousoperationId
                                ),
                                'contain' => false,
                                'order' => array('DesktopmanagerSousoperation.ordre ASC')
                                    )
                            );
                            foreach ($operationsCompos as $i => $compo) {
                                $operationCompo[$compo['DesktopmanagerSousoperation']['ordre']][] = $operationsCompos[$i];
                            }
                            foreach ($operationCompo as $n => $op) {
                                $intitulesagentsIds = Hash::extract($op, '{n}.DesktopmanagerSousoperation.intituleagent_id');
                                // ajout des associations entre OP, poste et bureaux
                                $associations[$n] = $this->Soustype->Type->Operation->Intituleagentbyoperation->find(
                                        'all', array(
                                    'conditions' => array(
                                        'Intituleagentbyoperation.operation_id' => $courrier['Operation']['id'],
                                        'Intituleagentbyoperation.intituleagent_id' => $intitulesagentsIds
                                    ),
                                    'contain' => array(
                                        'Desktopmanager'
                                    )
                                        )
                                );
                            }
                        }
                    }
                }
                $newetapes = array();
                foreach ($etapes as $k => $etape) {
                    $newetapes[$etape['Etape']['ordre']] = $etapes[$k];
                }
                foreach ($newetapes as $e => $etape) {
                    $desktopmanagerId = array();
                    $desktopmanagerName = array();
                    if (isset($associations)) {
                        foreach ($associations as $a => $association) {
                            foreach ($association as $v => $val) {
                                $desktopmanagerId[$a][$v] = $val['Desktopmanager']['id'];
                                $desktopmanagerName[$a][$v] = $val['Desktopmanager']['name'];
                            }
                        }
                    }
                    if (empty($etape['Composition'])) {
                        //FIXME: les compos ne sont pas "parfaites"
                        if (count($etapes) == count($operationCompo)) {
                            $compoTemp = array();
                            foreach ($operationCompo[$e] as $key => $info) {
                                $compoTemp[$key]['etape_id'] = $etape['Etape']['id']; // id de l'étape
                                $compoTemp[$key]['trigger_id'] = $desktopmanagerId[$e][$key]; // id du bureau
                                $compoTemp[$key]['Desktopmanager']['name'] = $desktopmanagerName[$e][$key]; // Nom du bureau
                                $compoTemp[$key]['type_validation'] = 'V';
                            }
                            $etapes[$e - 1]['Composition'] = $compoTemp;
                        }
                    }
                }
            }
        }
        $listOfDesktops = array();
        foreach ($etapes as $etape) {
            foreach ($etape['Composition'] as $composition) {
                $listOfDesktops[] = $composition['trigger_id'];
            }
        }
        return $listOfDesktops;
    }

    
    /**
     * Focntion permettant à l'admin (uniquement) de modifier le bancontenu pour repositionner le flux sur un autre profil
     * @param type $desktopId
     */
    public function repositionneFlux( $id, $desktopId ) {
        $sqBancontenu = $this->Bancontenu->sqDerniereBannette('Courrier.id');
        $courrier = $this->find(
            'first',
            array(
                'fields' => array_merge(
                    $this->fields(),
                    $this->Bancontenu->fields()
                ),
                'conditions' => array(
                    'Courrier.id' => $id
                ),
                'contain' => false,
                'joins' => array(
                    $this->join('Bancontenu', array('type' => 'INNER','conditions' => array("Bancontenu.id IN ( {$sqBancontenu} )") ) )
                )
            )
        );

        $group = $this->Desktop->find(
            'first',
            array(
                'conditions' => array(
                    'Desktop.id' => $desktopId
                ),
                'contain' => array(
                    'Group'
                ),
                'recursive' => -1
            )
        );

        if( $group['Group']['id'] == 7 ) {
            $banValue = BAN_AIGUILLAGE;
        }
        else if( in_array( $group['Group']['id'], array(4, 5) ) ) {
            $banValue = BAN_FLUX;
        }
        else if( $group['Group']['id'] == 3 ) {
            $banValue = BAN_DOCS;
        }

        $this->Bancontenu->updateAll(
            array(
                'Bancontenu.desktop_id' => $desktopId,
                'Bancontenu.bannette_id' => $banValue,
                'Bancontenu.etat' => 1
            ),
            array(
                'Bancontenu.id' => $courrier['Bancontenu']['id'],
                'Bancontenu.courrier_id' => $courrier['Bancontenu']['courrier_id']
            )
        );
    }
}

?>