BEGIN;

--
-- PostgreSQL database dump
--

SET statement_timeout = 0;
SET client_encoding = 'UTF8';
SET standard_conforming_strings = on;
SET check_function_bodies = false;
SET client_min_messages = warning;

--
-- Name: plpgsql; Type: EXTENSION; Schema: -; Owner: 
--

CREATE EXTENSION IF NOT EXISTS plpgsql WITH SCHEMA pg_catalog;


--
-- Name: EXTENSION plpgsql; Type: COMMENT; Schema: -; Owner: 
--

COMMENT ON EXTENSION plpgsql IS 'PL/pgSQL procedural language';


--
-- Name: fuzzystrmatch; Type: EXTENSION; Schema: -; Owner: 
--

CREATE EXTENSION IF NOT EXISTS fuzzystrmatch WITH SCHEMA public;


--
-- Name: EXTENSION fuzzystrmatch; Type: COMMENT; Schema: -; Owner: 
--

COMMENT ON EXTENSION fuzzystrmatch IS 'determine similarities and distance between strings';


SET search_path = public, pg_catalog;

--
-- Name: add_missing_constraint(text, text, text, text, text, boolean); Type: FUNCTION; Schema: public; Owner: postgres
--

CREATE FUNCTION add_missing_constraint(text, text, text, text, text, boolean) RETURNS boolean
LANGUAGE plpgsql
AS $_$
DECLARE
p_namespace   alias for $1;
p_table       alias for $2;
p_constraintname alias for $3;
p_foreigntable  alias for $4;
p_foreignkeyname alias for $5;
p_deletecascade  alias for $6;
v_row         record;
v_query       text;
BEGIN
SELECT 1 INTO v_row
FROM information_schema.table_constraints tc
LEFT JOIN information_schema.key_column_usage kcu ON (
  tc.constraint_catalog = kcu.constraint_catalog
  AND tc.constraint_schema = kcu.constraint_schema
  AND tc.constraint_name = kcu.constraint_name
)
LEFT JOIN information_schema.referential_constraints rc ON (
  tc.constraint_catalog = rc.constraint_catalog
  AND tc.constraint_schema = rc.constraint_schema
  AND tc.constraint_name = rc.constraint_name
)
LEFT JOIN information_schema.constraint_column_usage ccu ON (
  rc.unique_constraint_catalog = ccu.constraint_catalog
  AND rc.unique_constraint_schema = ccu.constraint_schema
  AND rc.unique_constraint_name = ccu.constraint_name
)
WHERE
tc.table_schema = p_namespace
AND tc.table_name = p_table
AND tc.constraint_type = 'FOREIGN KEY'
AND tc.constraint_name = substring( p_constraintname from 1 for 63 ) -- INFO: les noms sont juste tronqués, pas de chiffre à la fin -> ça ne devrait pas poser de problème
AND kcu.column_name = p_foreignkeyname
AND ccu.table_name = p_foreigntable
AND ccu.column_name = 'id';

IF NOT FOUND THEN
RAISE NOTICE 'Upgrade table %.% - add constraint %', p_namespace, p_table, p_constraintname;
v_query := 'alter table ' || p_namespace || '.' || p_table || ' add constraint ';
v_query := v_query || p_constraintname || ' FOREIGN KEY (' || p_foreignkeyname || ') REFERENCES ' || p_foreigntable || '(id)';

IF p_deletecascade THEN
v_query := v_query || ' ON DELETE CASCADE ON UPDATE CASCADE;';
ELSE
v_query := v_query || ' ON DELETE SET NULL ON UPDATE CASCADE;';
END IF;

EXECUTE v_query;
RETURN 't';
ELSE
RETURN 'f';
END IF;
END;
$_$;


ALTER FUNCTION public.add_missing_constraint(text, text, text, text, text, boolean) OWNER TO postgres;

--
-- Name: FUNCTION add_missing_constraint(text, text, text, text, text, boolean); Type: COMMENT; Schema: public; Owner: postgres
--

COMMENT ON FUNCTION add_missing_constraint(text, text, text, text, text, boolean) IS 'Add a constraint to a table if it is missing';


--
-- Name: add_missing_table_field(text, text, text, text); Type: FUNCTION; Schema: public; Owner: webgfc
--

CREATE FUNCTION add_missing_table_field(text, text, text, text) RETURNS boolean
LANGUAGE plpgsql
AS $_$
DECLARE
p_namespace alias for $1;
p_table     alias for $2;
p_field     alias for $3;
p_type      alias for $4;
v_row       record;
v_query     text;
BEGIN
select 1 into v_row from pg_namespace n, pg_class c, pg_attribute a
where
         --public.slon_quote_brute(n.nspname) = p_namespace and
         n.nspname = p_namespace and
         c.relnamespace = n.oid and
         --public.slon_quote_brute(c.relname) = p_table and
         c.relname = p_table and
         a.attrelid = c.oid and
         --public.slon_quote_brute(a.attname) = p_field;
         a.attname = p_field;
         if not found then
         raise notice 'Upgrade table %.% - add field %', p_namespace, p_table, p_field;
         v_query := 'alter table ' || p_namespace || '.' || p_table || ' add column ';
         v_query := v_query || p_field || ' ' || p_type || ';';
         execute v_query;
         return 't';
         else
         return 'f';
         end if;
         END;$_$;


         ALTER FUNCTION public.add_missing_table_field(text, text, text, text) OWNER TO webgfc;

--
-- Name: FUNCTION add_missing_table_field(text, text, text, text); Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON FUNCTION add_missing_table_field(text, text, text, text) IS 'Drops a column from a table if it exists.';


--
-- Name: alter_table_drop_column_if_exists(text, text, text); Type: FUNCTION; Schema: public; Owner: webgfc
--

CREATE FUNCTION alter_table_drop_column_if_exists(text, text, text) RETURNS boolean
LANGUAGE plpgsql
AS $_$
DECLARE
p_namespace alias for $1;
p_table     alias for $2;
p_field     alias for $3;
v_row       record;
v_query     text;
BEGIN
SELECT 1 INTO v_row FROM pg_namespace n, pg_class c, pg_attribute a
WHERE
n.nspname = p_namespace
AND c.relnamespace = n.oid
AND c.relname = p_table
AND a.attrelid = c.oid
AND a.attname = p_field;
IF FOUND THEN
RAISE NOTICE 'Upgrade table %.% - drop field %', p_namespace, p_table, p_field;
v_query := 'ALTER TABLE ' || p_namespace || '.' || p_table || ' DROP column ' || p_field || ';';
EXECUTE v_query;
RETURN 't';
ELSE
RETURN 'f';
END IF;
END;
$_$;


ALTER FUNCTION public.alter_table_drop_column_if_exists(text, text, text) OWNER TO webgfc;

--
-- Name: alter_table_drop_constraint_if_exists(text, text, text); Type: FUNCTION; Schema: public; Owner: webgfc
--

CREATE FUNCTION alter_table_drop_constraint_if_exists(text, text, text) RETURNS boolean
LANGUAGE plpgsql
AS $_$
DECLARE
p_namespace     alias for $1;
p_table       alias for $2;
p_constraintname    alias for $3;
v_row       record;
v_query       text;
BEGIN
SELECT
1 INTO v_row
FROM pg_constraint
INNER JOIN pg_namespace ON ( pg_constraint.connamespace = pg_namespace.oid )
INNER JOIN pg_class ON ( pg_constraint.conrelid = pg_class.oid )
WHERE
pg_namespace.nspname = p_namespace
AND pg_class.relname = p_table
AND pg_constraint.conname = p_constraintname;
IF FOUND THEN
RAISE NOTICE 'Alter table %.% - drop constraint %', p_namespace, p_table, p_constraintname;
v_query := 'ALTER TABLE ' || p_namespace || '.' || p_table || ' DROP constraint ' || p_constraintname || ';';
EXECUTE v_query;
RETURN 't';
ELSE
RETURN 'f';
END IF;
END;
$_$;


ALTER FUNCTION public.alter_table_drop_constraint_if_exists(text, text, text) OWNER TO webgfc;

--
-- Name: FUNCTION alter_table_drop_constraint_if_exists(text, text, text); Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON FUNCTION alter_table_drop_constraint_if_exists(text, text, text) IS 'Équivalent de la fonctionnalité ALTER TABLE <table> DROP CONSTRAINT <name> disponible à partir de PostgreSQl 9.1';


--
-- Name: cakephp_validate_in_list(text, text[]); Type: FUNCTION; Schema: public; Owner: webgfc
--

CREATE FUNCTION cakephp_validate_in_list(text, text[]) RETURNS boolean
LANGUAGE sql IMMUTABLE
AS $_$
SELECT $1 IS NULL OR ( ARRAY[CAST($1 AS TEXT)] <@ CAST($2 AS TEXT[]) );
$_$;


ALTER FUNCTION public.cakephp_validate_in_list(text, text[]) OWNER TO webgfc;

--
-- Name: FUNCTION cakephp_validate_in_list(text, text[]); Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON FUNCTION cakephp_validate_in_list(text, text[]) IS '@see http://api.cakephp.org/class/validation#method-ValidationinList';


--
-- Name: noaccents_upper(text); Type: FUNCTION; Schema: public; Owner: postgres
--

CREATE FUNCTION noaccents_upper(text) RETURNS text
LANGUAGE plpgsql IMMUTABLE STRICT
AS $_$
DECLARE
st text;

BEGIN
        -- On transforme les caractèes accentués et on passe en majuscule
        st:=translate($1,'aàäâeéèêëiïîoôöuùûücçñAÀÄÂEÉÈÊËIÏÎOÔÖUÙÛÜCÇÑ','AAAAEEEEEIIIOOOUUUUCCNAAAAEEEEEIIIOOOUUUUCCN');
        st:=upper(st);

        return st;
        END;
        $_$;


        ALTER FUNCTION public.noaccents_upper(text) OWNER TO postgres;

--
-- Name: rebuilt_sequences(); Type: FUNCTION; Schema: public; Owner: webgfc
--

CREATE FUNCTION rebuilt_sequences() RETURNS integer
LANGUAGE plpgsql
AS $$
DECLARE sequencedefs RECORD; c integer ;
BEGIN
FOR sequencedefs IN Select
constraint_column_usage.table_name as tablename,
constraint_column_usage.table_name as tablename,
constraint_column_usage.column_name as columnname,
replace(replace(columns.column_default,'''::regclass)',''),'nextval(''','') as sequencename
from information_schema.constraint_column_usage, information_schema.columns
where constraint_column_usage.table_schema ='public' AND
columns.table_schema = 'public' AND columns.table_name=constraint_column_usage.table_name
AND constraint_column_usage.column_name = columns.column_name
AND columns.column_default is not null
LOOP
EXECUTE 'select max('||sequencedefs.columnname||') from ' || sequencedefs.tablename INTO c;
IF c is null THEN c = 0; END IF;
IF c is not null THEN c = c+ 1; END IF;
EXECUTE 'alter sequence ' || sequencedefs.sequencename ||' restart  with ' || c;
END LOOP;

RETURN 1; END;
$$;


ALTER FUNCTION public.rebuilt_sequences() OWNER TO webgfc;

SET default_tablespace = '';

SET default_with_oids = false;

--
-- Name: acos; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE acos (
  id integer NOT NULL,
  parent_id integer,
  model character varying(255) DEFAULT NULL::character varying,
  foreign_key integer,
  alias character varying(255) DEFAULT NULL::character varying,
  lft integer,
  rght integer
);


ALTER TABLE public.acos OWNER TO webgfc;

--
-- Name: acos_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE acos_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.acos_id_seq OWNER TO webgfc;

--
-- Name: acos_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE acos_id_seq OWNED BY acos.id;


--
-- Name: activites; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE activites (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  active boolean DEFAULT true NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  description character varying(255)
);


ALTER TABLE public.activites OWNER TO postgres;

--
-- Name: TABLE activites; Type: COMMENT; Schema: public; Owner: postgres
--

COMMENT ON TABLE activites IS 'Table permettant de stocker les activités liées aux contacts (SAERP)';


--
-- Name: activites_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--

CREATE SEQUENCE activites_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.activites_id_seq OWNER TO postgres;

--
-- Name: activites_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--

ALTER SEQUENCE activites_id_seq OWNED BY activites.id;


--
-- Name: activites_organismes; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE activites_organismes (
  id integer NOT NULL,
  activite_id integer NOT NULL,
  organisme_id integer NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.activites_organismes OWNER TO postgres;

--
-- Name: TABLE activites_organismes; Type: COMMENT; Schema: public; Owner: postgres
--

COMMENT ON TABLE activites_organismes IS 'Table de liaison entre les activités et les organismes (SAERP)';


--
-- Name: activites_organismes_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--

CREATE SEQUENCE activites_organismes_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.activites_organismes_id_seq OWNER TO postgres;

--
-- Name: activites_organismes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--

ALTER SEQUENCE activites_organismes_id_seq OWNED BY activites_organismes.id;


--
-- Name: addressbooks; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE addressbooks (
  id integer NOT NULL,
  name character varying(255),
  description bytea,
  created timestamp without time zone DEFAULT now(),
  modified timestamp without time zone DEFAULT now(),
  foreign_key integer,
  active boolean DEFAULT true NOT NULL,
  private boolean DEFAULT false
);


ALTER TABLE public.addressbooks OWNER TO webgfc;

--
-- Name: addressbooks_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE addressbooks_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.addressbooks_id_seq OWNER TO webgfc;

--
-- Name: addressbooks_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE addressbooks_id_seq OWNED BY addressbooks.id;


--
-- Name: affaires; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE affaires (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  comment bytea,
  dossier_id integer,
  created timestamp without time zone,
  modified timestamp without time zone
);


ALTER TABLE public.affaires OWNER TO webgfc;

--
-- Name: affaires_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE affaires_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.affaires_id_seq OWNER TO webgfc;

--
-- Name: affaires_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE affaires_id_seq OWNED BY affaires.id;


--
-- Name: armodels; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE armodels (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  gedpath text,
  content bytea,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  soustype_id integer NOT NULL,
  sms_content character varying(160),
  email_content bytea,
  email_title character varying(255),
  format character varying(50),
  mime character varying(255),
  ext character varying(10),
  size integer,
  preview bytea
);


ALTER TABLE public.armodels OWNER TO webgfc;

--
-- Name: armodels_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE armodels_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.armodels_id_seq OWNER TO webgfc;

--
-- Name: armodels_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE armodels_id_seq OWNED BY armodels.id;


--
-- Name: aros; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE aros (
  id integer NOT NULL,
  parent_id integer,
  model character varying(255) DEFAULT NULL::character varying,
  foreign_key integer,
  alias character varying(255) DEFAULT NULL::character varying,
  lft integer,
  rght integer
);


ALTER TABLE public.aros OWNER TO webgfc;

--
-- Name: aros_acos; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE aros_acos (
  id integer NOT NULL,
  aro_id integer NOT NULL,
  aco_id integer NOT NULL,
  _create character varying(2) DEFAULT '0'::character varying(255) NOT NULL,
  _read character varying(2) DEFAULT '0'::character varying(255) NOT NULL,
  _update character varying(2) DEFAULT '0'::character varying(255) NOT NULL,
  _delete character varying(2) DEFAULT '0'::character varying(255) NOT NULL
);


ALTER TABLE public.aros_acos OWNER TO webgfc;

--
-- Name: aros_acos_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE aros_acos_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.aros_acos_id_seq OWNER TO webgfc;

--
-- Name: aros_acos_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE aros_acos_id_seq OWNED BY aros_acos.id;


--
-- Name: aros_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE aros_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.aros_id_seq OWNER TO webgfc;

--
-- Name: aros_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE aros_id_seq OWNED BY aros.id;


--
-- Name: ars; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE ars (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  gedpath text,
  content bytea,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  courrier_id integer NOT NULL,
  format character varying(50),
  sms_dest character varying(20),
  sms_content character varying(160),
  email_title character varying(255),
  email_content bytea,
  ext character varying(10),
  mime character varying(255),
  used boolean DEFAULT false NOT NULL,
  size integer,
  email_dest character varying(255),
  preview bytea
);


ALTER TABLE public.ars OWNER TO webgfc;

--
-- Name: ars_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE ars_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.ars_id_seq OWNER TO webgfc;

--
-- Name: ars_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE ars_id_seq OWNED BY ars.id;


--
-- Name: bancontenus; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE bancontenus (
  id integer NOT NULL,
  bannette_id integer,
  courrier_id integer,
  created timestamp without time zone,
  modified timestamp without time zone,
  etat integer DEFAULT 1 NOT NULL,
  desktop_id integer,
  read boolean DEFAULT false NOT NULL,
  desktopmanager_id integer
);


ALTER TABLE public.bancontenus OWNER TO webgfc;

--
-- Name: bancontenus_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE bancontenus_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.bancontenus_id_seq OWNER TO webgfc;

--
-- Name: bancontenus_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE bancontenus_id_seq OWNED BY bancontenus.id;


--
-- Name: bannettes; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE bannettes (
  id integer NOT NULL,
  name character varying(255)
);


ALTER TABLE public.bannettes OWNER TO webgfc;

--
-- Name: bannettes_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE bannettes_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.bannettes_id_seq OWNER TO webgfc;

--
-- Name: bannettes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE bannettes_id_seq OWNED BY bannettes.id;


--
-- Name: bans; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE bans (
  id integer NOT NULL,
  name character varying(100) NOT NULL,
  active boolean DEFAULT true,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.bans OWNER TO webgfc;

--
-- Name: TABLE bans; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE bans IS 'Table permettant le stockage des Bases Adresse Nationale';


--
-- Name: bans_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE bans_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.bans_id_seq OWNER TO webgfc;

--
-- Name: bans_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE bans_id_seq OWNED BY bans.id;


--
-- Name: bansadresses; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE bansadresses (
  id integer NOT NULL,
  bancommune_id integer NOT NULL,
  name character varying(255) NOT NULL,
  ident character varying(30) NOT NULL,
  nom_voie character varying(255),
  id_fantoir character varying(255),
  numero character varying(255),
  rep character varying(255),
  code_insee character varying(255),
  code_post character varying(255),
  alias character varying(255),
  nom_ld character varying(255),
  nom_afnor character varying(255),
  x character varying(255),
  y character varying(255),
  lon character varying(255),
  lat character varying(255),
  nom_commune character varying(255),
  active boolean DEFAULT true,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.bansadresses OWNER TO webgfc;

--
-- Name: TABLE bansadresses; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE bansadresses IS 'Table permettant le stockage des adresses par commune de la Base Adresse Nationale';


--
-- Name: bansadresses_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE bansadresses_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.bansadresses_id_seq OWNER TO webgfc;

--
-- Name: bansadresses_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE bansadresses_id_seq OWNED BY bansadresses.id;


--
-- Name: banscommunes; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE banscommunes (
  id integer NOT NULL,
  ban_id integer NOT NULL,
  name character varying(255) NOT NULL,
  active boolean DEFAULT true,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.banscommunes OWNER TO webgfc;

--
-- Name: TABLE banscommunes; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE banscommunes IS 'Table permettant le stockage des Communes par Base Adresse Nationale';


--
-- Name: banscommunes_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE banscommunes_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.banscommunes_id_seq OWNER TO webgfc;

--
-- Name: banscommunes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE banscommunes_id_seq OWNED BY banscommunes.id;


--
-- Name: comments; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE comments (
  id integer NOT NULL,
  slug character varying(255),
  content bytea,
  objet TEXT,
  private boolean DEFAULT false,
  owner_id integer NOT NULL,
  target_id integer NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.comments OWNER TO webgfc;

--
-- Name: comments_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE comments_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.comments_id_seq OWNER TO webgfc;

--
-- Name: comments_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE comments_id_seq OWNED BY comments.id;


--
-- Name: comments_readers; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE comments_readers (
  id integer NOT NULL,
  reader_id integer NOT NULL,
  comment_id integer NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.comments_readers OWNER TO webgfc;

--
-- Name: comments_readers_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE comments_readers_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.comments_readers_id_seq OWNER TO webgfc;

--
-- Name: comments_readers_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE comments_readers_id_seq OWNED BY comments_readers.id;


--
-- Name: compteurs; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE compteurs (
  id integer NOT NULL,
  nom character varying(255) NOT NULL,
  commentaire character varying(255) NOT NULL,
  def_compteur character varying(255) NOT NULL,
  sequence_id integer NOT NULL,
  def_reinit character varying(255) NOT NULL,
  created timestamp without time zone,
  modified timestamp without time zone,
  val_reinit character varying(255)
);


ALTER TABLE public.compteurs OWNER TO webgfc;

--
-- Name: compteurs_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE compteurs_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.compteurs_id_seq OWNER TO webgfc;

--
-- Name: compteurs_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE compteurs_id_seq OWNED BY compteurs.id;


--
-- Name: connecteurs; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE connecteurs (
  id integer NOT NULL,
  name character varying(250) DEFAULT NULL::character varying,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  use_signature boolean,
  signature_protocol character varying(30),
  host character varying(255),
  login character varying(255),
  pwd character varying(255),
  type character varying(255),
  visibility character varying(255),
  wsdl character varying(255),
  cacert bytea,
  clientcert bytea,
  cert bytea,
  certpwd bytea,
  nom_cert text,
  use_ged boolean,
  ged_url character varying(255),
  ged_login character varying(255),
  ged_passwd character varying(255),
  ged_repo character varying(255),
  ged_xml_version character varying(1),
  ged_cmis_version character varying(10)
);


ALTER TABLE public.connecteurs OWNER TO webgfc;

--
-- Name: TABLE connecteurs; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE connecteurs IS 'Table permettant la définition des différents connecteurs présents';


--
-- Name: connecteurs_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE connecteurs_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.connecteurs_id_seq OWNER TO webgfc;

--
-- Name: connecteurs_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE connecteurs_id_seq OWNED BY connecteurs.id;


--
-- Name: consultations; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE consultations (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  numero character varying(10) NOT NULL,
  objet character varying(255),
  datelimite date NOT NULL,
  heurelimite time without time zone NOT NULL,
  operation_id integer NOT NULL,
  chrono character varying(50),
  titulaire character varying(255),
  active boolean DEFAULT true NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.consultations OWNER TO postgres;

--
-- Name: TABLE consultations; Type: COMMENT; Schema: public; Owner: postgres
--

COMMENT ON TABLE consultations IS 'Table permettant de stocker les informations en lien avec les consultations (SAERP)';


--
-- Name: consultations_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--

CREATE SEQUENCE consultations_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.consultations_id_seq OWNER TO postgres;

--
-- Name: consultations_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--

ALTER SEQUENCE consultations_id_seq OWNED BY consultations.id;


--
-- Name: contactinfos; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE contactinfos (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  civilite character varying(255),
  nom character varying(255) NOT NULL,
  prenom character varying(255),
  email character varying(255),
  adresse character varying(255),
  compl character varying(255),
  cp character varying(5),
  ville character varying(255),
  region character varying(255),
  pays character varying(255),
  tel character varying(20),
  role character varying(255),
  organisation character varying(255),
  active boolean DEFAULT true NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  contact_id integer NOT NULL,
  slug character varying,
  numvoie character varying(20),
  typevoie character varying(4),
  nomvoie character varying(25),
  adressecomplete character varying(255),
  portable character varying(20),
  ban_id integer,
  bancommune_id integer,
  banadresse character varying(250)
);


ALTER TABLE public.contactinfos OWNER TO webgfc;

--
-- Name: contactinfos_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE contactinfos_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.contactinfos_id_seq OWNER TO webgfc;

--
-- Name: contactinfos_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE contactinfos_id_seq OWNED BY contactinfos.id;


--
-- Name: contacts; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE contacts (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  civilite character varying(255),
  nom character varying(255) NOT NULL,
  prenom character varying(255),
  created timestamp without time zone DEFAULT now(),
  modified timestamp without time zone DEFAULT now(),
  addressbook_id integer,
  foreign_key integer,
  active boolean DEFAULT true NOT NULL,
  citoyen boolean DEFAULT true NOT NULL,
  slug character varying,
  organisme_id integer NOT NULL,
  email character varying(255),
  adresse text,
  tel character varying(20),
  portable character varying(20),
  role character varying(255),
  ville character varying(255),
  ban_id integer,
  bancommune_id integer,
  banadresse character varying(255),
  numvoie character varying(255),
  compl character varying(255),
  cp character varying(5),
  nomvoie character varying(255),
  adressecomplete character varying(255),
  pays character varying(50) DEFAULT 'FRANCE'::character varying(255),
  lignedirecte character varying(20),
  observations text,
  activite_id integer,
  fonction_id integer
);


ALTER TABLE public.contacts OWNER TO webgfc;

--
-- Name: contacts_events; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE contacts_events (
  id integer NOT NULL,
  contact_id integer NOT NULL,
  event_id integer NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.contacts_events OWNER TO postgres;

--
-- Name: TABLE contacts_events; Type: COMMENT; Schema: public; Owner: postgres
--

COMMENT ON TABLE contacts_events IS 'Table de liaison entre les contacts et les événements(SAERP)';


--
-- Name: contacts_events_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--

CREATE SEQUENCE contacts_events_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.contacts_events_id_seq OWNER TO postgres;

--
-- Name: contacts_events_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--

ALTER SEQUENCE contacts_events_id_seq OWNED BY contacts_events.id;


--
-- Name: contacts_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE contacts_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.contacts_id_seq OWNER TO webgfc;

--
-- Name: contacts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE contacts_id_seq OWNED BY contacts.id;


--
-- Name: contacts_operations; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE contacts_operations (
  id integer NOT NULL,
  contact_id integer NOT NULL,
  operation_id integer NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.contacts_operations OWNER TO postgres;

--
-- Name: TABLE contacts_operations; Type: COMMENT; Schema: public; Owner: postgres
--

COMMENT ON TABLE contacts_operations IS 'Table de liaison entre les contacts et les opérations (SAERP)';


--
-- Name: contacts_operations_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--

CREATE SEQUENCE contacts_operations_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.contacts_operations_id_seq OWNER TO postgres;

--
-- Name: contacts_operations_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--

ALTER SEQUENCE contacts_operations_id_seq OWNED BY contacts_operations.id;


--
-- Name: contractants; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE contractants (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  description character varying(255),
  active boolean DEFAULT true NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.contractants OWNER TO postgres;

--
-- Name: TABLE contractants; Type: COMMENT; Schema: public; Owner: postgres
--

COMMENT ON TABLE contractants IS 'Table permettant de définir les types de contractant pour les marchés (SAERP)';


--
-- Name: contractants_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--

CREATE SEQUENCE contractants_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.contractants_id_seq OWNER TO postgres;

--
-- Name: contractants_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--

ALTER SEQUENCE contractants_id_seq OWNED BY contractants.id;


SET default_with_oids = true;

--
-- Name: courriers; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE courriers (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  fichier_associe character varying(255),
  objet text,
  ref_ancien smallint,
  soustype_id integer,
  lft integer,
  rght integer,
  parent_id integer,
  affaire_id integer,
  adressemail character varying(255),
  wav_associe character varying(255),
  video_associe character varying(255),
  numtelsms character varying(15),
  messagesms character varying(160),
  reference character varying(255),
  fichier_genere character varying(255),
  priorite integer DEFAULT 0 NOT NULL,
  user_creator_id integer,
  desktop_creator_id integer,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  date date DEFAULT now() NOT NULL,
  datereception date DEFAULT now() NOT NULL,
  direction integer,
  contactinfo_id integer,
  service_creator_id integer,
  delai_nb smallint,
  delai_unite smallint,
  affairesuiviepar_id integer,
  parapheur_etat integer,
  parapheur_id character varying(50),
  parapheur_commentaire text,
  parapheur_cible character varying(250),
  signee boolean,
  motifrefus text,
  dateenvoiparapheur timestamp without time zone,
  mail_retard_envoye boolean DEFAULT false,
  numero_depot integer,
  origineflux_id integer,
  typecontact character varying(20),
  contact_id integer,
  contact integer,
  organisme_id integer,
  marche_id integer,
  typedocument character varying(50),
  consultation_id integer,
  marcheconsult character varying(20),
  mail_clos_envoye boolean DEFAULT false
);


ALTER TABLE public.courriers OWNER TO webgfc;

--
-- Name: courriers_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE courriers_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.courriers_id_seq OWNER TO webgfc;

--
-- Name: courriers_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE courriers_id_seq OWNED BY courriers.id;


SET default_with_oids = false;

--
-- Name: courriers_metadonnees; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE courriers_metadonnees (
  id integer NOT NULL,
  valeur character varying(255),
  created timestamp without time zone,
  modified timestamp without time zone,
  courrier_id integer NOT NULL,
  metadonnee_id integer NOT NULL
);


ALTER TABLE public.courriers_metadonnees OWNER TO webgfc;

--
-- Name: courriers_metadonnees_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE courriers_metadonnees_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.courriers_metadonnees_id_seq OWNER TO webgfc;

--
-- Name: courriers_metadonnees_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE courriers_metadonnees_id_seq OWNED BY courriers_metadonnees.id;


--
-- Name: courriers_reference_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE courriers_reference_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.courriers_reference_seq OWNER TO webgfc;

--
-- Name: courriers_repertoires; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE courriers_repertoires (
  id integer NOT NULL,
  courrier_id integer,
  repertoire_id integer
);


ALTER TABLE public.courriers_repertoires OWNER TO webgfc;

--
-- Name: courriers_repertoires_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE courriers_repertoires_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.courriers_repertoires_id_seq OWNER TO webgfc;

--
-- Name: courriers_repertoires_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE courriers_repertoires_id_seq OWNED BY courriers_repertoires.id;


--
-- Name: dacos; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE dacos (
  id integer NOT NULL,
  parent_id integer,
  model character varying(255),
  foreign_key integer,
  alias character varying(255),
  lft integer,
  rght integer
);


ALTER TABLE public.dacos OWNER TO webgfc;

--
-- Name: dacos_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE dacos_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.dacos_id_seq OWNER TO webgfc;

--
-- Name: dacos_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE dacos_id_seq OWNED BY dacos.id;


--
-- Name: daros; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE daros (
  id integer NOT NULL,
  parent_id integer,
  model character varying(255),
  foreign_key integer,
  alias character varying(255),
  lft integer,
  rght integer
);


ALTER TABLE public.daros OWNER TO webgfc;

--
-- Name: daros_dacos; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE daros_dacos (
  id integer NOT NULL,
  daro_id integer NOT NULL,
  daco_id integer NOT NULL,
  _create character(2) DEFAULT 0 NOT NULL,
  _read character(2) DEFAULT 0 NOT NULL,
  _update character(2) DEFAULT 0 NOT NULL,
  _delete character(2) DEFAULT 0 NOT NULL
);


ALTER TABLE public.daros_dacos OWNER TO webgfc;

--
-- Name: daros_dacos_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE daros_dacos_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.daros_dacos_id_seq OWNER TO webgfc;

--
-- Name: daros_dacos_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE daros_dacos_id_seq OWNED BY daros_dacos.id;


--
-- Name: daros_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE daros_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.daros_id_seq OWNER TO webgfc;

--
-- Name: daros_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE daros_id_seq OWNED BY daros.id;


--
-- Name: desktops; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE desktops (
  name character varying(255) NOT NULL,
  group_id integer NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  id integer NOT NULL,
  active boolean DEFAULT true NOT NULL
);


ALTER TABLE public.desktops OWNER TO webgfc;

--
-- Name: desktops_desktopsmanagers; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE desktops_desktopsmanagers (
  id integer NOT NULL,
  desktop_id integer,
  desktopmanager_id integer,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.desktops_desktopsmanagers OWNER TO webgfc;

--
-- Name: desktops_desktopsmanagers_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE desktops_desktopsmanagers_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.desktops_desktopsmanagers_id_seq OWNER TO webgfc;

--
-- Name: desktops_desktopsmanagers_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE desktops_desktopsmanagers_id_seq OWNED BY desktops_desktopsmanagers.id;


--
-- Name: desktops_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE desktops_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.desktops_id_seq OWNER TO webgfc;

--
-- Name: desktops_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE desktops_id_seq OWNED BY desktops.id;


--
-- Name: desktops_services; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE desktops_services (
  id integer NOT NULL,
  service_id integer NOT NULL,
  desktop_id integer NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.desktops_services OWNER TO webgfc;

--
-- Name: desktops_services_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE desktops_services_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.desktops_services_id_seq OWNER TO webgfc;

--
-- Name: desktops_services_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE desktops_services_id_seq OWNED BY desktops_services.id;


--
-- Name: desktops_taches; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE desktops_taches (
  id integer NOT NULL,
  desktop_id integer NOT NULL,
  tache_id integer NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.desktops_taches OWNER TO webgfc;

--
-- Name: desktops_taches_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE desktops_taches_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.desktops_taches_id_seq OWNER TO webgfc;

--
-- Name: desktops_taches_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE desktops_taches_id_seq OWNED BY desktops_taches.id;


--
-- Name: desktops_users; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE desktops_users (
  id integer NOT NULL,
  user_id integer NOT NULL,
  desktop_id integer NOT NULL,
  delegation boolean DEFAULT false,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.desktops_users OWNER TO webgfc;

--
-- Name: desktops_users_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE desktops_users_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.desktops_users_id_seq OWNER TO webgfc;

--
-- Name: desktops_users_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE desktops_users_id_seq OWNED BY desktops_users.id;


--
-- Name: desktopsmanagers; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE desktopsmanagers (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  active boolean DEFAULT true NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  isdispatch boolean,
  intituleagent_id integer
);


ALTER TABLE public.desktopsmanagers OWNER TO webgfc;

--
-- Name: desktopsmanagers_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE desktopsmanagers_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.desktopsmanagers_id_seq OWNER TO webgfc;

--
-- Name: desktopsmanagers_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE desktopsmanagers_id_seq OWNED BY desktopsmanagers.id;


--
-- Name: desktopsmanagers_sousoperations; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE desktopsmanagers_sousoperations (
  id integer NOT NULL,
  desktopmanager_id integer,
  sousoperation_id integer NOT NULL,
  ordre character varying(10) NOT NULL,
  intituleagent_id integer NOT NULL,
  active boolean DEFAULT true NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  typeetape integer
);


ALTER TABLE public.desktopsmanagers_sousoperations OWNER TO webgfc;

--
-- Name: TABLE desktopsmanagers_sousoperations; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE desktopsmanagers_sousoperations IS 'Table de liaison entre les bureaux et les opérations (SAERP)';


--
-- Name: desktopsmanagers_sousoperations_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE desktopsmanagers_sousoperations_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.desktopsmanagers_sousoperations_id_seq OWNER TO webgfc;

--
-- Name: desktopsmanagers_sousoperations_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE desktopsmanagers_sousoperations_id_seq OWNED BY desktopsmanagers_sousoperations.id;


--
-- Name: documents; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE documents (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  gedpath text,
  content bytea,
  size integer NOT NULL,
  mime character varying(255),
  preview bytea,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  courrier_id integer NOT NULL,
  ext character varying(10),
  main_doc boolean DEFAULT false NOT NULL,
  parapheur_bordereau bytea,
  courrier_pdf bytea,
  signee boolean,
  signature bytea,
  desktop_creator_id integer,
  path text
);


ALTER TABLE public.documents OWNER TO webgfc;

--
-- Name: documents_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE documents_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.documents_id_seq OWNER TO webgfc;

--
-- Name: documents_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE documents_id_seq OWNED BY documents.id;


--
-- Name: documents_name_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE documents_name_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.documents_name_seq OWNER TO webgfc;

--
-- Name: dossiers; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE dossiers (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  comment bytea,
  created timestamp without time zone,
  modified timestamp without time zone
);


ALTER TABLE public.dossiers OWNER TO webgfc;

--
-- Name: dossiers_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE dossiers_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.dossiers_id_seq OWNER TO webgfc;

--
-- Name: dossiers_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE dossiers_id_seq OWNED BY dossiers.id;


--
-- Name: events; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE events (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  description character varying(255) NOT NULL,
  active boolean DEFAULT true NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.events OWNER TO postgres;

--
-- Name: TABLE events; Type: COMMENT; Schema: public; Owner: postgres
--

COMMENT ON TABLE events IS 'Table permettant de stocker les événements liés aux contacts (SAERP)';


--
-- Name: events_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--

CREATE SEQUENCE events_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.events_id_seq OWNER TO postgres;

--
-- Name: events_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--

ALTER SEQUENCE events_id_seq OWNED BY events.id;


--
-- Name: fonctions; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE fonctions (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  description character varying(255),
  active boolean DEFAULT true NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.fonctions OWNER TO postgres;

--
-- Name: TABLE fonctions; Type: COMMENT; Schema: public; Owner: postgres
--

COMMENT ON TABLE fonctions IS 'Table permettant de stocker les fonctions liées aux contacts (SAERP)';


--
-- Name: fonctions_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--

CREATE SEQUENCE fonctions_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.fonctions_id_seq OWNER TO postgres;

--
-- Name: fonctions_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--

ALTER SEQUENCE fonctions_id_seq OWNED BY fonctions.id;


--
-- Name: formatsreponse; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE formatsreponse (
  id integer NOT NULL,
  name character varying(255) NOT NULL
);


ALTER TABLE public.formatsreponse OWNER TO webgfc;

--
-- Name: TABLE formatsreponse; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE formatsreponse IS 'Table pour les formats de réponse';


--
-- Name: formatsreponse_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE formatsreponse_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.formatsreponse_id_seq OWNER TO webgfc;

--
-- Name: formatsreponse_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE formatsreponse_id_seq OWNED BY formatsreponse.id;


--
-- Name: formatsreponse_soustypes; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE formatsreponse_soustypes (
  id integer NOT NULL,
  formatreponse_id integer NOT NULL,
  soustype_id integer NOT NULL,
  created timestamp without time zone,
  modified timestamp without time zone
);


ALTER TABLE public.formatsreponse_soustypes OWNER TO webgfc;

--
-- Name: TABLE formatsreponse_soustypes; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE formatsreponse_soustypes IS 'Table pour lier les sous-types aux différents formats de réponse';


--
-- Name: formatsreponse_soustypes_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE formatsreponse_soustypes_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.formatsreponse_soustypes_id_seq OWNER TO webgfc;

--
-- Name: formatsreponse_soustypes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE formatsreponse_soustypes_id_seq OWNED BY formatsreponse_soustypes.id;


--
-- Name: gabaritsdocuments; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE gabaritsdocuments (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  content bytea,
  format character varying(50),
  mime character varying(255),
  ext character varying(255),
  size character varying(255),
  preview bytea,
  active boolean DEFAULT true NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.gabaritsdocuments OWNER TO webgfc;

--
-- Name: TABLE gabaritsdocuments; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE gabaritsdocuments IS 'Table permettant de stocker les gabarits de documents (SAERP)';


--
-- Name: gabaritsdocuments_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE gabaritsdocuments_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.gabaritsdocuments_id_seq OWNER TO webgfc;

--
-- Name: gabaritsdocuments_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE gabaritsdocuments_id_seq OWNED BY gabaritsdocuments.id;


--
-- Name: groups; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE groups (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  created timestamp without time zone,
  modified timestamp without time zone,
  readonly boolean DEFAULT false NOT NULL,
  active boolean DEFAULT true NOT NULL,
  group_id integer
);


ALTER TABLE public.groups OWNER TO webgfc;

--
-- Name: groups_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE groups_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.groups_id_seq OWNER TO webgfc;

--
-- Name: groups_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE groups_id_seq OWNED BY groups.id;


--
-- Name: inits_services; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE inits_services (
  id integer NOT NULL,
  desktop_id integer NOT NULL,
  service_id integer NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.inits_services OWNER TO webgfc;

--
-- Name: inits_services_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE inits_services_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.inits_services_id_seq OWNER TO webgfc;

--
-- Name: inits_services_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE inits_services_id_seq OWNED BY inits_services.id;


--
-- Name: intituleagentbyoperations; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE intituleagentbyoperations (
  id integer NOT NULL,
  intituleagent_id integer NOT NULL,
  operation_id integer NOT NULL,
  desktopmanager_id integer NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.intituleagentbyoperations OWNER TO postgres;

--
-- Name: TABLE intituleagentbyoperations; Type: COMMENT; Schema: public; Owner: postgres
--

COMMENT ON TABLE intituleagentbyoperations IS 'Table permettant de définir les bureaux pour un rôle donné dans une OP (SAERP)';


--
-- Name: intituleagentbyoperations_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--

CREATE SEQUENCE intituleagentbyoperations_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.intituleagentbyoperations_id_seq OWNER TO postgres;

--
-- Name: intituleagentbyoperations_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--

ALTER SEQUENCE intituleagentbyoperations_id_seq OWNED BY intituleagentbyoperations.id;


--
-- Name: intitulesagents; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE intitulesagents (
  id integer NOT NULL,
  name character varying(4) NOT NULL,
  description character varying(255),
  active boolean DEFAULT true NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.intitulesagents OWNER TO webgfc;

--
-- Name: TABLE intitulesagents; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE intitulesagents IS 'Table permettant de définir les intitulés d''agents (SAERP)';


--
-- Name: intitulesagents_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE intitulesagents_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.intitulesagents_id_seq OWNER TO webgfc;

--
-- Name: intitulesagents_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE intitulesagents_id_seq OWNED BY intitulesagents.id;


--
-- Name: keywordlists; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE keywordlists (
  id integer NOT NULL,
  nom character varying(255) NOT NULL,
  description bytea,
  version integer NOT NULL,
  active boolean DEFAULT true NOT NULL,
  keywordtype_id integer NOT NULL,
  scheme_id character varying(1024),
  scheme_name character varying(1024),
  scheme_agency_name character varying(1024),
  scheme_version_id character varying(1024),
  scheme_data_uri character varying(1024),
  scheme_uri character varying(1024),
  created_user_id integer,
  modified_user_id integer,
  created timestamp without time zone DEFAULT now(),
  modified timestamp without time zone DEFAULT now()
);


ALTER TABLE public.keywordlists OWNER TO webgfc;

--
-- Name: COLUMN keywordlists.scheme_id; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON COLUMN keywordlists.scheme_id IS 'Champ correpondant aux normes SEDA';


--
-- Name: COLUMN keywordlists.scheme_name; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON COLUMN keywordlists.scheme_name IS 'Champ correpondant aux normes SEDA';


--
-- Name: COLUMN keywordlists.scheme_agency_name; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON COLUMN keywordlists.scheme_agency_name IS 'Champ correpondant aux normes SEDA';


--
-- Name: COLUMN keywordlists.scheme_version_id; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON COLUMN keywordlists.scheme_version_id IS 'Champ correpondant aux normes SEDA';


--
-- Name: COLUMN keywordlists.scheme_data_uri; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON COLUMN keywordlists.scheme_data_uri IS 'Champ correpondant aux normes SEDA';


--
-- Name: COLUMN keywordlists.scheme_uri; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON COLUMN keywordlists.scheme_uri IS 'Champ correpondant aux normes SEDA';


--
-- Name: keywordlists_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE keywordlists_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.keywordlists_id_seq OWNER TO webgfc;

--
-- Name: keywordlists_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE keywordlists_id_seq OWNED BY keywordlists.id;


--
-- Name: keywords; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE keywords (
  id integer NOT NULL,
  version integer NOT NULL,
  code character varying(255) NOT NULL,
  libelle bytea,
  parent_id integer,
  lft integer,
  rght integer,
  keywordlist_id integer NOT NULL,
  created_user_id integer,
  modified_user_id integer,
  created timestamp without time zone DEFAULT now(),
  modified timestamp without time zone DEFAULT now()
);


ALTER TABLE public.keywords OWNER TO webgfc;

--
-- Name: keywords_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE keywords_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.keywords_id_seq OWNER TO webgfc;

--
-- Name: keywords_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE keywords_id_seq OWNED BY keywords.id;


--
-- Name: keywordtypes; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE keywordtypes (
  id integer NOT NULL,
  code character varying(10) NOT NULL,
  documentation character varying(250) NOT NULL,
  created timestamp without time zone DEFAULT now(),
  modified timestamp without time zone DEFAULT now()
);


ALTER TABLE public.keywordtypes OWNER TO webgfc;

--
-- Name: keywordtypes_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE keywordtypes_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.keywordtypes_id_seq OWNER TO webgfc;

--
-- Name: keywordtypes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE keywordtypes_id_seq OWNED BY keywordtypes.id;


--
-- Name: marches; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE marches (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  numero character varying(50) NOT NULL,
  objet character varying(255),
  datelimite date,
  heurelimite time without time zone,
  operation_id integer NOT NULL,
  chrono character varying(50),
  titulaire character varying(255),
  active boolean DEFAULT true NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  datenotification date,
  premiermois character varying(255),
  contractant_id integer NOT NULL,
  uniterif integer,
  uniterif_id integer,
  typecontractant character varying(255)
);


ALTER TABLE public.marches OWNER TO webgfc;

--
-- Name: TABLE marches; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE marches IS 'Table permettant de stocker les informations en lien avec les marchés (SAERP)';


--
-- Name: marches_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE marches_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.marches_id_seq OWNER TO webgfc;

--
-- Name: marches_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE marches_id_seq OWNED BY marches.id;


--
-- Name: metadonnees; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE metadonnees (
  id integer NOT NULL,
  name character varying(255),
  typemetadonnee_id integer NOT NULL,
  created timestamp without time zone,
  modified timestamp without time zone,
  champfusion character varying(255)
);


ALTER TABLE public.metadonnees OWNER TO webgfc;

--
-- Name: metadonnees_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE metadonnees_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.metadonnees_id_seq OWNER TO webgfc;

--
-- Name: metadonnees_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE metadonnees_id_seq OWNED BY metadonnees.id;


--
-- Name: metadonnees_soustypes; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE metadonnees_soustypes (
  id integer NOT NULL,
  metadonnee_id integer NOT NULL,
  soustype_id integer NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  obligatoire boolean
);


ALTER TABLE public.metadonnees_soustypes OWNER TO webgfc;

--
-- Name: TABLE metadonnees_soustypes; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE metadonnees_soustypes IS 'Table de liaison entre les métadonnées et les sous types';


--
-- Name: metadonnees_soustypes_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE metadonnees_soustypes_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.metadonnees_soustypes_id_seq OWNER TO webgfc;

--
-- Name: metadonnees_soustypes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE metadonnees_soustypes_id_seq OWNED BY metadonnees_soustypes.id;


--
-- Name: notifications; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE notifications (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  description bytea,
  typenotif_id integer NOT NULL,
  desktop_id integer NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.notifications OWNER TO webgfc;

--
-- Name: notifications_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE notifications_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.notifications_id_seq OWNER TO webgfc;

--
-- Name: notifications_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE notifications_id_seq OWNED BY notifications.id;


--
-- Name: notifieddesktops; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE notifieddesktops (
  id integer NOT NULL,
  courrier_id integer,
  notification_id integer NOT NULL,
  desktop_id integer NOT NULL,
  read boolean DEFAULT false NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.notifieddesktops OWNER TO webgfc;

--
-- Name: notifieddesktops_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE notifieddesktops_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.notifieddesktops_id_seq OWNER TO webgfc;

--
-- Name: notifieddesktops_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE notifieddesktops_id_seq OWNED BY notifieddesktops.id;


--
-- Name: operations; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE operations (
  id integer NOT NULL,
  type_id integer,
  name character varying(4) NOT NULL,
  nomoperation character varying(255) NOT NULL,
  ville character varying(255) NOT NULL,
  numvoie character varying(20),
  nomvoie character varying(255),
  compl character varying(255),
  codepostal character varying(5),
  commune character varying(255),
  numconvention character varying(255),
  nature character varying(255),
  lieu character varying(255),
  active boolean DEFAULT true NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.operations OWNER TO webgfc;

--
-- Name: TABLE operations; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE operations IS 'Table permettant de définir les opérations (SAERP)';


--
-- Name: operations_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE operations_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.operations_id_seq OWNER TO webgfc;

--
-- Name: operations_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE operations_id_seq OWNED BY operations.id;


--
-- Name: ordresservices; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE ordresservices (
  id integer NOT NULL,
  name character varying(255),
  marche_id integer NOT NULL,
  numero INTEGER,
  objet character varying(255),
  montant character varying(10),
  active boolean DEFAULT true NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  courrier_id integer,
  organisme_id integer
);


ALTER TABLE public.ordresservices OWNER TO webgfc;

--
-- Name: TABLE ordresservices; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE ordresservices IS 'Table permettant de stocker les ordres de service (SAERP)';


--
-- Name: ordresservices_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE ordresservices_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.ordresservices_id_seq OWNER TO webgfc;

--
-- Name: ordresservices_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE ordresservices_id_seq OWNED BY ordresservices.id;


--
-- Name: organismes; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE organismes (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  addressbook_id integer NOT NULL,
  ban_id integer,
  bancommune_id integer,
  banadresse character varying(255),
  numvoie character varying(255),
  adressecomplete character varying(255),
  compl character varying(255),
  cp character varying(5),
  email character varying(255),
  tel character varying(20),
  portable character varying(20),
  fax character varying(20),
  slug character varying(255),
  active boolean DEFAULT true NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  infoscontact boolean,
  ville character varying(255),
  nomvoie character varying(255),
  pays character varying(50) DEFAULT 'FRANCE'::character varying(255),
  antenne character varying(255),
  siret character varying(255)
);


ALTER TABLE public.organismes OWNER TO webgfc;

--
-- Name: organismes_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE organismes_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.organismes_id_seq OWNER TO webgfc;

--
-- Name: organismes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE organismes_id_seq OWNED BY organismes.id;


--
-- Name: organismes_operations; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE organismes_operations (
  id integer NOT NULL,
  organisme_id integer NOT NULL,
  operation_id integer NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.organismes_operations OWNER TO postgres;

--
-- Name: TABLE organismes_operations; Type: COMMENT; Schema: public; Owner: postgres
--

COMMENT ON TABLE organismes_operations IS 'Table de liaison entre les organismes et les opérations (SAERP)';


--
-- Name: organismes_operations_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--

CREATE SEQUENCE organismes_operations_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.organismes_operations_id_seq OWNER TO postgres;

--
-- Name: organismes_operations_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--

ALTER SEQUENCE organismes_operations_id_seq OWNED BY organismes_operations.id;


--
-- Name: plandelegations; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE plandelegations (
  id integer NOT NULL,
  user_id integer NOT NULL,
  desktop_id integer NOT NULL,
  date_start timestamp without time zone NOT NULL,
  date_end timestamp without time zone NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.plandelegations OWNER TO webgfc;

--
-- Name: plandelegations_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE plandelegations_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.plandelegations_id_seq OWNER TO webgfc;

--
-- Name: plandelegations_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE plandelegations_id_seq OWNED BY plandelegations.id;


--
-- Name: plisconsultatifs; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE plisconsultatifs (
  id integer NOT NULL,
  name character varying(255),
  consultation_id integer NOT NULL,
  origineflux_id integer,
  courrier_id integer,
  numero INTEGER NOT NULL,
  date date NOT NULL,
  heure time without time zone NOT NULL,
  objet character varying(255),
  active boolean DEFAULT true NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  lot character varying(10)
);


ALTER TABLE public.plisconsultatifs OWNER TO postgres;

--
-- Name: TABLE plisconsultatifs; Type: COMMENT; Schema: public; Owner: postgres
--

COMMENT ON TABLE plisconsultatifs IS 'Table permettant de stocker les activités liées aux contacts (SAERP)';


--
-- Name: plisconsultatifs_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--

CREATE SEQUENCE plisconsultatifs_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.plisconsultatifs_id_seq OWNER TO postgres;

--
-- Name: plisconsultatifs_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--

ALTER SEQUENCE plisconsultatifs_id_seq OWNED BY plisconsultatifs.id;


--
-- Name: recheche_contactinfos; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE recheche_contactinfos (
  id integer NOT NULL,
  recherche_id integer NOT NULL,
  contactinfo_id integer NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.recheche_contactinfos OWNER TO webgfc;

--
-- Name: recheche_contactinfos_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE recheche_contactinfos_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.recheche_contactinfos_id_seq OWNER TO webgfc;

--
-- Name: recheche_contactinfos_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE recheche_contactinfos_id_seq OWNED BY recheche_contactinfos.id;


--
-- Name: recherches; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE recherches (
  id integer NOT NULL,
  name character varying(255),
  cname character varying(255),
  cdate date,
  cdatereception date,
  user_id integer,
  created timestamp without time zone,
  modified timestamp without time zone,
  cdatefin date,
  cdatereceptionfin date,
  contactname character varying(255),
  cobjet character varying(255),
  creference character varying(255),
  ccomment character varying(255),
  cretard character varying(255)
);


ALTER TABLE public.recherches OWNER TO webgfc;

--
-- Name: recherches_addressbooks; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE recherches_addressbooks (
  id integer NOT NULL,
  addressbook_id integer NOT NULL,
  recherche_id integer NOT NULL,
  created timestamp without time zone DEFAULT now(),
  modified timestamp without time zone DEFAULT now()
);


ALTER TABLE public.recherches_addressbooks OWNER TO webgfc;

--
-- Name: recherches_addressbooks_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE recherches_addressbooks_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.recherches_addressbooks_id_seq OWNER TO webgfc;

--
-- Name: recherches_addressbooks_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE recherches_addressbooks_id_seq OWNED BY recherches_addressbooks.id;


--
-- Name: recherches_affaires; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE recherches_affaires (
  id integer NOT NULL,
  affaire_id integer NOT NULL,
  created timestamp without time zone,
  modified timestamp without time zone,
  recherche_id integer NOT NULL
);


ALTER TABLE public.recherches_affaires OWNER TO webgfc;

--
-- Name: recherches_affaires_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE recherches_affaires_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.recherches_affaires_id_seq OWNER TO webgfc;

--
-- Name: recherches_affaires_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE recherches_affaires_id_seq OWNED BY recherches_affaires.id;


--
-- Name: recherches_circuits; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE recherches_circuits (
  id integer NOT NULL,
  recherche_id integer,
  circuit_id integer,
  created timestamp without time zone,
  modified timestamp without time zone
);


ALTER TABLE public.recherches_circuits OWNER TO webgfc;

--
-- Name: recherches_circuits_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE recherches_circuits_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.recherches_circuits_id_seq OWNER TO webgfc;

--
-- Name: recherches_circuits_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE recherches_circuits_id_seq OWNED BY recherches_circuits.id;


--
-- Name: recherches_contacts; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE recherches_contacts (
  id integer NOT NULL,
  contact_id integer NOT NULL,
  recherche_id integer NOT NULL,
  created timestamp without time zone DEFAULT now(),
  modified timestamp without time zone DEFAULT now()
);


ALTER TABLE public.recherches_contacts OWNER TO webgfc;

--
-- Name: recherches_contacts_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE recherches_contacts_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.recherches_contacts_id_seq OWNER TO webgfc;

--
-- Name: recherches_contacts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE recherches_contacts_id_seq OWNED BY recherches_contacts.id;


--
-- Name: recherches_desktops; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE recherches_desktops (
  id integer NOT NULL,
  recherche_id integer NOT NULL,
  desktop_id integer NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.recherches_desktops OWNER TO webgfc;

--
-- Name: TABLE recherches_desktops; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE recherches_desktops IS 'Table de liaison entre les recherches et les desktops';


--
-- Name: recherches_desktops_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE recherches_desktops_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.recherches_desktops_id_seq OWNER TO webgfc;

--
-- Name: recherches_desktops_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE recherches_desktops_id_seq OWNED BY recherches_desktops.id;


--
-- Name: recherches_dossiers; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE recherches_dossiers (
  id integer NOT NULL,
  dossier_id integer NOT NULL,
  recherche_id integer NOT NULL,
  created timestamp without time zone,
  modified timestamp without time zone
);


ALTER TABLE public.recherches_dossiers OWNER TO webgfc;

--
-- Name: recherches_dossiers_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE recherches_dossiers_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.recherches_dossiers_id_seq OWNER TO webgfc;

--
-- Name: recherches_dossiers_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE recherches_dossiers_id_seq OWNED BY recherches_dossiers.id;


--
-- Name: recherches_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE recherches_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.recherches_id_seq OWNER TO webgfc;

--
-- Name: recherches_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE recherches_id_seq OWNED BY recherches.id;


--
-- Name: recherches_metadonnees; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE recherches_metadonnees (
  id integer NOT NULL,
  recherche_id integer,
  metadonnee_id integer,
  created timestamp without time zone,
  modified timestamp without time zone
);


ALTER TABLE public.recherches_metadonnees OWNER TO webgfc;

--
-- Name: recherches_metadonnees_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE recherches_metadonnees_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.recherches_metadonnees_id_seq OWNER TO webgfc;

--
-- Name: recherches_metadonnees_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE recherches_metadonnees_id_seq OWNED BY recherches_metadonnees.id;


--
-- Name: recherches_organismes; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE recherches_organismes (
  id integer NOT NULL,
  recherche_id integer NOT NULL,
  organisme_id integer NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.recherches_organismes OWNER TO webgfc;

--
-- Name: TABLE recherches_organismes; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE recherches_organismes IS 'Table de liaison entre les recherches et les organismes';


--
-- Name: recherches_organismes_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE recherches_organismes_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.recherches_organismes_id_seq OWNER TO webgfc;

--
-- Name: recherches_organismes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE recherches_organismes_id_seq OWNED BY recherches_organismes.id;


--
-- Name: recherches_selectvaluesmetadonnees; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE recherches_selectvaluesmetadonnees (
  id integer NOT NULL,
  recherche_id integer NOT NULL,
  selectvaluemetadonnee_id integer,
  metadonnee_id integer,
  valeur character varying(255),
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.recherches_selectvaluesmetadonnees OWNER TO webgfc;

--
-- Name: TABLE recherches_selectvaluesmetadonnees; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE recherches_selectvaluesmetadonnees IS 'Table de liaison entre les recherches et les selectvaluesmetadonnees';


--
-- Name: recherches_selectvaluesmetadonnees_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE recherches_selectvaluesmetadonnees_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.recherches_selectvaluesmetadonnees_id_seq OWNER TO webgfc;

--
-- Name: recherches_selectvaluesmetadonnees_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE recherches_selectvaluesmetadonnees_id_seq OWNED BY recherches_selectvaluesmetadonnees.id;


--
-- Name: recherches_services; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE recherches_services (
  id integer NOT NULL,
  recherche_id integer,
  service_id integer,
  created timestamp without time zone,
  modified timestamp without time zone
);


ALTER TABLE public.recherches_services OWNER TO webgfc;

--
-- Name: recherches_services_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE recherches_services_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.recherches_services_id_seq OWNER TO webgfc;

--
-- Name: recherches_services_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE recherches_services_id_seq OWNED BY recherches_services.id;


--
-- Name: recherches_soustypes; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE recherches_soustypes (
  id integer NOT NULL,
  soustype_id integer,
  recherche_id integer,
  created timestamp without time zone,
  modified timestamp without time zone
);


ALTER TABLE public.recherches_soustypes OWNER TO webgfc;

--
-- Name: recherches_soustypes_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE recherches_soustypes_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.recherches_soustypes_id_seq OWNER TO webgfc;

--
-- Name: recherches_soustypes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE recherches_soustypes_id_seq OWNED BY recherches_soustypes.id;


--
-- Name: recherches_taches; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE recherches_taches (
  id integer NOT NULL,
  tache_id integer NOT NULL,
  created timestamp without time zone,
  modified timestamp without time zone,
  recherche_id integer NOT NULL
);


ALTER TABLE public.recherches_taches OWNER TO webgfc;

--
-- Name: recherches_taches_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE recherches_taches_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.recherches_taches_id_seq OWNER TO webgfc;

--
-- Name: recherches_taches_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE recherches_taches_id_seq OWNED BY recherches_taches.id;


--
-- Name: recherches_types; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE recherches_types (
  id integer NOT NULL,
  type_id integer,
  recherche_id integer,
  created timestamp without time zone,
  modified timestamp without time zone
);


ALTER TABLE public.recherches_types OWNER TO webgfc;

--
-- Name: recherches_types_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE recherches_types_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.recherches_types_id_seq OWNER TO webgfc;

--
-- Name: recherches_types_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE recherches_types_id_seq OWNED BY recherches_types.id;


--
-- Name: referentielsfantoir; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE referentielsfantoir (
  id integer NOT NULL,
  name character varying(50) NOT NULL,
  active boolean DEFAULT true,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.referentielsfantoir OWNER TO webgfc;

--
-- Name: TABLE referentielsfantoir; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE referentielsfantoir IS 'Table permettant le stockage des référentiels FANTOIR';


--
-- Name: referentielsfantoir_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE referentielsfantoir_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.referentielsfantoir_id_seq OWNER TO webgfc;

--
-- Name: referentielsfantoir_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE referentielsfantoir_id_seq OWNED BY referentielsfantoir.id;


--
-- Name: referentielsfantoircom; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE referentielsfantoircom (
  id integer NOT NULL,
  referentielfantoir_id integer NOT NULL,
  referentielfantoirdir_id integer NOT NULL,
  name character varying(255) NOT NULL,
  codecom character varying(3) NOT NULL,
  clerivoli character varying(1) NOT NULL,
  libellecom character varying(30) NOT NULL,
  typecom character varying(1) NOT NULL,
  caracrur character varying(1) DEFAULT NULL::character varying,
  caracpop character varying(1) DEFAULT NULL::character varying,
  popreelle character varying(7) DEFAULT NULL::character varying,
  popapart character varying(7) DEFAULT NULL::character varying,
  popfictive character varying(7) DEFAULT NULL::character varying,
  caracannul character varying(1) DEFAULT NULL::character varying,
  dateannul character varying(7) DEFAULT NULL::character varying,
  datecreation character varying(7) DEFAULT NULL::character varying,
  active boolean DEFAULT true,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  CONSTRAINT referentielsfantoircom_typecom_in_list_chk CHECK (cakephp_validate_in_list((typecom)::text, ARRAY['N'::text, 'R'::text]))
);


ALTER TABLE public.referentielsfantoircom OWNER TO webgfc;

--
-- Name: TABLE referentielsfantoircom; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE referentielsfantoircom IS 'Table permettant le stockage des référentiels FANTOIR de commune';


--
-- Name: referentielsfantoircom_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE referentielsfantoircom_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.referentielsfantoircom_id_seq OWNER TO webgfc;

--
-- Name: referentielsfantoircom_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE referentielsfantoircom_id_seq OWNED BY referentielsfantoircom.id;


--
-- Name: referentielsfantoirdir; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE referentielsfantoirdir (
  id integer NOT NULL,
  referentielfantoir_id integer NOT NULL,
  name character varying(255) NOT NULL,
  codedpt character varying(2) NOT NULL,
  codedir character varying(1) NOT NULL,
  libelledir character varying(30) NOT NULL,
  active boolean DEFAULT true,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.referentielsfantoirdir OWNER TO webgfc;

--
-- Name: TABLE referentielsfantoirdir; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE referentielsfantoirdir IS 'Table permettant le stockage des référentiels FANTOIR de direction';


--
-- Name: referentielsfantoirdir_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE referentielsfantoirdir_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.referentielsfantoirdir_id_seq OWNER TO webgfc;

--
-- Name: referentielsfantoirdir_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE referentielsfantoirdir_id_seq OWNED BY referentielsfantoirdir.id;


--
-- Name: referentielsfantoirvoie; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE referentielsfantoirvoie (
  id integer NOT NULL,
  referentielfantoir_id integer NOT NULL,
  referentielfantoircom_id integer NOT NULL,
  name character varying(255) NOT NULL,
  identvoie character varying(4) NOT NULL,
  clerivoli character varying(1) NOT NULL,
  codevoie character varying(4) DEFAULT NULL::character varying,
  libellevoie character varying(26) NOT NULL,
  typecom character varying(1) NOT NULL,
  caracrur character varying(1) DEFAULT NULL::character varying,
  caracvoie character varying(1) NOT NULL,
  caracpop character varying(1) DEFAULT NULL::character varying,
  popapart character varying(7) DEFAULT NULL::character varying,
  popfictive character varying(7) DEFAULT NULL::character varying,
  caracannul character varying(1) DEFAULT NULL::character varying,
  dateannul character varying(7) DEFAULT NULL::character varying,
  datecreation character varying(7) DEFAULT NULL::character varying,
  codemajic character varying(5) NOT NULL,
  typevoie character varying(1) NOT NULL,
  caraclieudit character varying(1) DEFAULT NULL::character varying,
  libentiervoie character varying(8) NOT NULL,
  active boolean DEFAULT true,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  CONSTRAINT referentielsfantoirvoie_caracvoie_in_list_chk CHECK (cakephp_validate_in_list((caracvoie)::text, ARRAY['1'::text, '0'::text])),
  CONSTRAINT referentielsfantoirvoie_typecom_in_list_chk CHECK (cakephp_validate_in_list((typecom)::text, ARRAY['N'::text, 'R'::text])),
  CONSTRAINT referentielsfantoirvoie_typevoie_in_list_chk CHECK (cakephp_validate_in_list((typevoie)::text, ARRAY['1'::text, '2'::text, '3'::text, '4'::text, '5'::text]))
);


ALTER TABLE public.referentielsfantoirvoie OWNER TO webgfc;

--
-- Name: TABLE referentielsfantoirvoie; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE referentielsfantoirvoie IS 'Table permettant le stockage des référentiels FANTOIR de voie';


--
-- Name: referentielsfantoirvoie_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE referentielsfantoirvoie_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.referentielsfantoirvoie_id_seq OWNER TO webgfc;

--
-- Name: referentielsfantoirvoie_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE referentielsfantoirvoie_id_seq OWNED BY referentielsfantoirvoie.id;


--
-- Name: relements; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE relements (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  courrier_id integer NOT NULL,
  mime character varying(255),
  preview bytea,
  content bytea
);


ALTER TABLE public.relements OWNER TO webgfc;

--
-- Name: relements_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE relements_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.relements_id_seq OWNER TO webgfc;

--
-- Name: relements_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE relements_id_seq OWNED BY relements.id;


--
-- Name: repertoires; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE repertoires (
  id integer NOT NULL,
  name character varying(255),
  user_id integer,
  lft integer,
  rght integer,
  parent_id integer
);


ALTER TABLE public.repertoires OWNER TO webgfc;

--
-- Name: repertoires_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE repertoires_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.repertoires_id_seq OWNER TO webgfc;

--
-- Name: repertoires_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE repertoires_id_seq OWNED BY repertoires.id;


--
-- Name: rights; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE rights (
  id integer NOT NULL,
  get_infos boolean DEFAULT false NOT NULL,
  get_meta boolean DEFAULT false NOT NULL,
  get_taches boolean DEFAULT false NOT NULL,
  get_affaire boolean DEFAULT false NOT NULL,
  create_infos boolean DEFAULT false NOT NULL,
  set_infos boolean DEFAULT false NOT NULL,
  set_meta boolean DEFAULT false NOT NULL,
  set_taches boolean DEFAULT false NOT NULL,
  set_affaire boolean DEFAULT false NOT NULL,
  administration boolean DEFAULT false NOT NULL,
  aiguillage boolean DEFAULT false NOT NULL,
  creation boolean DEFAULT false NOT NULL,
  validation boolean DEFAULT false NOT NULL,
  edition boolean DEFAULT false NOT NULL,
  documentation boolean DEFAULT false NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  mode_classique boolean DEFAULT false NOT NULL,
  aro_id integer,
  envoi_cpy boolean DEFAULT false NOT NULL,
  envoi_wkf boolean DEFAULT false NOT NULL,
  envoi_ged boolean DEFAULT false NOT NULL,
  document_upload boolean DEFAULT false NOT NULL,
  document_suppr boolean DEFAULT false NOT NULL,
  document_setasdefault boolean DEFAULT false NOT NULL,
  relements boolean DEFAULT false NOT NULL,
  relements_upload boolean DEFAULT false NOT NULL,
  relements_suppr boolean DEFAULT false NOT NULL,
  ar_gen boolean DEFAULT false NOT NULL,
  mes_services boolean DEFAULT false NOT NULL,
  recherches boolean DEFAULT false NOT NULL,
  suppression_flux boolean DEFAULT false NOT NULL,
  model character varying(255),
  foreign_key integer,
  consult_scan boolean,
  carnet_adresse boolean DEFAULT false NOT NULL,
  statistiques boolean DEFAULT false NOT NULL
);


ALTER TABLE public.rights OWNER TO webgfc;

--
-- Name: rights_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE rights_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.rights_id_seq OWNER TO webgfc;

--
-- Name: rights_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE rights_id_seq OWNED BY rights.id;


--
-- Name: rmodelgabarits; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE rmodelgabarits (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  gedpath text,
  content bytea,
  size integer NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  soustype_id integer NOT NULL,
  mime character varying(255),
  preview bytea,
  ext character varying(10),
  path character varying(255)
);


ALTER TABLE public.rmodelgabarits OWNER TO webgfc;

--
-- Name: rmodelgabarits_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE rmodelgabarits_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.rmodelgabarits_id_seq OWNER TO webgfc;

--
-- Name: rmodelgabarits_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE rmodelgabarits_id_seq OWNED BY rmodelgabarits.id;


--
-- Name: rmodels; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE rmodels (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  gedpath text,
  content bytea,
  size integer NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  soustype_id integer NOT NULL,
  mime character varying(255),
  preview bytea,
  ext character varying(10)
);


ALTER TABLE public.rmodels OWNER TO webgfc;

--
-- Name: rmodels_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE rmodels_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.rmodels_id_seq OWNER TO webgfc;

--
-- Name: rmodels_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE rmodels_id_seq OWNED BY rmodels.id;


--
-- Name: scanemails; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE scanemails (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  username character varying(255) NOT NULL,
  mail character varying(255),
  hostname character varying(255) NOT NULL,
  port character varying(24) DEFAULT '143'::character varying,
  password character varying(255) NOT NULL,
  desktop_id integer,
  type_id integer,
  soustype_id integer,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  desktopmanager_id integer,
  scriptname character varying(255) NOT NULL
);


ALTER TABLE public.scanemails OWNER TO webgfc;

--
-- Name: TABLE scanemails; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE scanemails IS 'Table permettant de stocker les emails et destinataires pour chaque collectivité';


--
-- Name: scanemails_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE scanemails_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.scanemails_id_seq OWNER TO webgfc;

--
-- Name: scanemails_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE scanemails_id_seq OWNED BY scanemails.id;


--
-- Name: selectvaluesmetadonnees; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE selectvaluesmetadonnees (
  id integer NOT NULL,
  metadonnee_id integer NOT NULL,
  name character varying(250) DEFAULT NULL::character varying,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.selectvaluesmetadonnees OWNER TO webgfc;

--
-- Name: selectvaluesmetadonnees_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE selectvaluesmetadonnees_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.selectvaluesmetadonnees_id_seq OWNER TO webgfc;

--
-- Name: selectvaluesmetadonnees_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE selectvaluesmetadonnees_id_seq OWNED BY selectvaluesmetadonnees.id;


--
-- Name: sendsmails; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE sendsmails (
  id integer NOT NULL,
  email character varying(255) NOT NULL,
  sujet character varying(255),
  message text,
  desktop_id integer,
  courrier_id integer,
  document_id integer,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  ccmail character varying(255),
  templatemail_id integer
);


ALTER TABLE public.sendsmails OWNER TO webgfc;

--
-- Name: sendsmails_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE sendsmails_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.sendsmails_id_seq OWNER TO webgfc;

--
-- Name: sendsmails_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE sendsmails_id_seq OWNED BY sendsmails.id;


--
-- Name: sequences; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE sequences (
  id integer NOT NULL,
  nom character varying(255) NOT NULL,
  commentaire character varying(255) NOT NULL,
  num_sequence integer NOT NULL,
  created timestamp without time zone,
  modified timestamp without time zone
);


ALTER TABLE public.sequences OWNER TO webgfc;

--
-- Name: sequences_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE sequences_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.sequences_id_seq OWNER TO webgfc;

--
-- Name: sequences_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE sequences_id_seq OWNED BY sequences.id;


--
-- Name: services; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE services (
  name character varying(255) NOT NULL,
  created timestamp without time zone,
  modified timestamp without time zone,
  parent_id smallint,
  lft smallint,
  rght smallint,
  id integer NOT NULL,
  email character varying(255),
  active boolean DEFAULT true NOT NULL
);


ALTER TABLE public.services OWNER TO webgfc;

--
-- Name: services_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE services_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.services_id_seq OWNER TO webgfc;

--
-- Name: services_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE services_id_seq OWNED BY services.id;


--
-- Name: sousactivites; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE sousactivites (
  id integer NOT NULL,
  activite_id integer NOT NULL,
  name character varying(255) NOT NULL,
  active boolean DEFAULT true NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL,
  description character varying(255)
);


ALTER TABLE public.sousactivites OWNER TO postgres;

--
-- Name: TABLE sousactivites; Type: COMMENT; Schema: public; Owner: postgres
--

COMMENT ON TABLE sousactivites IS 'Table permettant de stocker les sous-activités liées aux contacts (SAERP)';


--
-- Name: sousactivites_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--

CREATE SEQUENCE sousactivites_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.sousactivites_id_seq OWNER TO postgres;

--
-- Name: sousactivites_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--

ALTER SEQUENCE sousactivites_id_seq OWNED BY sousactivites.id;


--
-- Name: sousoperations; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE sousoperations (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  soustype_id integer,
  operation_id integer NOT NULL,
  active boolean DEFAULT true NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.sousoperations OWNER TO webgfc;

--
-- Name: TABLE sousoperations; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE sousoperations IS 'Table de liaison entre les bureaux et les opérations (SAERP)';


--
-- Name: sousoperations_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE sousoperations_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.sousoperations_id_seq OWNER TO webgfc;

--
-- Name: sousoperations_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE sousoperations_id_seq OWNED BY sousoperations.id;


--
-- Name: soustypes; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE soustypes (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  created timestamp without time zone,
  modified timestamp without time zone,
  type_id integer NOT NULL,
  circuit_id integer,
  entrant smallint DEFAULT 1 NOT NULL,
  delai_nb smallint,
  delai_unite smallint,
  lft integer,
  rght integer,
  parent_id integer,
  valeuremail bytea,
  compteur_id integer,
  information text,
  active boolean DEFAULT true NOT NULL
);


ALTER TABLE public.soustypes OWNER TO webgfc;

--
-- Name: soustypes_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE soustypes_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.soustypes_id_seq OWNER TO webgfc;

--
-- Name: soustypes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE soustypes_id_seq OWNED BY soustypes.id;


SET default_with_oids = true;

--
-- Name: taches; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE taches (
  id integer NOT NULL,
  courrier_id integer NOT NULL,
  created timestamp without time zone,
  modified timestamp without time zone,
  name character varying(255) NOT NULL,
  delai_nb integer,
  delai_unite integer,
  statut integer DEFAULT 0 NOT NULL,
  act_desktop_id integer DEFAULT 0 NOT NULL
);


ALTER TABLE public.taches OWNER TO webgfc;

--
-- Name: taches_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE taches_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.taches_id_seq OWNER TO webgfc;

--
-- Name: taches_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE taches_id_seq OWNED BY taches.id;


SET default_with_oids = false;

--
-- Name: templatemails; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE templatemails (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  subject character varying(255) NOT NULL,
  object text NOT NULL,
  active boolean DEFAULT true NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.templatemails OWNER TO webgfc;

--
-- Name: TABLE templatemails; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE templatemails IS 'Table permettant de définir des mails-type à envoyer';


--
-- Name: templatemails_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE templatemails_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.templatemails_id_seq OWNER TO webgfc;

--
-- Name: templatemails_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE templatemails_id_seq OWNED BY templatemails.id;



--
-- Name: notifquotidiens; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE notifquotidiens (
  id SERIAL NOT NULL,
  notification_id integer,
  courrier_id integer NOT NULL,
  desktop_id integer,
  user_id integer NOT NULL ,
  name VARCHAR(255) NOT NULL,
  description VARCHAR(255),
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.notifquotidiens OWNER TO webgfc;

--
-- Name: TABLE notifquotidiens; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE notifquotidiens IS 'Table permettant de stocker les notifications quotidiennes';


-- --
-- -- Name: notifquotidiens_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
-- --

-- CREATE SEQUENCE notifquotidiens_id_seq
-- START WITH 1
-- INCREMENT BY 1
-- NO MINVALUE
-- NO MAXVALUE
-- CACHE 1;


-- ALTER TABLE public.notifquotidiens_id_seq OWNER TO webgfc;

-- --
-- -- Name: notifquotidiens_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
-- --

-- ALTER SEQUENCE notifquotidiens_id_seq OWNED BY notifquotidiens.id;



--
-- Name: templatenotifications; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE templatenotifications (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  subject character varying(255) NOT NULL,
  object text NOT NULL,
  active boolean DEFAULT true NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.templatenotifications OWNER TO webgfc;

--
-- Name: TABLE templatenotifications; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE templatenotifications IS 'Table permettant de définir les mails adressés par notifications';


--
-- Name: templatenotifications_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE templatenotifications_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.templatenotifications_id_seq OWNER TO webgfc;

--
-- Name: templatenotifications_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE templatenotifications_id_seq OWNED BY templatenotifications.id;


--
-- Name: typemetadonnees; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE typemetadonnees (
  id integer NOT NULL,
  name character varying(255),
  created timestamp without time zone,
  modified timestamp without time zone
);


ALTER TABLE public.typemetadonnees OWNER TO webgfc;

--
-- Name: typemetadonnees_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE typemetadonnees_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.typemetadonnees_id_seq OWNER TO webgfc;

--
-- Name: typemetadonnees_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE typemetadonnees_id_seq OWNED BY typemetadonnees.id;


--
-- Name: typenotifs; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE typenotifs (
  id integer NOT NULL,
  name character varying(255) NOT NULL
);


ALTER TABLE public.typenotifs OWNER TO webgfc;

--
-- Name: typenotifs_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE typenotifs_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.typenotifs_id_seq OWNER TO webgfc;

--
-- Name: typenotifs_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE typenotifs_id_seq OWNED BY typenotifs.id;


--
-- Name: typenotifs_users; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE typenotifs_users (
  id integer NOT NULL,
  typenotif_id integer NOT NULL,
  user_id integer NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.typenotifs_users OWNER TO webgfc;

--
-- Name: typenotifs_users_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE typenotifs_users_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.typenotifs_users_id_seq OWNER TO webgfc;

--
-- Name: typenotifs_users_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE typenotifs_users_id_seq OWNED BY typenotifs_users.id;


SET default_with_oids = true;

--
-- Name: types; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE types (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  created timestamp without time zone,
  modified timestamp without time zone,
  active boolean DEFAULT true NOT NULL
);


ALTER TABLE public.types OWNER TO webgfc;

--
-- Name: types_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE types_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.types_id_seq OWNER TO webgfc;

--
-- Name: types_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE types_id_seq OWNED BY types.id;


SET default_with_oids = false;

--
-- Name: originesflux; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE originesflux (
  id integer NOT NULL,
  name character varying(100) NOT NULL,
  active boolean DEFAULT true,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.originesflux OWNER TO webgfc;

--
-- Name: TABLE originesflux; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE originesflux IS 'Table permettant le stockage des origines de flux';


--
-- Name: originesflux; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE originesflux_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.originesflux_id_seq OWNER TO webgfc;

--
-- Name: origineflux_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE originesflux_id_seq OWNED BY originesflux.id;


--
-- Name: uniterifs; Type: TABLE; Schema: public; Owner: postgres; Tablespace: 
--

CREATE TABLE uniterifs (
  id integer NOT NULL,
  name character varying(255) NOT NULL,
  description character varying(255),
  active boolean DEFAULT true NOT NULL,
  created timestamp without time zone DEFAULT now() NOT NULL,
  modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.uniterifs OWNER TO postgres;

--
-- Name: TABLE uniterifs; Type: COMMENT; Schema: public; Owner: postgres
--

COMMENT ON TABLE uniterifs IS 'Table permettant de définir les Unités RIF (SAERP)';


--
-- Name: uniterifs_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--

CREATE SEQUENCE uniterifs_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.uniterifs_id_seq OWNER TO postgres;

--
-- Name: uniterifs_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--

ALTER SEQUENCE uniterifs_id_seq OWNED BY uniterifs.id;


SET default_with_oids = true;

--
-- Name: users; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE users (
  id integer NOT NULL,
  username character varying(255) NOT NULL,
  password character varying(255) NOT NULL,
  nom character varying(255) NOT NULL,
  prenom character varying(255) NOT NULL,
  created timestamp without time zone,
  modified timestamp without time zone,
  mail character varying(255),
  desktop_id integer,
  last_login timestamp without time zone,
  last_logout timestamp without time zone,
  active boolean DEFAULT true NOT NULL,
  pwdmodified boolean DEFAULT false NOT NULL,
  numtel character varying(20),
  accept_notif boolean DEFAULT false,
  mail_insertion boolean DEFAULT false,
  mail_traitement boolean DEFAULT false,
  mail_refus boolean DEFAULT false,
  mail_retard_validation boolean DEFAULT false,
  mail_copy boolean DEFAULT false,
  mail_insertion_reponse boolean DEFAULT false,
  mail_aiguillage boolean DEFAULT false,
  mail_delegation boolean DEFAULT false,
  mail_clos boolean DEFAULT true,
  addressbook_private boolean DEFAULT false,
  typeabonnement VARCHAR(255) DEFAULT 'event'::character varying
);


ALTER TABLE public.users OWNER TO webgfc;

--
-- Name: users_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE users_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.users_id_seq OWNER TO webgfc;

--
-- Name: users_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE users_id_seq OWNED BY users.id;


SET default_with_oids = false;

--
-- Name: wkf_circuits; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE wkf_circuits (
  id integer NOT NULL,
  nom character varying(250) NOT NULL,
  description bytea,
  actif integer DEFAULT 1 NOT NULL,
  defaut integer DEFAULT 0 NOT NULL,
  created_user_id integer NOT NULL,
  modified_user_id integer,
  created timestamp without time zone,
  modified timestamp without time zone
);


ALTER TABLE public.wkf_circuits OWNER TO webgfc;

--
-- Name: wkf_circuits_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE wkf_circuits_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.wkf_circuits_id_seq OWNER TO webgfc;

--
-- Name: wkf_circuits_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE wkf_circuits_id_seq OWNED BY wkf_circuits.id;


--
-- Name: wkf_compositions; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE wkf_compositions (
  id integer NOT NULL,
  etape_id integer NOT NULL,
  type_validation character varying(1) NOT NULL,
  created_user_id integer,
  modified_user_id integer,
  created timestamp without time zone,
  modified timestamp without time zone,
  trigger_id integer,
  soustype integer,
  type_composition character varying(20) DEFAULT 'USER'::character varying
);


ALTER TABLE public.wkf_compositions OWNER TO webgfc;

--
-- Name: wkf_compositions_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE wkf_compositions_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.wkf_compositions_id_seq OWNER TO webgfc;

--
-- Name: wkf_compositions_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE wkf_compositions_id_seq OWNED BY wkf_compositions.id;


--
-- Name: wkf_etapes; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE wkf_etapes (
  id integer NOT NULL,
  circuit_id integer NOT NULL,
  nom character varying(250) NOT NULL,
  description bytea,
  type integer NOT NULL,
  ordre integer NOT NULL,
  created_user_id integer NOT NULL,
  modified_user_id integer,
  created timestamp without time zone,
  modified timestamp without time zone,
  soustype integer,
  cpt_retard integer
);


ALTER TABLE public.wkf_etapes OWNER TO webgfc;

--
-- Name: wkf_etapes_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE wkf_etapes_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.wkf_etapes_id_seq OWNER TO webgfc;

--
-- Name: wkf_etapes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE wkf_etapes_id_seq OWNED BY wkf_etapes.id;


--
-- Name: wkf_signatures; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE wkf_signatures (
  id integer NOT NULL,
  type_signature character varying(100) NOT NULL,
  signature bytea NOT NULL,
  visa_id integer
);


ALTER TABLE public.wkf_signatures OWNER TO webgfc;

--
-- Name: wkf_signatures_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE wkf_signatures_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.wkf_signatures_id_seq OWNER TO webgfc;

--
-- Name: wkf_signatures_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE wkf_signatures_id_seq OWNED BY wkf_signatures.id;


--
-- Name: wkf_traitements; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE wkf_traitements (
  id integer NOT NULL,
  circuit_id integer NOT NULL,
  target_id integer NOT NULL,
  numero_traitement integer DEFAULT 1 NOT NULL,
  created_user_id integer,
  modified_user_id integer,
  created timestamp without time zone,
  modified timestamp without time zone,
  treated boolean
);


ALTER TABLE public.wkf_traitements OWNER TO webgfc;

--
-- Name: wkf_traitements_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE wkf_traitements_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.wkf_traitements_id_seq OWNER TO webgfc;

--
-- Name: wkf_traitements_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE wkf_traitements_id_seq OWNED BY wkf_traitements.id;


--
-- Name: wkf_visas; Type: TABLE; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE TABLE wkf_visas (
  id integer NOT NULL,
  traitement_id integer NOT NULL,
  trigger_id integer NOT NULL,
  signature_id integer,
  etape_nom character varying(250),
  etape_type integer NOT NULL,
  action character varying(2) NOT NULL,
  commentaire text,
  date timestamp without time zone,
  type_validation character varying(1) NOT NULL,
  numero_traitement integer NOT NULL,
  etape_id integer,
  date_retard timestamp without time zone
);


ALTER TABLE public.wkf_visas OWNER TO webgfc;

--
-- Name: wkf_visas_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE wkf_visas_id_seq
START WITH 1
INCREMENT BY 1
NO MINVALUE
NO MAXVALUE
CACHE 1;


ALTER TABLE public.wkf_visas_id_seq OWNER TO webgfc;

--
-- Name: wkf_visas_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE wkf_visas_id_seq OWNED BY wkf_visas.id;


CREATE TABLE journalevents (
    id SERIAL NOT NULL,
    user_id integer NOT NULL,
    username VARCHAR(255) NOT NULL ,
    date timestamp without time zone DEFAULT now() NOT NULL,
    desktop_id integer NOT NULL,
    courrier_id integer,
    reference VARCHAR(255),
    action VARCHAR(255) NOT NULL,
    level VARCHAR(255) NOT NULL,
    message TEXT NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);
COMMENT ON TABLE journalevents IS 'Table permettant de stocker les logs utilisateurs par actions';

ALTER TABLE public.journalevents OWNER TO webgfc;

--
-- Name: journalevents_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

-- CREATE SEQUENCE journalevents_id_seq
-- START WITH 1
-- INCREMENT BY 1
-- NO MINVALUE
-- NO MAXVALUE
-- CACHE 1;


ALTER TABLE public.journalevents_id_seq OWNER TO webgfc;

--
-- Name: wkf_visas_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE journalevents_id_seq OWNED BY journalevents.id;



--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY acos ALTER COLUMN id SET DEFAULT nextval('acos_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY activites ALTER COLUMN id SET DEFAULT nextval('activites_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY activites_organismes ALTER COLUMN id SET DEFAULT nextval('activites_organismes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY addressbooks ALTER COLUMN id SET DEFAULT nextval('addressbooks_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY affaires ALTER COLUMN id SET DEFAULT nextval('affaires_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY armodels ALTER COLUMN id SET DEFAULT nextval('armodels_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY aros ALTER COLUMN id SET DEFAULT nextval('aros_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY aros_acos ALTER COLUMN id SET DEFAULT nextval('aros_acos_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY ars ALTER COLUMN id SET DEFAULT nextval('ars_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY bancontenus ALTER COLUMN id SET DEFAULT nextval('bancontenus_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY bannettes ALTER COLUMN id SET DEFAULT nextval('bannettes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY bans ALTER COLUMN id SET DEFAULT nextval('bans_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY bansadresses ALTER COLUMN id SET DEFAULT nextval('bansadresses_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY banscommunes ALTER COLUMN id SET DEFAULT nextval('banscommunes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY comments ALTER COLUMN id SET DEFAULT nextval('comments_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY comments_readers ALTER COLUMN id SET DEFAULT nextval('comments_readers_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY compteurs ALTER COLUMN id SET DEFAULT nextval('compteurs_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY connecteurs ALTER COLUMN id SET DEFAULT nextval('connecteurs_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY consultations ALTER COLUMN id SET DEFAULT nextval('consultations_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY contactinfos ALTER COLUMN id SET DEFAULT nextval('contactinfos_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY contacts ALTER COLUMN id SET DEFAULT nextval('contacts_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY contacts_events ALTER COLUMN id SET DEFAULT nextval('contacts_events_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY contacts_operations ALTER COLUMN id SET DEFAULT nextval('contacts_operations_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY contractants ALTER COLUMN id SET DEFAULT nextval('contractants_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY courriers ALTER COLUMN id SET DEFAULT nextval('courriers_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY courriers_metadonnees ALTER COLUMN id SET DEFAULT nextval('courriers_metadonnees_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY courriers_repertoires ALTER COLUMN id SET DEFAULT nextval('courriers_repertoires_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY dacos ALTER COLUMN id SET DEFAULT nextval('dacos_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY daros ALTER COLUMN id SET DEFAULT nextval('daros_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY daros_dacos ALTER COLUMN id SET DEFAULT nextval('daros_dacos_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY desktops ALTER COLUMN id SET DEFAULT nextval('desktops_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY desktops_desktopsmanagers ALTER COLUMN id SET DEFAULT nextval('desktops_desktopsmanagers_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY desktops_services ALTER COLUMN id SET DEFAULT nextval('desktops_services_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY desktops_taches ALTER COLUMN id SET DEFAULT nextval('desktops_taches_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY desktops_users ALTER COLUMN id SET DEFAULT nextval('desktops_users_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY desktopsmanagers ALTER COLUMN id SET DEFAULT nextval('desktopsmanagers_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY desktopsmanagers_sousoperations ALTER COLUMN id SET DEFAULT nextval('desktopsmanagers_sousoperations_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY documents ALTER COLUMN id SET DEFAULT nextval('documents_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY dossiers ALTER COLUMN id SET DEFAULT nextval('dossiers_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY events ALTER COLUMN id SET DEFAULT nextval('events_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY fonctions ALTER COLUMN id SET DEFAULT nextval('fonctions_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY formatsreponse ALTER COLUMN id SET DEFAULT nextval('formatsreponse_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY formatsreponse_soustypes ALTER COLUMN id SET DEFAULT nextval('formatsreponse_soustypes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY gabaritsdocuments ALTER COLUMN id SET DEFAULT nextval('gabaritsdocuments_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY groups ALTER COLUMN id SET DEFAULT nextval('groups_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY inits_services ALTER COLUMN id SET DEFAULT nextval('inits_services_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY intituleagentbyoperations ALTER COLUMN id SET DEFAULT nextval('intituleagentbyoperations_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY intitulesagents ALTER COLUMN id SET DEFAULT nextval('intitulesagents_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY journalevents ALTER COLUMN id SET DEFAULT nextval('journalevents_id_seq'::regclass);

--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY keywordlists ALTER COLUMN id SET DEFAULT nextval('keywordlists_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY keywords ALTER COLUMN id SET DEFAULT nextval('keywords_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY keywordtypes ALTER COLUMN id SET DEFAULT nextval('keywordtypes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY marches ALTER COLUMN id SET DEFAULT nextval('marches_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY metadonnees ALTER COLUMN id SET DEFAULT nextval('metadonnees_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY metadonnees_soustypes ALTER COLUMN id SET DEFAULT nextval('metadonnees_soustypes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY notifications ALTER COLUMN id SET DEFAULT nextval('notifications_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY notifieddesktops ALTER COLUMN id SET DEFAULT nextval('notifieddesktops_id_seq'::regclass);

--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY notifquotidiens ALTER COLUMN id SET DEFAULT nextval('notifquotidiens_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY operations ALTER COLUMN id SET DEFAULT nextval('operations_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY ordresservices ALTER COLUMN id SET DEFAULT nextval('ordresservices_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY organismes ALTER COLUMN id SET DEFAULT nextval('organismes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY organismes_operations ALTER COLUMN id SET DEFAULT nextval('organismes_operations_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY plandelegations ALTER COLUMN id SET DEFAULT nextval('plandelegations_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY plisconsultatifs ALTER COLUMN id SET DEFAULT nextval('plisconsultatifs_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recheche_contactinfos ALTER COLUMN id SET DEFAULT nextval('recheche_contactinfos_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches ALTER COLUMN id SET DEFAULT nextval('recherches_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_addressbooks ALTER COLUMN id SET DEFAULT nextval('recherches_addressbooks_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_affaires ALTER COLUMN id SET DEFAULT nextval('recherches_affaires_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_circuits ALTER COLUMN id SET DEFAULT nextval('recherches_circuits_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_contacts ALTER COLUMN id SET DEFAULT nextval('recherches_contacts_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_desktops ALTER COLUMN id SET DEFAULT nextval('recherches_desktops_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_dossiers ALTER COLUMN id SET DEFAULT nextval('recherches_dossiers_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_metadonnees ALTER COLUMN id SET DEFAULT nextval('recherches_metadonnees_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_organismes ALTER COLUMN id SET DEFAULT nextval('recherches_organismes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_selectvaluesmetadonnees ALTER COLUMN id SET DEFAULT nextval('recherches_selectvaluesmetadonnees_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_soustypes ALTER COLUMN id SET DEFAULT nextval('recherches_soustypes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_taches ALTER COLUMN id SET DEFAULT nextval('recherches_taches_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_types ALTER COLUMN id SET DEFAULT nextval('recherches_types_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY referentielsfantoir ALTER COLUMN id SET DEFAULT nextval('referentielsfantoir_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY referentielsfantoircom ALTER COLUMN id SET DEFAULT nextval('referentielsfantoircom_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY referentielsfantoirdir ALTER COLUMN id SET DEFAULT nextval('referentielsfantoirdir_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY referentielsfantoirvoie ALTER COLUMN id SET DEFAULT nextval('referentielsfantoirvoie_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY relements ALTER COLUMN id SET DEFAULT nextval('relements_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY repertoires ALTER COLUMN id SET DEFAULT nextval('repertoires_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY rights ALTER COLUMN id SET DEFAULT nextval('rights_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY rmodelgabarits ALTER COLUMN id SET DEFAULT nextval('rmodelgabarits_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY rmodels ALTER COLUMN id SET DEFAULT nextval('rmodels_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY scanemails ALTER COLUMN id SET DEFAULT nextval('scanemails_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY selectvaluesmetadonnees ALTER COLUMN id SET DEFAULT nextval('selectvaluesmetadonnees_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY sendsmails ALTER COLUMN id SET DEFAULT nextval('sendsmails_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY sequences ALTER COLUMN id SET DEFAULT nextval('sequences_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY services ALTER COLUMN id SET DEFAULT nextval('services_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY sousactivites ALTER COLUMN id SET DEFAULT nextval('sousactivites_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY sousoperations ALTER COLUMN id SET DEFAULT nextval('sousoperations_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY soustypes ALTER COLUMN id SET DEFAULT nextval('soustypes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY taches ALTER COLUMN id SET DEFAULT nextval('taches_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY templatemails ALTER COLUMN id SET DEFAULT nextval('templatemails_id_seq'::regclass);



--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY templatenotifications ALTER COLUMN id SET DEFAULT nextval('templatenotifications_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY typemetadonnees ALTER COLUMN id SET DEFAULT nextval('typemetadonnees_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY typenotifs ALTER COLUMN id SET DEFAULT nextval('typenotifs_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY typenotifs_users ALTER COLUMN id SET DEFAULT nextval('typenotifs_users_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY types ALTER COLUMN id SET DEFAULT nextval('types_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY originesflux ALTER COLUMN id SET DEFAULT nextval('originesflux_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY uniterifs ALTER COLUMN id SET DEFAULT nextval('uniterifs_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY users ALTER COLUMN id SET DEFAULT nextval('users_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY wkf_circuits ALTER COLUMN id SET DEFAULT nextval('wkf_circuits_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY wkf_compositions ALTER COLUMN id SET DEFAULT nextval('wkf_compositions_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY wkf_etapes ALTER COLUMN id SET DEFAULT nextval('wkf_etapes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY wkf_signatures ALTER COLUMN id SET DEFAULT nextval('wkf_signatures_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY wkf_traitements ALTER COLUMN id SET DEFAULT nextval('wkf_traitements_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY wkf_visas ALTER COLUMN id SET DEFAULT nextval('wkf_visas_id_seq'::regclass);


--
-- Name: acos_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY acos
ADD CONSTRAINT acos_pkey PRIMARY KEY (id);


--
-- Name: activites_organismes_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY activites_organismes
ADD CONSTRAINT activites_organismes_pkey PRIMARY KEY (id);


--
-- Name: activites_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY activites
ADD CONSTRAINT activites_pkey PRIMARY KEY (id);


--
-- Name: addressbooks_name_key; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY addressbooks
ADD CONSTRAINT addressbooks_name_key UNIQUE (name);


--
-- Name: addressbooks_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY addressbooks
ADD CONSTRAINT addressbooks_pkey PRIMARY KEY (id);


--
-- Name: affaires_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY affaires
ADD CONSTRAINT affaires_pkey PRIMARY KEY (id);


--
-- Name: armodels_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY armodels
ADD CONSTRAINT armodels_pkey PRIMARY KEY (id);


--
-- Name: aros_acos_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY aros_acos
ADD CONSTRAINT aros_acos_pkey PRIMARY KEY (id);


--
-- Name: aros_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY aros
ADD CONSTRAINT aros_pkey PRIMARY KEY (id);


--
-- Name: ars_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY ars
ADD CONSTRAINT ars_pkey PRIMARY KEY (id);


--
-- Name: bancontenus_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY bancontenus
ADD CONSTRAINT bancontenus_pkey PRIMARY KEY (id);


--
-- Name: bannettes_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY bannettes
ADD CONSTRAINT bannettes_pkey PRIMARY KEY (id);


--
-- Name: bans_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY bans
ADD CONSTRAINT bans_pkey PRIMARY KEY (id);


--
-- Name: bansadresses_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY bansadresses
ADD CONSTRAINT bansadresses_pkey PRIMARY KEY (id);


--
-- Name: banscommunes_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY banscommunes
ADD CONSTRAINT banscommunes_pkey PRIMARY KEY (id);


--
-- Name: comments_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY comments
ADD CONSTRAINT comments_pkey PRIMARY KEY (id);


--
-- Name: comments_readers_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY comments_readers
ADD CONSTRAINT comments_readers_pkey PRIMARY KEY (id);


--
-- Name: compteurs_nom_key; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY compteurs
ADD CONSTRAINT compteurs_nom_key UNIQUE (nom);


--
-- Name: compteurs_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY compteurs
ADD CONSTRAINT compteurs_pkey PRIMARY KEY (id);


--
-- Name: connecteurs_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY connecteurs
ADD CONSTRAINT connecteurs_pkey PRIMARY KEY (id);


--
-- Name: consultations_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY consultations
ADD CONSTRAINT consultations_pkey PRIMARY KEY (id);


--
-- Name: contactinfos_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY contactinfos
ADD CONSTRAINT contactinfos_pkey PRIMARY KEY (id);


--
-- Name: contacts_events_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY contacts_events
ADD CONSTRAINT contacts_events_pkey PRIMARY KEY (id);


--
-- Name: contacts_operations_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY contacts_operations
ADD CONSTRAINT contacts_operations_pkey PRIMARY KEY (id);


--
-- Name: contacts_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY contacts
ADD CONSTRAINT contacts_pkey PRIMARY KEY (id);


--
-- Name: contractants_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY contractants
ADD CONSTRAINT contractants_pkey PRIMARY KEY (id);


--
-- Name: courriers_metadonnees_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY courriers_metadonnees
ADD CONSTRAINT courriers_metadonnees_pkey PRIMARY KEY (id);


--
-- Name: courriers_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY courriers
ADD CONSTRAINT courriers_pkey PRIMARY KEY (id);


--
-- Name: courriers_repertoires_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY courriers_repertoires
ADD CONSTRAINT courriers_repertoires_pkey PRIMARY KEY (id);


--
-- Name: dacos_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY dacos
ADD CONSTRAINT dacos_pkey PRIMARY KEY (id);


--
-- Name: daros_dacos_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY daros_dacos
ADD CONSTRAINT daros_dacos_pkey PRIMARY KEY (id);


--
-- Name: daros_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY daros
ADD CONSTRAINT daros_pkey PRIMARY KEY (id);


--
-- Name: desktops_desktopsmanagers_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY desktops_desktopsmanagers
ADD CONSTRAINT desktops_desktopsmanagers_pkey PRIMARY KEY (id);


--
-- Name: desktops_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY desktops
ADD CONSTRAINT desktops_pkey PRIMARY KEY (id);


--
-- Name: desktops_services_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY desktops_services
ADD CONSTRAINT desktops_services_pkey PRIMARY KEY (id);


--
-- Name: desktops_taches_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY desktops_taches
ADD CONSTRAINT desktops_taches_pkey PRIMARY KEY (id);


--
-- Name: desktops_users_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY desktops_users
ADD CONSTRAINT desktops_users_pkey PRIMARY KEY (id);


--
-- Name: desktopsmanagers_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY desktopsmanagers
ADD CONSTRAINT desktopsmanagers_pkey PRIMARY KEY (id);


--
-- Name: desktopsmanagers_sousoperations_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY desktopsmanagers_sousoperations
ADD CONSTRAINT desktopsmanagers_sousoperations_pkey PRIMARY KEY (id);


--
-- Name: documents_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY documents
ADD CONSTRAINT documents_pkey PRIMARY KEY (id);


--
-- Name: dossiers_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY dossiers
ADD CONSTRAINT dossiers_pkey PRIMARY KEY (id);


--
-- Name: events_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY events
ADD CONSTRAINT events_pkey PRIMARY KEY (id);


--
-- Name: fonctions_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY fonctions
ADD CONSTRAINT fonctions_pkey PRIMARY KEY (id);


--
-- Name: formatsreponse_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY formatsreponse
ADD CONSTRAINT formatsreponse_pkey PRIMARY KEY (id);


--
-- Name: formatsreponse_soustypes_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY formatsreponse_soustypes
ADD CONSTRAINT formatsreponse_soustypes_pkey PRIMARY KEY (id);


--
-- Name: gabaritsdocuments_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY gabaritsdocuments
ADD CONSTRAINT gabaritsdocuments_pkey PRIMARY KEY (id);


--
-- Name: groups_name_unique; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY groups
ADD CONSTRAINT groups_name_unique UNIQUE (name);


--
-- Name: groups_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY groups
ADD CONSTRAINT groups_pkey PRIMARY KEY (id);


--
-- Name: inits_services_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY inits_services
ADD CONSTRAINT inits_services_pkey PRIMARY KEY (id);


--
-- Name: intituleagentbyoperations_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY intituleagentbyoperations
ADD CONSTRAINT intituleagentbyoperations_pkey PRIMARY KEY (id);


--
-- Name: intitulesagents_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY intitulesagents
ADD CONSTRAINT intitulesagents_pkey PRIMARY KEY (id);


--
-- Name: activites_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY journalevents
ADD CONSTRAINT journalevents_pkey PRIMARY KEY (id);


--
-- Name: keywordlists_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY keywordlists
ADD CONSTRAINT keywordlists_pkey PRIMARY KEY (id);


--
-- Name: keywords_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY keywords
ADD CONSTRAINT keywords_pkey PRIMARY KEY (id);


--
-- Name: keywordtypes_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY keywordtypes
ADD CONSTRAINT keywordtypes_pkey PRIMARY KEY (id);


--
-- Name: marches_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY marches
ADD CONSTRAINT marches_pkey PRIMARY KEY (id);


--
-- Name: metadonnees_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY metadonnees
ADD CONSTRAINT metadonnees_pkey PRIMARY KEY (id);


--
-- Name: metadonnees_soustypes_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY metadonnees_soustypes
ADD CONSTRAINT metadonnees_soustypes_pkey PRIMARY KEY (id);


--
-- Name: notifications_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY notifications
ADD CONSTRAINT notifications_pkey PRIMARY KEY (id);


--
-- Name: notifieddesktops_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY notifieddesktops
ADD CONSTRAINT notifieddesktops_pkey PRIMARY KEY (id);


--
-- Name: notifquotidiens_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY notifquotidiens
ADD CONSTRAINT notifquotidiens_pkey PRIMARY KEY (id);

--
-- Name: operations_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY operations
ADD CONSTRAINT operations_pkey PRIMARY KEY (id);


--
-- Name: ordresservices_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY ordresservices
ADD CONSTRAINT ordresservices_pkey PRIMARY KEY (id);


--
-- Name: organismes_operations_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY organismes_operations
ADD CONSTRAINT organismes_operations_pkey PRIMARY KEY (id);


--
-- Name: organismes_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY organismes
ADD CONSTRAINT organismes_pkey PRIMARY KEY (id);


--
-- Name: plandelegations_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY plandelegations
ADD CONSTRAINT plandelegations_pkey PRIMARY KEY (id);


--
-- Name: plisconsultatifs_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY plisconsultatifs
ADD CONSTRAINT plisconsultatifs_pkey PRIMARY KEY (id);


--
-- Name: recheche_contactinfos_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY recheche_contactinfos
ADD CONSTRAINT recheche_contactinfos_pkey PRIMARY KEY (id);


--
-- Name: recherches_addressbooks_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY recherches_addressbooks
ADD CONSTRAINT recherches_addressbooks_pkey PRIMARY KEY (id);


--
-- Name: recherches_circuits_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY recherches_circuits
ADD CONSTRAINT recherches_circuits_pkey PRIMARY KEY (id);


--
-- Name: recherches_contacts_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY recherches_contacts
ADD CONSTRAINT recherches_contacts_pkey PRIMARY KEY (id);


--
-- Name: recherches_desktops_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY recherches_desktops
ADD CONSTRAINT recherches_desktops_pkey PRIMARY KEY (id);


--
-- Name: recherches_metadonnees_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY recherches_metadonnees
ADD CONSTRAINT recherches_metadonnees_pkey PRIMARY KEY (id);


--
-- Name: recherches_organismes_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY recherches_organismes
ADD CONSTRAINT recherches_organismes_pkey PRIMARY KEY (id);


--
-- Name: recherches_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY recherches
ADD CONSTRAINT recherches_pkey PRIMARY KEY (id);


--
-- Name: recherches_selectvaluesmetadonnees_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY recherches_selectvaluesmetadonnees
ADD CONSTRAINT recherches_selectvaluesmetadonnees_pkey PRIMARY KEY (id);


--
-- Name: referentielsfantoir_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY referentielsfantoir
ADD CONSTRAINT referentielsfantoir_pkey PRIMARY KEY (id);


--
-- Name: referentielsfantoircom_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY referentielsfantoircom
ADD CONSTRAINT referentielsfantoircom_pkey PRIMARY KEY (id);


--
-- Name: referentielsfantoirdir_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY referentielsfantoirdir
ADD CONSTRAINT referentielsfantoirdir_pkey PRIMARY KEY (id);


--
-- Name: referentielsfantoirvoie_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY referentielsfantoirvoie
ADD CONSTRAINT referentielsfantoirvoie_pkey PRIMARY KEY (id);


--
-- Name: relements_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY relements
ADD CONSTRAINT relements_pkey PRIMARY KEY (id);


--
-- Name: repertoires_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY repertoires
ADD CONSTRAINT repertoires_pkey PRIMARY KEY (id);


--
-- Name: rights_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY rights
ADD CONSTRAINT rights_pkey PRIMARY KEY (id);


--
-- Name: rmodelgabarits_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY rmodelgabarits
ADD CONSTRAINT rmodelgabarits_pkey PRIMARY KEY (id);


--
-- Name: rmodels_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY rmodels
ADD CONSTRAINT rmodels_pkey PRIMARY KEY (id);


--
-- Name: scanemails_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY scanemails
ADD CONSTRAINT scanemails_pkey PRIMARY KEY (id);


--
-- Name: selectvaluesmetadonnees_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY selectvaluesmetadonnees
ADD CONSTRAINT selectvaluesmetadonnees_pkey PRIMARY KEY (id);


--
-- Name: sendsmails_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY sendsmails
ADD CONSTRAINT sendsmails_pkey PRIMARY KEY (id);


--
-- Name: sequences_nom_key; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY sequences
ADD CONSTRAINT sequences_nom_key UNIQUE (nom);


--
-- Name: sequences_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY sequences
ADD CONSTRAINT sequences_pkey PRIMARY KEY (id);


--
-- Name: services_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY services
ADD CONSTRAINT services_pkey PRIMARY KEY (id);


--
-- Name: sousactivites_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY sousactivites
ADD CONSTRAINT sousactivites_pkey PRIMARY KEY (id);


--
-- Name: sousoperations_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY sousoperations
ADD CONSTRAINT sousoperations_pkey PRIMARY KEY (id);


--
-- Name: soustypes_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY soustypes
ADD CONSTRAINT soustypes_pkey PRIMARY KEY (id);


--
-- Name: taches_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY taches
ADD CONSTRAINT taches_pkey PRIMARY KEY (id);


--
-- Name: templatemails_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY templatemails
ADD CONSTRAINT templatemails_pkey PRIMARY KEY (id);


--
-- Name: templatenotifications_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY templatenotifications
ADD CONSTRAINT templatenotifications_pkey PRIMARY KEY (id);


--
-- Name: typemetadonnees_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY typemetadonnees
ADD CONSTRAINT typemetadonnees_pkey PRIMARY KEY (id);


--
-- Name: typenotifs_name_key; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY typenotifs
ADD CONSTRAINT typenotifs_name_key UNIQUE (name);


--
-- Name: typenotifs_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY typenotifs
ADD CONSTRAINT typenotifs_pkey PRIMARY KEY (id);


--
-- Name: typenotifs_users_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY typenotifs_users
ADD CONSTRAINT typenotifs_users_pkey PRIMARY KEY (id);


--
-- Name: types_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY types
ADD CONSTRAINT types_pkey PRIMARY KEY (id);


--
-- Name: originesflux_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY originesflux
ADD CONSTRAINT originesflux_pkey PRIMARY KEY (id);


--
-- Name: uniterifs_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres; Tablespace: 
--

ALTER TABLE ONLY uniterifs
ADD CONSTRAINT uniterifs_pkey PRIMARY KEY (id);


--
-- Name: users_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY users
ADD CONSTRAINT users_pkey PRIMARY KEY (id);


--
-- Name: wkf_circuits_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY wkf_circuits
ADD CONSTRAINT wkf_circuits_pkey PRIMARY KEY (id);


--
-- Name: wkf_compositions_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY wkf_compositions
ADD CONSTRAINT wkf_compositions_pkey PRIMARY KEY (id);


--
-- Name: wkf_etapes_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY wkf_etapes
ADD CONSTRAINT wkf_etapes_pkey PRIMARY KEY (id);


--
-- Name: wkf_signatures_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY wkf_signatures
ADD CONSTRAINT wkf_signatures_pkey PRIMARY KEY (id);


--
-- Name: wkf_traitements_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY wkf_traitements
ADD CONSTRAINT wkf_traitements_pkey PRIMARY KEY (id);


--
-- Name: wkf_visas_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc; Tablespace: 
--

ALTER TABLE ONLY wkf_visas
ADD CONSTRAINT wkf_visas_pkey PRIMARY KEY (id);


--
-- Name: acos_alias_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX acos_alias_idx ON acos USING btree (alias);


--
-- Name: acos_lft_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX acos_lft_idx ON acos USING btree (lft);


--
-- Name: acos_parent_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX acos_parent_id_idx ON acos USING btree (parent_id);


--
-- Name: acos_rght_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX acos_rght_idx ON acos USING btree (rght);


--
-- Name: affaires_dossier_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX affaires_dossier_id_idx ON affaires USING btree (dossier_id);


--
-- Name: aro_aco_key; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE UNIQUE INDEX aro_aco_key ON aros_acos USING btree (aro_id, aco_id);


--
-- Name: aros_acos_aco_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX aros_acos_aco_id_idx ON aros_acos USING btree (aco_id);


--
-- Name: aros_acos_aro_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX aros_acos_aro_id_idx ON aros_acos USING btree (aro_id);


--
-- Name: aros_alias_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX aros_alias_idx ON aros USING btree (alias);


--
-- Name: aros_aros_aco_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX aros_aros_aco_id_idx ON aros_acos USING btree (aco_id);


--
-- Name: aros_aros_aro_id_aco_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE UNIQUE INDEX aros_aros_aro_id_aco_id_idx ON aros_acos USING btree (aro_id, aco_id);


--
-- Name: aros_aros_aro_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX aros_aros_aro_id_idx ON aros_acos USING btree (aro_id);


--
-- Name: aros_foreign_key_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX aros_foreign_key_idx ON aros USING btree (foreign_key);


--
-- Name: aros_lft_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX aros_lft_idx ON aros USING btree (lft);


--
-- Name: aros_model_alias_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE UNIQUE INDEX aros_model_alias_idx ON aros USING btree (model, alias);


--
-- Name: aros_model_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX aros_model_idx ON aros USING btree (model);


--
-- Name: aros_parent_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX aros_parent_id_idx ON aros USING btree (parent_id);


--
-- Name: aros_rght_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX aros_rght_idx ON aros USING btree (rght);


--
-- Name: bancontenus_bannette_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX bancontenus_bannette_id_idx ON bancontenus USING btree (bannette_id);


--
-- Name: bancontenus_courrier_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX bancontenus_courrier_id_idx ON bancontenus USING btree (courrier_id);


--
-- Name: bancontenus_etat_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX bancontenus_etat_id_idx ON bancontenus USING btree (etat);


--
-- Name: bans_name_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE UNIQUE INDEX bans_name_idx ON bans USING btree (name);


--
-- Name: banscommunes_name_ban_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE UNIQUE INDEX banscommunes_name_ban_id_idx ON banscommunes USING btree (name, ban_id);


--
-- Name: comments_slug_noaccents_upper; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX comments_slug_noaccents_upper ON comments USING btree (noaccents_upper((slug)::text));


--
-- Name: compositions_etape_id_trigger_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE UNIQUE INDEX compositions_etape_id_trigger_id_idx ON wkf_compositions USING btree (etape_id, trigger_id);


--
-- Name: compteurs_sequence_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX compteurs_sequence_id_idx ON compteurs USING btree (sequence_id);


--
-- Name: consultations_name_idx; Type: INDEX; Schema: public; Owner: postgres; Tablespace: 
--

CREATE UNIQUE INDEX consultations_name_idx ON consultations USING btree (name);


--
-- Name: consultations_operation_id_idx; Type: INDEX; Schema: public; Owner: postgres; Tablespace: 
--

CREATE INDEX consultations_operation_id_idx ON consultations USING btree (operation_id);


--
-- Name: courriers_affaire_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX courriers_affaire_id_idx ON courriers USING btree (affaire_id);


--
-- Name: courriers_metadonnees_courrier_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX courriers_metadonnees_courrier_id_idx ON courriers_metadonnees USING btree (courrier_id);


--
-- Name: courriers_metadonnees_courrier_id_metadonnee_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE UNIQUE INDEX courriers_metadonnees_courrier_id_metadonnee_id_idx ON courriers_metadonnees USING btree (courrier_id, metadonnee_id);


--
-- Name: courriers_metadonnees_metadonnee_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX courriers_metadonnees_metadonnee_id_idx ON courriers_metadonnees USING btree (metadonnee_id);


--
-- Name: courriers_parent_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX courriers_parent_id_idx ON courriers USING btree (parent_id);


--
-- Name: courriers_reference_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE UNIQUE INDEX courriers_reference_idx ON courriers USING btree (reference);


--
-- Name: courriers_repertoires_courrier_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX courriers_repertoires_courrier_id_idx ON courriers_repertoires USING btree (courrier_id);


--
-- Name: courriers_repertoires_courrier_id_repertoire_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE UNIQUE INDEX courriers_repertoires_courrier_id_repertoire_id_idx ON courriers_repertoires USING btree (courrier_id, repertoire_id);


--
-- Name: courriers_repertoires_repertoire_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX courriers_repertoires_repertoire_id_idx ON courriers_repertoires USING btree (repertoire_id);


--
-- Name: desktops_desktopsmanagers_desktop_id_desktopmanager_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE UNIQUE INDEX desktops_desktopsmanagers_desktop_id_desktopmanager_id_idx ON desktops_desktopsmanagers USING btree (desktop_id, desktopmanager_id);


--
-- Name: desktops_desktopsmanagers_desktop_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX desktops_desktopsmanagers_desktop_id_idx ON desktops_desktopsmanagers USING btree (desktop_id);


--
-- Name: desktops_desktopsmanagers_desktopmanager_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX desktops_desktopsmanagers_desktopmanager_id_idx ON desktops_desktopsmanagers USING btree (desktopmanager_id);


--
-- Name: desktopsmanagers_sousoperations_desktopmanager_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX desktopsmanagers_sousoperations_desktopmanager_id_idx ON desktopsmanagers_sousoperations USING btree (desktopmanager_id);


--
-- Name: desktopsmanagers_sousoperations_desktopmanager_id_sousoperation; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE UNIQUE INDEX desktopsmanagers_sousoperations_desktopmanager_id_sousoperation ON desktopsmanagers_sousoperations USING btree (desktopmanager_id, sousoperation_id);


--
-- Name: desktopsmanagers_sousoperations_intituleagent_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX desktopsmanagers_sousoperations_intituleagent_id_idx ON desktopsmanagers_sousoperations USING btree (intituleagent_id);


--
-- Name: desktopsmanagers_sousoperations_sousoperation_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX desktopsmanagers_sousoperations_sousoperation_id_idx ON desktopsmanagers_sousoperations USING btree (sousoperation_id);


--
-- Name: formatsreponse_soustypes_formatreponse_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX formatsreponse_soustypes_formatreponse_id_idx ON formatsreponse_soustypes USING btree (formatreponse_id);


--
-- Name: formatsreponse_soustypes_soustype_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX formatsreponse_soustypes_soustype_id_idx ON formatsreponse_soustypes USING btree (soustype_id);


--
-- Name: intituleagentbyoperations_desktopmanager_id_idx; Type: INDEX; Schema: public; Owner: postgres; Tablespace: 
--

CREATE INDEX intituleagentbyoperations_desktopmanager_id_idx ON intituleagentbyoperations USING btree (desktopmanager_id);


--
-- Name: intituleagentbyoperations_intituleagent_id_idx; Type: INDEX; Schema: public; Owner: postgres; Tablespace: 
--

CREATE INDEX intituleagentbyoperations_intituleagent_id_idx ON intituleagentbyoperations USING btree (intituleagent_id);


--
-- Name: intituleagentbyoperations_operation_id_idx; Type: INDEX; Schema: public; Owner: postgres; Tablespace: 
--

CREATE INDEX intituleagentbyoperations_operation_id_idx ON intituleagentbyoperations USING btree (operation_id);


--
-- Name: intituleagentbyoperations_operation_id_intituleagent_id_idx; Type: INDEX; Schema: public; Owner: postgres; Tablespace: 
--

CREATE UNIQUE INDEX intituleagentbyoperations_operation_id_intituleagent_id_idx ON intituleagentbyoperations USING btree (intituleagent_id, operation_id);


--
-- Name: marches_operation_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX marches_operation_id_idx ON marches USING btree (operation_id);


--
-- Name: metadonnees_soustypes_metadonnee_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX metadonnees_soustypes_metadonnee_id_idx ON metadonnees_soustypes USING btree (metadonnee_id);


--
-- Name: metadonnees_soustypes_metadonnee_id_soustype_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE UNIQUE INDEX metadonnees_soustypes_metadonnee_id_soustype_id_idx ON metadonnees_soustypes USING btree (metadonnee_id, soustype_id);


--
-- Name: metadonnees_soustypes_soustype_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX metadonnees_soustypes_soustype_id_idx ON metadonnees_soustypes USING btree (soustype_id);


--
-- Name: metadonnees_typemetadonnee_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX metadonnees_typemetadonnee_id_idx ON metadonnees USING btree (typemetadonnee_id);


--
-- Name: operations_type_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX operations_type_id_idx ON operations USING btree (type_id);


--
-- Name: ordresservices_marche_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX ordresservices_marche_id_idx ON ordresservices USING btree (marche_id);


--
-- Name: ordresservices_marche_id_numero_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE UNIQUE INDEX ordresservices_marche_id_numero_idx ON ordresservices USING btree (marche_id, numero);


--
-- Name: organismes_addressbook_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX organismes_addressbook_id_idx ON organismes USING btree (addressbook_id);


--
-- Name: organismes_ban_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX organismes_ban_id_idx ON organismes USING btree (ban_id);


--
-- Name: organismes_bancommune_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX organismes_bancommune_id_idx ON organismes USING btree (bancommune_id);


--
-- Name: plisconsultatifs_consultation_id_idx; Type: INDEX; Schema: public; Owner: postgres; Tablespace: 
--

CREATE INDEX plisconsultatifs_consultation_id_idx ON plisconsultatifs USING btree (consultation_id);


--
-- Name: plisconsultatifs_typologieflux_id_idx; Type: INDEX; Schema: public; Owner: postgres; Tablespace: 
--

CREATE INDEX plisconsultatifs_origineflux_id_idx ON plisconsultatifs USING btree (origineflux_id);


--
-- Name: recherches_affaires_affaire_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX recherches_affaires_affaire_id_idx ON recherches_affaires USING btree (affaire_id);


--
-- Name: recherches_affaires_recherche_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX recherches_affaires_recherche_id_idx ON recherches_affaires USING btree (recherche_id);


--
-- Name: recherches_circuits_circuit_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX recherches_circuits_circuit_id_idx ON recherches_circuits USING btree (circuit_id);


--
-- Name: recherches_circuits_recherche_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX recherches_circuits_recherche_id_idx ON recherches_circuits USING btree (recherche_id);


--
-- Name: recherches_desktops_desktop_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX recherches_desktops_desktop_id_idx ON recherches_desktops USING btree (desktop_id);


--
-- Name: recherches_desktops_recherche_id_desktop_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE UNIQUE INDEX recherches_desktops_recherche_id_desktop_id_idx ON recherches_desktops USING btree (recherche_id, desktop_id);


--
-- Name: recherches_desktops_recherche_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX recherches_desktops_recherche_id_idx ON recherches_desktops USING btree (recherche_id);


--
-- Name: recherches_dossiers_dossier_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX recherches_dossiers_dossier_id_idx ON recherches_dossiers USING btree (dossier_id);


--
-- Name: recherches_dossiers_recherche_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX recherches_dossiers_recherche_id_idx ON recherches_dossiers USING btree (recherche_id);


--
-- Name: recherches_metadonnees_metadonnee_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX recherches_metadonnees_metadonnee_id_idx ON recherches_metadonnees USING btree (metadonnee_id);


--
-- Name: recherches_metadonnees_recherche_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX recherches_metadonnees_recherche_id_idx ON recherches_metadonnees USING btree (recherche_id);


--
-- Name: recherches_organismes_organisme_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX recherches_organismes_organisme_id_idx ON recherches_organismes USING btree (organisme_id);


--
-- Name: recherches_organismes_recherche_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX recherches_organismes_recherche_id_idx ON recherches_organismes USING btree (recherche_id);


--
-- Name: recherches_organismes_recherche_id_organisme_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE UNIQUE INDEX recherches_organismes_recherche_id_organisme_id_idx ON recherches_organismes USING btree (recherche_id, organisme_id);


--
-- Name: recherches_selectvaluesmetadonnees_recherche_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX recherches_selectvaluesmetadonnees_recherche_id_idx ON recherches_selectvaluesmetadonnees USING btree (recherche_id);


--
-- Name: recherches_selectvaluesmetadonnees_recherche_id_selectvaluemeta; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE UNIQUE INDEX recherches_selectvaluesmetadonnees_recherche_id_selectvaluemeta ON recherches_selectvaluesmetadonnees USING btree (recherche_id, selectvaluemetadonnee_id);


--
-- Name: recherches_soustypes_recherche_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX recherches_soustypes_recherche_id_idx ON recherches_soustypes USING btree (recherche_id);


--
-- Name: recherches_soustypes_soustype_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX recherches_soustypes_soustype_id_idx ON recherches_soustypes USING btree (soustype_id);


--
-- Name: recherches_taches_recherche_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX recherches_taches_recherche_id_idx ON recherches_taches USING btree (recherche_id);


--
-- Name: recherches_taches_tache_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX recherches_taches_tache_id_idx ON recherches_taches USING btree (tache_id);


--
-- Name: recherches_types_recherche_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX recherches_types_recherche_id_idx ON recherches_types USING btree (recherche_id);


--
-- Name: recherches_types_type_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX recherches_types_type_id_idx ON recherches_types USING btree (type_id);


--
-- Name: recherches_user_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX recherches_user_id_idx ON recherches USING btree (user_id);


--
-- Name: repertoires_parent_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX repertoires_parent_id_idx ON repertoires USING btree (parent_id);


--
-- Name: repertoires_user_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX repertoires_user_id_idx ON repertoires USING btree (user_id);


--
-- Name: rmodelgabarits_soustype_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX rmodelgabarits_soustype_id_idx ON rmodelgabarits USING btree (soustype_id);


--
-- Name: scanemails_desktop_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX scanemails_desktop_id_idx ON scanemails USING btree (desktop_id);


--
-- Name: scanemails_name_desktop_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE UNIQUE INDEX scanemails_name_desktop_id_idx ON scanemails USING btree (name, desktop_id);


--
-- Name: scanemails_soustype_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX scanemails_soustype_id_idx ON scanemails USING btree (soustype_id);


--
-- Name: scanemails_type_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX scanemails_type_id_idx ON scanemails USING btree (type_id);


--
-- Name: sendsmails_courrier_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX sendsmails_courrier_id_idx ON sendsmails USING btree (courrier_id);


--
-- Name: sendsmails_desktop_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX sendsmails_desktop_id_idx ON sendsmails USING btree (desktop_id);


--
-- Name: sendsmails_document_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX sendsmails_document_id_idx ON sendsmails USING btree (document_id);


--
-- Name: services_parent_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX services_parent_id_idx ON services USING btree (parent_id);


--
-- Name: sousoperations_operation_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX sousoperations_operation_id_idx ON sousoperations USING btree (operation_id);


--
-- Name: sousoperations_soustype_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX sousoperations_soustype_id_idx ON sousoperations USING btree (soustype_id);


--
-- Name: soustypes_circuit_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX soustypes_circuit_id_idx ON soustypes USING btree (circuit_id);


--
-- Name: soustypes_compteur_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX soustypes_compteur_id_idx ON soustypes USING btree (compteur_id);


--
-- Name: soustypes_parent_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX soustypes_parent_id_idx ON soustypes USING btree (parent_id);


--
-- Name: soustypes_type_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX soustypes_type_id_idx ON soustypes USING btree (type_id);


--
-- Name: taches_act_user_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX taches_act_user_id_idx ON taches USING btree (act_desktop_id);


--
-- Name: taches_courrier_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX taches_courrier_id_idx ON taches USING btree (courrier_id);


--
-- Name: wkf_cicruits_unique_nom; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE UNIQUE INDEX wkf_cicruits_unique_nom ON wkf_circuits USING btree (nom);


--
-- Name: wkf_circuits_created_user_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX wkf_circuits_created_user_id_idx ON wkf_circuits USING btree (created_user_id);


--
-- Name: wkf_circuits_modified_user_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX wkf_circuits_modified_user_id_idx ON wkf_circuits USING btree (modified_user_id);


--
-- Name: wkf_compositions_created_user_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX wkf_compositions_created_user_id_idx ON wkf_compositions USING btree (created_user_id);


--
-- Name: wkf_compositions_etape_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX wkf_compositions_etape_id_idx ON wkf_compositions USING btree (etape_id);


--
-- Name: wkf_compositions_modified_user_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX wkf_compositions_modified_user_id_idx ON wkf_compositions USING btree (modified_user_id);


--
-- Name: wkf_compositions_trigger_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX wkf_compositions_trigger_id_idx ON wkf_compositions USING btree (trigger_id);


--
-- Name: wkf_etapes_circuit_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX wkf_etapes_circuit_id_idx ON wkf_etapes USING btree (circuit_id);


--
-- Name: wkf_etapes_created_user_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX wkf_etapes_created_user_id_idx ON wkf_etapes USING btree (created_user_id);


--
-- Name: wkf_etapes_modified_user_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX wkf_etapes_modified_user_id_idx ON wkf_etapes USING btree (modified_user_id);


--
-- Name: wkf_traitements_circuit_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX wkf_traitements_circuit_id_idx ON wkf_traitements USING btree (circuit_id);


--
-- Name: wkf_traitements_created_user_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX wkf_traitements_created_user_id_idx ON wkf_traitements USING btree (created_user_id);


--
-- Name: wkf_traitements_modified_user_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX wkf_traitements_modified_user_id_idx ON wkf_traitements USING btree (modified_user_id);


--
-- Name: wkf_traitements_target_id_key; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX wkf_traitements_target_id_key ON wkf_traitements USING btree (target_id);


--
-- Name: wkf_visas_signature_id_idx; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX wkf_visas_signature_id_idx ON wkf_visas USING btree (signature_id);


--
-- Name: wkf_visas_traitement_id_key; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX wkf_visas_traitement_id_key ON wkf_visas USING btree (traitement_id);


--
-- Name: wkf_visas_trigger_id_key; Type: INDEX; Schema: public; Owner: webgfc; Tablespace: 
--

CREATE INDEX wkf_visas_trigger_id_key ON wkf_visas USING btree (trigger_id);


--
-- Name: aco_fk; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY aros_acos
ADD CONSTRAINT aco_fk FOREIGN KEY (aco_id) REFERENCES acos(id);


--
-- Name: activites_organismes_activite_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY activites_organismes
ADD CONSTRAINT activites_organismes_activite_id_fkey FOREIGN KEY (activite_id) REFERENCES activites(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: activites_organismes_organisme_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY activites_organismes
ADD CONSTRAINT activites_organismes_organisme_id_fkey FOREIGN KEY (organisme_id) REFERENCES organismes(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: affaires_dossiers_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY affaires
ADD CONSTRAINT affaires_dossiers_fkey FOREIGN KEY (dossier_id) REFERENCES dossiers(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: armodels_soustype_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY armodels
ADD CONSTRAINT armodels_soustype_id_fkey FOREIGN KEY (soustype_id) REFERENCES soustypes(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: aro_fk; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY aros_acos
ADD CONSTRAINT aro_fk FOREIGN KEY (aro_id) REFERENCES aros(id);


--
-- Name: ars_courrier_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY ars
ADD CONSTRAINT ars_courrier_id_fkey FOREIGN KEY (courrier_id) REFERENCES courriers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: bancontenus_bannettes_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY bancontenus
ADD CONSTRAINT bancontenus_bannettes_fkey FOREIGN KEY (bannette_id) REFERENCES bannettes(id);


--
-- Name: bancontenus_courriers_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY bancontenus
ADD CONSTRAINT bancontenus_courriers_fkey FOREIGN KEY (courrier_id) REFERENCES courriers(id);


--
-- Name: bancontenus_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY bancontenus
ADD CONSTRAINT bancontenus_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES desktops(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: bancontenus_desktopmanager_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY bancontenus
ADD CONSTRAINT bancontenus_desktopmanager_id_fkey FOREIGN KEY (desktopmanager_id) REFERENCES desktopsmanagers(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: bansadresses_bancommune_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY bansadresses
ADD CONSTRAINT bansadresses_bancommune_id_fkey FOREIGN KEY (bancommune_id) REFERENCES banscommunes(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: banscommunes_ban_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY banscommunes
ADD CONSTRAINT banscommunes_ban_id_fkey FOREIGN KEY (ban_id) REFERENCES bans(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: comments_readers_comment_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY comments_readers
ADD CONSTRAINT comments_readers_comment_id_fkey FOREIGN KEY (comment_id) REFERENCES comments(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: compteurs_sequence_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY compteurs
ADD CONSTRAINT compteurs_sequence_id_fkey FOREIGN KEY (sequence_id) REFERENCES sequences(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: consultations_operation_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY consultations
ADD CONSTRAINT consultations_operation_id_fkey FOREIGN KEY (operation_id) REFERENCES operations(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: contactinfos_contact_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY contactinfos
ADD CONSTRAINT contactinfos_contact_id_fkey FOREIGN KEY (contact_id) REFERENCES contacts(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: contacts_addressbook_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY contacts
ADD CONSTRAINT contacts_addressbook_id_fkey FOREIGN KEY (addressbook_id) REFERENCES addressbooks(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: contacts_ban_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY contacts
ADD CONSTRAINT contacts_ban_id_fkey FOREIGN KEY (ban_id) REFERENCES bans(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: contacts_bancommune_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY contacts
ADD CONSTRAINT contacts_bancommune_id_fkey FOREIGN KEY (bancommune_id) REFERENCES banscommunes(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: contacts_events_contact_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY contacts_events
ADD CONSTRAINT contacts_events_contact_id_fkey FOREIGN KEY (contact_id) REFERENCES contacts(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: contacts_events_event_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY contacts_events
ADD CONSTRAINT contacts_events_event_id_fkey FOREIGN KEY (event_id) REFERENCES events(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: contacts_fonction_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY contacts
ADD CONSTRAINT contacts_fonction_id_fkey FOREIGN KEY (fonction_id) REFERENCES fonctions(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: contacts_operations_contact_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY contacts_operations
ADD CONSTRAINT contacts_operations_contact_id_fkey FOREIGN KEY (contact_id) REFERENCES contacts(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: contacts_operations_operation_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY contacts_operations
ADD CONSTRAINT contacts_operations_operation_id_fkey FOREIGN KEY (operation_id) REFERENCES operations(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: contacts_organisme_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY contacts
ADD CONSTRAINT contacts_organisme_id_fkey FOREIGN KEY (organisme_id) REFERENCES organismes(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: couriers_affaires_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY courriers
ADD CONSTRAINT couriers_affaires_fkey FOREIGN KEY (affaire_id) REFERENCES affaires(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: courriers_affairesuiviepar_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY courriers
ADD CONSTRAINT courriers_affairesuiviepar_id_fkey FOREIGN KEY (affairesuiviepar_id) REFERENCES desktops(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: courriers_consultation_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY courriers
ADD CONSTRAINT courriers_consultation_id_fkey FOREIGN KEY (consultation_id) REFERENCES consultations(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: courriers_contactinfo_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY courriers
ADD CONSTRAINT courriers_contactinfo_id_fkey FOREIGN KEY (contactinfo_id) REFERENCES contactinfos(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: courriers_desktop_creator_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY courriers
ADD CONSTRAINT courriers_desktop_creator_id_fkey FOREIGN KEY (desktop_creator_id) REFERENCES desktops(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: courriers_marche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY courriers
ADD CONSTRAINT courriers_marche_id_fkey FOREIGN KEY (marche_id) REFERENCES marches(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: courriers_metadonnees_courriers_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY courriers_metadonnees
ADD CONSTRAINT courriers_metadonnees_courriers_fkey FOREIGN KEY (courrier_id) REFERENCES courriers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: courriers_metadonnees_metadonnees_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY courriers_metadonnees
ADD CONSTRAINT courriers_metadonnees_metadonnees_fkey FOREIGN KEY (metadonnee_id) REFERENCES metadonnees(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: courriers_organisme_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY courriers
ADD CONSTRAINT courriers_organisme_id_fkey FOREIGN KEY (organisme_id) REFERENCES organismes(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: courriers_repertoires_courriers_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY courriers_repertoires
ADD CONSTRAINT courriers_repertoires_courriers_fkey FOREIGN KEY (courrier_id) REFERENCES courriers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: courriers_repertoires_repertoires_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY courriers_repertoires
ADD CONSTRAINT courriers_repertoires_repertoires_fkey FOREIGN KEY (repertoire_id) REFERENCES repertoires(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: courriers_service_creator_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY courriers
ADD CONSTRAINT courriers_service_creator_id_fkey FOREIGN KEY (service_creator_id) REFERENCES services(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: courriers_soustypes_soustype_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY courriers
ADD CONSTRAINT courriers_soustypes_soustype_id_fkey FOREIGN KEY (soustype_id) REFERENCES soustypes(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: courriers_taches_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY taches
ADD CONSTRAINT courriers_taches_fkey FOREIGN KEY (courrier_id) REFERENCES courriers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: courriers_user_creator_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY courriers
ADD CONSTRAINT courriers_user_creator_id_fkey FOREIGN KEY (user_creator_id) REFERENCES users(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: desktops_desktopsmanagers_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY desktops_desktopsmanagers
ADD CONSTRAINT desktops_desktopsmanagers_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES desktops(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: desktops_desktopsmanagers_desktopmanager_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY desktops_desktopsmanagers
ADD CONSTRAINT desktops_desktopsmanagers_desktopmanager_id_fkey FOREIGN KEY (desktopmanager_id) REFERENCES desktopsmanagers(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: desktops_group_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY desktops
ADD CONSTRAINT desktops_group_id_fkey FOREIGN KEY (group_id) REFERENCES groups(id);


--
-- Name: desktops_services_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY desktops_services
ADD CONSTRAINT desktops_services_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES desktops(id);


--
-- Name: desktops_services_service_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY desktops_services
ADD CONSTRAINT desktops_services_service_id_fkey FOREIGN KEY (service_id) REFERENCES services(id);


--
-- Name: desktops_taches_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY desktops_taches
ADD CONSTRAINT desktops_taches_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES desktops(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: desktops_taches_tache_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY desktops_taches
ADD CONSTRAINT desktops_taches_tache_id_fkey FOREIGN KEY (tache_id) REFERENCES taches(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: desktops_users_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY desktops_users
ADD CONSTRAINT desktops_users_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES desktops(id);


--
-- Name: desktops_users_user_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY desktops_users
ADD CONSTRAINT desktops_users_user_id_fkey FOREIGN KEY (user_id) REFERENCES users(id);


--
-- Name: desktopsmanagers_sousoperations_desktopmanager_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY desktopsmanagers_sousoperations
ADD CONSTRAINT desktopsmanagers_sousoperations_desktopmanager_id_fkey FOREIGN KEY (desktopmanager_id) REFERENCES desktopsmanagers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: desktopsmanagers_sousoperations_intituleagent_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY desktopsmanagers_sousoperations
ADD CONSTRAINT desktopsmanagers_sousoperations_intituleagent_id_fkey FOREIGN KEY (intituleagent_id) REFERENCES intitulesagents(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: desktopsmanagers_sousoperations_sousoperation_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY desktopsmanagers_sousoperations
ADD CONSTRAINT desktopsmanagers_sousoperations_sousoperation_id_fkey FOREIGN KEY (sousoperation_id) REFERENCES sousoperations(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: documents_courrier_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY documents
ADD CONSTRAINT documents_courrier_id_fkey FOREIGN KEY (courrier_id) REFERENCES courriers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: formatsreponse_soustypes_formatreponse_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY formatsreponse_soustypes
ADD CONSTRAINT formatsreponse_soustypes_formatreponse_id_fkey FOREIGN KEY (formatreponse_id) REFERENCES formatsreponse(id);


--
-- Name: formatsreponse_soustypes_soustype_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY formatsreponse_soustypes
ADD CONSTRAINT formatsreponse_soustypes_soustype_id_fkey FOREIGN KEY (soustype_id) REFERENCES soustypes(id);


--
-- Name: groups_group_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY groups
ADD CONSTRAINT groups_group_id_fkey FOREIGN KEY (group_id) REFERENCES groups(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: inits_services_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY inits_services
ADD CONSTRAINT inits_services_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES desktops(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: inits_services_service_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY inits_services
ADD CONSTRAINT inits_services_service_id_fkey FOREIGN KEY (service_id) REFERENCES services(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: intituleagentbyoperations_desktopmanager_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY intituleagentbyoperations
ADD CONSTRAINT intituleagentbyoperations_desktopmanager_id_fkey FOREIGN KEY (desktopmanager_id) REFERENCES desktopsmanagers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: intituleagentbyoperations_intituleagent_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY intituleagentbyoperations
ADD CONSTRAINT intituleagentbyoperations_intituleagent_id_fkey FOREIGN KEY (intituleagent_id) REFERENCES intitulesagents(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: intituleagentbyoperations_operation_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY intituleagentbyoperations
ADD CONSTRAINT intituleagentbyoperations_operation_id_fkey FOREIGN KEY (operation_id) REFERENCES operations(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: journalevents_user_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY journalevents
ADD CONSTRAINT journalevents_user_id_fkey FOREIGN KEY (user_id) REFERENCES users(id) ON UPDATE CASCADE ON DELETE CASCADE;

--
-- Name: journalevents_user_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY journalevents
ADD CONSTRAINT journalevents_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES desktops(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: keywordlists_keywodtype_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY keywordlists
ADD CONSTRAINT keywordlists_keywodtype_id_fkey FOREIGN KEY (keywordtype_id) REFERENCES keywordtypes(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: keywords_keywordlist_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY keywords
ADD CONSTRAINT keywords_keywordlist_id_fkey FOREIGN KEY (keywordlist_id) REFERENCES keywordlists(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: marches_contractant_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY marches
ADD CONSTRAINT marches_contractant_id_fkey FOREIGN KEY (contractant_id) REFERENCES contractants(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: marches_operation_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY marches
ADD CONSTRAINT marches_operation_id_fkey FOREIGN KEY (operation_id) REFERENCES operations(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: marches_uniterif_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY marches
ADD CONSTRAINT marches_uniterif_id_fkey FOREIGN KEY (uniterif_id) REFERENCES uniterifs(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: metadonnees_soustypes_metadonnee_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY metadonnees_soustypes
ADD CONSTRAINT metadonnees_soustypes_metadonnee_id_fkey FOREIGN KEY (metadonnee_id) REFERENCES metadonnees(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: metadonnees_soustypes_soustype_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY metadonnees_soustypes
ADD CONSTRAINT metadonnees_soustypes_soustype_id_fkey FOREIGN KEY (soustype_id) REFERENCES soustypes(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: metadonnees_typemetadonnees_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY metadonnees
ADD CONSTRAINT metadonnees_typemetadonnees_fkey FOREIGN KEY (typemetadonnee_id) REFERENCES typemetadonnees(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: notifications_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY notifications
ADD CONSTRAINT notifications_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES desktops(id);


--
-- Name: notifications_typenotif_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY notifications
ADD CONSTRAINT notifications_typenotif_id_fkey FOREIGN KEY (typenotif_id) REFERENCES typenotifs(id);


--
-- Name: notifieddesktops_courrier_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY notifieddesktops
ADD CONSTRAINT notifieddesktops_courrier_id_fkey FOREIGN KEY (courrier_id) REFERENCES courriers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: notifieddesktops_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY notifieddesktops
ADD CONSTRAINT notifieddesktops_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES desktops(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: notifieddesktops_notification_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY notifieddesktops
ADD CONSTRAINT notifieddesktops_notification_id_fkey FOREIGN KEY (notification_id) REFERENCES notifications(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: notifquotidiens_notification_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY notifquotidiens
ADD CONSTRAINT notifquotidiens_notification_id_fkey FOREIGN KEY (notification_id) REFERENCES notifications(id) ON UPDATE CASCADE ON DELETE CASCADE;

-- --
-- -- Name: notifquotidiens_courrier_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
-- --

ALTER TABLE ONLY notifquotidiens
ADD CONSTRAINT notifquotidiens_courrier_id_fkey FOREIGN KEY (courrier_id) REFERENCES courriers(id) ON UPDATE CASCADE ON DELETE CASCADE;

-- --
-- -- Name: notifquotidiens_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
-- --

ALTER TABLE ONLY notifquotidiens
ADD CONSTRAINT notifquotidiens_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES desktops(id) ON UPDATE CASCADE ON DELETE CASCADE;

-- --
-- -- Name: notifquotidiens_user_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
-- --

ALTER TABLE ONLY notifquotidiens
ADD CONSTRAINT notifquotidiens_user_id_fkey FOREIGN KEY (user_id) REFERENCES users(id) ON UPDATE CASCADE ON DELETE CASCADE;

--
-- Name: operations_type_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY operations
ADD CONSTRAINT operations_type_id_fkey FOREIGN KEY (type_id) REFERENCES types(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: ordresservices_courrier_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY ordresservices
ADD CONSTRAINT ordresservices_courrier_id_fkey FOREIGN KEY (courrier_id) REFERENCES courriers(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: ordresservices_marche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY ordresservices
ADD CONSTRAINT ordresservices_marche_id_fkey FOREIGN KEY (marche_id) REFERENCES marches(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: ordresservices_organisme_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY ordresservices
ADD CONSTRAINT ordresservices_organisme_id_fkey FOREIGN KEY (organisme_id) REFERENCES organismes(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: organismes_addressbook_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY organismes
ADD CONSTRAINT organismes_addressbook_id_fkey FOREIGN KEY (addressbook_id) REFERENCES addressbooks(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: organismes_ban_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY organismes
ADD CONSTRAINT organismes_ban_id_fkey FOREIGN KEY (ban_id) REFERENCES bans(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: organismes_bancommune_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY organismes
ADD CONSTRAINT organismes_bancommune_id_fkey FOREIGN KEY (bancommune_id) REFERENCES banscommunes(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: organismes_operations_operation_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY organismes_operations
ADD CONSTRAINT organismes_operations_operation_id_fkey FOREIGN KEY (operation_id) REFERENCES operations(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: organismes_operations_organisme_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY organismes_operations
ADD CONSTRAINT organismes_operations_organisme_id_fkey FOREIGN KEY (organisme_id) REFERENCES organismes(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: plandelegations_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY plandelegations
ADD CONSTRAINT plandelegations_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES desktops(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: plandelegations_user_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY plandelegations
ADD CONSTRAINT plandelegations_user_id_fkey FOREIGN KEY (user_id) REFERENCES users(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: plisconsultatifs_consultation_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY plisconsultatifs
ADD CONSTRAINT plisconsultatifs_consultation_id_fkey FOREIGN KEY (consultation_id) REFERENCES consultations(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: plisconsultatifs_courrier_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY plisconsultatifs
ADD CONSTRAINT plisconsultatifs_courrier_id_fkey FOREIGN KEY (courrier_id) REFERENCES courriers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: plisconsultatifs_typologieflux_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY plisconsultatifs
ADD CONSTRAINT plisconsultatifs_origineflux_id_fkey FOREIGN KEY (origineflux_id) REFERENCES originesflux(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: reacherches_affaires_recherche_id; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_affaires
ADD CONSTRAINT reacherches_affaires_recherche_id FOREIGN KEY (recherche_id) REFERENCES recherches(id);


--
-- Name: recheche_contactinfos_contactinfo_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recheche_contactinfos
ADD CONSTRAINT recheche_contactinfos_contactinfo_id_fkey FOREIGN KEY (contactinfo_id) REFERENCES recherches(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recheche_contactinfos_recherche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recheche_contactinfos
ADD CONSTRAINT recheche_contactinfos_recherche_id_fkey FOREIGN KEY (recherche_id) REFERENCES recherches(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_addressbooks_addressbook_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_addressbooks
ADD CONSTRAINT recherches_addressbooks_addressbook_id_fkey FOREIGN KEY (addressbook_id) REFERENCES addressbooks(id);


--
-- Name: recherches_addressbooks_recherche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_addressbooks
ADD CONSTRAINT recherches_addressbooks_recherche_id_fkey FOREIGN KEY (recherche_id) REFERENCES recherches(id);


--
-- Name: recherches_affaires_affaire_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_affaires
ADD CONSTRAINT recherches_affaires_affaire_id_fkey FOREIGN KEY (affaire_id) REFERENCES affaires(id);


--
-- Name: recherches_circuits_circuits_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_circuits
ADD CONSTRAINT recherches_circuits_circuits_fkey FOREIGN KEY (circuit_id) REFERENCES wkf_circuits(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_circuits_recherches_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_circuits
ADD CONSTRAINT recherches_circuits_recherches_fkey FOREIGN KEY (recherche_id) REFERENCES recherches(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_contacts_contact_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_contacts
ADD CONSTRAINT recherches_contacts_contact_id_fkey FOREIGN KEY (contact_id) REFERENCES contacts(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_contacts_recherche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_contacts
ADD CONSTRAINT recherches_contacts_recherche_id_fkey FOREIGN KEY (recherche_id) REFERENCES recherches(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_desktops_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_desktops
ADD CONSTRAINT recherches_desktops_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES desktops(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_desktops_recherche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_desktops
ADD CONSTRAINT recherches_desktops_recherche_id_fkey FOREIGN KEY (recherche_id) REFERENCES recherches(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_dossiers_dossier_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_dossiers
ADD CONSTRAINT recherches_dossiers_dossier_id_fkey FOREIGN KEY (dossier_id) REFERENCES dossiers(id);


--
-- Name: recherches_dossiers_recherche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_dossiers
ADD CONSTRAINT recherches_dossiers_recherche_id_fkey FOREIGN KEY (recherche_id) REFERENCES recherches(id);


--
-- Name: recherches_metadonnees_metadonnees_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_metadonnees
ADD CONSTRAINT recherches_metadonnees_metadonnees_fkey FOREIGN KEY (metadonnee_id) REFERENCES metadonnees(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_metadonnees_recherches_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_metadonnees
ADD CONSTRAINT recherches_metadonnees_recherches_fkey FOREIGN KEY (recherche_id) REFERENCES recherches(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_organismes_organisme_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_organismes
ADD CONSTRAINT recherches_organismes_organisme_id_fkey FOREIGN KEY (organisme_id) REFERENCES organismes(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_organismes_recherche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_organismes
ADD CONSTRAINT recherches_organismes_recherche_id_fkey FOREIGN KEY (recherche_id) REFERENCES recherches(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_selectvaluesmetadonnee_selectvaluemetadonnee_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_selectvaluesmetadonnees
ADD CONSTRAINT recherches_selectvaluesmetadonnee_selectvaluemetadonnee_id_fkey FOREIGN KEY (selectvaluemetadonnee_id) REFERENCES selectvaluesmetadonnees(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_selectvaluesmetadonnees_metadonnee_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_selectvaluesmetadonnees
ADD CONSTRAINT recherches_selectvaluesmetadonnees_metadonnee_id_fkey FOREIGN KEY (metadonnee_id) REFERENCES metadonnees(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_selectvaluesmetadonnees_recherche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_selectvaluesmetadonnees
ADD CONSTRAINT recherches_selectvaluesmetadonnees_recherche_id_fkey FOREIGN KEY (recherche_id) REFERENCES recherches(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_soustypes_recherche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_soustypes
ADD CONSTRAINT recherches_soustypes_recherche_id_fkey FOREIGN KEY (recherche_id) REFERENCES recherches(id);


--
-- Name: recherches_soustypes_soustype_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_soustypes
ADD CONSTRAINT recherches_soustypes_soustype_id_fkey FOREIGN KEY (soustype_id) REFERENCES soustypes(id);


--
-- Name: recherches_taches_recherche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_taches
ADD CONSTRAINT recherches_taches_recherche_id_fkey FOREIGN KEY (recherche_id) REFERENCES recherches(id);


--
-- Name: recherches_taches_tache_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_taches
ADD CONSTRAINT recherches_taches_tache_id_fkey FOREIGN KEY (tache_id) REFERENCES taches(id);


--
-- Name: recherches_types_recherche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_types
ADD CONSTRAINT recherches_types_recherche_id_fkey FOREIGN KEY (recherche_id) REFERENCES recherches(id);


--
-- Name: recherches_types_type_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches_types
ADD CONSTRAINT recherches_types_type_id_fkey FOREIGN KEY (type_id) REFERENCES types(id);


--
-- Name: recherches_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY recherches
ADD CONSTRAINT recherches_users_fkey FOREIGN KEY (user_id) REFERENCES users(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: referentielsfantoircom_referentielfantoir_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY referentielsfantoircom
ADD CONSTRAINT referentielsfantoircom_referentielfantoir_id_fkey FOREIGN KEY (referentielfantoir_id) REFERENCES referentielsfantoir(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: referentielsfantoircom_referentielfantoirdir_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY referentielsfantoircom
ADD CONSTRAINT referentielsfantoircom_referentielfantoirdir_id_fkey FOREIGN KEY (referentielfantoirdir_id) REFERENCES referentielsfantoirdir(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: referentielsfantoirdir_referentielfantoir_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY referentielsfantoirdir
ADD CONSTRAINT referentielsfantoirdir_referentielfantoir_id_fkey FOREIGN KEY (referentielfantoir_id) REFERENCES referentielsfantoir(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: referentielsfantoirvoie_referentielfantoir_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY referentielsfantoirvoie
ADD CONSTRAINT referentielsfantoirvoie_referentielfantoir_id_fkey FOREIGN KEY (referentielfantoir_id) REFERENCES referentielsfantoir(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: referentielsfantoirvoie_referentielfantoircom_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY referentielsfantoirvoie
ADD CONSTRAINT referentielsfantoirvoie_referentielfantoircom_id_fkey FOREIGN KEY (referentielfantoircom_id) REFERENCES referentielsfantoircom(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: relements_courrier_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY relements
ADD CONSTRAINT relements_courrier_id_fkey FOREIGN KEY (courrier_id) REFERENCES courriers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: repertoires_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY repertoires
ADD CONSTRAINT repertoires_users_fkey FOREIGN KEY (user_id) REFERENCES users(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: rmodelgabarits_soustype_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY rmodelgabarits
ADD CONSTRAINT rmodelgabarits_soustype_id_fkey FOREIGN KEY (soustype_id) REFERENCES soustypes(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: rmodels_soustype_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY rmodels
ADD CONSTRAINT rmodels_soustype_id_fkey FOREIGN KEY (soustype_id) REFERENCES soustypes(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: scanemails_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY scanemails
ADD CONSTRAINT scanemails_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES desktops(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: scanemails_desktopmanager_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY scanemails
ADD CONSTRAINT scanemails_desktopmanager_id_fkey FOREIGN KEY (desktopmanager_id) REFERENCES desktopsmanagers(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: scanemails_soustype_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY scanemails
ADD CONSTRAINT scanemails_soustype_id_fkey FOREIGN KEY (soustype_id) REFERENCES soustypes(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: scanemails_type_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY scanemails
ADD CONSTRAINT scanemails_type_id_fkey FOREIGN KEY (type_id) REFERENCES types(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: selectvaluesmetadonnees_metadonnee_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY selectvaluesmetadonnees
ADD CONSTRAINT selectvaluesmetadonnees_metadonnee_id_fkey FOREIGN KEY (metadonnee_id) REFERENCES metadonnees(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: sendsmails_courrier_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY sendsmails
ADD CONSTRAINT sendsmails_courrier_id_fkey FOREIGN KEY (courrier_id) REFERENCES courriers(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: sendsmails_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY sendsmails
ADD CONSTRAINT sendsmails_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES desktops(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: sendsmails_document_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY sendsmails
ADD CONSTRAINT sendsmails_document_id_fkey FOREIGN KEY (document_id) REFERENCES documents(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: sendsmails_templatemail_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY sendsmails
ADD CONSTRAINT sendsmails_templatemail_id_fkey FOREIGN KEY (templatemail_id) REFERENCES templatemails(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: sousactivites_activite_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY sousactivites
ADD CONSTRAINT sousactivites_activite_id_fkey FOREIGN KEY (activite_id) REFERENCES activites(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: sousoperations_operation_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY sousoperations
ADD CONSTRAINT sousoperations_operation_id_fkey FOREIGN KEY (operation_id) REFERENCES operations(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: sousoperations_soustype_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY sousoperations
ADD CONSTRAINT sousoperations_soustype_id_fkey FOREIGN KEY (soustype_id) REFERENCES soustypes(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: soustypes_compteur_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY soustypes
ADD CONSTRAINT soustypes_compteur_id_fkey FOREIGN KEY (compteur_id) REFERENCES compteurs(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: soustypes_types_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY soustypes
ADD CONSTRAINT soustypes_types_fkey FOREIGN KEY (type_id) REFERENCES types(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: typenotifs_users_typenotif_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY typenotifs_users
ADD CONSTRAINT typenotifs_users_typenotif_id_fkey FOREIGN KEY (typenotif_id) REFERENCES typenotifs(id);


--
-- Name: typenotifs_users_user_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY typenotifs_users
ADD CONSTRAINT typenotifs_users_user_id_fkey FOREIGN KEY (user_id) REFERENCES users(id);


--
-- Name: users_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY users
ADD CONSTRAINT users_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES desktops(id) ON UPDATE CASCADE;


--
-- Name: users_wkf_circuits_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY wkf_circuits
ADD CONSTRAINT users_wkf_circuits_fkey FOREIGN KEY (created_user_id) REFERENCES users(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: users_wkf_circuits_fkey2; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY wkf_circuits
ADD CONSTRAINT users_wkf_circuits_fkey2 FOREIGN KEY (modified_user_id) REFERENCES users(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: wkf_circuits_wkf_compositions_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY wkf_etapes
ADD CONSTRAINT wkf_circuits_wkf_compositions_fkey FOREIGN KEY (circuit_id) REFERENCES wkf_circuits(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: wkf_compositions_etape_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY wkf_compositions
ADD CONSTRAINT wkf_compositions_etape_id_fkey FOREIGN KEY (etape_id) REFERENCES wkf_etapes(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: wkf_signatures_visa_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY wkf_signatures
ADD CONSTRAINT wkf_signatures_visa_id_fkey FOREIGN KEY (visa_id) REFERENCES wkf_visas(id);


--
-- Name: wkf_traitements_circuit_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY wkf_traitements
ADD CONSTRAINT wkf_traitements_circuit_id_fkey FOREIGN KEY (circuit_id) REFERENCES wkf_circuits(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: wkf_traitements_wkf_visas_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY wkf_visas
ADD CONSTRAINT wkf_traitements_wkf_visas_fkey FOREIGN KEY (traitement_id) REFERENCES wkf_traitements(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: wkf_visas_etape_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY wkf_visas
ADD CONSTRAINT wkf_visas_etape_id_fkey FOREIGN KEY (etape_id) REFERENCES wkf_etapes(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: public; Type: ACL; Schema: -; Owner: postgres
--

REVOKE ALL ON SCHEMA public FROM PUBLIC;
REVOKE ALL ON SCHEMA public FROM postgres;
GRANT ALL ON SCHEMA public TO postgres;
GRANT ALL ON SCHEMA public TO PUBLIC;


--
-- PostgreSQL database dump complete
--

-- DATAs

--
-- Data for Name: acos; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO acos VALUES (893, 2, NULL, NULL, 'isExistName', 39, 40);
INSERT INTO acos VALUES (2, 1, NULL, NULL, 'Addressbooks', 2, 43);
INSERT INTO acos VALUES (3, 2, NULL, NULL, 'index', 3, 4);
INSERT INTO acos VALUES (4, 2, NULL, NULL, 'get_addressbooks', 5, 6);
INSERT INTO acos VALUES (5, 2, NULL, NULL, 'add', 7, 8);
INSERT INTO acos VALUES (6, 2, NULL, NULL, 'edit', 9, 10);
INSERT INTO acos VALUES (7, 2, NULL, NULL, 'delete', 11, 12);
INSERT INTO acos VALUES (8, 2, NULL, NULL, 'userAclCheck', 13, 14);
INSERT INTO acos VALUES (9, 2, NULL, NULL, 'getSessionInitServices', 15, 16);
INSERT INTO acos VALUES (10, 2, NULL, NULL, 'getSessionAppDefinedServices', 17, 18);
INSERT INTO acos VALUES (11, 2, NULL, NULL, 'getSessionAppDefinedDesktops', 19, 20);
INSERT INTO acos VALUES (12, 2, NULL, NULL, 'getSessionInitDesktops', 21, 22);
INSERT INTO acos VALUES (13, 2, NULL, NULL, 'getSessionValDesktops', 23, 24);
INSERT INTO acos VALUES (14, 2, NULL, NULL, 'getSessionValeditDesktops', 25, 26);
INSERT INTO acos VALUES (15, 2, NULL, NULL, 'getSessionDispDesktops', 27, 28);
INSERT INTO acos VALUES (16, 2, NULL, NULL, 'getSessionArchDesktops', 29, 30);
INSERT INTO acos VALUES (17, 2, NULL, NULL, 'getSessionAllDesktops', 31, 32);
INSERT INTO acos VALUES (18, 2, NULL, NULL, 'getEnv', 33, 34);
INSERT INTO acos VALUES (19, 2, NULL, NULL, 'getAddParams', 35, 36);
INSERT INTO acos VALUES (20, 2, NULL, NULL, 'ldebug', 37, 38);
INSERT INTO acos VALUES (21, 1, NULL, NULL, 'Affaires', 44, 79);
INSERT INTO acos VALUES (22, 21, NULL, NULL, 'add', 45, 46);
INSERT INTO acos VALUES (23, 21, NULL, NULL, 'edit', 47, 48);
INSERT INTO acos VALUES (24, 21, NULL, NULL, 'delete', 49, 50);
INSERT INTO acos VALUES (25, 21, NULL, NULL, 'userAclCheck', 51, 52);
INSERT INTO acos VALUES (26, 21, NULL, NULL, 'getSessionInitServices', 53, 54);
INSERT INTO acos VALUES (27, 21, NULL, NULL, 'getSessionAppDefinedServices', 55, 56);
INSERT INTO acos VALUES (28, 21, NULL, NULL, 'getSessionAppDefinedDesktops', 57, 58);
INSERT INTO acos VALUES (29, 21, NULL, NULL, 'getSessionInitDesktops', 59, 60);
INSERT INTO acos VALUES (30, 21, NULL, NULL, 'getSessionValDesktops', 61, 62);
INSERT INTO acos VALUES (31, 21, NULL, NULL, 'getSessionValeditDesktops', 63, 64);
INSERT INTO acos VALUES (32, 21, NULL, NULL, 'getSessionDispDesktops', 65, 66);
INSERT INTO acos VALUES (33, 21, NULL, NULL, 'getSessionArchDesktops', 67, 68);
INSERT INTO acos VALUES (34, 21, NULL, NULL, 'getSessionAllDesktops', 69, 70);
INSERT INTO acos VALUES (35, 21, NULL, NULL, 'getEnv', 71, 72);
INSERT INTO acos VALUES (36, 21, NULL, NULL, 'getAddParams', 73, 74);
INSERT INTO acos VALUES (37, 21, NULL, NULL, 'ldebug', 75, 76);
INSERT INTO acos VALUES (38, 1, NULL, NULL, 'Allos', 80, 111);
INSERT INTO acos VALUES (39, 38, NULL, NULL, 'app_version', 81, 82);
INSERT INTO acos VALUES (40, 38, NULL, NULL, 'version', 83, 84);
INSERT INTO acos VALUES (41, 38, NULL, NULL, 'userAclCheck', 85, 86);
INSERT INTO acos VALUES (42, 38, NULL, NULL, 'getSessionInitServices', 87, 88);
INSERT INTO acos VALUES (43, 38, NULL, NULL, 'getSessionAppDefinedServices', 89, 90);
INSERT INTO acos VALUES (44, 38, NULL, NULL, 'getSessionAppDefinedDesktops', 91, 92);
INSERT INTO acos VALUES (53, 38, NULL, NULL, 'ldebug', 109, 110);
INSERT INTO acos VALUES (54, 1, NULL, NULL, 'Armodels', 112, 147);
INSERT INTO acos VALUES (55, 54, NULL, NULL, 'add', 113, 114);
INSERT INTO acos VALUES (56, 54, NULL, NULL, 'delete', 115, 116);
INSERT INTO acos VALUES (57, 54, NULL, NULL, 'download', 117, 118);
INSERT INTO acos VALUES (58, 54, NULL, NULL, 'getPreview', 119, 120);
INSERT INTO acos VALUES (59, 54, NULL, NULL, 'userAclCheck', 121, 122);
INSERT INTO acos VALUES (60, 54, NULL, NULL, 'getSessionInitServices', 123, 124);
INSERT INTO acos VALUES (61, 54, NULL, NULL, 'getSessionAppDefinedServices', 125, 126);
INSERT INTO acos VALUES (62, 54, NULL, NULL, 'getSessionAppDefinedDesktops', 127, 128);
INSERT INTO acos VALUES (63, 54, NULL, NULL, 'getSessionInitDesktops', 129, 130);
INSERT INTO acos VALUES (64, 54, NULL, NULL, 'getSessionValDesktops', 131, 132);
INSERT INTO acos VALUES (65, 54, NULL, NULL, 'getSessionValeditDesktops', 133, 134);
INSERT INTO acos VALUES (66, 54, NULL, NULL, 'getSessionDispDesktops', 135, 136);
INSERT INTO acos VALUES (67, 54, NULL, NULL, 'getSessionArchDesktops', 137, 138);
INSERT INTO acos VALUES (68, 54, NULL, NULL, 'getSessionAllDesktops', 139, 140);
INSERT INTO acos VALUES (69, 54, NULL, NULL, 'getEnv', 141, 142);
INSERT INTO acos VALUES (70, 54, NULL, NULL, 'getAddParams', 143, 144);
INSERT INTO acos VALUES (87, 72, NULL, NULL, 'getEnv', 177, 178);
INSERT INTO acos VALUES (98, 90, NULL, NULL, 'getSessionValeditDesktops', 199, 200);
INSERT INTO acos VALUES (99, 90, NULL, NULL, 'getSessionDispDesktops', 201, 202);
INSERT INTO acos VALUES (100, 90, NULL, NULL, 'getSessionArchDesktops', 203, 204);
INSERT INTO acos VALUES (101, 90, NULL, NULL, 'getSessionAllDesktops', 205, 206);
INSERT INTO acos VALUES (102, 90, NULL, NULL, 'getEnv', 207, 208);
INSERT INTO acos VALUES (103, 90, NULL, NULL, 'getAddParams', 209, 210);
INSERT INTO acos VALUES (104, 90, NULL, NULL, 'ldebug', 211, 212);
INSERT INTO acos VALUES (435, 423, NULL, NULL, 'getAddParams', 941, 942);
INSERT INTO acos VALUES (1074, 205, NULL, NULL, 'get_contactform', 485, 486);
INSERT INTO acos VALUES (894, 2, NULL, NULL, 'removeRepeat', 41, 42);
INSERT INTO acos VALUES (79, 72, NULL, NULL, 'getSessionAppDefinedServices', 161, 162);
INSERT INTO acos VALUES (80, 72, NULL, NULL, 'getSessionAppDefinedDesktops', 163, 164);
INSERT INTO acos VALUES (81, 72, NULL, NULL, 'getSessionInitDesktops', 165, 166);
INSERT INTO acos VALUES (82, 72, NULL, NULL, 'getSessionValDesktops', 167, 168);
INSERT INTO acos VALUES (83, 72, NULL, NULL, 'getSessionValeditDesktops', 169, 170);
INSERT INTO acos VALUES (84, 72, NULL, NULL, 'getSessionDispDesktops', 171, 172);
INSERT INTO acos VALUES (85, 72, NULL, NULL, 'getSessionArchDesktops', 173, 174);
INSERT INTO acos VALUES (86, 72, NULL, NULL, 'getSessionAllDesktops', 175, 176);
INSERT INTO acos VALUES (88, 72, NULL, NULL, 'getAddParams', 179, 180);
INSERT INTO acos VALUES (89, 72, NULL, NULL, 'ldebug', 181, 182);
INSERT INTO acos VALUES (91, 90, NULL, NULL, 'index', 185, 186);
INSERT INTO acos VALUES (92, 90, NULL, NULL, 'userAclCheck', 187, 188);
INSERT INTO acos VALUES (45, 38, NULL, NULL, 'getSessionInitDesktops', 93, 94);
INSERT INTO acos VALUES (46, 38, NULL, NULL, 'getSessionValDesktops', 95, 96);
INSERT INTO acos VALUES (47, 38, NULL, NULL, 'getSessionValeditDesktops', 97, 98);
INSERT INTO acos VALUES (48, 38, NULL, NULL, 'getSessionDispDesktops', 99, 100);
INSERT INTO acos VALUES (49, 38, NULL, NULL, 'getSessionArchDesktops', 101, 102);
INSERT INTO acos VALUES (50, 38, NULL, NULL, 'getSessionAllDesktops', 103, 104);
INSERT INTO acos VALUES (51, 38, NULL, NULL, 'getEnv', 105, 106);
INSERT INTO acos VALUES (52, 38, NULL, NULL, 'getAddParams', 107, 108);
INSERT INTO acos VALUES (93, 90, NULL, NULL, 'getSessionInitServices', 189, 190);
INSERT INTO acos VALUES (94, 90, NULL, NULL, 'getSessionAppDefinedServices', 191, 192);
INSERT INTO acos VALUES (95, 90, NULL, NULL, 'getSessionAppDefinedDesktops', 193, 194);
INSERT INTO acos VALUES (96, 90, NULL, NULL, 'getSessionInitDesktops', 195, 196);
INSERT INTO acos VALUES (97, 90, NULL, NULL, 'getSessionValDesktops', 197, 198);
INSERT INTO acos VALUES (71, 54, NULL, NULL, 'ldebug', 145, 146);
INSERT INTO acos VALUES (72, 1, NULL, NULL, 'Ars', 148, 183);
INSERT INTO acos VALUES (73, 72, NULL, NULL, 'select', 149, 150);
INSERT INTO acos VALUES (74, 72, NULL, NULL, 'delete', 151, 152);
INSERT INTO acos VALUES (75, 72, NULL, NULL, 'download', 153, 154);
INSERT INTO acos VALUES (76, 72, NULL, NULL, 'getPreview', 155, 156);
INSERT INTO acos VALUES (77, 72, NULL, NULL, 'userAclCheck', 157, 158);
INSERT INTO acos VALUES (78, 72, NULL, NULL, 'getSessionInitServices', 159, 160);
INSERT INTO acos VALUES (895, 21, NULL, NULL, 'getComment', 77, 78);
INSERT INTO acos VALUES (90, 1, NULL, NULL, 'Checks', 184, 219);
INSERT INTO acos VALUES (984, 90, NULL, NULL, 'readTimeout', 213, 214);
INSERT INTO acos VALUES (1066, 205, NULL, NULL, 'searchOrganismeinfo', 483, 484);
INSERT INTO acos VALUES (159, 147, NULL, NULL, 'getAddParams', 327, 328);
INSERT INTO acos VALUES (160, 147, NULL, NULL, 'ldebug', 329, 330);
INSERT INTO acos VALUES (1076, 1075, NULL, NULL, 'index', 2141, 2142);
INSERT INTO acos VALUES (1077, 1075, NULL, NULL, 'getCodePostal', 2143, 2144);
INSERT INTO acos VALUES (162, 161, NULL, NULL, 'index', 333, 334);
INSERT INTO acos VALUES (163, 161, NULL, NULL, 'getConnecteurs', 335, 336);
INSERT INTO acos VALUES (164, 161, NULL, NULL, 'edit', 337, 338);
INSERT INTO acos VALUES (165, 161, NULL, NULL, 'makeconf', 339, 340);
INSERT INTO acos VALUES (166, 161, NULL, NULL, 'getCircuitsParapheur', 341, 342);
INSERT INTO acos VALUES (167, 161, NULL, NULL, 'check', 343, 344);
INSERT INTO acos VALUES (168, 161, NULL, NULL, 'getRepositoryInfo', 345, 346);
INSERT INTO acos VALUES (169, 161, NULL, NULL, 'getRepositoryRetrieveInfo', 347, 348);
INSERT INTO acos VALUES (170, 161, NULL, NULL, 'getRootFolder', 349, 350);
INSERT INTO acos VALUES (171, 161, NULL, NULL, 'testObject', 351, 352);
INSERT INTO acos VALUES (172, 161, NULL, NULL, 'getObjectByPath', 353, 354);
INSERT INTO acos VALUES (184, 161, NULL, NULL, 'getEnv', 377, 378);
INSERT INTO acos VALUES (1089, 1075, NULL, NULL, 'getAddParams', 2167, 2168);
INSERT INTO acos VALUES (985, 90, NULL, NULL, 'sec2hms', 215, 216);
INSERT INTO acos VALUES (1075, 1, NULL, NULL, 'Bansadresses', 2140, 2171);
INSERT INTO acos VALUES (1090, 1075, NULL, NULL, 'ldebug', 2169, 2170);
INSERT INTO acos VALUES (374, 368, NULL, NULL, 'setDroitsFunc', 819, 820);
INSERT INTO acos VALUES (451, 437, NULL, NULL, 'getSessionArchDesktops', 973, 974);
INSERT INTO acos VALUES (452, 437, NULL, NULL, 'getSessionAllDesktops', 975, 976);
INSERT INTO acos VALUES (494, 476, NULL, NULL, 'ldebug', 1059, 1060);
INSERT INTO acos VALUES (154, 147, NULL, NULL, 'getSessionValeditDesktops', 317, 318);
INSERT INTO acos VALUES (155, 147, NULL, NULL, 'getSessionDispDesktops', 319, 320);
INSERT INTO acos VALUES (156, 147, NULL, NULL, 'getSessionArchDesktops', 321, 322);
INSERT INTO acos VALUES (157, 147, NULL, NULL, 'getSessionAllDesktops', 323, 324);
INSERT INTO acos VALUES (158, 147, NULL, NULL, 'getEnv', 325, 326);
INSERT INTO acos VALUES (639, 636, NULL, NULL, 'delete', 1361, 1362);
INSERT INTO acos VALUES (640, 636, NULL, NULL, 'checkTache', 1363, 1364);
INSERT INTO acos VALUES (641, 636, NULL, NULL, 'grabTache', 1365, 1366);
INSERT INTO acos VALUES (642, 636, NULL, NULL, 'releaseTache', 1367, 1368);
INSERT INTO acos VALUES (643, 636, NULL, NULL, 'getTaches', 1369, 1370);
INSERT INTO acos VALUES (644, 636, NULL, NULL, 'setFromTask', 1371, 1372);
INSERT INTO acos VALUES (645, 636, NULL, NULL, 'userAclCheck', 1373, 1374);
INSERT INTO acos VALUES (646, 636, NULL, NULL, 'getSessionInitServices', 1375, 1376);
INSERT INTO acos VALUES (647, 636, NULL, NULL, 'getSessionAppDefinedServices', 1377, 1378);
INSERT INTO acos VALUES (673, 658, NULL, NULL, 'getAddParams', 1429, 1430);
INSERT INTO acos VALUES (676, 675, NULL, NULL, 'index', 1435, 1436);
INSERT INTO acos VALUES (677, 675, NULL, NULL, 'getUsers', 1437, 1438);
INSERT INTO acos VALUES (678, 675, NULL, NULL, 'add', 1439, 1440);
INSERT INTO acos VALUES (679, 675, NULL, NULL, 'ajaxformvalid', 1441, 1442);
INSERT INTO acos VALUES (680, 675, NULL, NULL, 'edit', 1443, 1444);
INSERT INTO acos VALUES (681, 675, NULL, NULL, 'delete', 1445, 1446);
INSERT INTO acos VALUES (682, 675, NULL, NULL, 'setInfosPersos', 1447, 1448);
INSERT INTO acos VALUES (683, 675, NULL, NULL, 'getInfosCompte', 1449, 1450);
INSERT INTO acos VALUES (693, 675, NULL, NULL, 'userAclCheck', 1469, 1470);
INSERT INTO acos VALUES (694, 675, NULL, NULL, 'getSessionInitServices', 1471, 1472);
INSERT INTO acos VALUES (695, 675, NULL, NULL, 'getSessionAppDefinedServices', 1473, 1474);
INSERT INTO acos VALUES (696, 675, NULL, NULL, 'getSessionAppDefinedDesktops', 1475, 1476);
INSERT INTO acos VALUES (697, 675, NULL, NULL, 'getSessionInitDesktops', 1477, 1478);
INSERT INTO acos VALUES (698, 675, NULL, NULL, 'getSessionValDesktops', 1479, 1480);
INSERT INTO acos VALUES (699, 675, NULL, NULL, 'getSessionValeditDesktops', 1481, 1482);
INSERT INTO acos VALUES (700, 675, NULL, NULL, 'getSessionDispDesktops', 1483, 1484);
INSERT INTO acos VALUES (701, 675, NULL, NULL, 'getSessionArchDesktops', 1485, 1486);
INSERT INTO acos VALUES (702, 675, NULL, NULL, 'getSessionAllDesktops', 1487, 1488);
INSERT INTO acos VALUES (703, 675, NULL, NULL, 'getEnv', 1489, 1490);
INSERT INTO acos VALUES (704, 675, NULL, NULL, 'getAddParams', 1491, 1492);
INSERT INTO acos VALUES (705, 675, NULL, NULL, 'ldebug', 1493, 1494);
INSERT INTO acos VALUES (721, 706, NULL, NULL, 'getSessionValDesktops', 1529, 1530);
INSERT INTO acos VALUES (722, 706, NULL, NULL, 'getSessionValeditDesktops', 1531, 1532);
INSERT INTO acos VALUES (105, 1, NULL, NULL, 'Collectivites', 220, 269);
INSERT INTO acos VALUES (106, 105, NULL, NULL, 'index', 221, 222);
INSERT INTO acos VALUES (107, 105, NULL, NULL, 'getCollectivites', 223, 224);
INSERT INTO acos VALUES (108, 105, NULL, NULL, 'ajaxformvalid', 225, 226);
INSERT INTO acos VALUES (109, 105, NULL, NULL, 'getConnList', 227, 228);
INSERT INTO acos VALUES (110, 105, NULL, NULL, 'add', 229, 230);
INSERT INTO acos VALUES (111, 105, NULL, NULL, 'edit', 231, 232);
INSERT INTO acos VALUES (112, 105, NULL, NULL, 'delete', 233, 234);
INSERT INTO acos VALUES (113, 105, NULL, NULL, 'upload', 235, 236);
INSERT INTO acos VALUES (114, 105, NULL, NULL, 'getLogo', 237, 238);
INSERT INTO acos VALUES (115, 105, NULL, NULL, 'admin', 239, 240);
INSERT INTO acos VALUES (116, 105, NULL, NULL, 'deleteLogo', 241, 242);
INSERT INTO acos VALUES (117, 105, NULL, NULL, 'userAclCheck', 243, 244);
INSERT INTO acos VALUES (118, 105, NULL, NULL, 'getSessionInitServices', 245, 246);
INSERT INTO acos VALUES (119, 105, NULL, NULL, 'getSessionAppDefinedServices', 247, 248);
INSERT INTO acos VALUES (120, 105, NULL, NULL, 'getSessionAppDefinedDesktops', 249, 250);
INSERT INTO acos VALUES (121, 105, NULL, NULL, 'getSessionInitDesktops', 251, 252);
INSERT INTO acos VALUES (122, 105, NULL, NULL, 'getSessionValDesktops', 253, 254);
INSERT INTO acos VALUES (123, 105, NULL, NULL, 'getSessionValeditDesktops', 255, 256);
INSERT INTO acos VALUES (161, 1, NULL, NULL, 'Connecteurs', 332, 385);
INSERT INTO acos VALUES (202, 187, NULL, NULL, 'getEnv', 415, 416);
INSERT INTO acos VALUES (124, 105, NULL, NULL, 'getSessionDispDesktops', 257, 258);
INSERT INTO acos VALUES (125, 105, NULL, NULL, 'getSessionArchDesktops', 259, 260);
INSERT INTO acos VALUES (126, 105, NULL, NULL, 'getSessionAllDesktops', 261, 262);
INSERT INTO acos VALUES (127, 105, NULL, NULL, 'getEnv', 263, 264);
INSERT INTO acos VALUES (128, 105, NULL, NULL, 'getAddParams', 265, 266);
INSERT INTO acos VALUES (129, 105, NULL, NULL, 'ldebug', 267, 268);
INSERT INTO acos VALUES (130, 1, NULL, NULL, 'Comments', 270, 303);
INSERT INTO acos VALUES (131, 130, NULL, NULL, 'add', 271, 272);
INSERT INTO acos VALUES (132, 130, NULL, NULL, 'edit', 273, 274);
INSERT INTO acos VALUES (133, 130, NULL, NULL, 'delete', 275, 276);
INSERT INTO acos VALUES (134, 130, NULL, NULL, 'userAclCheck', 277, 278);
INSERT INTO acos VALUES (135, 130, NULL, NULL, 'getSessionInitServices', 279, 280);
INSERT INTO acos VALUES (136, 130, NULL, NULL, 'getSessionAppDefinedServices', 281, 282);
INSERT INTO acos VALUES (144, 130, NULL, NULL, 'getEnv', 297, 298);
INSERT INTO acos VALUES (986, 90, NULL, NULL, 'timeout', 217, 218);
INSERT INTO acos VALUES (1078, 1075, NULL, NULL, 'userAclCheck', 2145, 2146);
INSERT INTO acos VALUES (1079, 1075, NULL, NULL, 'getSessionInitServices', 2147, 2148);
INSERT INTO acos VALUES (1080, 1075, NULL, NULL, 'getSessionAppDefinedServices', 2149, 2150);
INSERT INTO acos VALUES (1081, 1075, NULL, NULL, 'getSessionAppDefinedDesktops', 2151, 2152);
INSERT INTO acos VALUES (1082, 1075, NULL, NULL, 'getSessionInitDesktops', 2153, 2154);
INSERT INTO acos VALUES (1083, 1075, NULL, NULL, 'getSessionValDesktops', 2155, 2156);
INSERT INTO acos VALUES (1084, 1075, NULL, NULL, 'getSessionValeditDesktops', 2157, 2158);
INSERT INTO acos VALUES (1085, 1075, NULL, NULL, 'getSessionDispDesktops', 2159, 2160);
INSERT INTO acos VALUES (1086, 1075, NULL, NULL, 'getSessionArchDesktops', 2161, 2162);
INSERT INTO acos VALUES (1087, 1075, NULL, NULL, 'getSessionAllDesktops', 2163, 2164);
INSERT INTO acos VALUES (1088, 1075, NULL, NULL, 'getEnv', 2165, 2166);
INSERT INTO acos VALUES (1091, 1045, NULL, NULL, 'getHomonymsOrgs', 2133, 2134);
INSERT INTO acos VALUES (440, 437, NULL, NULL, 'add', 951, 952);
INSERT INTO acos VALUES (553, 537, NULL, NULL, 'getSessionAllDesktops', 1175, 1176);
INSERT INTO acos VALUES (674, 658, NULL, NULL, 'ldebug', 1431, 1432);
INSERT INTO acos VALUES (723, 706, NULL, NULL, 'getSessionDispDesktops', 1533, 1534);
INSERT INTO acos VALUES (724, 706, NULL, NULL, 'getSessionArchDesktops', 1535, 1536);
INSERT INTO acos VALUES (725, 706, NULL, NULL, 'getSessionAllDesktops', 1537, 1538);
INSERT INTO acos VALUES (726, 706, NULL, NULL, 'getEnv', 1539, 1540);
INSERT INTO acos VALUES (727, 706, NULL, NULL, 'getAddParams', 1541, 1542);
INSERT INTO acos VALUES (728, 706, NULL, NULL, 'ldebug', 1543, 1544);
INSERT INTO acos VALUES (729, 1, NULL, NULL, 'Workflows', 1546, 1593);
INSERT INTO acos VALUES (730, 729, NULL, NULL, 'index', 1547, 1548);
INSERT INTO acos VALUES (731, 729, NULL, NULL, 'getWorkflows', 1549, 1550);
INSERT INTO acos VALUES (732, 729, NULL, NULL, 'add', 1551, 1552);
INSERT INTO acos VALUES (733, 729, NULL, NULL, 'edit', 1553, 1554);
INSERT INTO acos VALUES (734, 729, NULL, NULL, 'setSchema', 1555, 1556);
INSERT INTO acos VALUES (735, 729, NULL, NULL, 'delete', 1557, 1558);
INSERT INTO acos VALUES (736, 729, NULL, NULL, 'moveEtape', 1559, 1560);
INSERT INTO acos VALUES (737, 729, NULL, NULL, 'copieCircuit', 1561, 1562);
INSERT INTO acos VALUES (738, 729, NULL, NULL, 'userEnough', 1563, 1564);
INSERT INTO acos VALUES (739, 729, NULL, NULL, 'userAclCheck', 1565, 1566);
INSERT INTO acos VALUES (740, 729, NULL, NULL, 'getSessionInitServices', 1567, 1568);
INSERT INTO acos VALUES (808, 805, NULL, NULL, 'add', 1704, 1705);
INSERT INTO acos VALUES (809, 805, NULL, NULL, 'edit', 1706, 1707);
INSERT INTO acos VALUES (810, 805, NULL, NULL, 'delete', 1708, 1709);
INSERT INTO acos VALUES (811, 805, NULL, NULL, 'moveUp', 1710, 1711);
INSERT INTO acos VALUES (812, 805, NULL, NULL, 'moveDown', 1712, 1713);
INSERT INTO acos VALUES (813, 805, NULL, NULL, 'setCreatedModifiedUser', 1714, 1715);
INSERT INTO acos VALUES (814, 805, NULL, NULL, 'formatUser', 1716, 1717);
INSERT INTO acos VALUES (815, 805, NULL, NULL, 'formatLinkedModel', 1718, 1719);
INSERT INTO acos VALUES (816, 805, NULL, NULL, 'formatLinkedModels', 1720, 1721);
INSERT INTO acos VALUES (817, 805, NULL, NULL, 'listLinkedModel', 1722, 1723);
INSERT INTO acos VALUES (818, 805, NULL, NULL, 'userAclCheck', 1724, 1725);
INSERT INTO acos VALUES (880, 872, NULL, NULL, 'getSessionAppDefinedDesktops', 1848, 1849);
INSERT INTO acos VALUES (882, 872, NULL, NULL, 'getSessionValDesktops', 1852, 1853);
INSERT INTO acos VALUES (883, 872, NULL, NULL, 'getSessionValeditDesktops', 1854, 1855);
INSERT INTO acos VALUES (884, 872, NULL, NULL, 'getSessionDispDesktops', 1856, 1857);
INSERT INTO acos VALUES (885, 872, NULL, NULL, 'getSessionArchDesktops', 1858, 1859);
INSERT INTO acos VALUES (916, 915, NULL, NULL, 'index', 1915, 1916);
INSERT INTO acos VALUES (917, 915, NULL, NULL, 'edit', 1917, 1918);
INSERT INTO acos VALUES (137, 130, NULL, NULL, 'getSessionAppDefinedDesktops', 283, 284);
INSERT INTO acos VALUES (138, 130, NULL, NULL, 'getSessionInitDesktops', 285, 286);
INSERT INTO acos VALUES (139, 130, NULL, NULL, 'getSessionValDesktops', 287, 288);
INSERT INTO acos VALUES (140, 130, NULL, NULL, 'getSessionValeditDesktops', 289, 290);
INSERT INTO acos VALUES (141, 130, NULL, NULL, 'getSessionDispDesktops', 291, 292);
INSERT INTO acos VALUES (142, 130, NULL, NULL, 'getSessionArchDesktops', 293, 294);
INSERT INTO acos VALUES (143, 130, NULL, NULL, 'getSessionAllDesktops', 295, 296);
INSERT INTO acos VALUES (145, 130, NULL, NULL, 'getAddParams', 299, 300);
INSERT INTO acos VALUES (146, 130, NULL, NULL, 'ldebug', 301, 302);
INSERT INTO acos VALUES (147, 1, NULL, NULL, 'Compteurs', 304, 331);
INSERT INTO acos VALUES (148, 147, NULL, NULL, 'userAclCheck', 305, 306);
INSERT INTO acos VALUES (149, 147, NULL, NULL, 'getSessionInitServices', 307, 308);
INSERT INTO acos VALUES (150, 147, NULL, NULL, 'getSessionAppDefinedServices', 309, 310);
INSERT INTO acos VALUES (151, 147, NULL, NULL, 'getSessionAppDefinedDesktops', 311, 312);
INSERT INTO acos VALUES (152, 147, NULL, NULL, 'getSessionInitDesktops', 313, 314);
INSERT INTO acos VALUES (153, 147, NULL, NULL, 'getSessionValDesktops', 315, 316);
INSERT INTO acos VALUES (173, 161, NULL, NULL, 'getFolderRetrieveInfo', 355, 356);
INSERT INTO acos VALUES (174, 161, NULL, NULL, 'userAclCheck', 357, 358);
INSERT INTO acos VALUES (175, 161, NULL, NULL, 'getSessionInitServices', 359, 360);
INSERT INTO acos VALUES (176, 161, NULL, NULL, 'getSessionAppDefinedServices', 361, 362);
INSERT INTO acos VALUES (177, 161, NULL, NULL, 'getSessionAppDefinedDesktops', 363, 364);
INSERT INTO acos VALUES (178, 161, NULL, NULL, 'getSessionInitDesktops', 365, 366);
INSERT INTO acos VALUES (179, 161, NULL, NULL, 'getSessionValDesktops', 367, 368);
INSERT INTO acos VALUES (180, 161, NULL, NULL, 'getSessionValeditDesktops', 369, 370);
INSERT INTO acos VALUES (181, 161, NULL, NULL, 'getSessionDispDesktops', 371, 372);
INSERT INTO acos VALUES (182, 161, NULL, NULL, 'getSessionArchDesktops', 373, 374);
INSERT INTO acos VALUES (183, 161, NULL, NULL, 'getSessionAllDesktops', 375, 376);
INSERT INTO acos VALUES (185, 161, NULL, NULL, 'getAddParams', 379, 380);
INSERT INTO acos VALUES (186, 161, NULL, NULL, 'ldebug', 381, 382);
INSERT INTO acos VALUES (281, 227, NULL, NULL, 'getSessionAllDesktops', 595, 596);
INSERT INTO acos VALUES (282, 227, NULL, NULL, 'getEnv', 597, 598);
INSERT INTO acos VALUES (283, 227, NULL, NULL, 'getAddParams', 599, 600);
INSERT INTO acos VALUES (284, 227, NULL, NULL, 'ldebug', 601, 602);
INSERT INTO acos VALUES (285, 1, NULL, NULL, 'Desktops', 626, 669);
INSERT INTO acos VALUES (355, 344, NULL, NULL, 'userAclCheck', 781, 782);
INSERT INTO acos VALUES (356, 344, NULL, NULL, 'getSessionInitServices', 783, 784);
INSERT INTO acos VALUES (357, 344, NULL, NULL, 'getSessionAppDefinedServices', 785, 786);
INSERT INTO acos VALUES (358, 344, NULL, NULL, 'getSessionAppDefinedDesktops', 787, 788);
INSERT INTO acos VALUES (359, 344, NULL, NULL, 'getSessionInitDesktops', 789, 790);
INSERT INTO acos VALUES (360, 344, NULL, NULL, 'getSessionValDesktops', 791, 792);
INSERT INTO acos VALUES (361, 344, NULL, NULL, 'getSessionValeditDesktops', 793, 794);
INSERT INTO acos VALUES (362, 344, NULL, NULL, 'getSessionDispDesktops', 795, 796);
INSERT INTO acos VALUES (363, 344, NULL, NULL, 'getSessionArchDesktops', 797, 798);
INSERT INTO acos VALUES (364, 344, NULL, NULL, 'getSessionAllDesktops', 799, 800);
INSERT INTO acos VALUES (365, 344, NULL, NULL, 'getEnv', 801, 802);
INSERT INTO acos VALUES (366, 344, NULL, NULL, 'getAddParams', 803, 804);
INSERT INTO acos VALUES (367, 344, NULL, NULL, 'ldebug', 805, 806);
INSERT INTO acos VALUES (368, 1, NULL, NULL, 'Groups', 808, 853);
INSERT INTO acos VALUES (369, 368, NULL, NULL, 'getGroups', 809, 810);
INSERT INTO acos VALUES (370, 368, NULL, NULL, 'view', 811, 812);
INSERT INTO acos VALUES (371, 368, NULL, NULL, 'add', 813, 814);
INSERT INTO acos VALUES (372, 368, NULL, NULL, 'edit', 815, 816);
INSERT INTO acos VALUES (373, 368, NULL, NULL, 'setInfos', 817, 818);
INSERT INTO acos VALUES (375, 368, NULL, NULL, 'getDroitsFunc', 821, 822);
INSERT INTO acos VALUES (376, 368, NULL, NULL, 'delete', 823, 824);
INSERT INTO acos VALUES (377, 368, NULL, NULL, 'ajaxformvalid', 825, 826);
INSERT INTO acos VALUES (378, 368, NULL, NULL, 'userAclCheck', 827, 828);
INSERT INTO acos VALUES (379, 368, NULL, NULL, 'getSessionInitServices', 829, 830);
INSERT INTO acos VALUES (380, 368, NULL, NULL, 'getSessionAppDefinedServices', 831, 832);
INSERT INTO acos VALUES (381, 368, NULL, NULL, 'getSessionAppDefinedDesktops', 833, 834);
INSERT INTO acos VALUES (382, 368, NULL, NULL, 'getSessionInitDesktops', 835, 836);
INSERT INTO acos VALUES (383, 368, NULL, NULL, 'getSessionValDesktops', 837, 838);
INSERT INTO acos VALUES (384, 368, NULL, NULL, 'getSessionValeditDesktops', 839, 840);
INSERT INTO acos VALUES (385, 368, NULL, NULL, 'getSessionDispDesktops', 841, 842);
INSERT INTO acos VALUES (403, 391, NULL, NULL, 'getSessionDispDesktops', 877, 878);
INSERT INTO acos VALUES (409, 1, NULL, NULL, 'Keywords', 890, 917);
INSERT INTO acos VALUES (410, 409, NULL, NULL, 'userAclCheck', 891, 892);
INSERT INTO acos VALUES (431, 423, NULL, NULL, 'getSessionDispDesktops', 933, 934);
INSERT INTO acos VALUES (432, 423, NULL, NULL, 'getSessionArchDesktops', 935, 936);
INSERT INTO acos VALUES (433, 423, NULL, NULL, 'getSessionAllDesktops', 937, 938);
INSERT INTO acos VALUES (436, 423, NULL, NULL, 'ldebug', 943, 944);
INSERT INTO acos VALUES (437, 1, NULL, NULL, 'Metadonnees', 946, 983);
INSERT INTO acos VALUES (438, 437, NULL, NULL, 'index', 947, 948);
INSERT INTO acos VALUES (439, 437, NULL, NULL, 'getMetadonnees', 949, 950);
INSERT INTO acos VALUES (441, 437, NULL, NULL, 'edit', 953, 954);
INSERT INTO acos VALUES (442, 437, NULL, NULL, 'delete', 955, 956);
INSERT INTO acos VALUES (443, 437, NULL, NULL, 'userAclCheck', 957, 958);
INSERT INTO acos VALUES (444, 437, NULL, NULL, 'getSessionInitServices', 959, 960);
INSERT INTO acos VALUES (445, 437, NULL, NULL, 'getSessionAppDefinedServices', 961, 962);
INSERT INTO acos VALUES (446, 437, NULL, NULL, 'getSessionAppDefinedDesktops', 963, 964);
INSERT INTO acos VALUES (447, 437, NULL, NULL, 'getSessionInitDesktops', 965, 966);
INSERT INTO acos VALUES (449, 437, NULL, NULL, 'getSessionValeditDesktops', 969, 970);
INSERT INTO acos VALUES (450, 437, NULL, NULL, 'getSessionDispDesktops', 971, 972);
INSERT INTO acos VALUES (1071, 1, NULL, NULL, 'Addressbook', 2138, 2139);
INSERT INTO acos VALUES (959, 945, NULL, NULL, 'getEnv', 1975, 1976);
INSERT INTO acos VALUES (960, 945, NULL, NULL, 'getAddParams', 1977, 1978);
INSERT INTO acos VALUES (961, 945, NULL, NULL, 'ldebug', 1979, 1980);
INSERT INTO acos VALUES (962, 1, NULL, NULL, 'Typologiesflux', 1982, 2021);
INSERT INTO acos VALUES (963, 962, NULL, NULL, 'index', 1983, 1984);
INSERT INTO acos VALUES (964, 962, NULL, NULL, 'add', 1985, 1986);
INSERT INTO acos VALUES (965, 962, NULL, NULL, 'edit', 1987, 1988);
INSERT INTO acos VALUES (966, 962, NULL, NULL, 'delete', 1989, 1990);
INSERT INTO acos VALUES (967, 962, NULL, NULL, 'getTypologiesflux', 1991, 1992);
INSERT INTO acos VALUES (968, 962, NULL, NULL, 'ajaxformvalid', 1993, 1994);
INSERT INTO acos VALUES (1049, 1045, NULL, NULL, 'getAdresses', 2103, 2104);
INSERT INTO acos VALUES (1050, 1045, NULL, NULL, 'get_organismes', 2105, 2106);
INSERT INTO acos VALUES (1051, 1045, NULL, NULL, 'userAclCheck', 2107, 2108);
INSERT INTO acos VALUES (1052, 1045, NULL, NULL, 'getSessionInitServices', 2109, 2110);
INSERT INTO acos VALUES (1053, 1045, NULL, NULL, 'getSessionAppDefinedServices', 2111, 2112);
INSERT INTO acos VALUES (1054, 1045, NULL, NULL, 'getSessionAppDefinedDesktops', 2113, 2114);
INSERT INTO acos VALUES (1055, 1045, NULL, NULL, 'getSessionInitDesktops', 2115, 2116);
INSERT INTO acos VALUES (1056, 1045, NULL, NULL, 'getSessionValDesktops', 2117, 2118);
INSERT INTO acos VALUES (1057, 1045, NULL, NULL, 'getSessionValeditDesktops', 2119, 2120);
INSERT INTO acos VALUES (1058, 1045, NULL, NULL, 'getSessionDispDesktops', 2121, 2122);
INSERT INTO acos VALUES (1059, 1045, NULL, NULL, 'getSessionArchDesktops', 2123, 2124);
INSERT INTO acos VALUES (1060, 1045, NULL, NULL, 'getSessionAllDesktops', 2125, 2126);
INSERT INTO acos VALUES (1061, 1045, NULL, NULL, 'getEnv', 2127, 2128);
INSERT INTO acos VALUES (1062, 1045, NULL, NULL, 'getAddParams', 2129, 2130);
INSERT INTO acos VALUES (1063, 1045, NULL, NULL, 'ldebug', 2131, 2132);
INSERT INTO acos VALUES (1068, 227, NULL, NULL, 'add_contact', 617, 618);
INSERT INTO acos VALUES (1069, 227, NULL, NULL, 'edit_contact', 619, 620);
INSERT INTO acos VALUES (1072, 227, NULL, NULL, 'add_organisme', 621, 622);
INSERT INTO acos VALUES (1073, 227, NULL, NULL, 'edit_organisme', 623, 624);
INSERT INTO acos VALUES (423, 1, NULL, NULL, 'Keywordtypes', 918, 945);
INSERT INTO acos VALUES (424, 423, NULL, NULL, 'userAclCheck', 919, 920);
INSERT INTO acos VALUES (425, 423, NULL, NULL, 'getSessionInitServices', 921, 922);
INSERT INTO acos VALUES (426, 423, NULL, NULL, 'getSessionAppDefinedServices', 923, 924);
INSERT INTO acos VALUES (512, 495, NULL, NULL, 'getSessionValeditDesktops', 1095, 1096);
INSERT INTO acos VALUES (513, 495, NULL, NULL, 'getSessionDispDesktops', 1097, 1098);
INSERT INTO acos VALUES (514, 495, NULL, NULL, 'getSessionArchDesktops', 1099, 1100);
INSERT INTO acos VALUES (515, 495, NULL, NULL, 'getSessionAllDesktops', 1101, 1102);
INSERT INTO acos VALUES (516, 495, NULL, NULL, 'getEnv', 1103, 1104);
INSERT INTO acos VALUES (517, 495, NULL, NULL, 'getAddParams', 1105, 1106);
INSERT INTO acos VALUES (518, 495, NULL, NULL, 'ldebug', 1107, 1108);
INSERT INTO acos VALUES (520, 519, NULL, NULL, 'add', 1111, 1112);
INSERT INTO acos VALUES (521, 519, NULL, NULL, 'delete', 1113, 1114);
INSERT INTO acos VALUES (522, 519, NULL, NULL, 'download', 1115, 1116);
INSERT INTO acos VALUES (523, 519, NULL, NULL, 'getPreview', 1117, 1118);
INSERT INTO acos VALUES (524, 519, NULL, NULL, 'userAclCheck', 1119, 1120);
INSERT INTO acos VALUES (525, 519, NULL, NULL, 'getSessionInitServices', 1121, 1122);
INSERT INTO acos VALUES (527, 519, NULL, NULL, 'getSessionAppDefinedDesktops', 1125, 1126);
INSERT INTO acos VALUES (528, 519, NULL, NULL, 'getSessionInitDesktops', 1127, 1128);
INSERT INTO acos VALUES (529, 519, NULL, NULL, 'getSessionValDesktops', 1129, 1130);
INSERT INTO acos VALUES (530, 519, NULL, NULL, 'getSessionValeditDesktops', 1131, 1132);
INSERT INTO acos VALUES (531, 519, NULL, NULL, 'getSessionDispDesktops', 1133, 1134);
INSERT INTO acos VALUES (532, 519, NULL, NULL, 'getSessionArchDesktops', 1135, 1136);
INSERT INTO acos VALUES (533, 519, NULL, NULL, 'getSessionAllDesktops', 1137, 1138);
INSERT INTO acos VALUES (534, 519, NULL, NULL, 'getEnv', 1139, 1140);
INSERT INTO acos VALUES (535, 519, NULL, NULL, 'getAddParams', 1141, 1142);
INSERT INTO acos VALUES (536, 519, NULL, NULL, 'ldebug', 1143, 1144);
INSERT INTO acos VALUES (537, 1, NULL, NULL, 'Scanemails', 1146, 1183);
INSERT INTO acos VALUES (538, 537, NULL, NULL, 'index', 1147, 1148);
INSERT INTO acos VALUES (539, 537, NULL, NULL, 'add', 1149, 1150);
INSERT INTO acos VALUES (540, 537, NULL, NULL, 'edit', 1151, 1152);
INSERT INTO acos VALUES (541, 537, NULL, NULL, 'delete', 1153, 1154);
INSERT INTO acos VALUES (542, 537, NULL, NULL, 'getScanemails', 1155, 1156);
INSERT INTO acos VALUES (544, 537, NULL, NULL, 'userAclCheck', 1157, 1158);
INSERT INTO acos VALUES (545, 537, NULL, NULL, 'getSessionInitServices', 1159, 1160);
INSERT INTO acos VALUES (546, 537, NULL, NULL, 'getSessionAppDefinedServices', 1161, 1162);
INSERT INTO acos VALUES (547, 537, NULL, NULL, 'getSessionAppDefinedDesktops', 1163, 1164);
INSERT INTO acos VALUES (548, 537, NULL, NULL, 'getSessionInitDesktops', 1165, 1166);
INSERT INTO acos VALUES (549, 537, NULL, NULL, 'getSessionValDesktops', 1167, 1168);
INSERT INTO acos VALUES (550, 537, NULL, NULL, 'getSessionValeditDesktops', 1169, 1170);
INSERT INTO acos VALUES (551, 537, NULL, NULL, 'getSessionDispDesktops', 1171, 1172);
INSERT INTO acos VALUES (552, 537, NULL, NULL, 'getSessionArchDesktops', 1173, 1174);
INSERT INTO acos VALUES (554, 537, NULL, NULL, 'getEnv', 1177, 1178);
INSERT INTO acos VALUES (555, 537, NULL, NULL, 'getAddParams', 1179, 1180);
INSERT INTO acos VALUES (556, 537, NULL, NULL, 'ldebug', 1181, 1182);
INSERT INTO acos VALUES (557, 1, NULL, NULL, 'Selectvaluesmetadonnees', 1184, 1217);
INSERT INTO acos VALUES (558, 557, NULL, NULL, 'add', 1185, 1186);
INSERT INTO acos VALUES (559, 557, NULL, NULL, 'edit', 1187, 1188);
INSERT INTO acos VALUES (560, 557, NULL, NULL, 'delete', 1189, 1190);
INSERT INTO acos VALUES (561, 557, NULL, NULL, 'userAclCheck', 1191, 1192);
INSERT INTO acos VALUES (562, 557, NULL, NULL, 'getSessionInitServices', 1193, 1194);
INSERT INTO acos VALUES (563, 557, NULL, NULL, 'getSessionAppDefinedServices', 1195, 1196);
INSERT INTO acos VALUES (564, 557, NULL, NULL, 'getSessionAppDefinedDesktops', 1197, 1198);
INSERT INTO acos VALUES (1, NULL, NULL, NULL, 'controllers', 1, 2172);
INSERT INTO acos VALUES (1045, 1, NULL, NULL, 'Organismes', 2096, 2135);
INSERT INTO acos VALUES (411, 409, NULL, NULL, 'getSessionInitServices', 893, 894);
INSERT INTO acos VALUES (412, 409, NULL, NULL, 'getSessionAppDefinedServices', 895, 896);
INSERT INTO acos VALUES (413, 409, NULL, NULL, 'getSessionAppDefinedDesktops', 897, 898);
INSERT INTO acos VALUES (414, 409, NULL, NULL, 'getSessionInitDesktops', 899, 900);
INSERT INTO acos VALUES (415, 409, NULL, NULL, 'getSessionValDesktops', 901, 902);
INSERT INTO acos VALUES (416, 409, NULL, NULL, 'getSessionValeditDesktops', 903, 904);
INSERT INTO acos VALUES (417, 409, NULL, NULL, 'getSessionDispDesktops', 905, 906);
INSERT INTO acos VALUES (418, 409, NULL, NULL, 'getSessionArchDesktops', 907, 908);
INSERT INTO acos VALUES (419, 409, NULL, NULL, 'getSessionAllDesktops', 909, 910);
INSERT INTO acos VALUES (420, 409, NULL, NULL, 'getEnv', 911, 912);
INSERT INTO acos VALUES (421, 409, NULL, NULL, 'getAddParams', 913, 914);
INSERT INTO acos VALUES (422, 409, NULL, NULL, 'ldebug', 915, 916);
INSERT INTO acos VALUES (969, 962, NULL, NULL, 'userAclCheck', 1995, 1996);
INSERT INTO acos VALUES (970, 962, NULL, NULL, 'getSessionInitServices', 1997, 1998);
INSERT INTO acos VALUES (971, 962, NULL, NULL, 'getSessionAppDefinedServices', 1999, 2000);
INSERT INTO acos VALUES (972, 962, NULL, NULL, 'getSessionAppDefinedDesktops', 2001, 2002);
INSERT INTO acos VALUES (973, 962, NULL, NULL, 'getSessionInitDesktops', 2003, 2004);
INSERT INTO acos VALUES (974, 962, NULL, NULL, 'getSessionValDesktops', 2005, 2006);
INSERT INTO acos VALUES (975, 962, NULL, NULL, 'getSessionValeditDesktops', 2007, 2008);
INSERT INTO acos VALUES (976, 962, NULL, NULL, 'getSessionDispDesktops', 2009, 2010);
INSERT INTO acos VALUES (977, 962, NULL, NULL, 'getSessionArchDesktops', 2011, 2012);
INSERT INTO acos VALUES (978, 962, NULL, NULL, 'getSessionAllDesktops', 2013, 2014);
INSERT INTO acos VALUES (979, 962, NULL, NULL, 'getEnv', 2015, 2016);
INSERT INTO acos VALUES (980, 962, NULL, NULL, 'getAddParams', 2017, 2018);
INSERT INTO acos VALUES (981, 962, NULL, NULL, 'ldebug', 2019, 2020);
INSERT INTO acos VALUES (982, 729, NULL, NULL, 'getDescription', 1591, 1592);
INSERT INTO acos VALUES (988, 987, NULL, NULL, 'index', 2023, 2024);
INSERT INTO acos VALUES (990, 987, NULL, NULL, 'add', 2025, 2026);
INSERT INTO acos VALUES (991, 987, NULL, NULL, 'edit', 2027, 2028);
INSERT INTO acos VALUES (1008, 1006, NULL, NULL, 'download', 2063, 2064);
INSERT INTO acos VALUES (1009, 1006, NULL, NULL, 'getPreview', 2065, 2066);
INSERT INTO acos VALUES (1010, 1006, NULL, NULL, 'add', 2067, 2068);
INSERT INTO acos VALUES (1011, 1006, NULL, NULL, 'userAclCheck', 2069, 2070);
INSERT INTO acos VALUES (1012, 1006, NULL, NULL, 'getSessionInitServices', 2071, 2072);
INSERT INTO acos VALUES (1013, 1006, NULL, NULL, 'getSessionAppDefinedServices', 2073, 2074);
INSERT INTO acos VALUES (1014, 1006, NULL, NULL, 'getSessionAppDefinedDesktops', 2075, 2076);
INSERT INTO acos VALUES (1015, 1006, NULL, NULL, 'getSessionInitDesktops', 2077, 2078);
INSERT INTO acos VALUES (1016, 1006, NULL, NULL, 'getSessionValDesktops', 2079, 2080);
INSERT INTO acos VALUES (1017, 1006, NULL, NULL, 'getSessionValeditDesktops', 2081, 2082);
INSERT INTO acos VALUES (1018, 1006, NULL, NULL, 'getSessionDispDesktops', 2083, 2084);
INSERT INTO acos VALUES (1019, 1006, NULL, NULL, 'getSessionArchDesktops', 2085, 2086);
INSERT INTO acos VALUES (1020, 1006, NULL, NULL, 'getSessionAllDesktops', 2087, 2088);
INSERT INTO acos VALUES (1021, 1006, NULL, NULL, 'getEnv', 2089, 2090);
INSERT INTO acos VALUES (1022, 1006, NULL, NULL, 'getAddParams', 2091, 2092);
INSERT INTO acos VALUES (1023, 1006, NULL, NULL, 'ldebug', 2093, 2094);
INSERT INTO acos VALUES (1039, 227, NULL, NULL, 'execTraitementlot', 609, 610);
INSERT INTO acos VALUES (1044, 987, NULL, NULL, 'get_bureaux', 2057, 2058);
INSERT INTO acos VALUES (1046, 1045, NULL, NULL, 'add', 2097, 2098);
INSERT INTO acos VALUES (1047, 1045, NULL, NULL, 'edit', 2099, 2100);
INSERT INTO acos VALUES (1048, 1045, NULL, NULL, 'delete', 2101, 2102);
INSERT INTO acos VALUES (327, 325, NULL, NULL, 'add', 723, 724);
INSERT INTO acos VALUES (427, 423, NULL, NULL, 'getSessionAppDefinedDesktops', 925, 926);
INSERT INTO acos VALUES (428, 423, NULL, NULL, 'getSessionInitDesktops', 927, 928);
INSERT INTO acos VALUES (429, 423, NULL, NULL, 'getSessionValDesktops', 929, 930);
INSERT INTO acos VALUES (430, 423, NULL, NULL, 'getSessionValeditDesktops', 931, 932);
INSERT INTO acos VALUES (434, 423, NULL, NULL, 'getEnv', 939, 940);
INSERT INTO acos VALUES (565, 557, NULL, NULL, 'getSessionInitDesktops', 1199, 1200);
INSERT INTO acos VALUES (566, 557, NULL, NULL, 'getSessionValDesktops', 1201, 1202);
INSERT INTO acos VALUES (567, 557, NULL, NULL, 'getSessionValeditDesktops', 1203, 1204);
INSERT INTO acos VALUES (568, 557, NULL, NULL, 'getSessionDispDesktops', 1205, 1206);
INSERT INTO acos VALUES (569, 557, NULL, NULL, 'getSessionArchDesktops', 1207, 1208);
INSERT INTO acos VALUES (188, 187, NULL, NULL, 'add', 387, 388);
INSERT INTO acos VALUES (189, 187, NULL, NULL, 'getHomonyms', 389, 390);
INSERT INTO acos VALUES (190, 187, NULL, NULL, 'edit', 391, 392);
INSERT INTO acos VALUES (191, 187, NULL, NULL, 'delete', 393, 394);
INSERT INTO acos VALUES (192, 187, NULL, NULL, 'userAclCheck', 395, 396);
INSERT INTO acos VALUES (193, 187, NULL, NULL, 'getSessionInitServices', 397, 398);
INSERT INTO acos VALUES (1033, 161, NULL, NULL, 'delete', 383, 384);
INSERT INTO acos VALUES (187, 1, NULL, NULL, 'Contactinfos', 386, 425);
INSERT INTO acos VALUES (648, 636, NULL, NULL, 'getSessionAppDefinedDesktops', 1379, 1380);
INSERT INTO acos VALUES (649, 636, NULL, NULL, 'getSessionInitDesktops', 1381, 1382);
INSERT INTO acos VALUES (650, 636, NULL, NULL, 'getSessionValDesktops', 1383, 1384);
INSERT INTO acos VALUES (651, 636, NULL, NULL, 'getSessionValeditDesktops', 1385, 1386);
INSERT INTO acos VALUES (652, 636, NULL, NULL, 'getSessionDispDesktops', 1387, 1388);
INSERT INTO acos VALUES (653, 636, NULL, NULL, 'getSessionArchDesktops', 1389, 1390);
INSERT INTO acos VALUES (654, 636, NULL, NULL, 'getSessionAllDesktops', 1391, 1392);
INSERT INTO acos VALUES (656, 636, NULL, NULL, 'getAddParams', 1395, 1396);
INSERT INTO acos VALUES (657, 636, NULL, NULL, 'ldebug', 1397, 1398);
INSERT INTO acos VALUES (658, 1, NULL, NULL, 'Types', 1400, 1433);
INSERT INTO acos VALUES (659, 658, NULL, NULL, 'add', 1401, 1402);
INSERT INTO acos VALUES (660, 658, NULL, NULL, 'edit', 1403, 1404);
INSERT INTO acos VALUES (661, 658, NULL, NULL, 'delete', 1405, 1406);
INSERT INTO acos VALUES (662, 658, NULL, NULL, 'userAclCheck', 1407, 1408);
INSERT INTO acos VALUES (663, 658, NULL, NULL, 'getSessionInitServices', 1409, 1410);
INSERT INTO acos VALUES (664, 658, NULL, NULL, 'getSessionAppDefinedServices', 1411, 1412);
INSERT INTO acos VALUES (665, 658, NULL, NULL, 'getSessionAppDefinedDesktops', 1413, 1414);
INSERT INTO acos VALUES (666, 658, NULL, NULL, 'getSessionInitDesktops', 1415, 1416);
INSERT INTO acos VALUES (667, 658, NULL, NULL, 'getSessionValDesktops', 1417, 1418);
INSERT INTO acos VALUES (668, 658, NULL, NULL, 'getSessionValeditDesktops', 1419, 1420);
INSERT INTO acos VALUES (669, 658, NULL, NULL, 'getSessionDispDesktops', 1421, 1422);
INSERT INTO acos VALUES (670, 658, NULL, NULL, 'getSessionArchDesktops', 1423, 1424);
INSERT INTO acos VALUES (671, 658, NULL, NULL, 'getSessionAllDesktops', 1425, 1426);
INSERT INTO acos VALUES (672, 658, NULL, NULL, 'getEnv', 1427, 1428);
INSERT INTO acos VALUES (819, 805, NULL, NULL, 'getSessionInitServices', 1726, 1727);
INSERT INTO acos VALUES (820, 805, NULL, NULL, 'getSessionAppDefinedServices', 1728, 1729);
INSERT INTO acos VALUES (821, 805, NULL, NULL, 'getSessionAppDefinedDesktops', 1730, 1731);
INSERT INTO acos VALUES (822, 805, NULL, NULL, 'getSessionInitDesktops', 1732, 1733);
INSERT INTO acos VALUES (823, 805, NULL, NULL, 'getSessionValDesktops', 1734, 1735);
INSERT INTO acos VALUES (824, 805, NULL, NULL, 'getSessionValeditDesktops', 1736, 1737);
INSERT INTO acos VALUES (825, 805, NULL, NULL, 'getSessionDispDesktops', 1738, 1739);
INSERT INTO acos VALUES (826, 805, NULL, NULL, 'getSessionArchDesktops', 1740, 1741);
INSERT INTO acos VALUES (827, 805, NULL, NULL, 'getSessionAllDesktops', 1742, 1743);
INSERT INTO acos VALUES (868, 855, NULL, NULL, 'getEnv', 1824, 1825);
INSERT INTO acos VALUES (869, 855, NULL, NULL, 'getAddParams', 1826, 1827);
INSERT INTO acos VALUES (870, 855, NULL, NULL, 'ldebug', 1828, 1829);
INSERT INTO acos VALUES (871, 1, NULL, NULL, 'Imap', 1832, 1869);
INSERT INTO acos VALUES (872, 871, NULL, NULL, 'Imaps', 1833, 1868);
INSERT INTO acos VALUES (873, 872, NULL, NULL, 'index', 1834, 1835);
INSERT INTO acos VALUES (874, 872, NULL, NULL, 'retrieve', 1836, 1837);
INSERT INTO acos VALUES (875, 872, NULL, NULL, 'getMail', 1838, 1839);
INSERT INTO acos VALUES (876, 872, NULL, NULL, 'getMailPart', 1840, 1841);
INSERT INTO acos VALUES (877, 872, NULL, NULL, 'userAclCheck', 1842, 1843);
INSERT INTO acos VALUES (878, 872, NULL, NULL, 'getSessionInitServices', 1844, 1845);
INSERT INTO acos VALUES (879, 872, NULL, NULL, 'getSessionAppDefinedServices', 1846, 1847);
INSERT INTO acos VALUES (881, 872, NULL, NULL, 'getSessionInitDesktops', 1850, 1851);
INSERT INTO acos VALUES (519, 1, NULL, NULL, 'Rmodels', 1110, 1145);
INSERT INTO acos VALUES (570, 557, NULL, NULL, 'getSessionAllDesktops', 1209, 1210);
INSERT INTO acos VALUES (571, 557, NULL, NULL, 'getEnv', 1211, 1212);
INSERT INTO acos VALUES (572, 557, NULL, NULL, 'getAddParams', 1213, 1214);
INSERT INTO acos VALUES (574, 1, NULL, NULL, 'Services', 1218, 1261);
INSERT INTO acos VALUES (575, 574, NULL, NULL, 'index', 1219, 1220);
INSERT INTO acos VALUES (576, 574, NULL, NULL, 'getServices', 1221, 1222);
INSERT INTO acos VALUES (587, 574, NULL, NULL, 'getSessionInitDesktops', 1243, 1244);
INSERT INTO acos VALUES (612, 596, NULL, NULL, 'getSessionValeditDesktops', 1293, 1294);
INSERT INTO acos VALUES (613, 596, NULL, NULL, 'getSessionDispDesktops', 1295, 1296);
INSERT INTO acos VALUES (614, 596, NULL, NULL, 'getSessionArchDesktops', 1297, 1298);
INSERT INTO acos VALUES (615, 596, NULL, NULL, 'getSessionAllDesktops', 1299, 1300);
INSERT INTO acos VALUES (617, 596, NULL, NULL, 'getAddParams', 1303, 1304);
INSERT INTO acos VALUES (618, 596, NULL, NULL, 'ldebug', 1305, 1306);
INSERT INTO acos VALUES (619, 1, NULL, NULL, 'Statistiques', 1308, 1355);
INSERT INTO acos VALUES (620, 619, NULL, NULL, 'index', 1309, 1310);
INSERT INTO acos VALUES (623, 619, NULL, NULL, 'userAclCheck', 1311, 1312);
INSERT INTO acos VALUES (624, 619, NULL, NULL, 'getSessionInitServices', 1313, 1314);
INSERT INTO acos VALUES (625, 619, NULL, NULL, 'getSessionAppDefinedServices', 1315, 1316);
INSERT INTO acos VALUES (626, 619, NULL, NULL, 'getSessionAppDefinedDesktops', 1317, 1318);
INSERT INTO acos VALUES (627, 619, NULL, NULL, 'getSessionInitDesktops', 1319, 1320);
INSERT INTO acos VALUES (628, 619, NULL, NULL, 'getSessionValDesktops', 1321, 1322);
INSERT INTO acos VALUES (629, 619, NULL, NULL, 'getSessionValeditDesktops', 1323, 1324);
INSERT INTO acos VALUES (630, 619, NULL, NULL, 'getSessionDispDesktops', 1325, 1326);
INSERT INTO acos VALUES (631, 619, NULL, NULL, 'getSessionArchDesktops', 1327, 1328);
INSERT INTO acos VALUES (632, 619, NULL, NULL, 'getSessionAllDesktops', 1329, 1330);
INSERT INTO acos VALUES (633, 619, NULL, NULL, 'getEnv', 1331, 1332);
INSERT INTO acos VALUES (634, 619, NULL, NULL, 'getAddParams', 1333, 1334);
INSERT INTO acos VALUES (635, 619, NULL, NULL, 'ldebug', 1335, 1336);
INSERT INTO acos VALUES (636, 1, NULL, NULL, 'Taches', 1356, 1399);
INSERT INTO acos VALUES (637, 636, NULL, NULL, 'add', 1357, 1358);
INSERT INTO acos VALUES (638, 636, NULL, NULL, 'edit', 1359, 1360);
INSERT INTO acos VALUES (300, 285, NULL, NULL, 'getSessionDispDesktops', 655, 656);
INSERT INTO acos VALUES (301, 285, NULL, NULL, 'getSessionArchDesktops', 657, 658);
INSERT INTO acos VALUES (302, 285, NULL, NULL, 'getSessionAllDesktops', 659, 660);
INSERT INTO acos VALUES (303, 285, NULL, NULL, 'getEnv', 661, 662);
INSERT INTO acos VALUES (304, 285, NULL, NULL, 'getAddParams', 663, 664);
INSERT INTO acos VALUES (305, 285, NULL, NULL, 'ldebug', 665, 666);
INSERT INTO acos VALUES (306, 1, NULL, NULL, 'Documents', 670, 719);
INSERT INTO acos VALUES (307, 306, NULL, NULL, 'setMainDoc', 671, 672);
INSERT INTO acos VALUES (308, 306, NULL, NULL, 'upload', 673, 674);
INSERT INTO acos VALUES (309, 306, NULL, NULL, 'delete', 675, 676);
INSERT INTO acos VALUES (310, 306, NULL, NULL, 'download', 677, 678);
INSERT INTO acos VALUES (311, 306, NULL, NULL, 'getPreview', 679, 680);
INSERT INTO acos VALUES (886, 872, NULL, NULL, 'getSessionAllDesktops', 1860, 1861);
INSERT INTO acos VALUES (887, 872, NULL, NULL, 'getEnv', 1862, 1863);
INSERT INTO acos VALUES (888, 872, NULL, NULL, 'getAddParams', 1864, 1865);
INSERT INTO acos VALUES (889, 872, NULL, NULL, 'ldebug', 1866, 1867);
INSERT INTO acos VALUES (890, 1, NULL, NULL, 'Jsonmsg', 1870, 1871);
INSERT INTO acos VALUES (891, 1, NULL, NULL, 'Pgsqlcake', 1872, 1873);
INSERT INTO acos VALUES (892, 1, NULL, NULL, 'SimpleComment', 1874, 1875);
INSERT INTO acos VALUES (896, 1, NULL, NULL, 'Bans', 1876, 1913);
INSERT INTO acos VALUES (897, 896, NULL, NULL, 'index', 1877, 1878);
INSERT INTO acos VALUES (898, 896, NULL, NULL, 'getBan', 1879, 1880);
INSERT INTO acos VALUES (899, 896, NULL, NULL, 'edit', 1881, 1882);
INSERT INTO acos VALUES (900, 896, NULL, NULL, 'import', 1883, 1884);
INSERT INTO acos VALUES (901, 896, NULL, NULL, 'delete', 1885, 1886);
INSERT INTO acos VALUES (902, 896, NULL, NULL, 'userAclCheck', 1887, 1888);
INSERT INTO acos VALUES (903, 896, NULL, NULL, 'getSessionInitServices', 1889, 1890);
INSERT INTO acos VALUES (904, 896, NULL, NULL, 'getSessionAppDefinedServices', 1891, 1892);
INSERT INTO acos VALUES (905, 896, NULL, NULL, 'getSessionAppDefinedDesktops', 1893, 1894);
INSERT INTO acos VALUES (906, 896, NULL, NULL, 'getSessionInitDesktops', 1895, 1896);
INSERT INTO acos VALUES (907, 896, NULL, NULL, 'getSessionValDesktops', 1897, 1898);
INSERT INTO acos VALUES (908, 896, NULL, NULL, 'getSessionValeditDesktops', 1899, 1900);
INSERT INTO acos VALUES (909, 896, NULL, NULL, 'getSessionDispDesktops', 1901, 1902);
INSERT INTO acos VALUES (910, 896, NULL, NULL, 'getSessionArchDesktops', 1903, 1904);
INSERT INTO acos VALUES (911, 896, NULL, NULL, 'getSessionAllDesktops', 1905, 1906);
INSERT INTO acos VALUES (912, 896, NULL, NULL, 'getEnv', 1907, 1908);
INSERT INTO acos VALUES (913, 896, NULL, NULL, 'getAddParams', 1909, 1910);
INSERT INTO acos VALUES (914, 896, NULL, NULL, 'ldebug', 1911, 1912);
INSERT INTO acos VALUES (915, 1, NULL, NULL, 'Banscommunes', 1914, 1947);
INSERT INTO acos VALUES (828, 805, NULL, NULL, 'getEnv', 1744, 1745);
INSERT INTO acos VALUES (205, 1, NULL, NULL, 'Contacts', 426, 487);
INSERT INTO acos VALUES (227, 1, NULL, NULL, 'Courriers', 488, 625);
INSERT INTO acos VALUES (228, 227, NULL, NULL, 'view', 489, 490);
INSERT INTO acos VALUES (229, 227, NULL, NULL, 'edit', 491, 492);
INSERT INTO acos VALUES (312, 306, NULL, NULL, 'userAclCheck', 681, 682);
INSERT INTO acos VALUES (313, 306, NULL, NULL, 'getSessionInitServices', 683, 684);
INSERT INTO acos VALUES (314, 306, NULL, NULL, 'getSessionAppDefinedServices', 685, 686);
INSERT INTO acos VALUES (315, 306, NULL, NULL, 'getSessionAppDefinedDesktops', 687, 688);
INSERT INTO acos VALUES (316, 306, NULL, NULL, 'getSessionInitDesktops', 689, 690);
INSERT INTO acos VALUES (317, 306, NULL, NULL, 'getSessionValDesktops', 691, 692);
INSERT INTO acos VALUES (318, 306, NULL, NULL, 'getSessionValeditDesktops', 693, 694);
INSERT INTO acos VALUES (319, 306, NULL, NULL, 'getSessionDispDesktops', 695, 696);
INSERT INTO acos VALUES (346, 344, NULL, NULL, 'index', 763, 764);
INSERT INTO acos VALUES (1040, 227, NULL, NULL, 'refusparlot', 611, 612);
INSERT INTO acos VALUES (194, 187, NULL, NULL, 'getSessionAppDefinedServices', 399, 400);
INSERT INTO acos VALUES (195, 187, NULL, NULL, 'getSessionAppDefinedDesktops', 401, 402);
INSERT INTO acos VALUES (196, 187, NULL, NULL, 'getSessionInitDesktops', 403, 404);
INSERT INTO acos VALUES (197, 187, NULL, NULL, 'getSessionValDesktops', 405, 406);
INSERT INTO acos VALUES (198, 187, NULL, NULL, 'getSessionValeditDesktops', 407, 408);
INSERT INTO acos VALUES (199, 187, NULL, NULL, 'getSessionDispDesktops', 409, 410);
INSERT INTO acos VALUES (200, 187, NULL, NULL, 'getSessionArchDesktops', 411, 412);
INSERT INTO acos VALUES (201, 187, NULL, NULL, 'getSessionAllDesktops', 413, 414);
INSERT INTO acos VALUES (203, 187, NULL, NULL, 'getAddParams', 417, 418);
INSERT INTO acos VALUES (204, 187, NULL, NULL, 'ldebug', 419, 420);
INSERT INTO acos VALUES (206, 205, NULL, NULL, 'get_contacts', 427, 428);
INSERT INTO acos VALUES (207, 205, NULL, NULL, 'add', 429, 430);
INSERT INTO acos VALUES (208, 205, NULL, NULL, 'edit', 431, 432);
INSERT INTO acos VALUES (209, 205, NULL, NULL, 'delete', 433, 434);
INSERT INTO acos VALUES (210, 205, NULL, NULL, 'import', 435, 436);
INSERT INTO acos VALUES (211, 205, NULL, NULL, 'search', 437, 438);
INSERT INTO acos VALUES (212, 205, NULL, NULL, 'searchContactinfo', 439, 440);
INSERT INTO acos VALUES (213, 205, NULL, NULL, 'getHomonyms', 441, 442);
INSERT INTO acos VALUES (214, 205, NULL, NULL, 'userAclCheck', 443, 444);
INSERT INTO acos VALUES (215, 205, NULL, NULL, 'getSessionInitServices', 445, 446);
INSERT INTO acos VALUES (216, 205, NULL, NULL, 'getSessionAppDefinedServices', 447, 448);
INSERT INTO acos VALUES (217, 205, NULL, NULL, 'getSessionAppDefinedDesktops', 449, 450);
INSERT INTO acos VALUES (218, 205, NULL, NULL, 'getSessionInitDesktops', 451, 452);
INSERT INTO acos VALUES (219, 205, NULL, NULL, 'getSessionValDesktops', 453, 454);
INSERT INTO acos VALUES (220, 205, NULL, NULL, 'getSessionValeditDesktops', 455, 456);
INSERT INTO acos VALUES (221, 205, NULL, NULL, 'getSessionDispDesktops', 457, 458);
INSERT INTO acos VALUES (222, 205, NULL, NULL, 'getSessionArchDesktops', 459, 460);
INSERT INTO acos VALUES (223, 205, NULL, NULL, 'getSessionAllDesktops', 461, 462);
INSERT INTO acos VALUES (224, 205, NULL, NULL, 'getEnv', 463, 464);
INSERT INTO acos VALUES (225, 205, NULL, NULL, 'getAddParams', 465, 466);
INSERT INTO acos VALUES (226, 205, NULL, NULL, 'ldebug', 467, 468);
INSERT INTO acos VALUES (404, 391, NULL, NULL, 'getSessionArchDesktops', 879, 880);
INSERT INTO acos VALUES (405, 391, NULL, NULL, 'getSessionAllDesktops', 881, 882);
INSERT INTO acos VALUES (406, 391, NULL, NULL, 'getEnv', 883, 884);
INSERT INTO acos VALUES (407, 391, NULL, NULL, 'getAddParams', 885, 886);
INSERT INTO acos VALUES (408, 391, NULL, NULL, 'ldebug', 887, 888);
INSERT INTO acos VALUES (251, 227, NULL, NULL, 'send', 535, 536);
INSERT INTO acos VALUES (252, 227, NULL, NULL, 'dispsend', 537, 538);
INSERT INTO acos VALUES (253, 227, NULL, NULL, 'delete', 539, 540);
INSERT INTO acos VALUES (254, 227, NULL, NULL, 'getInsertEnd', 541, 542);
INSERT INTO acos VALUES (255, 227, NULL, NULL, 'getArs', 543, 544);
INSERT INTO acos VALUES (256, 227, NULL, NULL, 'setArs', 545, 546);
INSERT INTO acos VALUES (257, 227, NULL, NULL, 'genResponse', 547, 548);
INSERT INTO acos VALUES (258, 227, NULL, NULL, 'getContext', 549, 550);
INSERT INTO acos VALUES (259, 227, NULL, NULL, 'getRelements', 551, 552);
INSERT INTO acos VALUES (260, 227, NULL, NULL, 'getDocuments', 553, 554);
INSERT INTO acos VALUES (261, 227, NULL, NULL, 'getParent', 555, 556);
INSERT INTO acos VALUES (262, 227, NULL, NULL, 'acquisition', 557, 558);
INSERT INTO acos VALUES (263, 227, NULL, NULL, 'isInTraite', 559, 560);
INSERT INTO acos VALUES (264, 227, NULL, NULL, 'fluxIdInTraite', 561, 562);
INSERT INTO acos VALUES (265, 227, NULL, NULL, 'sendBackToDisp', 563, 564);
INSERT INTO acos VALUES (266, 227, NULL, NULL, 'dispmultiplesend', 565, 566);
INSERT INTO acos VALUES (267, 227, NULL, NULL, 'historiqueGlobal', 567, 568);
INSERT INTO acos VALUES (268, 227, NULL, NULL, 'sendToGed', 569, 570);
INSERT INTO acos VALUES (1034, 187, NULL, NULL, 'addFromCourrier', 421, 422);
INSERT INTO acos VALUES (1041, 227, NULL, NULL, 'sendcopylot', 613, 614);
INSERT INTO acos VALUES (453, 437, NULL, NULL, 'getEnv', 977, 978);
INSERT INTO acos VALUES (918, 915, NULL, NULL, 'delete', 1919, 1920);
INSERT INTO acos VALUES (919, 915, NULL, NULL, 'userAclCheck', 1921, 1922);
INSERT INTO acos VALUES (920, 915, NULL, NULL, 'getSessionInitServices', 1923, 1924);
INSERT INTO acos VALUES (921, 915, NULL, NULL, 'getSessionAppDefinedServices', 1925, 1926);
INSERT INTO acos VALUES (922, 915, NULL, NULL, 'getSessionAppDefinedDesktops', 1927, 1928);
INSERT INTO acos VALUES (923, 915, NULL, NULL, 'getSessionInitDesktops', 1929, 1930);
INSERT INTO acos VALUES (924, 915, NULL, NULL, 'getSessionValDesktops', 1931, 1932);
INSERT INTO acos VALUES (925, 915, NULL, NULL, 'getSessionValeditDesktops', 1933, 1934);
INSERT INTO acos VALUES (926, 915, NULL, NULL, 'getSessionDispDesktops', 1935, 1936);
INSERT INTO acos VALUES (927, 915, NULL, NULL, 'getSessionArchDesktops', 1937, 1938);
INSERT INTO acos VALUES (928, 915, NULL, NULL, 'getSessionAllDesktops', 1939, 1940);
INSERT INTO acos VALUES (929, 915, NULL, NULL, 'getEnv', 1941, 1942);
INSERT INTO acos VALUES (930, 915, NULL, NULL, 'getAddParams', 1943, 1944);
INSERT INTO acos VALUES (931, 915, NULL, NULL, 'ldebug', 1945, 1946);
INSERT INTO acos VALUES (945, 1, NULL, NULL, 'Referentielsfantoir', 1948, 1981);
INSERT INTO acos VALUES (946, 945, NULL, NULL, 'index', 1949, 1950);
INSERT INTO acos VALUES (947, 945, NULL, NULL, 'getReferentiel', 1951, 1952);
INSERT INTO acos VALUES (948, 945, NULL, NULL, 'import', 1953, 1954);
INSERT INTO acos VALUES (949, 945, NULL, NULL, 'userAclCheck', 1955, 1956);
INSERT INTO acos VALUES (950, 945, NULL, NULL, 'getSessionInitServices', 1957, 1958);
INSERT INTO acos VALUES (951, 945, NULL, NULL, 'getSessionAppDefinedServices', 1959, 1960);
INSERT INTO acos VALUES (952, 945, NULL, NULL, 'getSessionAppDefinedDesktops', 1961, 1962);
INSERT INTO acos VALUES (953, 945, NULL, NULL, 'getSessionInitDesktops', 1963, 1964);
INSERT INTO acos VALUES (954, 945, NULL, NULL, 'getSessionValDesktops', 1965, 1966);
INSERT INTO acos VALUES (955, 945, NULL, NULL, 'getSessionValeditDesktops', 1967, 1968);
INSERT INTO acos VALUES (956, 945, NULL, NULL, 'getSessionDispDesktops', 1969, 1970);
INSERT INTO acos VALUES (957, 945, NULL, NULL, 'getSessionArchDesktops', 1971, 1972);
INSERT INTO acos VALUES (958, 945, NULL, NULL, 'getSessionAllDesktops', 1973, 1974);
INSERT INTO acos VALUES (454, 437, NULL, NULL, 'getAddParams', 979, 980);
INSERT INTO acos VALUES (455, 437, NULL, NULL, 'ldebug', 981, 982);
INSERT INTO acos VALUES (456, 1, NULL, NULL, 'Recherches', 984, 1023);
INSERT INTO acos VALUES (457, 456, NULL, NULL, 'index', 985, 986);
INSERT INTO acos VALUES (458, 456, NULL, NULL, 'formulaire', 987, 988);
INSERT INTO acos VALUES (459, 456, NULL, NULL, 'getRecherches', 989, 990);
INSERT INTO acos VALUES (460, 456, NULL, NULL, 'delete', 991, 992);
INSERT INTO acos VALUES (461, 456, NULL, NULL, 'recherche', 993, 994);
INSERT INTO acos VALUES (462, 456, NULL, NULL, 'exportcsv', 995, 996);
INSERT INTO acos VALUES (463, 456, NULL, NULL, 'userAclCheck', 997, 998);
INSERT INTO acos VALUES (464, 456, NULL, NULL, 'getSessionInitServices', 999, 1000);
INSERT INTO acos VALUES (465, 456, NULL, NULL, 'getSessionAppDefinedServices', 1001, 1002);
INSERT INTO acos VALUES (466, 456, NULL, NULL, 'getSessionAppDefinedDesktops', 1003, 1004);
INSERT INTO acos VALUES (467, 456, NULL, NULL, 'getSessionInitDesktops', 1005, 1006);
INSERT INTO acos VALUES (468, 456, NULL, NULL, 'getSessionValDesktops', 1007, 1008);
INSERT INTO acos VALUES (469, 456, NULL, NULL, 'getSessionValeditDesktops', 1009, 1010);
INSERT INTO acos VALUES (470, 456, NULL, NULL, 'getSessionDispDesktops', 1011, 1012);
INSERT INTO acos VALUES (471, 456, NULL, NULL, 'getSessionArchDesktops', 1013, 1014);
INSERT INTO acos VALUES (472, 456, NULL, NULL, 'getSessionAllDesktops', 1015, 1016);
INSERT INTO acos VALUES (473, 456, NULL, NULL, 'getEnv', 1017, 1018);
INSERT INTO acos VALUES (474, 456, NULL, NULL, 'getAddParams', 1019, 1020);
INSERT INTO acos VALUES (475, 456, NULL, NULL, 'ldebug', 1021, 1022);
INSERT INTO acos VALUES (476, 1, NULL, NULL, 'Relements', 1024, 1061);
INSERT INTO acos VALUES (477, 476, NULL, NULL, 'add', 1025, 1026);
INSERT INTO acos VALUES (478, 476, NULL, NULL, 'delete', 1027, 1028);
INSERT INTO acos VALUES (479, 476, NULL, NULL, 'download', 1029, 1030);
INSERT INTO acos VALUES (480, 476, NULL, NULL, 'getPreview', 1031, 1032);
INSERT INTO acos VALUES (481, 476, NULL, NULL, 'addFromFile', 1033, 1034);
INSERT INTO acos VALUES (482, 476, NULL, NULL, 'userAclCheck', 1035, 1036);
INSERT INTO acos VALUES (483, 476, NULL, NULL, 'getSessionInitServices', 1037, 1038);
INSERT INTO acos VALUES (484, 476, NULL, NULL, 'getSessionAppDefinedServices', 1039, 1040);
INSERT INTO acos VALUES (1042, 227, NULL, NULL, 'detache', 615, 616);
INSERT INTO acos VALUES (1035, 187, NULL, NULL, 'editFromCourrier', 423, 424);
INSERT INTO acos VALUES (573, 557, NULL, NULL, 'ldebug', 1215, 1216);
INSERT INTO acos VALUES (326, 325, NULL, NULL, 'index', 721, 722);
INSERT INTO acos VALUES (485, 476, NULL, NULL, 'getSessionAppDefinedDesktops', 1041, 1042);
INSERT INTO acos VALUES (486, 476, NULL, NULL, 'getSessionInitDesktops', 1043, 1044);
INSERT INTO acos VALUES (487, 476, NULL, NULL, 'getSessionValDesktops', 1045, 1046);
INSERT INTO acos VALUES (488, 476, NULL, NULL, 'getSessionValeditDesktops', 1047, 1048);
INSERT INTO acos VALUES (489, 476, NULL, NULL, 'getSessionDispDesktops', 1049, 1050);
INSERT INTO acos VALUES (490, 476, NULL, NULL, 'getSessionArchDesktops', 1051, 1052);
INSERT INTO acos VALUES (491, 476, NULL, NULL, 'getSessionAllDesktops', 1053, 1054);
INSERT INTO acos VALUES (493, 476, NULL, NULL, 'getAddParams', 1057, 1058);
INSERT INTO acos VALUES (495, 1, NULL, NULL, 'Repertoires', 1062, 1109);
INSERT INTO acos VALUES (496, 495, NULL, NULL, 'index', 1063, 1064);
INSERT INTO acos VALUES (497, 495, NULL, NULL, 'getRepertoires', 1065, 1066);
INSERT INTO acos VALUES (498, 495, NULL, NULL, 'add', 1067, 1068);
INSERT INTO acos VALUES (499, 495, NULL, NULL, 'edit', 1069, 1070);
INSERT INTO acos VALUES (500, 495, NULL, NULL, 'getFlux', 1071, 1072);
INSERT INTO acos VALUES (501, 495, NULL, NULL, 'delete', 1073, 1074);
INSERT INTO acos VALUES (502, 495, NULL, NULL, 'linkGroup', 1075, 1076);
INSERT INTO acos VALUES (503, 495, NULL, NULL, 'link', 1077, 1078);
INSERT INTO acos VALUES (504, 495, NULL, NULL, 'unlinkGroup', 1079, 1080);
INSERT INTO acos VALUES (505, 495, NULL, NULL, 'unlink', 1081, 1082);
INSERT INTO acos VALUES (506, 495, NULL, NULL, 'userAclCheck', 1083, 1084);
INSERT INTO acos VALUES (507, 495, NULL, NULL, 'getSessionInitServices', 1085, 1086);
INSERT INTO acos VALUES (508, 495, NULL, NULL, 'getSessionAppDefinedServices', 1087, 1088);
INSERT INTO acos VALUES (509, 495, NULL, NULL, 'getSessionAppDefinedDesktops', 1089, 1090);
INSERT INTO acos VALUES (510, 495, NULL, NULL, 'getSessionInitDesktops', 1091, 1092);
INSERT INTO acos VALUES (511, 495, NULL, NULL, 'getSessionValDesktops', 1093, 1094);
INSERT INTO acos VALUES (269, 227, NULL, NULL, 'sendDocumentToGed', 571, 572);
INSERT INTO acos VALUES (270, 227, NULL, NULL, 'sendmail', 573, 574);
INSERT INTO acos VALUES (271, 227, NULL, NULL, 'sendwkfsignaturemanuelle', 575, 576);
INSERT INTO acos VALUES (272, 227, NULL, NULL, 'userAclCheck', 577, 578);
INSERT INTO acos VALUES (273, 227, NULL, NULL, 'getSessionInitServices', 579, 580);
INSERT INTO acos VALUES (274, 227, NULL, NULL, 'getSessionAppDefinedServices', 581, 582);
INSERT INTO acos VALUES (275, 227, NULL, NULL, 'getSessionAppDefinedDesktops', 583, 584);
INSERT INTO acos VALUES (276, 227, NULL, NULL, 'getSessionInitDesktops', 585, 586);
INSERT INTO acos VALUES (277, 227, NULL, NULL, 'getSessionValDesktops', 587, 588);
INSERT INTO acos VALUES (278, 227, NULL, NULL, 'getSessionValeditDesktops', 589, 590);
INSERT INTO acos VALUES (279, 227, NULL, NULL, 'getSessionDispDesktops', 591, 592);
INSERT INTO acos VALUES (280, 227, NULL, NULL, 'getSessionArchDesktops', 593, 594);
INSERT INTO acos VALUES (323, 306, NULL, NULL, 'getAddParams', 703, 704);
INSERT INTO acos VALUES (324, 306, NULL, NULL, 'ldebug', 705, 706);
INSERT INTO acos VALUES (325, 1, NULL, NULL, 'Dossiers', 720, 759);
INSERT INTO acos VALUES (577, 574, NULL, NULL, 'add', 1223, 1224);
INSERT INTO acos VALUES (578, 574, NULL, NULL, 'edit', 1225, 1226);
INSERT INTO acos VALUES (579, 574, NULL, NULL, 'setInfos', 1227, 1228);
INSERT INTO acos VALUES (580, 574, NULL, NULL, 'setDroitsData', 1229, 1230);
INSERT INTO acos VALUES (581, 574, NULL, NULL, 'setDroitsMeta', 1231, 1232);
INSERT INTO acos VALUES (582, 574, NULL, NULL, 'delete', 1233, 1234);
INSERT INTO acos VALUES (583, 574, NULL, NULL, 'userAclCheck', 1235, 1236);
INSERT INTO acos VALUES (584, 574, NULL, NULL, 'getSessionInitServices', 1237, 1238);
INSERT INTO acos VALUES (585, 574, NULL, NULL, 'getSessionAppDefinedServices', 1239, 1240);
INSERT INTO acos VALUES (586, 574, NULL, NULL, 'getSessionAppDefinedDesktops', 1241, 1242);
INSERT INTO acos VALUES (588, 574, NULL, NULL, 'getSessionValDesktops', 1245, 1246);
INSERT INTO acos VALUES (589, 574, NULL, NULL, 'getSessionValeditDesktops', 1247, 1248);
INSERT INTO acos VALUES (590, 574, NULL, NULL, 'getSessionDispDesktops', 1249, 1250);
INSERT INTO acos VALUES (591, 574, NULL, NULL, 'getSessionArchDesktops', 1251, 1252);
INSERT INTO acos VALUES (592, 574, NULL, NULL, 'getSessionAllDesktops', 1253, 1254);
INSERT INTO acos VALUES (593, 574, NULL, NULL, 'getEnv', 1255, 1256);
INSERT INTO acos VALUES (594, 574, NULL, NULL, 'getAddParams', 1257, 1258);
INSERT INTO acos VALUES (595, 574, NULL, NULL, 'ldebug', 1259, 1260);
INSERT INTO acos VALUES (596, 1, NULL, NULL, 'Soustypes', 1262, 1307);
INSERT INTO acos VALUES (597, 596, NULL, NULL, 'index', 1263, 1264);
INSERT INTO acos VALUES (598, 596, NULL, NULL, 'getTypesSoustypes', 1265, 1266);
INSERT INTO acos VALUES (599, 596, NULL, NULL, 'setInfos', 1267, 1268);
INSERT INTO acos VALUES (600, 596, NULL, NULL, 'setArmodels', 1269, 1270);
INSERT INTO acos VALUES (601, 596, NULL, NULL, 'setRmodels', 1271, 1272);
INSERT INTO acos VALUES (602, 596, NULL, NULL, 'add', 1273, 1274);
INSERT INTO acos VALUES (603, 596, NULL, NULL, 'edit', 1275, 1276);
INSERT INTO acos VALUES (604, 596, NULL, NULL, 'delete', 1277, 1278);
INSERT INTO acos VALUES (605, 596, NULL, NULL, 'getRefSimul', 1279, 1280);
INSERT INTO acos VALUES (606, 596, NULL, NULL, 'userAclCheck', 1281, 1282);
INSERT INTO acos VALUES (607, 596, NULL, NULL, 'getSessionInitServices', 1283, 1284);
INSERT INTO acos VALUES (608, 596, NULL, NULL, 'getSessionAppDefinedServices', 1285, 1286);
INSERT INTO acos VALUES (609, 596, NULL, NULL, 'getSessionAppDefinedDesktops', 1287, 1288);
INSERT INTO acos VALUES (610, 596, NULL, NULL, 'getSessionInitDesktops', 1289, 1290);
INSERT INTO acos VALUES (611, 596, NULL, NULL, 'getSessionValDesktops', 1291, 1292);
INSERT INTO acos VALUES (616, 596, NULL, NULL, 'getEnv', 1301, 1302);
INSERT INTO acos VALUES (655, 636, NULL, NULL, 'getEnv', 1393, 1394);
INSERT INTO acos VALUES (492, 476, NULL, NULL, 'getEnv', 1055, 1056);
INSERT INTO acos VALUES (777, 755, NULL, NULL, 'getSessionAllDesktops', 1642, 1643);
INSERT INTO acos VALUES (778, 755, NULL, NULL, 'getEnv', 1644, 1645);
INSERT INTO acos VALUES (779, 755, NULL, NULL, 'getAddParams', 1646, 1647);
INSERT INTO acos VALUES (780, 755, NULL, NULL, 'ldebug', 1648, 1649);
INSERT INTO acos VALUES (781, 754, NULL, NULL, 'Compositions', 1651, 1698);
INSERT INTO acos VALUES (782, 781, NULL, NULL, 'index', 1652, 1653);
INSERT INTO acos VALUES (783, 781, NULL, NULL, 'view', 1654, 1655);
INSERT INTO acos VALUES (784, 781, NULL, NULL, 'add', 1656, 1657);
INSERT INTO acos VALUES (341, 325, NULL, NULL, 'getEnv', 751, 752);
INSERT INTO acos VALUES (342, 325, NULL, NULL, 'getAddParams', 753, 754);
INSERT INTO acos VALUES (343, 325, NULL, NULL, 'ldebug', 755, 756);
INSERT INTO acos VALUES (344, 1, NULL, NULL, 'Environnement', 760, 807);
INSERT INTO acos VALUES (345, 344, NULL, NULL, 'getBancontenusServices', 761, 762);
INSERT INTO acos VALUES (347, 344, NULL, NULL, 'superadminenv', 765, 766);
INSERT INTO acos VALUES (386, 368, NULL, NULL, 'getSessionArchDesktops', 843, 844);
INSERT INTO acos VALUES (387, 368, NULL, NULL, 'getSessionAllDesktops', 845, 846);
INSERT INTO acos VALUES (388, 368, NULL, NULL, 'getEnv', 847, 848);
INSERT INTO acos VALUES (389, 368, NULL, NULL, 'getAddParams', 849, 850);
INSERT INTO acos VALUES (390, 368, NULL, NULL, 'ldebug', 851, 852);
INSERT INTO acos VALUES (391, 1, NULL, NULL, 'Keywordlists', 854, 889);
INSERT INTO acos VALUES (392, 391, NULL, NULL, 'index', 855, 856);
INSERT INTO acos VALUES (393, 391, NULL, NULL, 'get_keywordlists', 857, 858);
INSERT INTO acos VALUES (394, 391, NULL, NULL, 'edit', 859, 860);
INSERT INTO acos VALUES (395, 391, NULL, NULL, 'view', 861, 862);
INSERT INTO acos VALUES (396, 391, NULL, NULL, 'userAclCheck', 863, 864);
INSERT INTO acos VALUES (397, 391, NULL, NULL, 'getSessionInitServices', 865, 866);
INSERT INTO acos VALUES (398, 391, NULL, NULL, 'getSessionAppDefinedServices', 867, 868);
INSERT INTO acos VALUES (399, 391, NULL, NULL, 'getSessionAppDefinedDesktops', 869, 870);
INSERT INTO acos VALUES (400, 391, NULL, NULL, 'getSessionInitDesktops', 871, 872);
INSERT INTO acos VALUES (401, 391, NULL, NULL, 'getSessionValDesktops', 873, 874);
INSERT INTO acos VALUES (402, 391, NULL, NULL, 'getSessionValeditDesktops', 875, 876);
INSERT INTO acos VALUES (1043, 285, NULL, NULL, 'setMain', 667, 668);
INSERT INTO acos VALUES (684, 675, NULL, NULL, 'editinfo', 1451, 1452);
INSERT INTO acos VALUES (685, 675, NULL, NULL, 'changemdp', 1453, 1454);
INSERT INTO acos VALUES (686, 675, NULL, NULL, 'rights', 1455, 1456);
INSERT INTO acos VALUES (687, 675, NULL, NULL, 'getRights', 1457, 1458);
INSERT INTO acos VALUES (688, 675, NULL, NULL, 'login', 1459, 1460);
INSERT INTO acos VALUES (689, 675, NULL, NULL, 'logout', 1461, 1462);
INSERT INTO acos VALUES (690, 675, NULL, NULL, 'isAway', 1463, 1464);
INSERT INTO acos VALUES (691, 675, NULL, NULL, 'mdplost', 1465, 1466);
INSERT INTO acos VALUES (692, 675, NULL, NULL, 'changenotif', 1467, 1468);
INSERT INTO acos VALUES (932, 205, NULL, NULL, 'download_model', 469, 470);
INSERT INTO acos VALUES (933, 205, NULL, NULL, 'import_contacts', 471, 472);
INSERT INTO acos VALUES (934, 205, NULL, NULL, 'searchAdresse', 473, 474);
INSERT INTO acos VALUES (935, 205, NULL, NULL, 'searchAdresseSpecifique', 475, 476);
INSERT INTO acos VALUES (936, 205, NULL, NULL, 'searchAdresseNomvoie', 477, 478);
INSERT INTO acos VALUES (1036, 205, NULL, NULL, 'addFromCourrier', 479, 480);
INSERT INTO acos VALUES (320, 306, NULL, NULL, 'getSessionArchDesktops', 697, 698);
INSERT INTO acos VALUES (321, 306, NULL, NULL, 'getSessionAllDesktops', 699, 700);
INSERT INTO acos VALUES (322, 306, NULL, NULL, 'getEnv', 701, 702);
INSERT INTO acos VALUES (328, 325, NULL, NULL, 'edit', 725, 726);
INSERT INTO acos VALUES (329, 325, NULL, NULL, 'delete', 727, 728);
INSERT INTO acos VALUES (330, 325, NULL, NULL, 'getDossiers', 729, 730);
INSERT INTO acos VALUES (331, 325, NULL, NULL, 'userAclCheck', 731, 732);
INSERT INTO acos VALUES (332, 325, NULL, NULL, 'getSessionInitServices', 733, 734);
INSERT INTO acos VALUES (333, 325, NULL, NULL, 'getSessionAppDefinedServices', 735, 736);
INSERT INTO acos VALUES (334, 325, NULL, NULL, 'getSessionAppDefinedDesktops', 737, 738);
INSERT INTO acos VALUES (335, 325, NULL, NULL, 'getSessionInitDesktops', 739, 740);
INSERT INTO acos VALUES (336, 325, NULL, NULL, 'getSessionValDesktops', 741, 742);
INSERT INTO acos VALUES (337, 325, NULL, NULL, 'getSessionValeditDesktops', 743, 744);
INSERT INTO acos VALUES (338, 325, NULL, NULL, 'getSessionDispDesktops', 745, 746);
INSERT INTO acos VALUES (339, 325, NULL, NULL, 'getSessionArchDesktops', 747, 748);
INSERT INTO acos VALUES (340, 325, NULL, NULL, 'getSessionAllDesktops', 749, 750);
INSERT INTO acos VALUES (348, 344, NULL, NULL, 'adminenv', 767, 768);
INSERT INTO acos VALUES (349, 344, NULL, NULL, 'getNotifications', 769, 770);
INSERT INTO acos VALUES (350, 344, NULL, NULL, 'setNotificationRead', 771, 772);
INSERT INTO acos VALUES (351, 344, NULL, NULL, 'setAllNotificationRead', 773, 774);
INSERT INTO acos VALUES (352, 344, NULL, NULL, 'userenv', 775, 776);
INSERT INTO acos VALUES (353, 344, NULL, NULL, 'dispenv', 777, 778);
INSERT INTO acos VALUES (354, 344, NULL, NULL, 'historique', 779, 780);
INSERT INTO acos VALUES (526, 519, NULL, NULL, 'getSessionAppDefinedServices', 1123, 1124);
INSERT INTO acos VALUES (844, 831, NULL, NULL, 'getSessionInitDesktops', 1776, 1777);
INSERT INTO acos VALUES (845, 831, NULL, NULL, 'getSessionValDesktops', 1778, 1779);
INSERT INTO acos VALUES (846, 831, NULL, NULL, 'getSessionValeditDesktops', 1780, 1781);
INSERT INTO acos VALUES (847, 831, NULL, NULL, 'getSessionDispDesktops', 1782, 1783);
INSERT INTO acos VALUES (938, 306, NULL, NULL, 'getFichierScane', 707, 708);
INSERT INTO acos VALUES (939, 306, NULL, NULL, 'mergeFile', 709, 710);
INSERT INTO acos VALUES (940, 306, NULL, NULL, 'sousRepertoireMerge', 711, 712);
INSERT INTO acos VALUES (941, 306, NULL, NULL, 'nouveauFlux', 713, 714);
INSERT INTO acos VALUES (942, 306, NULL, NULL, 'getScanFilePreview', 715, 716);
INSERT INTO acos VALUES (943, 306, NULL, NULL, 'getFusionPreview', 717, 718);
INSERT INTO acos VALUES (944, 325, NULL, NULL, 'getComment', 757, 758);
INSERT INTO acos VALUES (987, 1, NULL, NULL, 'Desktopsmanagers', 2022, 2059);
INSERT INTO acos VALUES (992, 987, NULL, NULL, 'delete', 2029, 2030);
INSERT INTO acos VALUES (993, 987, NULL, NULL, 'userAclCheck', 2031, 2032);
INSERT INTO acos VALUES (994, 987, NULL, NULL, 'getSessionInitServices', 2033, 2034);
INSERT INTO acos VALUES (995, 987, NULL, NULL, 'getSessionAppDefinedServices', 2035, 2036);
INSERT INTO acos VALUES (996, 987, NULL, NULL, 'getSessionAppDefinedDesktops', 2037, 2038);
INSERT INTO acos VALUES (997, 987, NULL, NULL, 'getSessionInitDesktops', 2039, 2040);
INSERT INTO acos VALUES (998, 987, NULL, NULL, 'getSessionValDesktops', 2041, 2042);
INSERT INTO acos VALUES (999, 987, NULL, NULL, 'getSessionValeditDesktops', 2043, 2044);
INSERT INTO acos VALUES (1000, 987, NULL, NULL, 'getSessionDispDesktops', 2045, 2046);
INSERT INTO acos VALUES (1001, 987, NULL, NULL, 'getSessionArchDesktops', 2047, 2048);
INSERT INTO acos VALUES (1002, 987, NULL, NULL, 'getSessionAllDesktops', 2049, 2050);
INSERT INTO acos VALUES (1003, 987, NULL, NULL, 'getEnv', 2051, 2052);
INSERT INTO acos VALUES (1004, 987, NULL, NULL, 'getAddParams', 2053, 2054);
INSERT INTO acos VALUES (1005, 987, NULL, NULL, 'ldebug', 2055, 2056);
INSERT INTO acos VALUES (1006, 1, NULL, NULL, 'Rmodelgabarits', 2060, 2095);
INSERT INTO acos VALUES (1007, 1006, NULL, NULL, 'delete', 2061, 2062);
INSERT INTO acos VALUES (1024, 619, NULL, NULL, 'indicateursGeo', 1337, 1338);
INSERT INTO acos VALUES (1025, 619, NULL, NULL, 'indicateursQuant', 1339, 1340);
INSERT INTO acos VALUES (1026, 619, NULL, NULL, 'indicateursQual', 1341, 1342);
INSERT INTO acos VALUES (1027, 619, NULL, NULL, 'exportqual', 1343, 1344);
INSERT INTO acos VALUES (1028, 619, NULL, NULL, 'exportqualattente', 1345, 1346);
INSERT INTO acos VALUES (1029, 619, NULL, NULL, 'exportquant', 1347, 1348);
INSERT INTO acos VALUES (1030, 619, NULL, NULL, 'exportquantmeta', 1349, 1350);
INSERT INTO acos VALUES (1031, 619, NULL, NULL, 'exportgeo', 1351, 1352);
INSERT INTO acos VALUES (1032, 619, NULL, NULL, 'getAdresses', 1353, 1354);
INSERT INTO acos VALUES (706, 1, NULL, NULL, 'Webservices', 1500, 1545);
INSERT INTO acos VALUES (707, 706, NULL, NULL, 'chk', 1501, 1502);
INSERT INTO acos VALUES (708, 706, NULL, NULL, 'service', 1503, 1504);
INSERT INTO acos VALUES (709, 706, NULL, NULL, 'echotest', 1505, 1506);
INSERT INTO acos VALUES (710, 706, NULL, NULL, 'getGFCCollectivites', 1507, 1508);
INSERT INTO acos VALUES (711, 706, NULL, NULL, 'getGFCTypes', 1509, 1510);
INSERT INTO acos VALUES (712, 706, NULL, NULL, 'getGFCSoustypes', 1511, 1512);
INSERT INTO acos VALUES (713, 706, NULL, NULL, 'createCourrier', 1513, 1514);
INSERT INTO acos VALUES (714, 706, NULL, NULL, 'scanMass', 1515, 1516);
INSERT INTO acos VALUES (715, 706, NULL, NULL, 'getStatut', 1517, 1518);
INSERT INTO acos VALUES (716, 706, NULL, NULL, 'userAclCheck', 1519, 1520);
INSERT INTO acos VALUES (717, 706, NULL, NULL, 'getSessionInitServices', 1521, 1522);
INSERT INTO acos VALUES (718, 706, NULL, NULL, 'getSessionAppDefinedServices', 1523, 1524);
INSERT INTO acos VALUES (719, 706, NULL, NULL, 'getSessionAppDefinedDesktops', 1525, 1526);
INSERT INTO acos VALUES (720, 706, NULL, NULL, 'getSessionInitDesktops', 1527, 1528);
INSERT INTO acos VALUES (1064, 675, NULL, NULL, 'exportcsv', 1495, 1496);
INSERT INTO acos VALUES (448, 437, NULL, NULL, 'getSessionValDesktops', 967, 968);
INSERT INTO acos VALUES (763, 755, NULL, NULL, 'setCreatedModifiedUser', 1614, 1615);
INSERT INTO acos VALUES (764, 755, NULL, NULL, 'formatUser', 1616, 1617);
INSERT INTO acos VALUES (765, 755, NULL, NULL, 'formatLinkedModel', 1618, 1619);
INSERT INTO acos VALUES (766, 755, NULL, NULL, 'formatLinkedModels', 1620, 1621);
INSERT INTO acos VALUES (767, 755, NULL, NULL, 'listLinkedModel', 1622, 1623);
INSERT INTO acos VALUES (768, 755, NULL, NULL, 'userAclCheck', 1624, 1625);
INSERT INTO acos VALUES (769, 755, NULL, NULL, 'getSessionInitServices', 1626, 1627);
INSERT INTO acos VALUES (770, 755, NULL, NULL, 'getSessionAppDefinedServices', 1628, 1629);
INSERT INTO acos VALUES (771, 755, NULL, NULL, 'getSessionAppDefinedDesktops', 1630, 1631);
INSERT INTO acos VALUES (772, 755, NULL, NULL, 'getSessionInitDesktops', 1632, 1633);
INSERT INTO acos VALUES (773, 755, NULL, NULL, 'getSessionValDesktops', 1634, 1635);
INSERT INTO acos VALUES (785, 781, NULL, NULL, 'edit', 1658, 1659);
INSERT INTO acos VALUES (286, 285, NULL, NULL, 'add', 627, 628);
INSERT INTO acos VALUES (287, 285, NULL, NULL, 'edit', 629, 630);
INSERT INTO acos VALUES (288, 285, NULL, NULL, 'setRights', 631, 632);
INSERT INTO acos VALUES (289, 285, NULL, NULL, 'setDeleg', 633, 634);
INSERT INTO acos VALUES (290, 285, NULL, NULL, 'getRights', 635, 636);
INSERT INTO acos VALUES (291, 285, NULL, NULL, 'rights', 637, 638);
INSERT INTO acos VALUES (292, 285, NULL, NULL, 'delete', 639, 640);
INSERT INTO acos VALUES (293, 285, NULL, NULL, 'userAclCheck', 641, 642);
INSERT INTO acos VALUES (294, 285, NULL, NULL, 'getSessionInitServices', 643, 644);
INSERT INTO acos VALUES (295, 285, NULL, NULL, 'getSessionAppDefinedServices', 645, 646);
INSERT INTO acos VALUES (296, 285, NULL, NULL, 'getSessionAppDefinedDesktops', 647, 648);
INSERT INTO acos VALUES (297, 285, NULL, NULL, 'getSessionInitDesktops', 649, 650);
INSERT INTO acos VALUES (298, 285, NULL, NULL, 'getSessionValDesktops', 651, 652);
INSERT INTO acos VALUES (299, 285, NULL, NULL, 'getSessionValeditDesktops', 653, 654);
INSERT INTO acos VALUES (1037, 205, NULL, NULL, 'editFromCourrier', 481, 482);
INSERT INTO acos VALUES (675, 1, NULL, NULL, 'Users', 1434, 1499);
INSERT INTO acos VALUES (741, 729, NULL, NULL, 'getSessionAppDefinedServices', 1569, 1570);
INSERT INTO acos VALUES (742, 729, NULL, NULL, 'getSessionAppDefinedDesktops', 1571, 1572);
INSERT INTO acos VALUES (743, 729, NULL, NULL, 'getSessionInitDesktops', 1573, 1574);
INSERT INTO acos VALUES (744, 729, NULL, NULL, 'getSessionValDesktops', 1575, 1576);
INSERT INTO acos VALUES (749, 729, NULL, NULL, 'getEnv', 1585, 1586);
INSERT INTO acos VALUES (745, 729, NULL, NULL, 'getSessionValeditDesktops', 1577, 1578);
INSERT INTO acos VALUES (746, 729, NULL, NULL, 'getSessionDispDesktops', 1579, 1580);
INSERT INTO acos VALUES (747, 729, NULL, NULL, 'getSessionArchDesktops', 1581, 1582);
INSERT INTO acos VALUES (748, 729, NULL, NULL, 'getSessionAllDesktops', 1583, 1584);
INSERT INTO acos VALUES (750, 729, NULL, NULL, 'getAddParams', 1587, 1588);
INSERT INTO acos VALUES (751, 729, NULL, NULL, 'ldebug', 1589, 1590);
INSERT INTO acos VALUES (752, 1, NULL, NULL, 'AclExtras', 1594, 1595);
INSERT INTO acos VALUES (754, 1, NULL, NULL, 'Cakeflow', 1598, 1795);
INSERT INTO acos VALUES (755, 754, NULL, NULL, 'Circuits', 1599, 1650);
INSERT INTO acos VALUES (756, 755, NULL, NULL, 'index', 1600, 1601);
INSERT INTO acos VALUES (757, 755, NULL, NULL, 'view', 1602, 1603);
INSERT INTO acos VALUES (758, 755, NULL, NULL, 'add', 1604, 1605);
INSERT INTO acos VALUES (759, 755, NULL, NULL, 'edit', 1606, 1607);
INSERT INTO acos VALUES (760, 755, NULL, NULL, 'delete', 1608, 1609);
INSERT INTO acos VALUES (761, 755, NULL, NULL, 'visuCircuit', 1610, 1611);
INSERT INTO acos VALUES (762, 755, NULL, NULL, 'reorder', 1612, 1613);
INSERT INTO acos VALUES (774, 755, NULL, NULL, 'getSessionValeditDesktops', 1636, 1637);
INSERT INTO acos VALUES (775, 755, NULL, NULL, 'getSessionDispDesktops', 1638, 1639);
INSERT INTO acos VALUES (776, 755, NULL, NULL, 'getSessionArchDesktops', 1640, 1641);
INSERT INTO acos VALUES (1065, 675, NULL, NULL, 'cutCircuit', 1497, 1498);
INSERT INTO acos VALUES (786, 781, NULL, NULL, 'delete', 1660, 1661);
INSERT INTO acos VALUES (787, 781, NULL, NULL, 'setCreatedModifiedUser', 1662, 1663);
INSERT INTO acos VALUES (788, 781, NULL, NULL, 'formatUser', 1664, 1665);
INSERT INTO acos VALUES (789, 781, NULL, NULL, 'formatLinkedModel', 1666, 1667);
INSERT INTO acos VALUES (790, 781, NULL, NULL, 'formatLinkedModels', 1668, 1669);
INSERT INTO acos VALUES (791, 781, NULL, NULL, 'listLinkedModel', 1670, 1671);
INSERT INTO acos VALUES (792, 781, NULL, NULL, 'userAclCheck', 1672, 1673);
INSERT INTO acos VALUES (793, 781, NULL, NULL, 'getSessionInitServices', 1674, 1675);
INSERT INTO acos VALUES (794, 781, NULL, NULL, 'getSessionAppDefinedServices', 1676, 1677);
INSERT INTO acos VALUES (795, 781, NULL, NULL, 'getSessionAppDefinedDesktops', 1678, 1679);
INSERT INTO acos VALUES (796, 781, NULL, NULL, 'getSessionInitDesktops', 1680, 1681);
INSERT INTO acos VALUES (797, 781, NULL, NULL, 'getSessionValDesktops', 1682, 1683);
INSERT INTO acos VALUES (798, 781, NULL, NULL, 'getSessionValeditDesktops', 1684, 1685);
INSERT INTO acos VALUES (799, 781, NULL, NULL, 'getSessionDispDesktops', 1686, 1687);
INSERT INTO acos VALUES (800, 781, NULL, NULL, 'getSessionArchDesktops', 1688, 1689);
INSERT INTO acos VALUES (801, 781, NULL, NULL, 'getSessionAllDesktops', 1690, 1691);
INSERT INTO acos VALUES (802, 781, NULL, NULL, 'getEnv', 1692, 1693);
INSERT INTO acos VALUES (803, 781, NULL, NULL, 'getAddParams', 1694, 1695);
INSERT INTO acos VALUES (804, 781, NULL, NULL, 'ldebug', 1696, 1697);
INSERT INTO acos VALUES (805, 754, NULL, NULL, 'Etapes', 1699, 1750);
INSERT INTO acos VALUES (806, 805, NULL, NULL, 'index', 1700, 1701);
INSERT INTO acos VALUES (807, 805, NULL, NULL, 'view', 1702, 1703);
INSERT INTO acos VALUES (829, 805, NULL, NULL, 'getAddParams', 1746, 1747);
INSERT INTO acos VALUES (830, 805, NULL, NULL, 'ldebug', 1748, 1749);
INSERT INTO acos VALUES (831, 754, NULL, NULL, 'Traitements', 1751, 1794);
INSERT INTO acos VALUES (832, 831, NULL, NULL, 'visuTraitement', 1752, 1753);
INSERT INTO acos VALUES (833, 831, NULL, NULL, 'majTraitementsParapheur', 1754, 1755);
INSERT INTO acos VALUES (834, 831, NULL, NULL, 'traiterDelegationsPassees', 1756, 1757);
INSERT INTO acos VALUES (835, 831, NULL, NULL, 'setCreatedModifiedUser', 1758, 1759);
INSERT INTO acos VALUES (836, 831, NULL, NULL, 'formatUser', 1760, 1761);
INSERT INTO acos VALUES (837, 831, NULL, NULL, 'formatLinkedModel', 1762, 1763);
INSERT INTO acos VALUES (230, 227, NULL, NULL, 'add', 493, 494);
INSERT INTO acos VALUES (231, 227, NULL, NULL, 'getInfos', 495, 496);
INSERT INTO acos VALUES (232, 227, NULL, NULL, 'getCircuit', 497, 498);
INSERT INTO acos VALUES (233, 227, NULL, NULL, 'getMeta', 499, 500);
INSERT INTO acos VALUES (234, 227, NULL, NULL, 'getTache', 501, 502);
INSERT INTO acos VALUES (235, 227, NULL, NULL, 'getAffaire', 503, 504);
INSERT INTO acos VALUES (236, 227, NULL, NULL, 'createInfos', 505, 506);
INSERT INTO acos VALUES (237, 227, NULL, NULL, 'setInfos', 507, 508);
INSERT INTO acos VALUES (238, 227, NULL, NULL, 'setComments', 509, 510);
INSERT INTO acos VALUES (239, 227, NULL, NULL, 'getComments', 511, 512);
INSERT INTO acos VALUES (240, 227, NULL, NULL, 'getContacts', 513, 514);
INSERT INTO acos VALUES (241, 227, NULL, NULL, 'getFlowControls', 515, 516);
INSERT INTO acos VALUES (242, 227, NULL, NULL, 'setMeta', 517, 518);
INSERT INTO acos VALUES (243, 227, NULL, NULL, 'setTache', 519, 520);
INSERT INTO acos VALUES (244, 227, NULL, NULL, 'setAffaire', 521, 522);
INSERT INTO acos VALUES (245, 227, NULL, NULL, 'reponse', 523, 524);
INSERT INTO acos VALUES (246, 227, NULL, NULL, 'valid', 525, 526);
INSERT INTO acos VALUES (247, 227, NULL, NULL, 'insert', 527, 528);
INSERT INTO acos VALUES (248, 227, NULL, NULL, 'refus', 529, 530);
INSERT INTO acos VALUES (249, 227, NULL, NULL, 'sendcpy', 531, 532);
INSERT INTO acos VALUES (250, 227, NULL, NULL, 'sendwkf', 533, 534);
INSERT INTO acos VALUES (937, 227, NULL, NULL, 'validEtapeLastCol', 603, 604);
INSERT INTO acos VALUES (983, 227, NULL, NULL, 'ajaxformvalid', 605, 606);
INSERT INTO acos VALUES (1038, 227, NULL, NULL, 'traitementlot', 607, 608);
INSERT INTO acos VALUES (838, 831, NULL, NULL, 'formatLinkedModels', 1764, 1765);
INSERT INTO acos VALUES (839, 831, NULL, NULL, 'listLinkedModel', 1766, 1767);
INSERT INTO acos VALUES (840, 831, NULL, NULL, 'userAclCheck', 1768, 1769);
INSERT INTO acos VALUES (841, 831, NULL, NULL, 'getSessionInitServices', 1770, 1771);
INSERT INTO acos VALUES (842, 831, NULL, NULL, 'getSessionAppDefinedServices', 1772, 1773);
INSERT INTO acos VALUES (843, 831, NULL, NULL, 'getSessionAppDefinedDesktops', 1774, 1775);
INSERT INTO acos VALUES (848, 831, NULL, NULL, 'getSessionArchDesktops', 1784, 1785);
INSERT INTO acos VALUES (849, 831, NULL, NULL, 'getSessionAllDesktops', 1786, 1787);
INSERT INTO acos VALUES (850, 831, NULL, NULL, 'getEnv', 1788, 1789);
INSERT INTO acos VALUES (851, 831, NULL, NULL, 'getAddParams', 1790, 1791);
INSERT INTO acos VALUES (852, 831, NULL, NULL, 'ldebug', 1792, 1793);
INSERT INTO acos VALUES (853, 1, NULL, NULL, 'DataAcl', 1796, 1797);
INSERT INTO acos VALUES (854, 1, NULL, NULL, 'DebugKit', 1798, 1831);
INSERT INTO acos VALUES (855, 854, NULL, NULL, 'ToolbarAccess', 1799, 1830);
INSERT INTO acos VALUES (856, 855, NULL, NULL, 'history_state', 1800, 1801);
INSERT INTO acos VALUES (857, 855, NULL, NULL, 'sql_explain', 1802, 1803);
INSERT INTO acos VALUES (858, 855, NULL, NULL, 'userAclCheck', 1804, 1805);
INSERT INTO acos VALUES (859, 855, NULL, NULL, 'getSessionInitServices', 1806, 1807);
INSERT INTO acos VALUES (860, 855, NULL, NULL, 'getSessionAppDefinedServices', 1808, 1809);
INSERT INTO acos VALUES (861, 855, NULL, NULL, 'getSessionAppDefinedDesktops', 1810, 1811);
INSERT INTO acos VALUES (862, 855, NULL, NULL, 'getSessionInitDesktops', 1812, 1813);
INSERT INTO acos VALUES (863, 855, NULL, NULL, 'getSessionValDesktops', 1814, 1815);
INSERT INTO acos VALUES (864, 855, NULL, NULL, 'getSessionValeditDesktops', 1816, 1817);
INSERT INTO acos VALUES (865, 855, NULL, NULL, 'getSessionDispDesktops', 1818, 1819);
INSERT INTO acos VALUES (866, 855, NULL, NULL, 'getSessionArchDesktops', 1820, 1821);
INSERT INTO acos VALUES (867, 855, NULL, NULL, 'getSessionAllDesktops', 1822, 1823);


--
-- Name: acos_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('acos_id_seq', 1091, true);


--
-- Data for Name: aros; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO aros VALUES (1, NULL, 'Group', 1, 'Superadmin', 1, 2);
INSERT INTO aros VALUES (2, NULL, 'Group', 2, 'Admin', 3, 6);
INSERT INTO aros VALUES (3, NULL, 'Group', 3, 'Initiateur', 7, 160);
INSERT INTO aros VALUES (4, NULL, 'Group', 4, 'Valideur', 161, 164);
INSERT INTO aros VALUES (5, NULL, 'Group', 5, 'Valideur Editeur', 165, 214);
INSERT INTO aros VALUES (6, NULL, 'Group', 6, 'Documentaliste', 215, 218);
INSERT INTO aros VALUES (7, NULL, 'Group', 7, 'Aiguilleur', 219, 242);

-- Data for Name: aros_acos; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO aros_acos VALUES (85, 3, 346, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (86, 3, 354, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (87, 3, 352, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (88, 3, 349, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (89, 3, 350, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (90, 3, 351, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (91, 3, 495, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (92, 3, 456, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (93, 3, 636, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (94, 3, 289, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (95, 3, 130, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (96, 3, 239, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (97, 3, 238, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (98, 3, 267, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (99, 3, 268, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (100, 3, 269, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (101, 3, 270, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (102, 3, 263, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (103, 3, 205, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (104, 3, 187, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (105, 3, 684, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (106, 3, 685, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (107, 3, 692, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (108, 3, 682, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (109, 3, 683, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (110, 3, 679, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (111, 3, 690, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (112, 3, 253, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (113, 3, 230, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (114, 3, 228, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (115, 3, 229, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (117, 3, 308, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (118, 3, 309, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (119, 3, 310, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (120, 3, 307, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (121, 3, 311, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (122, 3, 260, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (123, 3, 255, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (124, 3, 259, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (125, 3, 476, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (126, 3, 523, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (127, 3, 58, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (128, 3, 76, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (129, 3, 522, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (130, 3, 256, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (131, 3, 257, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (132, 3, 72, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (133, 3, 57, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (134, 3, 261, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (135, 3, 477, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (136, 3, 478, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (137, 3, 241, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (138, 3, 232, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (139, 3, 246, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (140, 3, 251, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (141, 3, 249, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (142, 3, 250, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (143, 3, 247, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (144, 3, 254, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (145, 3, 248, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (146, 3, 245, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (147, 3, 265, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (148, 3, 236, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (149, 3, 207, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (150, 3, 208, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (151, 3, 213, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (152, 3, 211, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (153, 3, 212, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (154, 3, 188, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (155, 3, 190, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (156, 3, 189, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (157, 3, 605, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (168, 4, 354, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (170, 4, 349, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (171, 4, 350, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (172, 4, 351, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (173, 4, 495, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (174, 4, 456, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (175, 4, 636, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (176, 4, 289, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (177, 4, 130, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (178, 4, 239, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (179, 4, 238, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (180, 4, 267, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (181, 4, 268, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (183, 4, 270, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (184, 4, 263, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (185, 4, 205, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (186, 4, 187, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (187, 4, 684, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (188, 4, 685, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (189, 4, 692, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (190, 4, 682, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (191, 4, 683, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (192, 4, 679, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (193, 4, 690, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (194, 4, 253, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (196, 4, 228, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (197, 4, 229, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (199, 4, 308, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (200, 4, 309, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (201, 4, 310, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (202, 4, 307, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (203, 4, 311, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (204, 4, 260, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (205, 4, 255, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (206, 4, 259, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (207, 4, 476, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (208, 4, 523, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (210, 4, 76, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (211, 4, 522, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (212, 4, 256, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (213, 4, 257, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (214, 4, 72, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (215, 4, 57, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (216, 4, 261, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (217, 4, 477, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (218, 4, 478, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (219, 4, 241, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (220, 4, 232, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (221, 4, 246, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (223, 4, 249, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (224, 4, 250, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (225, 4, 247, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (226, 4, 254, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (227, 4, 248, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (228, 4, 245, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (229, 4, 231, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (230, 4, 207, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (231, 4, 213, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (232, 4, 211, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (233, 4, 212, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (234, 4, 188, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (236, 4, 605, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (237, 4, 233, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (238, 4, 242, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (239, 4, 234, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (240, 4, 243, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (241, 4, 235, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (242, 4, 236, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (243, 4, 208, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (244, 4, 190, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (245, 4, 237, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (246, 4, 244, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (158, 3, 231, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (160, 3, 233, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (161, 3, 242, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (162, 3, 234, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (163, 3, 243, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (164, 3, 345, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (165, 3, 235, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (166, 3, 244, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (247, 5, 346, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (248, 5, 354, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (249, 5, 352, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (250, 5, 349, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (251, 5, 350, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (253, 5, 495, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (254, 5, 456, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (255, 5, 636, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (256, 5, 289, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (257, 5, 130, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (258, 5, 239, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (259, 5, 238, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (260, 5, 267, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (261, 5, 268, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (262, 5, 269, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (263, 5, 270, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (264, 5, 263, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (266, 5, 187, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (267, 5, 684, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (268, 5, 685, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (269, 5, 692, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (270, 5, 682, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (271, 5, 683, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (272, 5, 679, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (273, 5, 690, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (274, 5, 253, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (275, 5, 345, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (276, 5, 228, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (277, 5, 229, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (280, 5, 309, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (281, 5, 310, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (282, 5, 307, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (283, 5, 311, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (284, 5, 260, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (285, 5, 255, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (286, 5, 259, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (287, 5, 476, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (288, 5, 523, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (289, 5, 58, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (290, 5, 76, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (291, 5, 522, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (293, 5, 257, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (294, 5, 72, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (295, 5, 57, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (296, 5, 261, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (297, 5, 477, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (298, 5, 478, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (299, 5, 241, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (300, 5, 232, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (301, 5, 246, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (302, 5, 251, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (303, 5, 249, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (304, 5, 250, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (306, 5, 254, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (316, 5, 605, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (318, 5, 242, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (319, 5, 234, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (320, 5, 243, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (321, 5, 235, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (322, 5, 237, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (323, 5, 208, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (324, 5, 190, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (325, 5, 236, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (326, 5, 244, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (327, 6, 346, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (328, 6, 354, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (329, 6, 352, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (331, 6, 350, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (332, 6, 351, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (333, 6, 495, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (334, 6, 456, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (335, 6, 636, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (336, 6, 289, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (337, 6, 130, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (338, 6, 239, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (339, 6, 238, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (340, 6, 267, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (341, 6, 268, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (342, 6, 269, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (344, 6, 263, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (345, 6, 205, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (346, 6, 187, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (347, 6, 684, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (348, 6, 685, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (349, 6, 692, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (350, 6, 682, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (351, 6, 683, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (352, 6, 679, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (353, 6, 690, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (354, 6, 253, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (355, 6, 345, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (357, 6, 229, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (359, 6, 308, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (360, 6, 309, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (361, 6, 310, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (362, 6, 307, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (363, 6, 311, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (364, 6, 260, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (365, 6, 255, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (366, 6, 259, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (367, 6, 476, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (368, 6, 523, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (369, 6, 58, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (371, 6, 522, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (372, 6, 256, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (373, 6, 257, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (374, 6, 72, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (375, 6, 57, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (376, 6, 261, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (377, 6, 477, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (378, 6, 478, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (379, 6, 241, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (380, 6, 232, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (381, 6, 246, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (382, 6, 251, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (384, 6, 250, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (385, 6, 247, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (386, 6, 254, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (387, 6, 248, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (388, 6, 245, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (389, 6, 231, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (390, 6, 207, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (391, 6, 213, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (392, 6, 211, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (393, 6, 212, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (394, 6, 188, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (395, 6, 189, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (397, 6, 233, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (398, 6, 242, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (399, 6, 234, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (400, 6, 243, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (401, 6, 235, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (402, 6, 244, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (403, 6, 236, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (404, 6, 208, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (405, 6, 190, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (406, 6, 237, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (407, 7, 346, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (490, 7, 353, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (408, 7, 354, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (410, 7, 349, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (411, 7, 350, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (412, 7, 351, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (413, 7, 495, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (414, 7, 456, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (415, 7, 636, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (416, 7, 289, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (417, 7, 130, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (418, 7, 239, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (419, 7, 238, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (420, 7, 267, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (421, 7, 268, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (423, 7, 270, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (424, 7, 263, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (425, 7, 205, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (426, 7, 187, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (427, 7, 684, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (428, 7, 685, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (429, 7, 692, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (430, 7, 682, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (431, 7, 683, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (432, 7, 679, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (433, 7, 690, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (434, 7, 253, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (436, 7, 229, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (437, 7, 228, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (439, 7, 308, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (440, 7, 309, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (441, 7, 310, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (442, 7, 307, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (443, 7, 311, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (444, 7, 260, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (445, 7, 255, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (446, 7, 259, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (447, 7, 476, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (448, 7, 523, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (450, 7, 76, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (451, 7, 522, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (452, 7, 256, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (453, 7, 257, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (454, 7, 72, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (455, 7, 57, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (456, 7, 261, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (457, 7, 252, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (458, 7, 266, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (459, 7, 241, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (460, 7, 232, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (461, 7, 246, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (463, 7, 249, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18288, 7, 984, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18289, 6, 984, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18290, 5, 984, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18291, 3, 984, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18292, 4, 984, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18294, 7, 985, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18295, 6, 985, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18296, 5, 985, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18297, 3, 985, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18298, 4, 985, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18300, 7, 986, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18301, 6, 986, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18302, 5, 986, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18303, 3, 986, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18304, 4, 986, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18306, 7, 987, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18307, 6, 987, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18308, 5, 987, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18309, 3, 987, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18310, 4, 987, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18312, 7, 988, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18313, 6, 988, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18314, 5, 988, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18315, 3, 988, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18316, 4, 988, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18287, 2, 984, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18324, 7, 990, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18325, 6, 990, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18326, 5, 990, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18327, 3, 990, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (485, 7, 477, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (486, 7, 478, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (472, 7, 211, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (473, 7, 212, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (474, 7, 188, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (475, 7, 189, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (476, 7, 605, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (477, 7, 237, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (478, 7, 208, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (479, 7, 190, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (480, 7, 233, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (481, 7, 242, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (482, 7, 234, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (483, 7, 243, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (484, 7, 345, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (487, 7, 235, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (488, 7, 236, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (489, 7, 244, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (18323, 2, 990, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18328, 4, 990, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18330, 7, 991, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18331, 6, 991, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18332, 5, 991, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18333, 3, 991, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18334, 4, 991, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18336, 7, 992, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18337, 6, 992, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18338, 5, 992, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18339, 3, 992, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18340, 4, 992, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18342, 7, 993, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18343, 6, 993, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18344, 5, 993, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18345, 3, 993, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18346, 4, 993, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18348, 7, 994, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18349, 6, 994, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18350, 5, 994, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18351, 3, 994, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18352, 4, 994, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18354, 7, 995, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18355, 6, 995, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18356, 5, 995, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18357, 3, 995, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18358, 4, 995, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18360, 7, 996, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18361, 6, 996, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18362, 5, 996, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18363, 3, 996, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18364, 4, 996, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18366, 7, 997, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18367, 6, 997, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18368, 5, 997, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18369, 3, 997, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18370, 4, 997, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18372, 7, 998, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18373, 6, 998, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18374, 5, 998, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18375, 3, 998, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18376, 4, 998, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18378, 7, 999, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18379, 6, 999, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18380, 5, 999, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18381, 3, 999, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18382, 4, 999, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18384, 7, 1000, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18385, 6, 1000, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18386, 5, 1000, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18387, 3, 1000, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18388, 4, 1000, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18390, 7, 1001, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18391, 6, 1001, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18392, 5, 1001, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18393, 3, 1001, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18394, 4, 1001, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18396, 7, 1002, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18397, 6, 1002, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18398, 5, 1002, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18399, 3, 1002, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18400, 4, 1002, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18402, 7, 1003, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18403, 6, 1003, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18404, 5, 1003, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18405, 3, 1003, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18406, 4, 1003, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18408, 7, 1004, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18409, 6, 1004, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18410, 5, 1004, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18411, 3, 1004, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18412, 4, 1004, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18414, 7, 1005, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18415, 6, 1005, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18416, 5, 1005, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18417, 3, 1005, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18418, 4, 1005, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18420, 7, 1006, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18421, 6, 1006, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18422, 5, 1006, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18423, 3, 1006, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18424, 4, 1006, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18426, 7, 1007, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18427, 6, 1007, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18428, 5, 1007, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18429, 3, 1007, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18430, 4, 1007, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18432, 7, 1008, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18433, 6, 1008, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18434, 5, 1008, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18435, 3, 1008, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18436, 4, 1008, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18438, 7, 1009, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18439, 6, 1009, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18440, 5, 1009, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18441, 3, 1009, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18442, 4, 1009, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18444, 7, 1010, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18445, 6, 1010, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18446, 5, 1010, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18447, 3, 1010, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18448, 4, 1010, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18450, 7, 1011, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18451, 6, 1011, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18452, 5, 1011, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18453, 3, 1011, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18454, 4, 1011, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18456, 7, 1012, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18457, 6, 1012, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18458, 5, 1012, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18459, 3, 1012, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18460, 4, 1012, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18462, 7, 1013, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18463, 6, 1013, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18464, 5, 1013, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18465, 3, 1013, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18466, 4, 1013, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18468, 7, 1014, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18469, 6, 1014, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18470, 5, 1014, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18471, 3, 1014, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18472, 4, 1014, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18474, 7, 1015, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18475, 6, 1015, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18476, 5, 1015, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18477, 3, 1015, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18478, 4, 1015, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18480, 7, 1016, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18481, 6, 1016, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18482, 5, 1016, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18483, 3, 1016, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18484, 4, 1016, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18486, 7, 1017, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18487, 6, 1017, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18488, 5, 1017, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18489, 3, 1017, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18490, 4, 1017, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18492, 7, 1018, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18493, 6, 1018, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18494, 5, 1018, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18495, 3, 1018, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18496, 4, 1018, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18498, 7, 1019, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18499, 6, 1019, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18500, 5, 1019, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18501, 3, 1019, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18502, 4, 1019, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18504, 7, 1020, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18505, 6, 1020, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18506, 5, 1020, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18507, 3, 1020, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18508, 4, 1020, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18510, 7, 1021, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18511, 6, 1021, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18512, 5, 1021, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18513, 3, 1021, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18514, 4, 1021, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18516, 7, 1022, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18517, 6, 1022, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18518, 5, 1022, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18519, 3, 1022, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18520, 4, 1022, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18522, 7, 1023, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18523, 6, 1023, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18524, 5, 1023, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18525, 3, 1023, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18526, 4, 1023, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18528, 7, 1024, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18529, 6, 1024, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18530, 5, 1024, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18531, 3, 1024, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18532, 4, 1024, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18534, 7, 1025, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18535, 6, 1025, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18536, 5, 1025, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18537, 3, 1025, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18538, 4, 1025, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18540, 7, 1026, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18541, 6, 1026, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18542, 5, 1026, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18543, 3, 1026, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18544, 4, 1026, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18546, 7, 1027, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18547, 6, 1027, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18548, 5, 1027, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18549, 3, 1027, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18550, 4, 1027, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18552, 7, 1028, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18553, 6, 1028, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18554, 5, 1028, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18555, 3, 1028, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18556, 4, 1028, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18558, 7, 1029, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18559, 6, 1029, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18560, 5, 1029, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18561, 3, 1029, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18562, 4, 1029, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18564, 7, 1030, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18565, 6, 1030, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18566, 5, 1030, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18567, 3, 1030, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18568, 4, 1030, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18570, 7, 1031, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (9, 2, 54, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (116, 3, 258, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (14341, 3, 983, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (14342, 4, 983, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (14343, 7, 983, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (14344, 5, 983, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (14345, 6, 983, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18571, 6, 1031, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18572, 5, 1031, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18573, 3, 1031, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18574, 4, 1031, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18576, 7, 1032, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18577, 6, 1032, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (358, 6, 258, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (438, 7, 258, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (167, 4, 346, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (169, 4, 352, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (182, 4, 269, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (195, 4, 345, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (198, 4, 258, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8793, 4, 938, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8794, 4, 939, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8795, 4, 940, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8796, 4, 941, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8797, 4, 943, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (209, 4, 58, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (222, 4, 251, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (235, 4, 189, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8798, 4, 944, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8799, 4, 895, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8800, 4, 934, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (18578, 5, 1032, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18579, 3, 1032, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18580, 4, 1032, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8801, 4, 935, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8802, 4, 936, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8803, 3, 938, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8804, 3, 939, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8805, 3, 940, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8806, 3, 941, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8807, 3, 943, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8808, 3, 934, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8809, 3, 935, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8810, 3, 936, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (159, 3, 237, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8811, 3, 944, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8812, 3, 895, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (252, 5, 351, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (265, 5, 205, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (278, 5, 258, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (279, 5, 308, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8813, 5, 938, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8814, 5, 939, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8815, 5, 940, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8816, 5, 941, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8817, 5, 943, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (292, 5, 256, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (305, 5, 247, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (307, 5, 248, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (308, 5, 245, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (309, 5, 231, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (310, 5, 207, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (311, 5, 213, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (312, 5, 211, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (313, 5, 212, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (314, 5, 188, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (315, 5, 189, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (317, 5, 233, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8818, 5, 944, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8819, 5, 895, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8820, 5, 934, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8821, 5, 935, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8822, 5, 936, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (330, 6, 349, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (343, 6, 270, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (356, 6, 228, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8823, 6, 938, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8824, 6, 939, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8825, 6, 940, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8826, 6, 941, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8827, 6, 943, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (370, 6, 76, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (383, 6, 249, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (396, 6, 605, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8828, 6, 944, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8829, 6, 895, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8830, 6, 934, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8831, 6, 935, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8832, 6, 936, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (409, 7, 352, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (422, 7, 269, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (435, 7, 230, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8833, 7, 938, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8834, 7, 939, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8835, 7, 940, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8836, 7, 941, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8837, 7, 943, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (449, 7, 58, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (462, 7, 251, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (464, 7, 250, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (465, 7, 247, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (466, 7, 254, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (467, 7, 248, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (468, 7, 245, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (469, 7, 231, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (470, 7, 207, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (471, 7, 213, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8838, 7, 934, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8839, 7, 935, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8840, 7, 936, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8841, 7, 944, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (8842, 7, 895, '-1', '-1', '-1', '-1');
INSERT INTO aros_acos VALUES (19793, 5, 1033, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19794, 7, 1033, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19795, 6, 1033, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19796, 3, 1033, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19797, 4, 1033, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19799, 5, 1034, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19800, 7, 1034, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19801, 6, 1034, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19802, 3, 1034, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19803, 4, 1034, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19805, 5, 1035, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19806, 7, 1035, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19807, 6, 1035, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19808, 3, 1035, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19809, 4, 1035, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19811, 5, 1036, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19812, 7, 1036, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19813, 6, 1036, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19814, 3, 1036, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19815, 4, 1036, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19817, 5, 1037, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19818, 7, 1037, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19819, 6, 1037, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19820, 3, 1037, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19821, 4, 1037, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19823, 5, 1038, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19824, 7, 1038, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19825, 6, 1038, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19826, 3, 1038, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19827, 4, 1038, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19829, 5, 1039, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19830, 7, 1039, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19831, 6, 1039, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19832, 3, 1039, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19833, 4, 1039, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19835, 5, 1040, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19836, 7, 1040, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19837, 6, 1040, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19838, 3, 1040, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19839, 4, 1040, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19841, 5, 1041, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19842, 7, 1041, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19843, 6, 1041, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19844, 3, 1041, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19845, 4, 1041, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19847, 5, 1042, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19848, 7, 1042, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19849, 6, 1042, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19850, 3, 1042, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19792, 2, 1033, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19798, 2, 1034, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19804, 2, 1035, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19810, 2, 1036, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19816, 2, 1037, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19822, 2, 1038, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19828, 2, 1039, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19834, 2, 1040, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19840, 2, 1041, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19846, 2, 1042, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19851, 4, 1042, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19853, 5, 1043, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19854, 7, 1043, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19855, 6, 1043, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19856, 3, 1043, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19857, 4, 1043, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19859, 5, 1044, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19860, 7, 1044, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19861, 6, 1044, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19862, 3, 1044, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19863, 4, 1044, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19865, 5, 1045, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19866, 7, 1045, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19867, 6, 1045, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19868, 3, 1045, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19869, 4, 1045, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19871, 5, 1046, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19872, 7, 1046, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19873, 6, 1046, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19874, 3, 1046, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19875, 4, 1046, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19877, 5, 1047, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19878, 7, 1047, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19879, 6, 1047, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19880, 3, 1047, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19881, 4, 1047, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19883, 5, 1048, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19884, 7, 1048, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19885, 6, 1048, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19886, 3, 1048, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19887, 4, 1048, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19889, 5, 1049, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19890, 7, 1049, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19891, 6, 1049, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19892, 3, 1049, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19893, 4, 1049, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19895, 5, 1050, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19896, 7, 1050, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19897, 6, 1050, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19898, 3, 1050, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19899, 4, 1050, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19901, 5, 1051, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19902, 7, 1051, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19903, 6, 1051, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19904, 3, 1051, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19905, 4, 1051, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19907, 5, 1052, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19908, 7, 1052, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19909, 6, 1052, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19910, 3, 1052, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19911, 4, 1052, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19913, 5, 1053, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19914, 7, 1053, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19915, 6, 1053, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19916, 3, 1053, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19917, 4, 1053, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19919, 5, 1054, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19920, 7, 1054, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19921, 6, 1054, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19922, 3, 1054, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19923, 4, 1054, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19925, 5, 1055, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19926, 7, 1055, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19927, 6, 1055, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19928, 3, 1055, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19929, 4, 1055, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19931, 5, 1056, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19932, 7, 1056, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19933, 6, 1056, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19934, 3, 1056, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19935, 4, 1056, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19937, 5, 1057, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19938, 7, 1057, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19939, 6, 1057, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19940, 3, 1057, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19941, 4, 1057, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19943, 5, 1058, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19944, 7, 1058, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19945, 6, 1058, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19946, 3, 1058, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19947, 4, 1058, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19949, 5, 1059, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19950, 7, 1059, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19951, 6, 1059, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19952, 3, 1059, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19953, 4, 1059, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19955, 5, 1060, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19956, 7, 1060, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19957, 6, 1060, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19958, 3, 1060, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19959, 4, 1060, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19961, 5, 1061, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19962, 7, 1061, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19963, 6, 1061, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19964, 3, 1061, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19965, 4, 1061, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19967, 5, 1062, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19968, 7, 1062, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19969, 6, 1062, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19970, 3, 1062, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19971, 4, 1062, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19973, 5, 1063, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19974, 7, 1063, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19975, 6, 1063, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19976, 3, 1063, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19977, 4, 1063, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19979, 5, 1064, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19980, 7, 1064, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19981, 6, 1064, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19982, 3, 1064, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19983, 4, 1064, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19985, 5, 1065, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19986, 7, 1065, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19987, 6, 1065, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19988, 3, 1065, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19989, 4, 1065, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19852, 2, 1043, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19858, 2, 1044, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19864, 2, 1045, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19870, 2, 1046, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19876, 2, 1047, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19882, 2, 1048, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19888, 2, 1049, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19894, 2, 1050, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19900, 2, 1051, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19906, 2, 1052, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19912, 2, 1053, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19918, 2, 1054, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19924, 2, 1055, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19930, 2, 1056, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19936, 2, 1057, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19942, 2, 1058, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19948, 2, 1059, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19954, 2, 1060, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20066, 5, 1066, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20067, 7, 1066, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20068, 6, 1066, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20069, 3, 1066, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20070, 4, 1066, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20078, 5, 1068, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20079, 7, 1068, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20080, 6, 1068, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20081, 3, 1068, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20082, 4, 1068, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20084, 5, 1069, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20085, 7, 1069, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20086, 6, 1069, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20087, 3, 1069, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20088, 4, 1069, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20065, 2, 1066, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20077, 2, 1068, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20083, 2, 1069, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20208, 1, 2, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20335, 5, 1071, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20336, 7, 1071, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20337, 6, 1071, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20338, 3, 1071, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20339, 4, 1071, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18293, 2, 985, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18299, 2, 986, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18305, 2, 987, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18311, 2, 988, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18329, 2, 991, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18335, 2, 992, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18341, 2, 993, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18347, 2, 994, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18353, 2, 995, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18359, 2, 996, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18365, 2, 997, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18371, 2, 998, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18377, 2, 999, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18383, 2, 1000, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18389, 2, 1001, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18395, 2, 1002, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18401, 2, 1003, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18407, 2, 1004, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18413, 2, 1005, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18419, 2, 1006, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18425, 2, 1007, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18431, 2, 1008, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18437, 2, 1009, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18443, 2, 1010, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18449, 2, 1011, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18455, 2, 1012, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18461, 2, 1013, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18467, 2, 1014, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18473, 2, 1015, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18479, 2, 1016, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18485, 2, 1017, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18491, 2, 1018, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (2, 2, 344, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18497, 2, 1019, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18503, 2, 1020, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18509, 2, 1021, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18515, 2, 1022, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18521, 2, 1023, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18527, 2, 1024, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18533, 2, 1025, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18539, 2, 1026, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18545, 2, 1027, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18551, 2, 1028, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (3, 2, 105, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (4, 2, 729, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (5, 2, 368, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (6, 2, 675, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (7, 2, 227, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8, 2, 306, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (10, 2, 519, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (11, 2, 476, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (12, 2, 72, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (13, 2, 574, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (14, 2, 596, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (15, 2, 658, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (16, 2, 754, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18, 2, 437, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19, 2, 325, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20, 2, 21, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (21, 2, 456, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (22, 2, 409, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (23, 2, 391, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (24, 2, 423, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (25, 2, 205, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (26, 2, 187, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (28, 2, 285, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (29, 2, 557, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (31, 2, 268, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (32, 2, 269, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (33, 2, 230, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (34, 2, 228, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (35, 2, 229, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (36, 2, 253, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (38, 2, 308, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (39, 2, 309, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (40, 2, 310, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (41, 2, 307, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (42, 2, 311, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (43, 2, 260, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (45, 2, 259, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (46, 2, 523, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (47, 2, 58, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (48, 2, 76, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (49, 2, 522, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (50, 2, 256, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (51, 2, 257, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (52, 2, 270, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (53, 2, 57, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (54, 2, 261, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (55, 2, 477, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18557, 2, 1029, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18563, 2, 1030, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18569, 2, 1031, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (56, 2, 478, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (57, 2, 241, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (58, 2, 232, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (59, 2, 246, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (60, 2, 251, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (61, 2, 249, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (62, 2, 250, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (63, 2, 247, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (64, 2, 254, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (65, 2, 248, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (66, 2, 245, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (67, 2, 265, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (68, 2, 236, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (69, 2, 207, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (70, 2, 208, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (71, 2, 213, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (72, 2, 211, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (73, 2, 212, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (74, 2, 188, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (75, 2, 190, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (76, 2, 189, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (77, 2, 605, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (78, 2, 231, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (79, 2, 237, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (80, 2, 233, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (81, 2, 242, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (82, 2, 234, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (83, 2, 243, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20334, 2, 1071, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20305, 6, 2, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (84, 2, 345, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (14340, 2, 983, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (18575, 2, 1032, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (1, 2, 90, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (17, 2, 636, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (30, 2, 161, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8780, 2, 537, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8781, 2, 962, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8782, 2, 945, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8783, 2, 896, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8784, 2, 915, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (37, 2, 258, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8785, 2, 938, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8786, 2, 939, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8787, 2, 940, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8788, 2, 941, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8789, 2, 943, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (44, 2, 255, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8790, 2, 934, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8791, 2, 935, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (8792, 2, 936, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19960, 2, 1061, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19966, 2, 1062, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19972, 2, 1063, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19978, 2, 1064, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (19984, 2, 1065, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20610, 1, 1071, '1', '1', '1', '0');
INSERT INTO aros_acos VALUES (20724, 2, 1072, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20725, 5, 1072, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20726, 7, 1072, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20727, 6, 1072, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20728, 3, 1072, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20729, 4, 1072, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20730, 2, 1073, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20731, 5, 1073, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20732, 7, 1073, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20733, 6, 1073, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20734, 3, 1073, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20735, 4, 1073, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20736, 2, 1074, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20737, 5, 1074, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20738, 7, 1074, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20739, 6, 1074, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20740, 3, 1074, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20741, 4, 1074, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20742, 2, 1075, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20743, 3, 1075, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20744, 5, 1075, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20745, 7, 1075, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20746, 6, 1075, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20747, 4, 1075, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20748, 2, 1076, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20749, 3, 1076, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20750, 5, 1076, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20751, 7, 1076, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20752, 6, 1076, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20753, 4, 1076, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20754, 2, 1077, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20755, 3, 1077, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20756, 5, 1077, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20757, 7, 1077, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20758, 6, 1077, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20759, 4, 1077, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20760, 2, 1078, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20761, 3, 1078, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20762, 5, 1078, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20763, 7, 1078, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20764, 6, 1078, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20765, 4, 1078, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20766, 2, 1079, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20767, 3, 1079, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20768, 5, 1079, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20769, 7, 1079, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20770, 6, 1079, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20771, 4, 1079, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20772, 2, 1080, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20773, 3, 1080, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20774, 5, 1080, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20775, 7, 1080, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20776, 6, 1080, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20777, 4, 1080, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20778, 2, 1081, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20779, 3, 1081, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20780, 5, 1081, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20781, 7, 1081, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20782, 6, 1081, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20783, 4, 1081, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20784, 2, 1082, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20785, 3, 1082, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20786, 5, 1082, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20787, 7, 1082, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20788, 6, 1082, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20789, 4, 1082, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20790, 2, 1083, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20791, 3, 1083, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20792, 5, 1083, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20793, 7, 1083, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20794, 6, 1083, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20795, 4, 1083, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20796, 2, 1084, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20797, 3, 1084, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20798, 5, 1084, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20799, 7, 1084, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20800, 6, 1084, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20801, 4, 1084, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20802, 2, 1085, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20803, 3, 1085, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20804, 5, 1085, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20805, 7, 1085, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20806, 6, 1085, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20807, 4, 1085, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20808, 2, 1086, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20809, 3, 1086, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20810, 5, 1086, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20811, 7, 1086, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20812, 6, 1086, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20813, 4, 1086, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20814, 2, 1087, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20815, 3, 1087, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20816, 5, 1087, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20817, 7, 1087, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20818, 6, 1087, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20819, 4, 1087, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20820, 2, 1088, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20821, 3, 1088, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20822, 5, 1088, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20823, 7, 1088, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20824, 6, 1088, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20825, 4, 1088, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20826, 2, 1089, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20827, 3, 1089, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20828, 5, 1089, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20829, 7, 1089, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20830, 6, 1089, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20831, 4, 1089, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20832, 2, 1090, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20833, 3, 1090, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20834, 5, 1090, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20835, 7, 1090, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20836, 6, 1090, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20837, 4, 1090, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20838, 2, 1091, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20839, 3, 1091, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20840, 5, 1091, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20841, 7, 1091, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20842, 6, 1091, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20843, 4, 1091, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20297, 5, 2, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20302, 7, 2, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20309, 3, 2, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20325, 4, 2, '1', '1', '1', '1');
INSERT INTO aros_acos VALUES (20327, 2, 2, '1', '1', '1', '1');


--
-- Name: aros_acos_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('aros_acos_id_seq', 20844, true);


--
-- Name: aros_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('aros_id_seq', 125, true);


--
-- Data for Name: bannettes; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO bannettes VALUES (1, 'docs');
INSERT INTO bannettes VALUES (2, 'refus');
INSERT INTO bannettes VALUES (3, 'reponses');
INSERT INTO bannettes VALUES (4, 'flux');
INSERT INTO bannettes VALUES (5, 'aiguillage');
INSERT INTO bannettes VALUES (6, 'infos');
INSERT INTO bannettes VALUES (7, 'copy');


--
-- Name: bannettes_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('bannettes_id_seq', 8, true);


--
-- Data for Name: connecteurs; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO connecteurs VALUES (1, 'Parapheur électronique (Signature)', '2014-08-05 13:53:10.780508', '2014-08-05 13:53:10.780508');
INSERT INTO connecteurs VALUES (2, 'GED (CMIS)', '2015-06-08 11:58:03.398832', '2015-06-08 11:58:03.398832');


--
-- Name: connecteurs_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('connecteurs_id_seq', 3, true);


--
-- Data for Name: dacos; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO dacos VALUES (1, NULL, NULL, NULL, 'data', 1, 2);
--
-- Name: dacos_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('dacos_id_seq', 2, true);


--
-- Data for Name: formatsreponse; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO formatsreponse VALUES (1, 'Modèle de document');
INSERT INTO formatsreponse VALUES (2, 'Fichier son (.wav)');
INSERT INTO formatsreponse VALUES (3, 'Fichier vidéo (.mpeg)');
INSERT INTO formatsreponse VALUES (4, 'SMS');
INSERT INTO formatsreponse VALUES (5, 'Email');


--
-- Name: formatsreponse_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('formatsreponse_id_seq', 6, true);


--
-- Data for Name: groups; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO groups VALUES (1, 'Superadmin', '2012-02-17 14:55:14.842819', '2012-02-17 14:55:14.842819', true, true, NULL);
INSERT INTO groups VALUES (2, 'Admin', '2012-02-17 15:15:43.236166', '2012-02-17 15:15:43.236166', true, true, NULL);
INSERT INTO groups VALUES (3, 'Initiateur', '2012-02-17 15:15:43.260401', '2012-02-17 15:15:43.260401', true, true, NULL);
INSERT INTO groups VALUES (4, 'Valideur', '2012-02-17 15:15:43.277671', '2012-02-17 15:15:43.277671', true, true, NULL);
INSERT INTO groups VALUES (5, 'Valideur Editeur', '2012-02-17 15:15:43.296607', '2012-02-17 15:15:43.296607', true, true, NULL);
INSERT INTO groups VALUES (6, 'Documentaliste', '2012-02-17 15:15:43.310943', '2012-02-17 15:15:43.310943', true, true, NULL);
INSERT INTO groups VALUES (7, 'Aiguilleur', '2012-07-06 11:51:30.769035', '2012-07-06 11:51:30.769035', true, true, NULL);


--
-- Name: groups_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('groups_id_seq', 9, true);


--
-- Data for Name: keywordtypes; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO keywordtypes VALUES (1, 'corpname', 'Collectivité', '2012-02-28 11:38:20.054222', '2012-02-28 11:38:20.054222');
INSERT INTO keywordtypes VALUES (2, 'famname', 'Nom de famille', '2012-02-28 11:38:20.088158', '2012-02-28 11:38:20.088158');
INSERT INTO keywordtypes VALUES (3, 'geogname', 'Nom géographique', '2012-02-28 11:38:20.096135', '2012-02-28 11:38:20.096135');
INSERT INTO keywordtypes VALUES (4, 'name', 'Nom', '2012-02-28 11:38:20.105176', '2012-02-28 11:38:20.105176');
INSERT INTO keywordtypes VALUES (5, 'occupation', 'Fonction', '2012-02-28 11:38:20.113056', '2012-02-28 11:38:20.113056');
INSERT INTO keywordtypes VALUES (6, 'persname', 'Nom de personne', '2012-02-28 11:38:20.12103', '2012-02-28 11:38:20.12103');
INSERT INTO keywordtypes VALUES (7, 'subject', 'Mot-matière', '2012-02-28 11:38:20.129994', '2012-02-28 11:38:20.129994');
INSERT INTO keywordtypes VALUES (8, 'genreform', 'Type de document', '2012-02-28 11:38:20.138126', '2012-02-28 11:38:20.138126');
INSERT INTO keywordtypes VALUES (9, 'function', 'Activité', '2012-02-28 11:38:20.146106', '2012-02-28 11:38:20.146106');


--
-- Data for Name: keywordlists; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO keywordlists VALUES (3, 'Liste d''autorité « Contexte historique »', '\x5365727669636520696e7465726d696e697374c3a97269656c20646573204172636869766573206465204672616e6365203a20766f636162756c6169726520636f6e7472c3b46cc3a92064657320636f6e74657874657320686973746f726971756573', 1, true, 7, 'LISTE D''AUTORITE « CONTEXTE HISTORIQUE »', 'Liste d''autorité « Contexte historique »', 'Service interministériel des Archives de France', '2010-04-21', 'http://www.archivesdefrance.culture.gouv.fr/thesaurus/data/Contexte?includeSchemes=true', 'http://www.archivesdefrance.culture.gouv.fr/thesaurus/resource/Contexte', 1, 1, '2011-05-25 10:09:29', '2011-05-25 10:09:29');
INSERT INTO keywordlists VALUES (4, 'Liste d''autorités « Typologie documentaire »', '\x5365727669636520696e7465726d696e697374c3a97269656c20646573204172636869766573206465204672616e6365203a20766f636162756c6169726520636f6e7472c3b46cc3a9206465206c61207479706f6c6f67696520646f63756d656e74616972652064616e73206c6573206172636869766573', 1, true, 7, 'LISTE D''AUTORITES « TYPOLOGIE DOCUMENTAIRE »', 'Liste d''autorités « Typologie documentaire »', 'Service interministériel des Archives de France', '2011-04-18', 'http://www.archivesdefrance.culture.gouv.fr/thesaurus/data/Typologie?includeSchemes=true', 'http://www.archivesdefrance.culture.gouv.fr/thesaurus/resource/Typologie', 1, 1, '2011-05-25 10:13:13', '2011-05-25 10:13:13');
INSERT INTO keywordlists VALUES (2, 'Liste d''autorités « Actions »', '\x5365727669636520696e7465726d696e697374c3a97269656c20646573204172636869766573206465204672616e6365203a20766f636162756c6169726520636f6e7472c3b46cc3a92064657320616374696f6e732064616e73206c6573206172636869766573', 1, true, 7, 'LISTE D''AUTORITES « ACTIONS »', 'Liste d''autorités « Actions »', 'Service interministériel des Archives de France', '2011-04-07', 'http://www.archivesdefrance.culture.gouv.fr/thesaurus/data/Actions?includeSchemes=true', 'http://www.archivesdefrance.culture.gouv.fr/thesaurus/resource/Actions', 1, 1, '2011-05-25 10:06:25', '2011-05-25 10:14:22');
INSERT INTO keywordlists VALUES (1, 'Thésaurus-matières', '\x5365727669636520696e7465726d696e697374c3a97269656c20646573204172636869766573206465204672616e6365203a206465736372697074696f6e20647520766f636162756c61697265', 1, false, 7, 'THESAURUS-MATIERES', 'Thésaurus-matières', 'Service interministériel des Archives de France', '2011-04-19', 'http://www.archivesdefrance.culture.gouv.fr/thesaurus/data/Matiere?includeSchemes=true', 'http://www.archivesdefrance.culture.gouv.fr/thesaurus/resource/Matiere', 1, 1, '2015-10-14 07:54:57.221605', '2015-10-14 07:54:57.221605');


--
-- Name: keywordlists_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('keywordlists_id_seq', 5, true);


--
-- Data for Name: keywords; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO keywords VALUES (2, 1, 'T1-543', '\xc3a9636f6e6f6d6965', NULL, 3, 4, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (1814, 1, 'T3-224', '\x7461726966', NULL, 3627, 3628, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1826, 1, 'T3-221', '\x7374617475742064276173736f63696174696f6e', NULL, 3651, 3652, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (297, 1, 'T1-90', '\x66697363616c6974c3a92070c3a974726f6c69657265', NULL, 593, 594, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (1827, 1, 'T3-51', '\x636172746f6e206427696e7669746174696f6e', NULL, 3653, 3654, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (424, 1, 'T1-1219', '\x696d6d6575626c65206465206772616e64652068617574657572', NULL, 847, 848, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (620, 1, 'T1-1173', '\x6c6f69736972', NULL, 1239, 1240, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (1126, 1, 'T1-743', '\x76696520706173746f72616c65', NULL, 2251, 2252, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1777, 1, 'T3-150', '\x6f75767261676520696d7072696dc3a9', NULL, 3553, 3554, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1746, 1, 'T3-44', '\x6361727465206427c3a96c656374657572', NULL, 3491, 3492, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1327, 1, 'T1-1440', '\x766f6c61696c6c65', NULL, 2653, 2654, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (546, 1, 'T1-1241', '\x73616c6c6520646520737065637461636c6573', NULL, 1091, 1092, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (82, 1, 'T1-21', '\x63c3a972c3a96d6f6e6965207075626c69717565', NULL, 163, 164, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (579, 1, 'T1-237', '\x61756265726765206465206a65756e65737365', NULL, 1157, 1158, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (372, 1, 'T1-484', '\x70617472696d6f696e6520736369656e7469666971756520657420746563686e69717565', NULL, 743, 744, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (250, 1, 'T1-80', '\x6469736369706c696e652070c3a96e6974656e746961697265', NULL, 499, 500, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (218, 1, 'T1-284', '\x696e63656e646965', NULL, 435, 436, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (219, 1, 'T1-698', '\x636174617374726f706865206e61747572656c6c65', NULL, 437, 438, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (701, 1, 'T1-325', '\x666f72c3aa7420636f6d6d756e616c65', NULL, 1401, 1402, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (889, 1, 'T1-1012', '\x6173737572616e6365206d617465726e6974c3a9', NULL, 1777, 1778, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (261, 1, 'T1-85', '\x656d706c6f6920c3a02074656d7073207061727469656c', NULL, 521, 522, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (302, 1, 'T1-1315', '\xc3a96e65726769652072656e6f7576656c61626c65', NULL, 603, 604, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (373, 1, 'T1-226', '\x6f6575767265206427617274', NULL, 745, 746, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (162, 1, 'T1-892', '\x6a75737469636520636976696c65', NULL, 323, 324, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (301, 1, 'T1-92', '\xc3a96e65726769652067c3a96f746865726d69717565', NULL, 601, 602, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (1569, 1, 'T4-68', '\x6d6f7576656d656e7420646573204c756d69c3a8726573', NULL, 3137, 3138, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (750, 1, 'T1-357', '\x73747275637475726520636f6d6d756e616c652064276169646520736f6369616c65', NULL, 1499, 1500, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (477, 1, 'T1-1264', '\x73c3a96375726974c3a9206475207472617661696c', NULL, 953, 954, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (486, 1, 'T1-182', '\x64c3a963656e7472616c69736174696f6e20696e647573747269656c6c65', NULL, 971, 972, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (970, 1, 'T1-514', '\x617274206472616d617469717565', NULL, 1939, 1940, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1212, 1, 'T1-792', '\x696e737472756374696f6e2063697669717565', NULL, 2423, 2424, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1114, 1, 'T1-1120', '\x766f6965207069c3a9746f6e6e65', NULL, 2227, 2228, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (240, 1, 'T1-75', '\x666f726d6174696f6e206d75736963616c65', NULL, 479, 480, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (165, 1, 'T1-48', '\x6f757672616765206427617274', NULL, 329, 330, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (1816, 1, 'T3-36', '\x62756c6c6574696e20646520766f7465', NULL, 3631, 3632, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (263, 1, 'T1-86', '\xc3a96475636174696f6e20706f70756c61697265', NULL, 525, 526, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (288, 1, 'T1-803', '\x6d6f7576656d656e74206175746f6e6f6d69737465', NULL, 575, 576, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (682, 1, 'T1-297', '\x636972717565', NULL, 1363, 1364, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (1061, 1, 'T1-708', '\x636972636f6e736372697074696f6e2066697363616c65', NULL, 2121, 2122, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1084, 1, 'T1-623', '\x64726f69747320646f6d616e69617578206427416e6369656e2052c3a967696d65', NULL, 2167, 2168, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1110, 1, 'T1-657', '\x6167656e74206e6f6e20746974756c61697265', NULL, 2219, 2220, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (738, 1, 'T1-476', '\xc389746174', NULL, 1475, 1476, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (1288, 1, 'T1-1342', '\x7465727261696e2064652063616d70696e67', NULL, 2575, 2576, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1122, 1, 'T1-1399', '\x707570696c6c65206465206c61206e6174696f6e', NULL, 2243, 2244, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1149, 1, 'T1-701', '\x736f6369c3a974c3a9206427c3a9636f6e6f6d6965206d69787465', NULL, 2297, 2298, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1678, 1, 'T3-170', '\x70726f63c3a8732d76657262616c', NULL, 3355, 3356, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1006, 1, 'T1-627', '\x70696c6f7465', NULL, 2011, 2012, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (711, 1, 'T1-336', '\x617373697374616e636520c3a96475636174697665', NULL, 1421, 1422, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (712, 1, 'T1-824', '\x70726f74656374696f6e206a756469636961697265206465206c61206a65756e65737365', NULL, 1423, 1424, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (723, 1, 'T1-959', '\x626c6f637573', NULL, 1445, 1446, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (725, 1, 'T1-630', '\x706f6c696365206465732066726f6e7469c3a8726573', NULL, 1449, 1450, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (749, 1, 'T1-356', '\x70616c616973206465206a757374696365', NULL, 1497, 1498, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (994, 1, 'T1-1201', '\x6d617269616765', NULL, 1987, 1988, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1140, 1, 'T1-1288', '\x65617520706c757669616c65', NULL, 2279, 2280, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (247, 1, 'T1-78', '\x6d696c6974616e7420706f6c697469717565', NULL, 493, 494, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (1530, 1, 'T4-128', '\x7472616974c3a9206465205665727361696c6c657320283139313929', NULL, 3059, 3060, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1300, 1, 'T1-1445', '\x73746174696f6e20726164696f20c3a96c65637472697175652070726976c3a965', NULL, 2599, 2600, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1331, 1, 'T1-1023', '\x67617a206e61747572656c', NULL, 2661, 2662, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (292, 1, 'T1-752', '\x696d7072696d657572', NULL, 583, 584, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (769, 1, 'T1-1283', '\x73c3bb726574c3a9206d6f62696c69c3a87265', NULL, 1537, 1538, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (1539, 1, 'T4-97', '\x63726973652070c3a974726f6c69c3a872652028313937332d3139373929', NULL, 3077, 3078, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1812, 1, 'T3-228', '\x74697472652064652063697263756c6174696f6e', NULL, 3623, 3624, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (293, 1, 'T1-1212', '\xc3a9646974657572', NULL, 585, 586, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (1813, 1, 'T3-12', '\x61726d6f7269616c', NULL, 3625, 3626, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1219, 1, 'T1-794', '\x68c3b474656c206465206c6120736f75732d7072c3a966656374757265', NULL, 2437, 2438, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1367, 1, 'T1-1161', '\x636f6d6d616e6465207075626c69717565', NULL, 2733, 2734, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (751, 1, 'T1-358', '\x7465727261696e206d696c697461697265', NULL, 1501, 1502, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (320, 1, 'T1-390', '\x6875697373696572206465206a757374696365', NULL, 639, 640, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (996, 1, 'T1-537', '\x65617520706f7461626c65', NULL, 1991, 1992, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (334, 1, 'T1-860', '\xc3a97472616e676572', NULL, 667, 668, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (669, 1, 'T1-982', '\x6d616c61646965206465732076c3a967c3a974617578', NULL, 1337, 1338, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (823, 1, 'T1-410', '\x64726f69747320646520646f75616e65', NULL, 1645, 1646, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (1201, 1, 'T1-775', '\xc3a96e657267696520736f6c61697265', NULL, 2401, 2402, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1365, 1, 'T1-1451', '\xc3a9746174732070726f76696e6369617578', NULL, 2729, 2730, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1354, 1, 'T1-1142', '\x6163636964656e742073636f6c61697265', NULL, 2707, 2708, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1355, 1, 'T1-1225', '\x62617461696c6c6f6e2073636f6c61697265', NULL, 2709, 2710, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1358, 1, 'T1-1125', '\x66c3aa7465206e6174696f6e616c65', NULL, 2715, 2716, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1326, 1, 'T1-1389', '\x616c696d656e742064752062c3a97461696c', NULL, 2651, 2652, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1266, 1, 'T1-893', '\x7072c3a9736964656e74206427c3a97461626c697373656d656e74207075626c696320646520636f6f70c3a9726174696f6e20696e746572636f6d6d756e616c65', NULL, 2531, 2532, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1292, 1, 'T1-916', '\x696e74657272757074696f6e20766f6c6f6e74616972652064652067726f737365737365', NULL, 2583, 2584, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1376, 1, 'T1-1401', '\x74616fc3af736d65', NULL, 2751, 2752, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1356, 1, 'T1-1122', '\x636f6e7365696c20636f6d6d756e61757461697265', NULL, 2711, 2712, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1360, 1, 'T1-1230', '\x63656e7472652076696c6c65', NULL, 2719, 2720, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1202, 1, 'T1-776', '\x6368616d62726520636f6e73756c61697265', NULL, 2403, 2404, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (766, 1, 'T1-1250', '\x7072c3a9736964656e7420647520636f6e7365696c2067c3a96ec3a972616c', NULL, 1531, 1532, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (399, 1, 'T1-135', '\x706f6964732d65742d6d657375726573', NULL, 797, 798, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (1193, 1, 'T1-765', '\x6cc3a96769736c6174696f6e', NULL, 2385, 2386, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1364, 1, 'T1-1156', '\x6661757820656e20c3a963726974757265', NULL, 2727, 2728, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (763, 1, 'T1-364', '\x666175782074c3a96d6f69676e616765', NULL, 1525, 1526, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (999, 1, 'T1-540', '\x61726368c3a96f6c6f6769652061c3a97269656e6e65', NULL, 1997, 1998, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (950, 1, 'T1-857', '\x656e736569676e656d656e74207072c3a9c3a96cc3a96d656e7461697265', NULL, 1899, 1900, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1368, 1, 'T1-1189', '\x746f7869636f6d616e65', NULL, 2735, 2736, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1375, 1, 'T1-1367', '\x70726f74657374616e7469736d65', NULL, 2749, 2750, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1745, 1, 'T3-178', '\x70726f66657373696f6e20646520666f69', NULL, 3489, 3490, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1672, 1, 'T3-208', '\x72c3a9706572746f69726520646573206f70c3a9726174696f6e7320646520646f75616e6573', NULL, 3343, 3344, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1209, 1, 'T1-1202', '\x6175746f726974c3a92064276f636375706174696f6e', NULL, 2417, 2418, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1370, 1, 'T1-1362', '\x6d616c616469652064657320616e696d617578', NULL, 2739, 2740, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (670, 1, 'T1-461', '\x616e696d616c206e75697369626c65', NULL, 1339, 1340, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (671, 1, 'T1-290', '\x6c6f67656d656e74', NULL, 1341, 1342, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (478, 1, 'T1-177', '\x63616d7061676e6520c3a96c6563746f72616c65', NULL, 955, 956, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (1115, 1, 'T1-1340', '\xc3a9636f6c65', NULL, 2229, 2230, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (164, 1, 'T1-47', '\x66697363616c6974c3a92064697265637465206427416e6369656e2052c3a967696d65', NULL, 327, 328, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (1377, 1, 'T1-1180', '\x636f6d7061676e6965207468c3a9c3a27472616c65', NULL, 2753, 2754, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1416, 1, 'T2-1', '\x61626f6c6974696f6e', NULL, 2831, 2832, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1417, 1, 'T2-50', '\x696e64656d6e69736174696f6e', NULL, 2833, 2834, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1488, 1, 'T4-46', '\x323165207369c3a8636c65', NULL, 2975, 2976, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1329, 1, 'T1-1360', '\x62617474616765', NULL, 2657, 2658, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1330, 1, 'T1-1020', '\x6f657576726520646520677565727265', NULL, 2659, 2660, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1531, 1, 'T4-87', '\x6166666169726520447265796675732028313839342d3139303629', NULL, 3061, 3062, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1323, 1, 'T1-1006', '\xc3a9636f6c65206d696c697461697265', NULL, 2645, 2646, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (248, 1, 'T1-79', '\x66c3aa746520666f7261696e65', NULL, 495, 496, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (824, 1, 'T1-456', '\x636f6d6d6572636520657874c3a97269657572', NULL, 1647, 1648, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (556, 1, 'T1-220', '\x636f757274696572206465206d61726368616e6469736573', NULL, 1111, 1112, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (557, 1, 'T1-221', '\x6a657578206f6c796d706971756573', NULL, 1113, 1114, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (1234, 1, 'T1-821', '\x646973736964656e74', NULL, 2467, 2468, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1205, 1, 'T1-1198', '\x6dc3a963c3a96e6174', NULL, 2409, 2410, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1206, 1, 'T1-834', '\x6d616e69666573746174696f6e2063756c747572656c6c65', NULL, 2411, 2412, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1207, 1, 'T1-1231', '\x72c3a973697374616e6365', NULL, 2413, 2414, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1141, 1, 'T1-1276', '\x6e61707065206427656175', NULL, 2281, 2282, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (264, 1, 'T1-786', '\x63756c74757265', NULL, 527, 528, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (1418, 1, 'T2-21', '\x636f6e737472756374696f6e', NULL, 2835, 2836, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1328, 1, 'T1-1026', '\x6173736f6c656d656e74', NULL, 2655, 2656, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1529, 1, 'T4-111', '\x6775657272652069737261c3a96c6f2d61726162652028313934382d2029', NULL, 3057, 3058, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1290, 1, 'T1-1119', '\x617273656e616c', NULL, 2579, 2580, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (636, 1, 'T1-268', '\x6d696e65726169', NULL, 1271, 1272, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (637, 1, 'T1-269', '\x6a756966', NULL, 1273, 1274, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (1204, 1, 'T1-941', '\x6d61726368c3a9206427696e74c3a972c3aa74206e6174696f6e616c', NULL, 2407, 2408, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1128, 1, 'T1-806', '\x736167652066656d6d65', NULL, 2255, 2256, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1588, 1, 'T4-17', '\x3965207369c3a8636c65', NULL, 3175, 3176, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1131, 1, 'T1-681', '\x6f70696e696f6e207075626c69717565', NULL, 2261, 2262, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (992, 1, 'T1-532', '\x756e69766572736974c3a9', NULL, 1983, 1984, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1244, 1, 'T1-1165', '\x66697363616c6974c3a920646972656374652072c3a9766f6c7574696f6e6e61697265', NULL, 2487, 2488, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (993, 1, 'T1-666', '\x6166666169726520636976696c65', NULL, 1985, 1986, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1366, 1, 'T1-1160', '\x68c3b474656c20636f6d6d756e61757461697265', NULL, 2731, 2732, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1371, 1, 'T1-1249', '\x6169646520736f6369616c6520666163756c746174697665', NULL, 2741, 2742, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1359, 1, 'T1-1258', '\x646f6d61696e65207075626c696320666c757669616c', NULL, 2717, 2718, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (715, 1, 'T1-1010', '\x6f66666963696e6520706861726d616365757469717565', NULL, 1429, 1430, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (1361, 1, 'T1-1205', '\x62617468796dc3a974726965', NULL, 2721, 2722, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1362, 1, 'T1-1147', '\x726f7574652064c3a970617274656d656e74616c65', NULL, 2723, 2724, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1363, 1, 'T1-1380', '\x766f696520696e746572636f6d6d756e616c65', NULL, 2725, 2726, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1232, 1, 'T1-818', '\x7365637465', NULL, 2463, 2464, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (767, 1, 'T1-721', '\x636f6e7365696c6c65722067c3a96ec3a972616c', NULL, 1533, 1534, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (1236, 1, 'T1-823', '\x63617365726e65', NULL, 2471, 2472, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (768, 1, 'T1-368', '\x61646d696e697374726174696f6e2063616e746f6e616c65206427c3a9706f7175652072c3a9766f6c7574696f6e6e61697265', NULL, 1535, 1536, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (1828, 1, 'T3-49', '\x63617274652064276964656e746974c3a92070726f66657373696f6e6e656c6c65', NULL, 3655, 3656, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (794, 1, 'T1-1091', '\x6368617373657572', NULL, 1587, 1588, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (1829, 1, 'T3-152', '\x706172746974696f6e206d75736963616c65', NULL, 3657, 3658, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (316, 1, 'T1-100', '\x707570696c6c65206465206c27c389746174', NULL, 631, 632, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (658, 1, 'T1-1179', '\x63756c7465', NULL, 1315, 1316, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (1197, 1, 'T1-769', '\x636f6e7365696c20642761646d696e697374726174696f6e', NULL, 2393, 2394, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1198, 1, 'T1-1280', '\x6d696c696365732070726f76696e6369616c6573', NULL, 2395, 2396, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1658, 1, 'T3-16', '\x617272c3aa74c3a9206475207072c3a9736964656e7420647520636f6e7365696c2072c3a967696f6e616c', NULL, 3315, 3316, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1659, 1, 'T3-27', '\x62696c616e2070c3a96461676f6769717565', NULL, 3317, 3318, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1692, 1, 'T3-7', '\x61666669636865', NULL, 3383, 3384, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1593, 1, 'T4-61', '\x677565727265206465206c61204c6967756520642741756773626f7572672028313638382d3136393729', NULL, 3185, 3186, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1645, 1, 'T3-156', '\x7065726d69732064652064c3a96d6f6c6972', NULL, 3289, 3290, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1654, 1, 'T3-52', '\x63617274756c61697265', NULL, 3307, 3308, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1655, 1, 'T3-132', '\x6d617472696365206427696d706f736974696f6e', NULL, 3309, 3310, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1508, 1, 'T4-83', '\x52c3a9766f6c7574696f6e20696e647573747269656c6c65', NULL, 3015, 3016, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1270, 1, 'T1-922', '\x617373656d626cc3a965206575726f70c3a9656e6e65', NULL, 2539, 2540, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (604, 1, 'T1-441', '\x7375727665696c6c616e6365206465732062c3a274696d656e7473', NULL, 1207, 1208, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (995, 1, 'T1-1233', '\x656e7472657072697365206465207472617661696c2074656d706f7261697265', NULL, 1989, 1990, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1594, 1, 'T4-60', '\x67756572726520646520486f6c6c616e64652028313637322d3136373829', NULL, 3187, 3188, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (873, 1, 'T1-451', '\x6175746f726f757465', NULL, 1745, 1746, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (1741, 1, 'T3-102', '\x657461742064652073656374696f6e', NULL, 3481, 3482, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1666, 1, 'T3-212', '\x72c3b46c65', NULL, 3331, 3332, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1667, 1, 'T3-129', '\x6d61726368c3a9207075626c6963', NULL, 3333, 3334, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1665, 1, 'T3-192', '\x7265676973747265206427617070656c', NULL, 3329, 3330, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1171, 1, 'T1-1185', '\x7a6f6e65206465206d6172616973', NULL, 2341, 2342, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (300, 1, 'T1-739', '\x657370616365206e61747572656c2073656e7369626c65', NULL, 599, 600, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (635, 1, 'T1-1141', '\xc3a9636f6c6520646520636f6e6475697465', NULL, 1269, 1270, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (267, 1, 'T1-110', '\x63656e747269736d65', NULL, 533, 534, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (949, 1, 'T1-952', '\x656e736569676e656d656e742070726976c3a9', NULL, 1897, 1898, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1200, 1, 'T1-774', '\x686162697461742070c3a9726975726261696e', NULL, 2399, 2400, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (296, 1, 'T1-672', '\x70726f66657373696f6e206dc3a9646963616c65', NULL, 591, 592, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (951, 1, 'T1-1349', '\x6f7269656e746174696f6e2073636f6c61697265', NULL, 1901, 1902, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (765, 1, 'T1-367', '\x636f6e7365696c2067c3a96ec3a972616c', NULL, 1529, 1530, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (1235, 1, 'T1-822', '\x6772c3a26365', NULL, 2469, 2470, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1563, 1, 'T4-30', '\x313865207369c3a8636c65', NULL, 3125, 3126, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1567, 1, 'T4-63', '\x677565727265206465205472656e746520416e732028313631382d3136343829', NULL, 3133, 3134, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1566, 1, 'T4-10', '\x416e746971756974c3a9206761756c6f697365', NULL, 3131, 3132, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1568, 1, 'T4-66', '\x6775657272652064652053756363657373696f6e20642741757472696368652028313734302d3137343829', NULL, 3135, 3136, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1589, 1, 'T4-22', '\x313465207369c3a8636c65', NULL, 3177, 3178, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (979, 1, 'T1-1190', '\x736f6369c3a974c3a920636f6d6d65726369616c65', NULL, 1957, 1958, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1466, 1, 'T2-47', '\x67657374696f6e20636f6d707461626c65', NULL, 2931, 2932, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (747, 1, 'T1-354', '\x66696e616e63657320696e746572636f6d6d756e616c6573', NULL, 1493, 1494, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (1635, 1, 'T3-214', '\x72c3b46c65206427696d706f736974696f6e', NULL, 3269, 3270, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1752, 1, 'T3-155', '\x7065726d697320646520636f6e73747275697265', NULL, 3503, 3504, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1008, 1, 'T1-1050', '\x63697263756c6174696f6e20726f757469c3a87265', NULL, 2015, 2016, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1029, 1, 'T1-1072', '\x6d696e6575722064c3a96c696e7175616e74', NULL, 2057, 2058, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1035, 1, 'T1-561', '\x6167656e7420736569676e65757269616c', NULL, 2069, 2070, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1668, 1, 'T3-231', '\x7472616374', NULL, 3335, 3336, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1030, 1, 'T1-848', '\x696e73657274696f6e2070726f66657373696f6e6e656c6c65', NULL, 2059, 2060, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1669, 1, 'T3-59', '\x6368726f6e6f', NULL, 3337, 3338, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1199, 1, 'T1-799', '\x63696d657469c3a87265206d696c697461697265', NULL, 2397, 2398, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1127, 1, 'T1-1032', '\x617578696c6961697265206dc3a9646963616c', NULL, 2253, 2254, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (997, 1, 'T1-565', '\x68796769c3a86e6520616c696d656e7461697265', NULL, 1993, 1994, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (998, 1, 'T1-539', '\x696e717569736974696f6e', NULL, 1995, 1996, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (627, 1, 'T1-261', '\x70617472696d6f696e652070726976c3a9', NULL, 1253, 1254, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (295, 1, 'T1-89', '\x6dc3a9646563696e', NULL, 589, 590, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (1211, 1, 'T1-791', '\x7265636f757672656d656e742066697363616c', NULL, 2421, 2422, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1111, 1, 'T1-659', '\x64c3a96c69742064277573616765', NULL, 2221, 2222, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1664, 1, 'T3-179', '\x70726f6772616d6d65', NULL, 3327, 3328, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (265, 1, 'T1-1407', '\x756e69766572736974c3a920706f70756c61697265', NULL, 529, 530, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (1210, 1, 'T1-1274', '\x70726f706167616e6465', NULL, 2419, 2420, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (771, 1, 'T1-1294', '\x636f6d70746162696c6974c3a9206427656e7472657072697365', NULL, 1541, 1542, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (1554, 1, 'T4-33', '\x417373656d626cc3a965206cc3a96769736c61746976652028313739312d3137393229', NULL, 3107, 3108, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1580, 1, 'T4-56', '\x52c3a9666f726d652070726f74657374616e7465', NULL, 3159, 3160, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (13, 1, 'T1-764', '\x706f70756c6174696f6e20727572616c65', NULL, 25, 26, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (158, 1, 'T1-1299', '\x70726f74656374696f6e20736f6369616c65', NULL, 315, 316, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (262, 1, 'T1-696', '\x656d706c6f69', NULL, 523, 524, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (1590, 1, 'T4-71', '\x63616d7061676e652064652052757373696520283138313229', NULL, 3179, 3180, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1591, 1, 'T4-70', '\x63616d7061676e65206465204672616e636520283138313429', NULL, 3181, 3182, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1608, 1, 'T3-203', '\x72c3a8676c656d656e74206427656e7472657072697365', NULL, 3215, 3216, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1000, 1, 'T1-648', '\x61726368c3a96f6c6f676965', NULL, 1999, 2000, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1527, 1, 'T4-118', '\x4f636375706174696f6e20616c6c656d616e64652028313934302d3139343429', NULL, 3053, 3054, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1372, 1, 'T1-1425', '\x6d6172c3a9636861757373c3a965', NULL, 2743, 2744, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1373, 1, 'T1-1200', '\x64697374696e6374696f6e20686f6e6f72696669717565', NULL, 2745, 2746, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1374, 1, 'T1-1175', '\x626f7564646869736d65', NULL, 2747, 2748, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1208, 1, 'T1-830', '\x636f6c6c61626f726174696f6e', NULL, 2415, 2416, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (298, 1, 'T1-647', '\x687964726f63617262757265', NULL, 595, 596, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (722, 1, 'T1-341', '\x66726f6e7469c3a87265', NULL, 1443, 1444, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (463, 1, 'T1-1321', '\x72c3a9717569736974696f6e73206d696c69746169726573', NULL, 925, 926, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (515, 1, 'T1-1096', '\x6173736f63696174696f6e20736f63696f2dc3a96475636174697665', NULL, 1029, 1030, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (555, 1, 'T1-219', '\x736f6369c3a974c3a92064652074c3a96cc3a9646966667573696f6e2070726976c3a965', NULL, 1109, 1110, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (1231, 1, 'T1-816', '\x75736167657220646573207472616e73706f727473', NULL, 2461, 2462, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1233, 1, 'T1-819', '\x64726f697420646573206f626c69676174696f6e73', NULL, 2465, 2466, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1172, 1, 'T1-740', '\x636174686f6c696369736d65', NULL, 2343, 2344, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (299, 1, 'T1-91', '\x64c3a9666f726573746174696f6e', NULL, 597, 598, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (1237, 1, 'T1-1444', '\xc3a96475636174696f6e207375727665696c6cc3a965', NULL, 2473, 2474, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (770, 1, 'T1-524', '\x6964656e746974c3a920636f6d6d65726369616c65', NULL, 1539, 1540, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (1679, 1, 'T3-209', '\x72c3a9706572746f6972652064276f66666963696572207075626c6963206d696e697374c3a97269656c', NULL, 3357, 3358, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (815, 1, 'T1-626', '\x666f796572', NULL, 1629, 1630, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (1001, 1, 'T1-541', '\x6d6f7274616c6974c3a920696e66616e74696c65', NULL, 2001, 2002, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1190, 1, 'T1-1108', '\x64c3a966656e736520636f6e747265206c61206d6572', NULL, 2379, 2380, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1191, 1, 'T1-951', '\x7369676e616c69736174696f6e206d61726974696d65', NULL, 2381, 2382, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (294, 1, 'T1-397', '\x6c69627261697265', NULL, 587, 588, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (1656, 1, 'T3-216', '\x736368c3a96d6120646972656374657572', NULL, 3311, 3312, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (266, 1, 'T1-919', '\x6661736369736d65', NULL, 531, 532, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (1189, 1, 'T1-761', '\x72656c6174696f6e732061766563206c65732075736167657273', NULL, 2377, 2378, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (631, 1, 'T1-1082', '\x736572766963652066696e616e63696572206465206c6120706f737465', NULL, 1261, 1262, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (249, 1, 'T1-1437', '\x66c3aa7465', NULL, 497, 498, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (1203, 1, 'T1-781', '\x726563656e73656d656e7420646520706f70756c6174696f6e', NULL, 2405, 2406, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1269, 1, 'T1-1063', '\xc3a970696365726965', NULL, 2537, 2538, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (654, 1, 'T1-279', '\x7472616e73667573696f6e2073616e6775696e65', NULL, 1307, 1308, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (679, 1, 'T1-1018', '\x707261746971756573206167726169726573', NULL, 1357, 1358, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (1348, 1, 'T1-1093', '\x706f6c6963652075726261696e65', NULL, 2695, 2696, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1429, 1, 'T2-55', '\x6f7267616e69736174696f6e', NULL, 2857, 2858, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1509, 1, 'T4-37', '\x4469726563746f6972652028313739352d3137393929', NULL, 3017, 3018, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1581, 1, 'T4-53', '\x4772616e6465732064c3a9636f75766572746573', NULL, 3161, 3162, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1582, 1, 'T4-52', '\xc3a9646974206465204e616e74657320283135393829', NULL, 3163, 3164, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1660, 1, 'T3-176', '\x70726f63c3a8732d76657262616c2064276578616d656e', NULL, 3319, 3320, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1742, 1, 'T3-134', '\x6dc3a96d6f697265', NULL, 3483, 3484, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1822, 1, 'T3-211', '\x726576756520646520707265737365', NULL, 3643, 3644, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (3, 1, 'T1-246', '\x6167726963756c74757265', NULL, 5, 6, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (4, 1, 'T1-1317', '\x61646d696e697374726174696f6e', NULL, 7, 8, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (5, 1, 'T1-307', '\x736f6369c3a974c3a9', NULL, 9, 10, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (7, 1, 'T1-1262', '\xc3a971756970656d656e74', NULL, 13, 14, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (8, 1, 'T1-1302', '\xc3a96475636174696f6e', NULL, 15, 16, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (9, 1, 'T1-483', '\x6a757374696365', NULL, 17, 18, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (10, 1, 'T1-906', '\x6f70696e696f6e', NULL, 19, 20, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (11, 1, 'T1-150', '\x657874c3a97269657572', NULL, 21, 22, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (12, 1, 'T1-1', '\x6861626974617420727572616c', NULL, 23, 24, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (304, 1, 'T1-94', '\x686f75696c6c65', NULL, 607, 608, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (39, 1, 'T1-643', '\x626f6973', NULL, 77, 78, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (40, 1, 'T1-1070', '\x70726f6475697420646f6d616e69616c', NULL, 79, 80, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (41, 1, 'T1-8', '\x6f70c3a9726174696f6e206d696c697461697265', NULL, 81, 82, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (42, 1, 'T1-250', '\x64c3a966656e7365206475207465727269746f697265', NULL, 83, 84, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (43, 1, 'T1-9', '\x6879647261756c697175652061677269636f6c65', NULL, 85, 86, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (44, 1, 'T1-1134', '\xc3a9636f6e6f6d696520727572616c65', NULL, 87, 88, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (45, 1, 'T1-242', '\x62617272616765206879647261756c69717565', NULL, 89, 90, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (46, 1, 'T1-133', '\x69727269676174696f6e', NULL, 91, 92, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (47, 1, 'T1-211', '\x647261696e616765', NULL, 93, 94, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (48, 1, 'T1-988', '\x616dc3a96e6167656d656e74206465732065617578', NULL, 95, 96, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (49, 1, 'T1-10', '\x696e647573747269652064752063756972', NULL, 97, 98, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (50, 1, 'T1-11', '\x70726f707269c3a974c3a9207075626c69717565', NULL, 99, 100, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (51, 1, 'T1-1222', '\x66696e616e636573207075626c6971756573', NULL, 101, 102, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (52, 1, 'T1-763', '\x646f6d61696e65206d6f62696c696572', NULL, 103, 104, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (1195, 1, 'T1-1366', '\x6d6167697374726174', NULL, 2389, 2390, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (61, 1, 'T1-383', '\x646f6d61696e6520696d6d6f62696c696572', NULL, 121, 122, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (62, 1, 'T1-377', '\x616c69c3a96e6174696f6e20646f6d616e69616c65', NULL, 123, 124, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (63, 1, 'T1-1113', '\x636f6e63657373696f6e20646f6d616e69616c65', NULL, 125, 126, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (64, 1, 'T1-838', '\x646f6e732d65742d6c656773', NULL, 127, 128, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (65, 1, 'T1-1095', '\x666f72c3aa7420646f6d616e69616c65', NULL, 129, 130, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (66, 1, 'T1-12', '\x67c3a96e6965206d696c697461697265', NULL, 131, 132, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (67, 1, 'T1-912', '\x696e667261737472756374757265206d696c697461697265', NULL, 133, 134, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (68, 1, 'T1-13', '\x76656e74652061757820656e6368c3a8726573', NULL, 135, 136, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (69, 1, 'T1-1248', '\x636f6d6d65726365', NULL, 137, 138, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (70, 1, 'T1-1438', '\x76656e7465207075626c6971756520696d6d6f62696c69c3a87265', NULL, 139, 140, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (71, 1, 'T1-1137', '\x636f6d6d697373616972652070726973657572', NULL, 141, 142, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (72, 1, 'T1-14', '\x706c616765', NULL, 143, 144, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (73, 1, 'T1-762', '\x6c6974746f72616c', NULL, 145, 146, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (74, 1, 'T1-342', '\x746f757269736d652062616c6ec3a961697265', NULL, 147, 148, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (75, 1, 'T1-15', '\x6672616e636f70686f6e6965', NULL, 149, 150, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (76, 1, 'T1-974', '\x72656c6174696f6e7320696e7465726e6174696f6e616c6573', NULL, 151, 152, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (77, 1, 'T1-16', '\x736f6369c3a974c3a92072c3a9766f6c7574696f6e6e61697265', NULL, 153, 154, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (78, 1, 'T1-17', '\x6d61696e7469656e206465206c276f72647265', NULL, 155, 156, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (79, 1, 'T1-471', '\x706f6c696365', NULL, 157, 158, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (80, 1, 'T1-926', '\x6d616e69666573746174696f6e2073706f7274697665', NULL, 159, 160, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (83, 1, 'T1-606', '\x766973697465206f6666696369656c6c65', NULL, 165, 166, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (84, 1, 'T1-796', '\x6d616e69666573746174696f6e2064652070726f746573746174696f6e', NULL, 167, 168, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (603, 1, 'T1-801', '\x7365727669636573', NULL, 1205, 1206, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (1799, 1, 'T3-144', '\x6e6f7465', NULL, 3597, 3598, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (14, 1, 'T1-40', '\x62c3a274696d656e742061677269636f6c65', NULL, 27, 28, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (15, 1, 'T1-1047', '\x616dc3a96e6167656d656e7420727572616c', NULL, 29, 30, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (16, 1, 'T1-366', '\x67c3ae746520727572616c', NULL, 31, 32, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (17, 1, 'T1-2', '\x766973697465757220646520707269736f6e', NULL, 33, 34, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (18, 1, 'T1-3', '\x72656368657263686520736369656e74696669717565', NULL, 35, 36, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (19, 1, 'T1-1356', '\x63756c7475726520657870c3a972696d656e74616c65', NULL, 37, 38, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (20, 1, 'T1-1413', '\x726563686572636865206170706c697175c3a965', NULL, 39, 40, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (21, 1, 'T1-618', '\x64c3a9636f75766572746520736369656e74696669717565', NULL, 41, 42, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (22, 1, 'T1-676', '\x72c3a9756e696f6e20736369656e74696669717565', NULL, 43, 44, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (23, 1, 'T1-1073', '\x736f6369c3a974c3a920736176616e7465', NULL, 45, 46, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (24, 1, 'T1-287', '\x6469736369706c696e6520736369656e74696669717565', NULL, 47, 48, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (25, 1, 'T1-1005', '\x736369656e6365732068756d61696e6573', NULL, 49, 50, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (26, 1, 'T1-319', '\x72656368657263686520666f6e64616d656e74616c65', NULL, 51, 52, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (27, 1, 'T1-981', '\x6f7267616e69736d6520646520726563686572636865', NULL, 53, 54, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (1449, 1, 'T2-15', '\x63657373696f6e', NULL, 2897, 2898, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (846, 1, 'T1-428', '\xc3a96cc3a87665', NULL, 1691, 1692, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (1613, 1, 'T3-18', '\x61746c6173', NULL, 3225, 3226, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (85, 1, 'T1-18', '\x6368616e736f6e2073756276657273697665', NULL, 169, 170, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (86, 1, 'T1-19', '\x6361727269c3a87265', NULL, 171, 172, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (87, 1, 'T1-481', '\x696e647573747269652065787472616374697665', NULL, 173, 174, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (88, 1, 'T1-20', '\x7472617661696c6c65757220736f6369616c', NULL, 175, 176, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (28, 1, 'T1-448', '\x636865726368657572', NULL, 55, 56, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (29, 1, 'T1-436', '\x6469736369706c696e6520756e6976657273697461697265', NULL, 57, 58, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (30, 1, 'T1-896', '\x76696520696e74656c6c65637475656c6c65', NULL, 59, 60, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (31, 1, 'T1-4', '\x647572c3a965206475207472617661696c', NULL, 61, 62, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (37, 1, 'T1-1079', '\x72c3a967696f6e', NULL, 73, 74, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (1788, 1, 'T3-220', '\x737461747574', NULL, 3575, 3576, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (53, 1, 'T1-1433', '\x6269656e7320696e746572636f6d6d756e617578', NULL, 105, 106, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (54, 1, 'T1-1051', '\x6269656e73206e6174696f6e617578', NULL, 107, 108, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (55, 1, 'T1-690', '\x6269656e7320636f6d6d756e617578', NULL, 109, 110, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (56, 1, 'T1-766', '\x6269656e2061646d696e69737472c3a9', NULL, 111, 112, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (57, 1, 'T1-333', '\x6163717569736974696f6e20646f6d616e69616c65', NULL, 113, 114, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (58, 1, 'T1-616', '\x6269656e732072c3a967696f6e617578', NULL, 115, 116, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (59, 1, 'T1-93', '\x636f6e63657373696f6e2066756ec3a97261697265', NULL, 117, 118, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (33, 1, 'T1-5', '\x726f757465206e6174696f6e616c65', NULL, 65, 66, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (34, 1, 'T1-1149', '\x72c3a97365617520726f7574696572', NULL, 67, 68, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (35, 1, 'T1-6', '\x66696e616e6365732072c3a967696f6e616c6573', NULL, 69, 70, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (690, 1, 'T1-312', '\x66656d6d65', NULL, 1379, 1380, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (1457, 1, 'T2-18', '\x636f6d6d756e69636174696f6e', NULL, 2913, 2914, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (90, 1, 'T1-350', '\xc3a96475636174657572', NULL, 179, 180, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (91, 1, 'T1-936', '\x656e7472c3a96520736f6c656e6e656c6c65', NULL, 181, 182, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (93, 1, 'T1-194', '\x72c3a963657074696f6e206f6666696369656c6c65', NULL, 185, 186, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (94, 1, 'T1-1335', '\x696e61756775726174696f6e', NULL, 187, 188, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (95, 1, 'T1-22', '\x76696520706f6c697469717565', NULL, 189, 190, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (96, 1, 'T1-1169', '\x706172746920706f6c697469717565', NULL, 191, 192, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (97, 1, 'T1-87', '\x6d6f7576656d656e7420706f6c697469717565', NULL, 193, 194, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (98, 1, 'T1-23', '\x636f6d7061676e6f6e6e616765', NULL, 195, 196, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (99, 1, 'T1-1025', '\x6f7267616e69736174696f6e2070726f66657373696f6e6e656c6c65', NULL, 197, 198, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (100, 1, 'T1-24', '\x6368656d696e2064652072616e646f6e6ec3a965', NULL, 199, 200, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (101, 1, 'T1-632', '\x746f757269736d65', NULL, 201, 202, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (102, 1, 'T1-908', '\x6368656d696e20727572616c', NULL, 203, 204, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (103, 1, 'T1-25', '\x696e63656e64696520766f6c6f6e7461697265', NULL, 205, 206, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (104, 1, 'T1-1403', '\x6372696d65732065742064c3a96c697473', NULL, 207, 208, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (157, 1, 'T1-45', '\x6578636c7573696f6e20736f6369616c65', NULL, 313, 314, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (243, 1, 'T1-76', '\x766f69652070726976c3a965', NULL, 485, 486, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (112, 1, 'T1-418', '\x64c3a974656e74696f6e20642761726d6573', NULL, 223, 224, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (113, 1, 'T1-204', '\x73c3a96a6f75722064657320c3a97472616e67657273', NULL, 225, 226, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (115, 1, 'T1-545', '\x7472616e73706f727420646520636f727073', NULL, 229, 230, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (116, 1, 'T1-518', '\x706967656f6e20766f796167657572', NULL, 231, 232, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (117, 1, 'T1-29', '\x70726976696cc3a86765732064657320636f6d6d756e617574c3a973', NULL, 233, 234, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (118, 1, 'T1-842', '\x72c3a967696d6520736569676e65757269616c', NULL, 235, 236, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (119, 1, 'T1-155', '\x636f7574756d6573', NULL, 237, 238, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (120, 1, 'T1-1333', '\x64726f6974732064277573616765', NULL, 239, 240, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (121, 1, 'T1-30', '\x766f69652066657272c3a9652070726976c3a965', NULL, 241, 242, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (122, 1, 'T1-31', '\x766163616e7473', NULL, 243, 244, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (123, 1, 'T1-32', '\x63697263756c6174696f6e2064657320706572736f6e6e6573', NULL, 245, 246, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (124, 1, 'T1-1269', '\x7375727665696c6c616e6365206475207465727269746f697265', NULL, 247, 248, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (125, 1, 'T1-33', '\x6d6f7576656d656e742064276964c3a96573', NULL, 249, 250, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (126, 1, 'T1-453', '\x746f6cc3a972616e6365', NULL, 251, 252, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (1657, 1, 'T3-105', '\xc3a974756465', NULL, 3313, 3314, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (127, 1, 'T1-1239', '\x64726f697473206465206c27686f6d6d65', NULL, 253, 254, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (128, 1, 'T1-1247', '\x617468c3a969736d65', NULL, 255, 256, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (129, 1, 'T1-143', '\x7068696c6f736f70686965', NULL, 257, 258, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (130, 1, 'T1-34', '\x62c3a274696d656e7420706f6c7976616c656e74', NULL, 259, 260, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (131, 1, 'T1-35', '\x646f6d61696e65207075626c6963206d61726974696d65', NULL, 261, 262, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (132, 1, 'T1-36', '\x64c3a96368657420726164696f6163746966', NULL, 263, 264, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (133, 1, 'T1-362', '\x7472616974656d656e74206465732064c3a96368657473', NULL, 265, 266, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (134, 1, 'T1-1443', '\x63656e7472616c65206e75636cc3a961697265', NULL, 267, 268, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (135, 1, 'T1-37', '\x746f757269736d6520666c757669616c', NULL, 269, 270, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (136, 1, 'T1-255', '\x6e617669676174696f6e20646520706c616973616e6365', NULL, 271, 272, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (137, 1, 'T1-38', '\x656e736569676e616e74', NULL, 273, 274, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (138, 1, 'T1-499', '\x6f7267616e69736174696f6e2073636f6c61697265', NULL, 275, 276, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (139, 1, 'T1-1029', '\x696e737469747574657572', NULL, 277, 278, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (140, 1, 'T1-1456', '\xc3a964756361746575722073706f72746966', NULL, 279, 280, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (141, 1, 'T1-614', '\x6d61c3ae74726520642761707072656e74697373616765', NULL, 281, 282, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (142, 1, 'T1-300', '\x70726f66657373657572', NULL, 283, 284, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (143, 1, 'T1-583', '\x666f726d6174657572', NULL, 285, 286, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (144, 1, 'T1-777', '\x6dc3a9646563696e20686f73706974616c696572', NULL, 287, 288, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (145, 1, 'T1-1056', '\x666f6e6374696f6e6e61697265206465206c27c389746174', NULL, 289, 290, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (146, 1, 'T1-39', '\xc3a9736f74657269736d65', NULL, 291, 292, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (147, 1, 'T1-172', '\x736369656e63657320706172616c6cc3a86c6573', NULL, 293, 294, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (1695, 1, 'T3-21', '\x6176616e742d70726f6a6574', NULL, 3389, 3390, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1784, 1, 'T3-92', '\x646f63756d656e746174696f6e', NULL, 3567, 3568, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (149, 1, 'T1-42', '\x72656472657373656d656e74206a756469636961697265', NULL, 297, 298, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (150, 1, 'T1-460', '\x6166666169726520636f6d6d65726369616c65', NULL, 299, 300, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (151, 1, 'T1-724', '\x656e747265707269736520656e20646966666963756c74c3a9', NULL, 301, 302, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (152, 1, 'T1-224', '\x61646d696e697374726174657572206a756469636961697265', NULL, 303, 304, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (153, 1, 'T1-43', '\x64616e7365', NULL, 305, 306, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (154, 1, 'T1-608', '\x617274', NULL, 307, 308, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (155, 1, 'T1-1228', '\x656e736569676e656d656e742061727469737469717565', NULL, 309, 310, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (156, 1, 'T1-44', '\xc3a96d6967726174696f6e', NULL, 311, 312, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (400, 1, 'T1-136', '\x76696e', NULL, 799, 800, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (105, 1, 'T1-26', '\x70726f626174696f6e', NULL, 209, 210, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (106, 1, 'T1-996', '\x6170706c69636174696f6e20646573207065696e6573', NULL, 211, 212, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (107, 1, 'T1-27', '\x736f6369c3a974c3a920636976696c652070726f66657373696f6e6e656c6c65', NULL, 213, 214, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (108, 1, 'T1-885', '\x656e7472657072697365', NULL, 215, 216, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (109, 1, 'T1-369', '\x656e72656769737472656d656e74206a756469636961697265', NULL, 217, 218, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (110, 1, 'T1-28', '\x706f6c6963652061646d696e697374726174697665', NULL, 219, 220, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (111, 1, 'T1-962', '\x70726f66657373696f6e2072c3a9676c656d656e74c3a965', NULL, 221, 222, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (159, 1, 'T1-678', '\x696e646967656e74', NULL, 317, 318, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (163, 1, 'T1-432', '\x73c3a96375726974c3a920736f6369616c65', NULL, 325, 326, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (1789, 1, 'T3-138', '\x6d696e757465206a75726964696374696f6e6e656c6c65', NULL, 3577, 3578, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (714, 1, 'T1-1028', '\x7375627374616e63652076c3a96ec3a96e65757365', NULL, 1427, 1428, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (166, 1, 'T1-1334', '\x766f696520646520636f6d6d756e69636174696f6e', NULL, 331, 332, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (167, 1, 'T1-277', '\x696e66726173747275637475726520706f72747561697265', NULL, 333, 334, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (168, 1, 'T1-1089', '\x6261727261676520687964726fc3a96c6563747269717565', NULL, 335, 336, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (169, 1, 'T1-49', '\x7669746963756c74757265', NULL, 337, 338, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (170, 1, 'T1-421', '\x70726f64756374696f6e2061677269636f6c65', NULL, 339, 340, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (171, 1, 'T1-50', '\x76c3a9686963756c65206175746f6d6f62696c65', NULL, 341, 342, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (172, 1, 'T1-548', '\x7472616e73706f7274', NULL, 343, 344, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (1630, 1, 'T3-23', '\x6261696c', NULL, 3259, 3260, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (181, 1, 'T1-554', '\x616dc3a96e6167656d656e74206475207465727269746f697265', NULL, 361, 362, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (182, 1, 'T1-334', '\x6d6f6e7461676e65', NULL, 363, 364, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (183, 1, 'T1-53', '\x61696465206dc3a9646963616c65', NULL, 365, 366, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (184, 1, 'T1-1168', '\x6169646520736f6369616c65', NULL, 367, 368, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (185, 1, 'T1-54', '\x7072c3a9736964656e7420647520636f6e7365696c2072c3a967696f6e616c', NULL, 369, 370, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (186, 1, 'T1-868', '\xc3a96c75', NULL, 371, 372, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (187, 1, 'T1-1237', '\x636f6e7365696c2072c3a967696f6e616c', NULL, 373, 374, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (188, 1, 'T1-55', '\x72c3a967696f6e616c69736174696f6e', NULL, 375, 376, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (189, 1, 'T1-56', '\x6dc3a9646563696e652076c3a974c3a972696e61697265', NULL, 377, 378, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (191, 1, 'T1-57', '\x746572726520696e63756c7465', NULL, 381, 382, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (192, 1, 'T1-105', '\x64c3a96672696368656d656e74', NULL, 383, 384, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (193, 1, 'T1-58', '\x636f6e6672c3a9726965', NULL, 385, 386, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (194, 1, 'T1-74', '\x6173736f63696174696f6e2063756c7475656c6c65', NULL, 387, 388, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (195, 1, 'T1-59', '\x696e64757374726965206d61726974696d65', NULL, 389, 390, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (196, 1, 'T1-60', '\x6e6f7461697265', NULL, 391, 392, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (197, 1, 'T1-101', '\x6f66666963696572206d696e697374c3a97269656c', NULL, 393, 394, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (198, 1, 'T1-1215', '\x7075626c69636974c3a920666f6e6369c3a87265', NULL, 395, 396, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (199, 1, 'T1-61', '\x696e6672616374696f6e206d696c697461697265', NULL, 397, 398, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (200, 1, 'T1-872', '\x726563727574656d656e74206d696c697461697265', NULL, 399, 400, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (201, 1, 'T1-359', '\x736572766963652063697669717565', NULL, 401, 402, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (202, 1, 'T1-62', '\x636f6e67c3a9732073636f6c6169726573', NULL, 403, 404, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (203, 1, 'T1-1404', '\x727974686d652073636f6c61697265', NULL, 405, 406, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (204, 1, 'T1-684', '\x63656e747265206465206c6f6973697273', NULL, 407, 408, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (205, 1, 'T1-311', '\x63656e74726520646520766163616e636573', NULL, 409, 410, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (206, 1, 'T1-63', '\xc3a9636c616972616765207075626c6963', NULL, 411, 412, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (207, 1, 'T1-755', '\x72c3a97365617520646520646973747269627574696f6e', NULL, 413, 414, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (208, 1, 'T1-64', '\x706c616e74652074696e63746f7269616c65', NULL, 415, 416, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (209, 1, 'T1-1325', '\x63756c7475726520696e647573747269656c6c65', NULL, 417, 418, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (210, 1, 'T1-203', '\x696e647573747269652074657874696c65', NULL, 419, 420, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (211, 1, 'T1-65', '\x6d61676173696e2067c3a96ec3a972616c', NULL, 421, 422, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (212, 1, 'T1-66', '\x636972636f6e736372697074696f6e206563636cc3a9736961737469717565', NULL, 423, 424, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (213, 1, 'T1-793', '\x696e737469747574696f6e206563636cc3a9736961737469717565', NULL, 425, 426, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (214, 1, 'T1-67', '\x73696e6973747265', NULL, 427, 428, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (215, 1, 'T1-147', '\x70726f74656374696f6e20636976696c65', NULL, 429, 430, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (216, 1, 'T1-1393', '\x696e6f6e646174696f6e', NULL, 431, 432, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (217, 1, 'T1-1055', '\x636174617374726f70686520696e647573747269656c6c65', NULL, 433, 434, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (221, 1, 'T1-68', '\x70726f707269c3a974c3a920696e647573747269656c6c65', NULL, 441, 442, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (222, 1, 'T1-69', '\x696d7069c3a974c3a9', NULL, 443, 444, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (223, 1, 'T1-485', '\x6d616e69666573746174696f6e20616e746972656c69676965757365', NULL, 445, 446, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (235, 1, 'T1-317', '\x636f6d6d756e65', NULL, 469, 470, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (479, 1, 'T1-178', '\x666572726f7574616765', NULL, 957, 958, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (736, 1, 'T1-345', '\x62616e6e697373656d656e74', NULL, 1471, 1472, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (1511, 1, 'T4-96', '\x63726973652064652031393239', NULL, 3021, 3022, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (224, 1, 'T1-70', '\x6d6f7576656d656e74206c61c3af717565', NULL, 447, 448, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (225, 1, 'T1-1106', '\x636f6e666c69742073636f6c61697265', NULL, 449, 450, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (226, 1, 'T1-71', '\x696d70c3b47420737572206c65732062c3a96ec3a96669636573', NULL, 451, 452, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (227, 1, 'T1-669', '\x66697363616c6974c3a92070726f66657373696f6e6e656c6c65', NULL, 453, 454, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (228, 1, 'T1-72', '\x72c3a9717569736974696f6e206465206c6f67656d656e74', NULL, 455, 456, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (229, 1, 'T1-73', '\x636f6c6c65637469766974c3a9206c6f63616c65', NULL, 457, 458, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (230, 1, 'T1-1345', '\x61646d696e697374726174696f6e2067c3a96ec3a972616c65', NULL, 459, 460, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (231, 1, 'T1-594', '\x726567726f7570656d656e7420646520636f6d6d756e6573', NULL, 461, 462, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (1268, 1, 'T1-1178', '\x626f756c616e6765726965', NULL, 2535, 2536, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (176, 1, 'T1-628', '\x73c3a96375726974c3a920726f757469c3a87265', NULL, 351, 352, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (177, 1, 'T1-51', '\x7072756427686f6d6d65', NULL, 353, 354, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (178, 1, 'T1-767', '\x6a756765', NULL, 355, 356, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (179, 1, 'T1-1061', '\xc3a96c656374696f6e2070726f66657373696f6e6e656c6c65', NULL, 357, 358, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (180, 1, 'T1-52', '\x706f6c697469717565206465206c61206d6f6e7461676e65', NULL, 359, 360, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (232, 1, 'T1-1449', '\x64c3a970617274656d656e74', NULL, 463, 464, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (233, 1, 'T1-199', '\x636f6f70c3a9726174696f6e20696e74657272c3a967696f6e616c65', NULL, 465, 466, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (1676, 1, 'T3-67', '\x636f6e74726174', NULL, 3351, 3352, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (89, 1, 'T1-200', '\x616374696f6e20736f6369616c65', NULL, 177, 178, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (324, 1, 'T1-446', '\x6e6f626c65737365', NULL, 647, 648, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (582, 1, 'T1-239', '\x686f6d6d616765', NULL, 1163, 1164, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (234, 1, 'T1-1238', '\xc3a97461626c697373656d656e74207075626c696320646520636f6f70c3a9726174696f6e20696e746572636f6d6d756e616c65', NULL, 467, 468, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (236, 1, 'T1-804', '\x7669652072656c69676965757365', NULL, 471, 472, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (237, 1, 'T1-1391', '\x736f6369c3a974c3a920646520636861726974c3a9', NULL, 473, 474, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (241, 1, 'T1-1311', '\x656e747265707269736520646520737065637461636c65', NULL, 481, 482, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (242, 1, 'T1-562', '\x6c616e6775652072c3a967696f6e616c65', NULL, 483, 484, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (244, 1, 'T1-77', '\x6e61697373616e6365', NULL, 487, 488, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (245, 1, 'T1-1139', '\xc3a974617420636976696c', NULL, 489, 490, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (246, 1, 'T1-492', '\x706f70756c6174696f6e20686f73706974616c69c3a87265', NULL, 491, 492, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (254, 1, 'T1-946', '\x656e66616e74', NULL, 507, 508, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (774, 1, 'T1-782', '\x616c696d656e746174696f6e', NULL, 1547, 1548, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (282, 1, 'T1-1390', '\x7261646963616c69736d65', NULL, 563, 564, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (251, 1, 'T1-81', '\x696e6475737472696520c3a96c656374726f6e69717565', NULL, 501, 502, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (252, 1, 'T1-82', '\x656e66616e742068616e6469636170c3a9', NULL, 503, 504, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (253, 1, 'T1-677', '\xc3a96475636174696f6e207370c3a96369616c65', NULL, 505, 506, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (255, 1, 'T1-83', '\x6f70c3a9726174696f6e206427757262616e69736d65', NULL, 509, 510, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (256, 1, 'T1-209', '\x757262616e69736d65', NULL, 511, 512, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (257, 1, 'T1-475', '\x6c6f74697373656d656e74', NULL, 513, 514, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (258, 1, 'T1-1097', '\x726573747275637475726174696f6e2075726261696e65', NULL, 515, 516, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (259, 1, 'T1-124', '\x726573746175726174696f6e20696d6d6f62696c69c3a87265', NULL, 517, 518, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (260, 1, 'T1-84', '\x757262616e69737465', NULL, 519, 520, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (575, 1, 'T1-233', '\x73697465', NULL, 1149, 1150, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (283, 1, 'T1-1322', '\x65787472c3aa6d652064726f697465', NULL, 565, 566, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (284, 1, 'T1-1207', '\x65787472c3aa6d6520676175636865', NULL, 567, 568, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (285, 1, 'T1-229', '\x726f79616c69736d65', NULL, 569, 570, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (286, 1, 'T1-174', '\x626f6e617061727469736d65', NULL, 571, 572, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (287, 1, 'T1-1218', '\x6d6f7576656d656e742066c3a96d696e69737465', NULL, 573, 574, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (289, 1, 'T1-115', '\x6d6f7576656d656e7420686f6d6f73657875656c', NULL, 577, 578, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (290, 1, 'T1-318', '\x6d6f7576656d656e7420706163696669737465', NULL, 579, 580, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (291, 1, 'T1-88', '\x6dc3a97469657273206475206c69767265', NULL, 581, 582, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (354, 1, 'T1-114', '\x6d7573c3a965', NULL, 707, 708, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (874, 1, 'T1-496', '\x72616369736d65', NULL, 1747, 1748, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (274, 1, 'T1-538', '\x636f6c6f6e69616c69736d65', NULL, 547, 548, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (275, 1, 'T1-526', '\x636f6d6d756e69736d65', NULL, 549, 550, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (276, 1, 'T1-605', '\x6761756c6c69736d65', NULL, 551, 552, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (277, 1, 'T1-720', '\x6d6f7576656d656e742072c3a967696f6e616c69737465', NULL, 553, 554, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (278, 1, 'T1-1339', '\x6d6f7576656d656e7420c3a974756469616e74', NULL, 555, 556, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (279, 1, 'T1-197', '\x636cc3a9726963616c69736d65', NULL, 557, 558, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (280, 1, 'T1-391', '\x6d6f7576656d656e74206f757672696572', NULL, 559, 560, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (281, 1, 'T1-266', '\x6c6962c3a972616c69736d65', NULL, 561, 562, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (303, 1, 'T1-773', '\x63696d657469c3a87265', NULL, 605, 606, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (305, 1, 'T1-1326', '\x6d617469c3a87265207072656d69c3a87265', NULL, 609, 610, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (306, 1, 'T1-1441', '\x63656e7472616c6520746865726d69717565', NULL, 611, 612, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (307, 1, 'T1-95', '\x74c3a96cc3a96d617469717565', NULL, 613, 614, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (308, 1, 'T1-1327', '\x74c3a96cc3a9636f6d6d756e69636174696f6e73', NULL, 615, 616, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (309, 1, 'T1-96', '\x636f6e63657373696f6e20666f7265737469c3a87265', NULL, 617, 618, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (310, 1, 'T1-832', '\x6578706c6f6974616e7420666f72657374696572', NULL, 619, 620, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (311, 1, 'T1-97', '\x736f6369c3a974c3a9206d757475616c69737465', NULL, 621, 622, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (312, 1, 'T1-98', '\x6173736f63696174696f6e20646520636f6e736f6d6d617465757273', NULL, 623, 624, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (313, 1, 'T1-581', '\x636f6e736f6d6d6174696f6e', NULL, 625, 626, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (314, 1, 'T1-99', '\x656e66616e74207365636f757275', NULL, 627, 628, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (315, 1, 'T1-566', '\x6d696e657572207375727665696c6cc3a9', NULL, 629, 630, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (317, 1, 'T1-158', '\x6f7267616e69736174696f6e206a756469636961697265', NULL, 633, 634, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (318, 1, 'T1-1166', '\x61766f75c3a9', NULL, 635, 636, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (321, 1, 'T1-102', '\x706f6c69636520646573207472616e73706f727473', NULL, 641, 642, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (322, 1, 'T1-103', '\x636f6e646974696f6e20736f6369616c65', NULL, 643, 644, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (323, 1, 'T1-942', '\x636f6e646974696f6e2064657320706572736f6e6e657320657420646573206269656e73', NULL, 645, 646, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (325, 1, 'T1-1394', '\x6573636c6176616765', NULL, 649, 650, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (326, 1, 'T1-723', '\x70726f6cc3a9746172696174', NULL, 651, 652, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (327, 1, 'T1-1181', '\x73657276616765', NULL, 653, 654, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (328, 1, 'T1-151', '\x746965727320c3a9746174', NULL, 655, 656, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (329, 1, 'T1-184', '\x626f757267656f69736965', NULL, 657, 658, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (330, 1, 'T1-104', '\x636f6e7365696c206a7572696469717565', NULL, 659, 660, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (331, 1, 'T1-487', '\x617578696c6961697265206465206a757374696365', NULL, 661, 662, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (332, 1, 'T1-400', '\x73796c766963756c74757265', NULL, 663, 664, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (333, 1, 'T1-106', '\x72c3a966756769c3a9', NULL, 665, 666, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (335, 1, 'T1-1243', '\x72c3a966756769c3a920646520677565727265', NULL, 669, 670, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (336, 1, 'T1-107', '\xc3a96c656374696f6e20706f6c697469717565', NULL, 671, 672, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (173, 1, 'T1-1221', '\x6163636964656e7420646573207472616e73706f727473', NULL, 345, 346, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (174, 1, 'T1-126', '\x617070617265696c20c3a0207072657373696f6e20646520766170657572', NULL, 347, 348, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (175, 1, 'T1-1374', '\x636f6e737472756374696f6e206175746f6d6f62696c65', NULL, 349, 350, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (337, 1, 'T1-1152', '\xc3a96c656374696f6e', NULL, 673, 674, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (339, 1, 'T1-1081', '\xc3a96c656374696f6e20726567696f6e616c65', NULL, 677, 678, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (340, 1, 'T1-624', '\xc3a96c656374696f6e20707265736964656e7469656c6c65', NULL, 679, 680, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (341, 1, 'T1-1383', '\xc3a96c656374696f6e2073c3a96e61746f7269616c65', NULL, 681, 682, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (342, 1, 'T1-923', '\xc3a96c656374696f6e2063616e746f6e616c65', NULL, 683, 684, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (343, 1, 'T1-321', '\x72c3a966c3a972656e64756d', NULL, 685, 686, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (773, 1, 'T1-372', '\x6a75676520636f6e73756c61697265', NULL, 1545, 1546, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (344, 1, 'T1-401', '\xc3a96c656374696f6e20617520636f6e7365696c2064276172726f6e64697373656d656e74', NULL, 687, 688, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (345, 1, 'T1-171', '\xc3a96c656374696f6e206d756e69636970616c65', NULL, 689, 690, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (346, 1, 'T1-1254', '\xc3a96c656374696f6e206cc3a96769736c6174697665', NULL, 691, 692, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (347, 1, 'T1-108', '\x61726d7572696572', NULL, 693, 694, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (348, 1, 'T1-109', '\x6dc3a964696174657572206465206a757374696365', NULL, 695, 696, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (349, 1, 'T1-111', '\x696e74c3a972657373656d656e7420646573207472617661696c6c65757273', NULL, 697, 698, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (350, 1, 'T1-536', '\x7472617661696c', NULL, 699, 700, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (351, 1, 'T1-112', '\x72656365747465206e6f6e2066697363616c65', NULL, 701, 702, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (352, 1, 'T1-113', '\x636f6e737469747574696f6e', NULL, 703, 704, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (353, 1, 'T1-1259', '\x64726f6974207075626c6963', NULL, 705, 706, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (355, 1, 'T1-213', '\xc3a971756970656d656e7420736f63696f2063756c747572656c', NULL, 709, 710, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (356, 1, 'T1-116', '\xc3a9636f6e6f6d6965206d6f6e7461676e61726465', NULL, 711, 712, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (357, 1, 'T1-148', '\x616374696f6e20c3a9636f6e6f6d69717565', NULL, 713, 714, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (358, 1, 'T1-1013', '\xc3a96c6576616765', NULL, 715, 716, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (359, 1, 'T1-117', '\x666f6e6473206575726f70c3a9656e', NULL, 717, 718, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (360, 1, 'T1-118', '\x6d61676173696e20636f6c6c6563746966', NULL, 719, 720, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (361, 1, 'T1-1158', '\x666f6e647320646520636f6d6d65726365', NULL, 721, 722, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (363, 1, 'T1-119', '\x616c70686162c3a97469736174696f6e', NULL, 725, 726, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (364, 1, 'T1-544', '\x616374696f6e2068756d616e697461697265', NULL, 727, 728, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (365, 1, 'T1-881', '\x656e736569676e656d656e74', NULL, 729, 730, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (366, 1, 'T1-120', '\x626f6d62617264656d656e74', NULL, 731, 732, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (367, 1, 'T1-1341', '\x677565727265', NULL, 733, 734, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (368, 1, 'T1-121', '\x726176697461696c6c656d656e74206d696c697461697265', NULL, 735, 736, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (369, 1, 'T1-122', '\x6d6172696e65', NULL, 737, 738, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (370, 1, 'T1-551', '\x61726dc3a965', NULL, 739, 740, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (371, 1, 'T1-123', '\x70617472696d6f696e652063756c747572656c', NULL, 741, 742, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (374, 1, 'T1-889', '\x70617472696d6f696e65206e756dc3a97269717565', NULL, 747, 748, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (375, 1, 'T1-995', '\x6d6f6e756d656e7420686973746f7269717565', NULL, 749, 750, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (376, 1, 'T1-1343', '\x70617472696d6f696e6520617564696f76697375656c', NULL, 751, 752, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (377, 1, 'T1-991', '\x70617472696d6f696e65206574686e6f6c6f6769717565', NULL, 753, 754, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (378, 1, 'T1-1337', '\x70617472696d6f696e6520c3a963726974', NULL, 755, 756, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (379, 1, 'T1-658', '\xc3a964696669636520636c617373c3a9', NULL, 757, 758, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (380, 1, 'T1-825', '\x6f626a6574206427617274', NULL, 759, 760, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (381, 1, 'T1-1223', '\x70617472696d6f696e652069636f6e6f677261706869717565', NULL, 761, 762, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (1739, 1, 'T3-50', '\x636172746520706f7374616c65', NULL, 3477, 3478, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (268, 1, 'T1-170', '\x72c3a97075626c6963616e69736d65', NULL, 535, 536, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (269, 1, 'T1-1021', '\x736f6369616c69736d65', NULL, 537, 538, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (270, 1, 'T1-288', '\x616e74696361706974616c69736d65', NULL, 539, 540, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (271, 1, 'T1-634', '\x616e6172636869736d65', NULL, 541, 542, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (272, 1, 'T1-1278', '\x6e6174696f6e616c69736d65', NULL, 543, 544, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (273, 1, 'T1-1177', '\x6d6f7576656d656e7420c3a9636f6c6f6769737465', NULL, 545, 546, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (385, 1, 'T1-127', '\x6f757672696572', NULL, 769, 770, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (386, 1, 'T1-128', '\x70726174697175652072656c69676965757365', NULL, 771, 772, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (387, 1, 'T1-129', '\x706c616e696669636174696f6e', NULL, 773, 774, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (388, 1, 'T1-130', '\x657370c3a863652070726f74c3a967c3a965', NULL, 775, 776, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (389, 1, 'T1-1395', '\x70c3aa636865206d61726974696d65', NULL, 777, 778, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (390, 1, 'T1-389', '\x636861737365', NULL, 779, 780, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (391, 1, 'T1-1182', '\x70c3aa63686520c3a0206c61206c69676e65', NULL, 781, 782, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (392, 1, 'T1-425', '\x70c3aa63686520656e2065617520646f756365', NULL, 783, 784, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (394, 1, 'T1-435', '\x7374727563747572652061646d696e697374726174697665', NULL, 787, 788, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (395, 1, 'T1-132', '\xc3a976616c756174696f6e20666f6e6369c3a87265', NULL, 789, 790, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (396, 1, 'T1-944', '\x7072697365206427656175', NULL, 791, 792, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (397, 1, 'T1-134', '\x7075c3a9726963756c7472696365', NULL, 793, 794, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (401, 1, 'T1-603', '\x626f6973736f6e20616c636f6f6c6973c3a965', NULL, 801, 802, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (402, 1, 'T1-137', '\x6368c3b46d616765207061727469656c', NULL, 803, 804, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (403, 1, 'T1-862', '\x6368c3b46d616765', NULL, 805, 806, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (405, 1, 'T1-866', '\x6173736f63696174696f6e2073796e646963616c652064652070726f707269c3a9746169726573', NULL, 809, 810, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (406, 1, 'T1-139', '\x6c69676e65206465206368656d696e20646520666572', NULL, 811, 812, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (407, 1, 'T1-164', '\x7472616e73706f727420666572726f766961697265', NULL, 813, 814, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (459, 1, 'T1-1019', '\x62696f2d636172627572616e74', NULL, 917, 918, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (408, 1, 'T1-140', '\x707265737365', NULL, 815, 816, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (409, 1, 'T1-813', '\x64c3a970c3b474206cc3a967616c', NULL, 817, 818, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (410, 1, 'T1-1186', '\x6a6f75726e616c69737465', NULL, 819, 820, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (411, 1, 'T1-968', '\x7075626c69636174696f6e20696e7465726e65', NULL, 821, 822, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (412, 1, 'T1-576', '\x7265706f727465722070686f746f677261706865', NULL, 823, 824, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (413, 1, 'T1-1358', '\x7072657373652072c3a967696f6e616c65', NULL, 825, 826, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (414, 1, 'T1-141', '\x646973747269627574696f6e206465732070726978', NULL, 827, 828, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (415, 1, 'T1-142', '\x616772657373696f6e2073657875656c6c65', NULL, 829, 830, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (439, 1, 'T1-584', '\x6d6172696e2070c3aa6368657572', NULL, 877, 878, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (440, 1, 'T1-157', '\x706f6c6c7574696f6e2076697375656c6c65', NULL, 879, 880, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (441, 1, 'T1-286', '\x706f6c6c7574696f6e', NULL, 881, 882, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (442, 1, 'T1-1132', '\x6a75726964696374696f6e', NULL, 883, 884, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (443, 1, 'T1-159', '\x636f6e7365696c206d756e69636970616c', NULL, 885, 886, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (444, 1, 'T1-1157', '\x6f7267616e652064c3a96c6962c3a972616e74', NULL, 887, 888, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (445, 1, 'T1-1251', '\x6d61697265', NULL, 889, 890, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (446, 1, 'T1-1188', '\x636f6e7365696c6c6572206d756e69636970616c', NULL, 891, 892, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (447, 1, 'T1-160', '\x617263686974656374757265', NULL, 893, 894, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (1289, 1, 'T1-910', '\x6861726b69', NULL, 2577, 2578, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1616, 1, 'T3-8', '\x6167656e6461', NULL, 3231, 3232, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (452, 1, 'T1-163', '\x73796e646963206465206661696c6c697465', NULL, 903, 904, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (454, 1, 'T1-493', '\x6c696169736f6e2066657272c3a96520696e7465726e6174696f6e616c65', NULL, 907, 908, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (455, 1, 'T1-165', '\x6d697261636c65', NULL, 909, 910, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (456, 1, 'T1-1210', '\x7669652073706972697475656c6c65', NULL, 911, 912, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (457, 1, 'T1-166', '\xc3a96e6572676965', NULL, 913, 914, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (460, 1, 'T1-1373', '\xc3a96e6572676965206e75636cc3a961697265', NULL, 919, 920, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (461, 1, 'T1-452', '\xc3a9636f6e6f6d6965206427c3a96e6572676965', NULL, 921, 922, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (462, 1, 'T1-744', '\xc3a971756970656d656e74206d696c697461697265', NULL, 923, 924, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (1740, 1, 'T3-108', '\x66696368696572', NULL, 3479, 3480, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (425, 1, 'T1-887', '\x696e7374616c6c6174696f6e20636c617373c3a965', NULL, 849, 850, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (426, 1, 'T1-1000', '\x7365636f757273', NULL, 851, 852, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (427, 1, 'T1-511', '\x64c3a976656c6f7070656d656e742064757261626c65', NULL, 853, 854, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (428, 1, 'T1-920', '\x61696465207075626c697175652061757820656e747265707269736573', NULL, 855, 856, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (429, 1, 'T1-939', '\x706f6c69636520c3a9636f6e6f6d69717565', NULL, 857, 858, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (430, 1, 'T1-1234', '\x6c6162656c206465207175616c6974c3a9', NULL, 859, 860, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (431, 1, 'T1-568', '\x70726978', NULL, 861, 862, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (432, 1, 'T1-149', '\x65617520736f757465727261696e65', NULL, 863, 864, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (1343, 1, 'T1-1318', '\x6d6572', NULL, 2685, 2686, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (464, 1, 'T1-506', '\xc3a97461742d6d616a6f72', NULL, 927, 928, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (465, 1, 'T1-905', '\x736572766963652073616e697461697265206d696c697461697265', NULL, 929, 930, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (466, 1, 'T1-168', '\x64c3a9636f726174696f6e2075726261696e65', NULL, 931, 932, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (467, 1, 'T1-169', '\x636f6e74656e746965757820c3a96c6563746f72616c', NULL, 933, 934, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (468, 1, 'T1-1154', '\x636f6e74656e74696575782061646d696e6973747261746966', NULL, 935, 936, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (469, 1, 'T1-1439', '\x63726f79616e63657320657420736369656e63657320706172616c6cc3a86c6573', NULL, 937, 938, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (470, 1, 'T1-1111', '\x617374726f6c6f676965', NULL, 939, 940, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (471, 1, 'T1-784', '\x616c6368696d6965', NULL, 941, 942, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (472, 1, 'T1-386', '\x67726170686f6c6f676965', NULL, 943, 944, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (473, 1, 'T1-901', '\x73706972697469736d65', NULL, 945, 946, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (474, 1, 'T1-173', '\x726563686572636865206d696e69c3a87265', NULL, 947, 948, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (475, 1, 'T1-175', '\x617373656d626cc3a965206e6174696f6e616c65', NULL, 949, 950, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (476, 1, 'T1-902', '\x7061726c656d656e7461697265', NULL, 951, 952, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (634, 1, 'T1-267', '\x6175746f6d6f62696c69737465', NULL, 1267, 1268, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (480, 1, 'T1-745', '\x7472616e73706f7274206d756c74696d6f64616c', NULL, 959, 960, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (481, 1, 'T1-179', '\x73616c617269c3a92061677269636f6c65', NULL, 961, 962, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (482, 1, 'T1-419', '\x6167726963756c74657572', NULL, 963, 964, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (483, 1, 'T1-667', '\x6d6967726174696f6e20736169736f6e6e69c3a87265', NULL, 965, 966, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (484, 1, 'T1-180', '\x68616c7465206761726465726965', NULL, 967, 968, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (485, 1, 'T1-181', '\x70657273c3a9637574696f6e2072656c69676965757365', NULL, 969, 970, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (487, 1, 'T1-183', '\x61637469766974c3a9206dc3a96e6167c3a87265', NULL, 973, 974, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (488, 1, 'T1-185', '\xc3a97461626c697373656d656e742064652073616e74c3a9', NULL, 975, 976, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (489, 1, 'T1-344', '\x73616e74c3a9', NULL, 977, 978, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (490, 1, 'T1-1457', '\xc3a97461626c697373656d656e742073616e6974616972652070726976c3a9', NULL, 979, 980, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (764, 1, 'T1-365', '\x706861726d616369656e', NULL, 1527, 1528, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (382, 1, 'T1-125', '\x726566756765206465206d6f6e7461676e65', NULL, 763, 764, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (383, 1, 'T1-909', '\xc3a971756970656d656e7420746f757269737469717565', NULL, 765, 766, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (384, 1, 'T1-176', '\x6d616368696e652064616e67657265757365', NULL, 767, 768, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (491, 1, 'T1-950', '\x636f6e737472756374696f6e20686f73706974616c69c3a87265', NULL, 981, 982, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (495, 1, 'T1-894', '\x73616e61746f7269756d', NULL, 989, 990, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (496, 1, 'T1-560', '\xc3a971756970656d656e74206dc3a96469636f20636869727572676963616c', NULL, 991, 992, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (497, 1, 'T1-651', '\x656e736569676e656d656e7420686f73706974616c696572', NULL, 993, 994, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (498, 1, 'T1-1414', '\x666f6e6374696f6e6e6169726520686f73706974616c696572', NULL, 995, 996, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (1369, 1, 'T1-1164', '\x766f6c', NULL, 2737, 2738, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (513, 1, 'T1-1090', '\x7472616e73706f7274206d61726974696d65', NULL, 1025, 1026, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (514, 1, 'T1-198', '\x63617274652073636f6c61697265', NULL, 1027, 1028, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (516, 1, 'T1-406', '\x6173736f63696174696f6e2066616d696c69616c65', NULL, 1031, 1032, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (517, 1, 'T1-201', '\x706f726e6f67726170686965', NULL, 1033, 1034, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (518, 1, 'T1-1396', '\x706f6c69636520646573206d6f65757273', NULL, 1035, 1036, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (519, 1, 'T1-1110', '\x63696ec3a96d61', NULL, 1037, 1038, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (520, 1, 'T1-202', '\x63616c616d6974c3a92061677269636f6c65', NULL, 1039, 1040, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (521, 1, 'T1-309', '\x73c3a96368657265737365', NULL, 1041, 1042, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (522, 1, 'T1-854', '\x67656cc3a965', NULL, 1043, 1044, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (523, 1, 'T1-683', '\x6772c3aa6c65', NULL, 1045, 1046, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (524, 1, 'T1-663', '\x6dc3a974c3a96f726f6c6f676965', NULL, 1047, 1048, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (525, 1, 'T1-447', '\x706c616e74652074657874696c65', NULL, 1049, 1050, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (526, 1, 'T1-205', '\x7461786520737572206c6520636869666672652064276166666169726573', NULL, 1051, 1052, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (527, 1, 'T1-206', '\x64c3a970656e7365206427696e76657374697373656d656e74', NULL, 1053, 1054, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (528, 1, 'T1-207', '\xc3a96d616e6369706174696f6e', NULL, 1055, 1056, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (529, 1, 'T1-489', '\x64726f6974206465206c612066616d696c6c65', NULL, 1057, 1058, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (619, 1, 'T1-1060', '\x6a6575782d65742d7061726973', NULL, 1237, 1238, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (530, 1, 'T1-208', '\xc3a96475636174696f6e2073657875656c6c65', NULL, 1059, 1060, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (531, 1, 'T1-940', '\x70726f6772616d6d652073636f6c61697265', NULL, 1061, 1062, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (532, 1, 'T1-1131', '\x6d6f62696c6965722075726261696e', NULL, 1063, 1064, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (533, 1, 'T1-1246', '\xc3a974616c656d656e742075726261696e', NULL, 1065, 1066, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (534, 1, 'T1-444', '\x7265636f6e737472756374696f6e', NULL, 1067, 1068, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (535, 1, 'T1-1105', '\x7365637465757220736175766567617264c3a9', NULL, 1069, 1070, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (536, 1, 'T1-778', '\x666c657572697373656d656e74', NULL, 1071, 1072, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (537, 1, 'T1-210', '\x656d706c6f79c3a9206465206d6169736f6e', NULL, 1073, 1074, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (538, 1, 'T1-480', '\x70726f66657373696f6e20706172746963756c69c3a87265', NULL, 1075, 1076, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (539, 1, 'T1-212', '\x6a757374696365206d756e69636970616c65', NULL, 1077, 1078, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (540, 1, 'T1-1115', '\x64726f69747320736569676e65757269617578', NULL, 1079, 1080, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (541, 1, 'T1-578', '\x6dc3a96469617468c3a8717565', NULL, 1081, 1082, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (542, 1, 'T1-693', '\x6b696f7371756520c3a0206d757369717565', NULL, 1083, 1084, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (543, 1, 'T1-254', '\x6172636869766573', NULL, 1085, 1086, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (544, 1, 'T1-871', '\x6d6169736f6e20646573206a65756e6573', NULL, 1087, 1088, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (545, 1, 'T1-305', '\x73616c6c65206465732066c3aa746573', NULL, 1089, 1090, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (547, 1, 'T1-704', '\x636f6e7365727661746f697265', NULL, 1093, 1094, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (549, 1, 'T1-332', '\x666f79657220727572616c', NULL, 1097, 1098, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (416, 1, 'T1-144', '\x656d706c6f6920646573206a65756e6573', NULL, 831, 832, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (417, 1, 'T1-558', '\x6a65756e65', NULL, 833, 834, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (418, 1, 'T1-145', '\x636c617373652064652064c3a9636f757665727465', NULL, 835, 836, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (419, 1, 'T1-146', '\x726176697461696c6c656d656e74', NULL, 837, 838, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (420, 1, 'T1-1150', '\x726174696f6e6e656d656e74', NULL, 839, 840, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (421, 1, 'T1-756', '\x6d61726368c3a9206e6f6972', NULL, 841, 842, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (422, 1, 'T1-1450', '\x73756273697374616e636573', NULL, 843, 844, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (423, 1, 'T1-1151', '\x7472616e73706f7274206465206d617469c3a87265732064616e6765726575736573', NULL, 845, 846, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (438, 1, 'T1-231', '\x706f7274', NULL, 875, 876, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (1662, 1, 'T3-135', '\x6d656e75', NULL, 3323, 3324, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (499, 1, 'T1-186', '\xc3a97461626c697373656d656e7420646520666f726d6174696f6e20646573206d61c3ae74726573', NULL, 997, 998, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (500, 1, 'T1-662', '\xc3a97461626c697373656d656e74206427656e736569676e656d656e74', NULL, 999, 1000, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (501, 1, 'T1-188', '\x696d70c3b47420737572206c6120666f7274756e65', NULL, 1001, 1002, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (502, 1, 'T1-1376', '\x66697363616c6974c3a92064657320706572736f6e6e6573', NULL, 1003, 1004, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (503, 1, 'T1-189', '\x7472617661696c2070726f74c3a967c3a9', NULL, 1005, 1006, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (504, 1, 'T1-1027', '\x7472617661696c6c6575722068616e6469636170c3a9', NULL, 1007, 1008, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (505, 1, 'T1-190', '\x6372c3a8636865', NULL, 1009, 1010, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (506, 1, 'T1-191', '\x73756963696465', NULL, 1011, 1012, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (779, 1, 'T1-900', '\x6d6f62696c696572', NULL, 1557, 1558, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (584, 1, 'T1-240', '\x6772616e646520c3a9636f6c65', NULL, 1167, 1168, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (510, 1, 'T1-195', '\x666f6e6374696f6e6e61697265206465206c27556e696f6e206575726f70c3a9656e6e65', NULL, 1019, 1020, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (511, 1, 'T1-805', '\x706572736f6e6e656c', NULL, 1021, 1022, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (512, 1, 'T1-196', '\x6c69676e65206d61726974696d65', NULL, 1023, 1024, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (448, 1, 'T1-738', '\x61726368697465637465', NULL, 895, 896, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (449, 1, 'T1-161', '\x7265706f7320686562646f6d616461697265', NULL, 897, 898, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (450, 1, 'T1-162', '\x6672756974', NULL, 899, 900, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (451, 1, 'T1-631', '\x686f72746963756c74757265', NULL, 901, 902, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (453, 1, 'T1-1232', '\x6661696c6c697465', NULL, 905, 906, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (436, 1, 'T1-154', '\x6173736f63696174696f6e20646520746f757269736d65', NULL, 871, 872, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (437, 1, 'T1-156', '\x706f72742064652070c3aa636865', NULL, 873, 874, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (492, 1, 'T1-1263', '\x6cc3a970726f7365726965', NULL, 983, 984, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (493, 1, 'T1-1062', '\x68c3b4706974616c2070737963686961747269717565', NULL, 985, 986, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (578, 1, 'T1-525', '\x6a7573746963652070c3a96e616c65', NULL, 1155, 1156, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (494, 1, 'T1-329', '\xc3a97461626c697373656d656e74207075626c6963206427686f73706974616c69736174696f6e', NULL, 987, 988, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (548, 1, 'T1-1459', '\x6269626c696f7468c3a8717565', NULL, 1095, 1096, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (550, 1, 'T1-1034', '\x63656e74726520646520646f63756d656e746174696f6e', NULL, 1099, 1100, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (551, 1, 'T1-214', '\x6d616c616469652073657875656c6c656d656e74207472616e736d69737369626c65', NULL, 1101, 1102, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (552, 1, 'T1-215', '\x6162616e646f6e2064652066616d696c6c65', NULL, 1103, 1104, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (553, 1, 'T1-217', '\x62617465617520646520706c616973616e6365', NULL, 1105, 1106, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (554, 1, 'T1-218', '\x6167656e742064652064726f69742070726976c3a9', NULL, 1107, 1108, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (558, 1, 'T1-639', '\x73706f7274', NULL, 1115, 1116, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (559, 1, 'T1-222', '\x766f796167652073636f6c61697265', NULL, 1117, 1118, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (560, 1, 'T1-223', '\x6561752d64652d766965', NULL, 1119, 1120, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (561, 1, 'T1-225', '\x68c3b474656c20646520746f757269736d65', NULL, 1121, 1122, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (562, 1, 'T1-227', '\x656e74726570726973652064752062c3a274696d656e74', NULL, 1123, 1124, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (563, 1, 'T1-915', '\x6d657373616765726965', NULL, 1125, 1126, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (564, 1, 'T1-1350', '\x7265646576616e6365207061726166697363616c65', NULL, 1127, 1128, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (565, 1, 'T1-230', '\x6f7267616e69736d652064652073c3a96375726974c3a920736f6369616c65', NULL, 1129, 1130, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (566, 1, 'T1-1214', '\xc3a96c656374696f6e20736f6369616c65', NULL, 1131, 1132, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (567, 1, 'T1-597', '\x706f7274206d61726974696d65', NULL, 1133, 1134, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (568, 1, 'T1-1434', '\x646f636b6572', NULL, 1135, 1136, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (569, 1, 'T1-567', '\x706f727420646520636f6d6d65726365', NULL, 1137, 1138, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (570, 1, 'T1-948', '\x706f727420666c757669616c', NULL, 1139, 1140, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (571, 1, 'T1-1253', '\x706f727420646520706c616973616e6365', NULL, 1141, 1142, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (572, 1, 'T1-928', '\x6f6666696369657220646520706f7274', NULL, 1143, 1144, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (573, 1, 'T1-1206', '\x766f69652066657272c3a96520646573207175616973', NULL, 1145, 1146, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (698, 1, 'T1-322', '\x6cc3a8707265', NULL, 1395, 1396, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (574, 1, 'T1-232', '\x656e736569676e656d656e742072656c696769657578', NULL, 1147, 1148, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (576, 1, 'T1-234', '\x6f66667265206427656d706c6f69', NULL, 1151, 1152, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (577, 1, 'T1-235', '\x6372696d696e616c6974c3a9', NULL, 1153, 1154, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (580, 1, 'T1-1412', '\x746f757269736d6520736f6369616c', NULL, 1159, 1160, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (581, 1, 'T1-238', '\x76657220c3a020736f6965', NULL, 1161, 1162, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (583, 1, 'T1-718', '\x66c3a96f64616c6974c3a9', NULL, 1165, 1166, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (585, 1, 'T1-241', '\x696d70c3b47473206c6f63617578', NULL, 1169, 1170, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (586, 1, 'T1-1316', '\x6f6374726f69', NULL, 1171, 1172, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (587, 1, 'T1-1016', '\x696d70c3b474732070726f76696e6369617578206427416e6369656e2052c3a967696d65', NULL, 1173, 1174, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (588, 1, 'T1-641', '\x64726f697420646520706c616365', NULL, 1175, 1176, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (589, 1, 'T1-298', '\x64726f6974206465732070617576726573', NULL, 1177, 1178, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (590, 1, 'T1-1002', '\x746178652064652073c3a96a6f7572', NULL, 1179, 1180, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (591, 1, 'T1-1298', '\x7461786520642768616269746174696f6e', NULL, 1181, 1182, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (592, 1, 'T1-243', '\x6368656e616c', NULL, 1183, 1184, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (593, 1, 'T1-244', '\x696e64757374726965206167726f20616c696d656e7461697265', NULL, 1185, 1186, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (594, 1, 'T1-869', '\x636f6d6d6572636520616c696d656e7461697265', NULL, 1187, 1188, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (595, 1, 'T1-1092', '\x61626174746f6972', NULL, 1189, 1190, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (596, 1, 'T1-245', '\x7472616e73706f72742073636f6c61697265', NULL, 1191, 1192, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (848, 1, 'T1-429', '\x61637175697474656d656e74', NULL, 1695, 1696, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (1775, 1, 'T3-42', '\x6361727465', NULL, 3549, 3550, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (611, 1, 'T1-251', '\x656e736569676e656d656e7420737570c3a97269657572', NULL, 1221, 1222, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (612, 1, 'T1-294', '\x64726f6974732064652073636f6c61726974c3a9', NULL, 1223, 1224, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (613, 1, 'T1-586', '\x746974726520756e6976657273697461697265', NULL, 1225, 1226, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (614, 1, 'T1-976', '\x74726f75626c6520756e6976657273697461697265', NULL, 1227, 1228, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (615, 1, 'T1-252', '\x64726f697473206427757361676520666f7265737469657273', NULL, 1229, 1230, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (616, 1, 'T1-1324', '\x7573616765732061677269636f6c6573206c6f63617578', NULL, 1231, 1232, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (617, 1, 'T1-637', '\x6c6f7420646520636861737365', NULL, 1233, 1234, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (618, 1, 'T1-253', '\x706f6c69636520646573206a657578', NULL, 1235, 1236, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (1188, 1, 'T1-1379', '\x666f6e646174696f6e', NULL, 2375, 2376, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (597, 1, 'T1-1296', '\x6f6575767265732073636f6c6169726573', NULL, 1193, 1194, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (598, 1, 'T1-1273', '\x7472616e73706f727420656e20636f6d6d756e', NULL, 1195, 1196, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (599, 1, 'T1-1405', '\x656e736569676e656d656e742061677269636f6c65', NULL, 1197, 1198, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (600, 1, 'T1-247', '\x6f66666963696572206d696c697461697265', NULL, 1199, 1200, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (601, 1, 'T1-1103', '\x6d696c697461697265', NULL, 1201, 1202, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (602, 1, 'T1-248', '\x6167656e742064652073c3a96375726974c3a9', NULL, 1203, 1204, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (605, 1, 'T1-249', '\x70726f706f7320737562766572736966', NULL, 1209, 1210, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (606, 1, 'T1-770', '\x6d696c69636573', NULL, 1211, 1212, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (607, 1, 'T1-1293', '\x6d6f62696c69736174696f6e', NULL, 1213, 1214, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (608, 1, 'T1-966', '\x706f6c696365206d696c697461697265', NULL, 1215, 1216, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (684, 1, 'T1-303', '\x68696e646f7569736d65', NULL, 1367, 1368, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (661, 1, 'T1-285', '\x7265636f6e6e61697373616e636573', NULL, 1321, 1322, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (662, 1, 'T1-1074', '\x656e7669726f6e6e656d656e74', NULL, 1323, 1324, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (663, 1, 'T1-714', '\x706f6c6c7574696f6e2061746d6f737068c3a97269717565', NULL, 1325, 1326, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (664, 1, 'T1-1420', '\x706f6c6c7574696f6e206465732065617578', NULL, 1327, 1328, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (665, 1, 'T1-375', '\x706f6c6c7574696f6e20736f6e6f7265', NULL, 1329, 1330, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (666, 1, 'T1-327', '\x706f6c6c7574696f6e206465206c61206d6572', NULL, 1331, 1332, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (667, 1, 'T1-1145', '\x696e64757374726965206368696d69717565', NULL, 1333, 1334, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (668, 1, 'T1-289', '\x70726f74656374696f6e206465732076c3a967c3a974617578', NULL, 1335, 1336, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (433, 1, 'T1-695', '\x656175', NULL, 865, 866, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (434, 1, 'T1-152', '\x63616e636572', NULL, 867, 868, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (609, 1, 'T1-1135', '\x67656e6461726d65726965', NULL, 1217, 1218, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (610, 1, 'T1-301', '\x61747465696e746520c3a0206c612073c3bb726574c3a9206465206c27c389746174', NULL, 1219, 1220, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (639, 1, 'T1-1297', '\x616e746973c3a96d697469736d65', NULL, 1277, 1278, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (640, 1, 'T1-270', '\x72c3a9636c7573696f6e206372696d696e656c6c65', NULL, 1279, 1280, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (641, 1, 'T1-615', '\x7065696e65', NULL, 1281, 1282, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (642, 1, 'T1-271', '\x676172646520c3a020767565', NULL, 1283, 1284, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (643, 1, 'T1-468', '\x706f6c696365206a756469636961697265', NULL, 1285, 1286, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (644, 1, 'T1-272', '\x636f6e737472756374696f6e2061c3a9726f6e61757469717565', NULL, 1287, 1288, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (645, 1, 'T1-849', '\x696e64757374726965206dc3a963616e69717565', NULL, 1289, 1290, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (646, 1, 'T1-1406', '\x61c3a9726f6e6566', NULL, 1291, 1292, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (647, 1, 'T1-273', '\x68c3b474656c2064752064c3a970617274656d656e74', NULL, 1293, 1294, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (983, 1, 'T1-1381', '\x636f6e74726176656e74696f6e', NULL, 1965, 1966, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (507, 1, 'T1-192', '\x6173737572616e636520766965696c6c65737365', NULL, 1013, 1014, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (508, 1, 'T1-404', '\x706572736f6e6e6520c3a267c3a965', NULL, 1015, 1016, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (509, 1, 'T1-193', '\x636f6e74656e74696575782066697363616c', NULL, 1017, 1018, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (621, 1, 'T1-256', '\x626f75727365206427c3a97475646573', NULL, 1241, 1242, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (622, 1, 'T1-257', '\x70726573627974c3a87265', NULL, 1243, 1244, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (623, 1, 'T1-258', '\x73796e64696361742070726f66657373696f6e6e656c', NULL, 1245, 1246, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (624, 1, 'T1-847', '\x72656c6174696f6e73206465207472617661696c', NULL, 1247, 1248, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (625, 1, 'T1-259', '\x6368697275726769656e2064656e7469737465', NULL, 1249, 1250, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (626, 1, 'T1-260', '\x61646d696e697374726174696f6e2064c3a970617274656d656e74616c65', NULL, 1251, 1252, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (672, 1, 'T1-1130', '\x6c6f79657220696d6d6f62696c696572', NULL, 1343, 1344, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (673, 1, 'T1-656', '\x6c6f67656d656e7420696e646976696475656c', NULL, 1345, 1346, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (674, 1, 'T1-772', '\x6c6f67656d656e7420636f6c6c6563746966', NULL, 1347, 1348, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (675, 1, 'T1-302', '\x657870756c73696f6e206c6f636174697665', NULL, 1349, 1350, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (676, 1, 'T1-291', '\x6173737572616e63652064c3a963c3a873', NULL, 1351, 1352, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (677, 1, 'T1-292', '\x736f6e646167652067c3a96f6c6f6769717565', NULL, 1353, 1354, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (678, 1, 'T1-293', '\xc3a9636f6275616765', NULL, 1355, 1356, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (680, 1, 'T1-295', '\x6672617564652066697363616c65', NULL, 1359, 1360, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (681, 1, 'T1-296', '\x657363726f717565726965', NULL, 1361, 1362, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (683, 1, 'T1-299', '\x726573736f7274206a756469636961697265', NULL, 1365, 1366, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (628, 1, 'T1-262', '\x636f7276c3a96520726f79616c65', NULL, 1255, 1256, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (629, 1, 'T1-263', '\x636169737365206427c3a9706172676e65', NULL, 1257, 1258, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (959, 1, 'T1-549', '\x61697265206465206a657578', NULL, 1917, 1918, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (630, 1, 'T1-314', '\xc3a97461626c697373656d656e7420646520637265646974', NULL, 1259, 1260, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (632, 1, 'T1-264', '\x6a75727920642761737369736573', NULL, 1263, 1264, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (633, 1, 'T1-265', '\xc3a9636865632073636f6c61697265', NULL, 1265, 1266, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (638, 1, 'T1-281', '\x6a756461c3af736d65', NULL, 1275, 1276, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (691, 1, 'T1-313', '\x6a7567652061646d696e6973747261746966', NULL, 1381, 1382, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (692, 1, 'T1-622', '\x7072c3aa74657572', NULL, 1383, 1384, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (693, 1, 'T1-1386', '\x62616e717565206d757475616c69737465', NULL, 1385, 1386, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (694, 1, 'T1-315', '\x62c3a274696d656e7420696e647573747269656c', NULL, 1387, 1388, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (695, 1, 'T1-316', '\x636f7272757074696f6e', NULL, 1389, 1390, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (696, 1, 'T1-888', '\x66696e616e63657320636f6d6d756e616c6573', NULL, 1391, 1392, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (697, 1, 'T1-320', '\x746572726f7269736d65', NULL, 1393, 1394, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (699, 1, 'T1-323', '\x6167726963756c747572652062696f6c6f6769717565', NULL, 1397, 1398, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (1601, 1, 'T3-53', '\x636174616c6f677565', NULL, 3201, 3202, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (648, 1, 'T1-274', '\x6d61726368c3a92064652064657461696c', NULL, 1295, 1296, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (649, 1, 'T1-1385', '\x636f6c706f7274657572', NULL, 1297, 1298, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (650, 1, 'T1-1078', '\x6d61726368616e6420666f7261696e', NULL, 1299, 1300, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (651, 1, 'T1-275', '\xc3a9746174732067c3a96ec3a972617578', NULL, 1301, 1302, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (652, 1, 'T1-276', '\x64c3a96c6f63616c69736174696f6e', NULL, 1303, 1304, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (653, 1, 'T1-278', '\x706f6c696365207365636f757273', NULL, 1305, 1306, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (655, 1, 'T1-1423', '\x6f7267616e69736174696f6e2073616e697461697265', NULL, 1309, 1310, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (656, 1, 'T1-280', '\x656e736569676e656d656e7420636f6d6d65726369616c', NULL, 1311, 1312, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (657, 1, 'T1-1331', '\x656e736569676e656d656e742070726f66657373696f6e6e656c', NULL, 1313, 1314, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (659, 1, 'T1-282', '\x64726f69747320686f6e6f7269666971756573', NULL, 1317, 1318, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (660, 1, 'T1-283', '\x6f6cc3a96167696e657578', NULL, 1319, 1320, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (685, 1, 'T1-304', '\x63656e74726520646520666f726d6174696f6e', NULL, 1369, 1370, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (686, 1, 'T1-1196', '\x666f726d6174696f6e2070726f66657373696f6e6e656c6c65', NULL, 1371, 1372, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (687, 1, 'T1-306', '\x656e72656769737472656d656e74', NULL, 1373, 1374, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (688, 1, 'T1-308', '\x636f6d6d756e69636174696f6e20696e737469747574696f6e6e656c6c65', NULL, 1375, 1376, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (689, 1, 'T1-310', '\x726570726f64756374696f6e20616e696d616c65', NULL, 1377, 1378, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (700, 1, 'T1-324', '\x63616d70206427696e7465726e656d656e74', NULL, 1399, 1400, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (728, 1, 'T1-748', '\x68796769c3a86e65', NULL, 1455, 1456, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (1072, 1, 'T1-611', '\x6c6963656e6369656d656e74', NULL, 2143, 2144, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (704, 1, 'T1-679', '\x7472616e73706f727420666c757669616c', NULL, 1407, 1408, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (705, 1, 'T1-328', '\x6e6176697265', NULL, 1409, 1410, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (706, 1, 'T1-330', '\x6372c3a96174696f6e20617564696f76697375656c6c65', NULL, 1411, 1412, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (795, 1, 'T1-1357', '\x6661756e652073617576616765', NULL, 1589, 1590, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (707, 1, 'T1-331', '\x7175616c696669636174696f6e20636f7272656374696f6e6e656c6c65', NULL, 1413, 1414, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (708, 1, 'T1-1004', '\x726573746175726174696f6e20646573207465727261696e7320656e206d6f6e7461676e65', NULL, 1415, 1416, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (709, 1, 'T1-335', '\x6469736369706c696e652073706f7274697665', NULL, 1417, 1418, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (710, 1, 'T1-687', '\x746f757269736d65206465206d6f6e7461676e65', NULL, 1419, 1420, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (729, 1, 'T1-1170', '\x616dc3a96c696f726174696f6e206465206c2768616269746174', NULL, 1457, 1458, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (730, 1, 'T1-500', '\x6d616c616469652070726f66657373696f6e6e656c6c65', NULL, 1459, 1460, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (731, 1, 'T1-1163', '\x68796769c3a86e6520736f6369616c65', NULL, 1461, 1462, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (732, 1, 'T1-702', '\x6dc3a9646563696e652070c3a96e6974656e746961697265', NULL, 1463, 1464, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (733, 1, 'T1-841', '\x6dc3a9646563696e65206475207472617661696c', NULL, 1465, 1466, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (734, 1, 'T1-998', '\x6dc3a9646563696e652073636f6c61697265', NULL, 1467, 1468, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (735, 1, 'T1-1330', '\x6173737572616e6365206d616c61646965', NULL, 1469, 1470, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (737, 1, 'T1-346', '\x6d6f6e617263686965', NULL, 1473, 1474, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (739, 1, 'T1-1083', '\x64c3a963656e7472616c69736174696f6e', NULL, 1477, 1478, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (740, 1, 'T1-1452', '\x64c3a9636f6e63656e74726174696f6e', NULL, 1479, 1480, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (741, 1, 'T1-348', '\x6c6f7465726965', NULL, 1481, 1482, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (742, 1, 'T1-349', '\x6c696d697465207465727269746f7269616c65', NULL, 1483, 1484, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (743, 1, 'T1-351', '\x636974c3a92061646d696e697374726174697665', NULL, 1485, 1486, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (744, 1, 'T1-352', '\x616e6369656e20636f6d62617474616e74', NULL, 1487, 1488, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (745, 1, 'T1-1312', '\x656d706c6f692072c3a973657276c3a9', NULL, 1489, 1490, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (746, 1, 'T1-353', '\x666f6e6374696f6e6e61697265207465727269746f7269616c', NULL, 1491, 1492, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (702, 1, 'T1-326', '\x766f6965206e6176696761626c65', NULL, 1403, 1404, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (713, 1, 'T1-337', '\x706861726d61636965', NULL, 1425, 1426, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (716, 1, 'T1-728', '\x70726f6475697420636f736dc3a97469717565', NULL, 1431, 1432, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (717, 1, 'T1-625', '\x7370c3a96369616c6974c3a920706861726d616365757469717565', NULL, 1433, 1434, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (718, 1, 'T1-338', '\x616374696f6e2063756c747572656c6c65', NULL, 1435, 1436, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (719, 1, 'T1-339', '\x6e6f6d616465', NULL, 1437, 1438, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (720, 1, 'T1-1128', '\x616972652064652073746174696f6e6e656d656e74', NULL, 1439, 1440, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (721, 1, 'T1-340', '\x6c6f742064652070c3aa636865', NULL, 1441, 1442, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (1301, 1, 'T1-947', '\x736f6c646573', NULL, 2601, 2602, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (778, 1, 'T1-416', '\x65737061636520646f6d65737469717565', NULL, 1555, 1556, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (780, 1, 'T1-374', '\x72c3a96d756ec3a9726174696f6e', NULL, 1559, 1560, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (782, 1, 'T1-379', '\x656e736569676e656d656e7420c3a96cc3a96d656e7461697265', NULL, 1563, 1564, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (783, 1, 'T1-380', '\x62616c207075626c6963', NULL, 1565, 1566, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (784, 1, 'T1-381', '\x70c3aa6368657572', NULL, 1567, 1568, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (785, 1, 'T1-382', '\x636c616e64657374696e6974c3a9', NULL, 1569, 1570, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (786, 1, 'T1-1193', '\x72c3a9736572766520666f6e6369c3a87265', NULL, 1571, 1572, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (787, 1, 'T1-384', '\x636f6c6f6e69736174696f6e', NULL, 1573, 1574, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (788, 1, 'T1-1069', '\x636f6c6f6e6965', NULL, 1575, 1576, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (789, 1, 'T1-385', '\x696e737469747574696f6e73', NULL, 1577, 1578, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (790, 1, 'T1-387', '\x6f7267616e69736d6520706172617075626c6963', NULL, 1579, 1580, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (791, 1, 'T1-760', '\x6173736f63696174696f6e', NULL, 1581, 1582, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (792, 1, 'T1-388', '\x616d656e6465', NULL, 1583, 1584, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (796, 1, 'T1-1114', '\x706f6c696365206465206c6120636861737365', NULL, 1591, 1592, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (797, 1, 'T1-886', '\x636f6e666c6974206475207472617661696c', NULL, 1593, 1594, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (798, 1, 'T1-392', '\x72c3a9686162696c69746174696f6e', NULL, 1595, 1596, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (799, 1, 'T1-393', '\x746175726f6d6163686965', NULL, 1597, 1598, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (800, 1, 'T1-394', '\x696e6672616374696f6e206d61726974696d65', NULL, 1599, 1600, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (801, 1, 'T1-395', '\x766f696520636f6d6d756e616c65', NULL, 1601, 1602, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (802, 1, 'T1-595', '\x6368616e74696572', NULL, 1603, 1604, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (804, 1, 'T1-654', '\x736f6369c3a974c3a920646520636f6e737472756374696f6e', NULL, 1607, 1608, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (805, 1, 'T1-1346', '\x6f7576726965722064752062c3a274696d656e74', NULL, 1609, 1610, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (806, 1, 'T1-975', '\x6d6f6e6f706f6c65732066697363617578', NULL, 1611, 1612, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (807, 1, 'T1-1086', '\x64726f6974732064652063697263756c6174696f6e', NULL, 1613, 1614, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (808, 1, 'T1-399', '\x616dc3a96e6167656d656e74206475206c6974746f72616c', NULL, 1615, 1616, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (816, 1, 'T1-405', '\x636f6e67c3a97320706179c3a973', NULL, 1631, 1632, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (726, 1, 'T1-1268', '\x746865726d6f636c696d617469736d65', NULL, 1451, 1452, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (727, 1, 'T1-343', '\x616d69616e7465', NULL, 1453, 1454, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (756, 1, 'T1-378', '\x64c3a96368657420616e696d616c', NULL, 1511, 1512, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (757, 1, 'T1-442', '\x64c3a96368657420696e647573747269656c', NULL, 1513, 1514, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (758, 1, 'T1-1129', '\x6f726475726573206dc3a96e6167c3a8726573', NULL, 1515, 1516, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (759, 1, 'T1-800', '\x64c3a96368657420686f73706974616c696572', NULL, 1517, 1518, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (760, 1, 'T1-722', '\x7573696e65206427696e63696ec3a9726174696f6e', NULL, 1519, 1520, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (761, 1, 'T1-1159', '\x64c3a9636861726765207075626c69717565', NULL, 1521, 1522, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (762, 1, 'T1-363', '\x636f6e6772c3a87320706f6c697469717565', NULL, 1523, 1524, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (752, 1, 'T1-360', '\x6c697474c3a9726174757265', NULL, 1503, 1504, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (753, 1, 'T1-361', '\x6672616e63206d61c3a76f6e', NULL, 1505, 1506, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (754, 1, 'T1-370', '\x736f6369c3a974c3a92073656372c3a87465', NULL, 1507, 1508, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (755, 1, 'T1-879', '\x76696374696d6520646520677565727265', NULL, 1509, 1510, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (817, 1, 'T1-411', '\x66616d696c6c65', NULL, 1633, 1634, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (818, 1, 'T1-407', '\x617373656d626cc3a9652067c3a96ec3a972616c6520647520636c657267c3a9', NULL, 1635, 1636, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (819, 1, 'T1-408', '\x616d6e6973746965', NULL, 1637, 1638, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (820, 1, 'T1-409', '\x6f66666963696572206465206a757374696365206427416e6369656e2052c3a967696d65', NULL, 1639, 1640, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (821, 1, 'T1-1417', '\x6167656e74206465206c2761646d696e697374726174696f6e20726f79616c65', NULL, 1641, 1642, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (822, 1, 'T1-780', '\x6f666669636573', NULL, 1643, 1644, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (908, 1, 'T1-477', '\x6578747261646974696f6e', NULL, 1815, 1816, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (772, 1, 'T1-371', '\x61696465206a756469636961697265', NULL, 1543, 1544, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (775, 1, 'T1-1042', '\x616e696d616c20646520636f6d7061676e6965', NULL, 1549, 1550, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (776, 1, 'T1-1252', '\x76c3aa74656d656e74', NULL, 1551, 1552, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (777, 1, 'T1-686', '\x636f6d70746162696c6974c3a92070726976c3a965', NULL, 1553, 1554, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (809, 1, 'T1-1213', '\x7265626f6973656d656e74', NULL, 1617, 1618, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (810, 1, 'T1-874', '\x636f6e7365696c2064276172726f6e64697373656d656e74', NULL, 1619, 1620, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (811, 1, 'T1-402', '\x666f7263657320616c6c69c3a96573', NULL, 1621, 1622, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (812, 1, 'T1-403', '\x6573706163652076657274', NULL, 1623, 1624, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (813, 1, 'T1-1057', '\x70617263206e61747572656c', NULL, 1625, 1626, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (814, 1, 'T1-591', '\x706c6163656d656e742066616d696c69616c', NULL, 1627, 1628, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (834, 1, 'T1-546', '\xc3a96c6576657572', NULL, 1667, 1668, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (1691, 1, 'T3-147', '\x6f626a6574', NULL, 3381, 3382, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (825, 1, 'T1-1415', '\x6d6169736f6e2066616d696c69616c6520646520766163616e636573', NULL, 1649, 1650, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (826, 1, 'T1-533', '\x616666616972652066616d696c69616c65', NULL, 1651, 1652, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (827, 1, 'T1-790', '\x70726573746174696f6e2066616d696c69616c65', NULL, 1653, 1654, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (828, 1, 'T1-412', '\x616c636f6f6c69717565', NULL, 1655, 1656, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (829, 1, 'T1-413', '\x72c3a973656175206427696e666f726d6174696f6e', NULL, 1657, 1658, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (830, 1, 'T1-414', '\x6578616d656e2070726f66657373696f6e6e656c', NULL, 1659, 1660, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (831, 1, 'T1-415', '\x61706963756c74757265', NULL, 1661, 1662, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (832, 1, 'T1-417', '\x636c616e64657374696e', NULL, 1663, 1664, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (835, 1, 'T1-449', '\x6a65756e65206167726963756c74657572', NULL, 1669, 1670, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (836, 1, 'T1-1209', '\x6578706c6f6974616e742061677269636f6c65', NULL, 1671, 1672, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (837, 1, 'T1-1107', '\x626572676572', NULL, 1673, 1674, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (838, 1, 'T1-986', '\x6171756163756c74757265', NULL, 1675, 1676, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (839, 1, 'T1-653', '\x70c3aa636865', NULL, 1677, 1678, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (840, 1, 'T1-785', '\x6d61726368c3a92061677269636f6c65', NULL, 1679, 1680, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (842, 1, 'T1-423', '\x6f636375706174696f6e2074656d706f726169726520647520646f6d61696e65207075626c6963', NULL, 1683, 1684, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (843, 1, 'T1-424', '\x6d696e69737472652064752063756c7465', NULL, 1685, 1686, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (844, 1, 'T1-426', '\x73616e7320646f6d6963696c652066697865', NULL, 1687, 1688, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (845, 1, 'T1-427', '\x666f7274696669636174696f6e', NULL, 1689, 1690, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (847, 1, 'T1-559', '\x706f70756c6174696f6e2073636f6c61697265', NULL, 1693, 1694, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (849, 1, 'T1-1242', '\x64c3a9636973696f6e206465206a757374696365', NULL, 1697, 1698, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (850, 1, 'T1-430', '\xc3a96e6572676965206879647261756c69717565', NULL, 1699, 1700, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (851, 1, 'T1-431', '\x636f6d6d657263652064652064c3a97461696c', NULL, 1701, 1702, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (1450, 1, 'T2-29', '\x636f6f70c3a9726174696f6e', NULL, 2899, 2900, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (852, 1, 'T1-433', '\x6c616e67756520c3a97472616e67c3a87265', NULL, 1703, 1704, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (853, 1, 'T1-434', '\x65786f6ec3a9726174696f6e2066697363616c65', NULL, 1705, 1706, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (854, 1, 'T1-1109', '\xc3a97461626c697373656d656e74207075626c6963206e6174696f6e616c', NULL, 1707, 1708, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (855, 1, 'T1-771', '\x6f7267616e69736d652061646d696e6973747261746966206575726f70c3a9656e', NULL, 1709, 1710, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (856, 1, 'T1-1044', '\x61646d696e697374726174696f6e2063656e7472616c65', NULL, 1711, 1712, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (857, 1, 'T1-575', '\x7365727669636520657874c3a97269657572', NULL, 1713, 1714, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (858, 1, 'T1-1422', '\xc3a97461626c697373656d656e74207075626c6963206c6f63616c', NULL, 1715, 1716, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (860, 1, 'T1-1040', '\x61646d696e697374726174696f6e206c6f63616c65206427416e6369656e2052c3a967696d65', NULL, 1719, 1720, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (861, 1, 'T1-1088', '\x736572766963652064c3a9636f6e63656e7472c3a9', NULL, 1721, 1722, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (862, 1, 'T1-621', '\x61646d696e697374726174696f6e207072c3a9666563746f72616c65', NULL, 1723, 1724, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (863, 1, 'T1-437', '\x6167656e7420696d6d6f62696c696572', NULL, 1725, 1726, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (864, 1, 'T1-706', '\x6d61726368c3a920696d6d6f62696c696572', NULL, 1727, 1728, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (865, 1, 'T1-438', '\x636f6e7365696c6c65722064276172726f6e64697373656d656e74', NULL, 1729, 1730, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (866, 1, 'T1-439', '\x706861726d616369652076c3a974c3a972696e61697265', NULL, 1731, 1732, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (867, 1, 'T1-440', '\x7472616e73706f727420c3a020646f73206427686f6d6d65', NULL, 1733, 1734, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (868, 1, 'T1-443', '\x766f69652066657272c3a965206427696e74c3a972c3aa74206c6f63616c', NULL, 1735, 1736, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (1041, 1, 'T1-579', '\x64656d692d736f6c6465', NULL, 2081, 2082, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (869, 1, 'T1-1453', '\x646f6d6d6167657320646520677565727265', NULL, 1737, 1738, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (870, 1, 'T1-450', '\xc3a96c656374657572', NULL, 1739, 1740, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (875, 1, 'T1-454', '\x736569676e6575726965', NULL, 1749, 1750, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (876, 1, 'T1-455', '\x6cc3a967696f6e20c3a97472616e67c3a87265', NULL, 1751, 1752, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (877, 1, 'T1-937', '\x646973747269627574696f6e20c3a96c6563747269717565', NULL, 1753, 1754, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (878, 1, 'T1-458', '\x6173736f63696174696f6e20646520646566656e7365206465206c27656e7669726f6e6e656d656e74', NULL, 1755, 1756, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (879, 1, 'T1-459', '\x64c3a9636f757061676520c3a96c6563746f72616c', NULL, 1757, 1758, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (880, 1, 'T1-645', '\x63616e746f6e', NULL, 1759, 1760, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (954, 1, 'T1-502', '\x7069737465206379636c61626c65', NULL, 1907, 1908, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1042, 1, 'T1-580', '\x6c7963c3a965', NULL, 2083, 2084, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (881, 1, 'T1-1338', '\x6c69657574656e616e74206465206c6f7576657465726965', NULL, 1761, 1762, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (882, 1, 'T1-462', '\x746178652065787472616f7264696e61697265206427416e6369656e2052c3a967696d65', NULL, 1763, 1764, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (883, 1, 'T1-463', '\x6361646173747265', NULL, 1765, 1766, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (884, 1, 'T1-1143', '\x636172746f67726170686965', NULL, 1767, 1768, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (885, 1, 'T1-464', '\x64c3a970656e736520646520666f6e6374696f6e6e656d656e74', NULL, 1769, 1770, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (886, 1, 'T1-465', '\x6368616d7020646520666f697265', NULL, 1771, 1772, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (887, 1, 'T1-1354', '\x6173737572616e636520696e76616c69646974c3a9', NULL, 1773, 1774, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (888, 1, 'T1-779', '\x6173737572616e6365206368c3b46d616765', NULL, 1775, 1776, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (1045, 1, 'T1-588', '\x6162616e646f6e206427656e66616e74', NULL, 2089, 2090, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (890, 1, 'T1-673', '\x6173737572616e63652076657576616765', NULL, 1779, 1780, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (891, 1, 'T1-467', '\x656e66616e7420656e206761726465', NULL, 1781, 1782, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (892, 1, 'T1-1054', '\x696e7465726469742064652073c3a96a6f7572', NULL, 1783, 1784, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (893, 1, 'T1-652', '\x696e646976696475207265636865726368c3a9', NULL, 1785, 1786, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (894, 1, 'T1-1411', '\x6172726573746174696f6e', NULL, 1787, 1788, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (896, 1, 'T1-629', '\x6167656e7420646520706f6c696365206a756469636961697265', NULL, 1791, 1792, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (897, 1, 'T1-1162', '\x6c6962c3a9726174696f6e20636f6e646974696f6e6e656c6c65', NULL, 1793, 1794, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (898, 1, 'T1-469', '\x66726963686520696e647573747269656c6c65', NULL, 1795, 1796, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (1007, 1, 'T1-1022', '\x61c3a9726f64726f6d65', NULL, 2013, 2014, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (841, 1, 'T1-422', '\x61646d696e697374726174696f6e2072c3a967696f6e616c65', NULL, 1681, 1682, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (907, 1, 'T1-710', '\x72c3a97075626c69717565', NULL, 1813, 1814, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (909, 1, 'T1-789', '\x70617263206465206c6f6973697273', NULL, 1817, 1818, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (910, 1, 'T1-585', '\x696e7374616c6c6174696f6e2073706f7274697665', NULL, 1819, 1820, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (911, 1, 'T1-826', '\x64c3a962697420646520626f6973736f6e73', NULL, 1821, 1822, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (912, 1, 'T1-933', '\x636173696e6f', NULL, 1823, 1824, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (913, 1, 'T1-1155', '\x6772616e6465207375726661636520636f6d6d65726369616c65', NULL, 1825, 1826, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (914, 1, 'T1-820', '\xc3a96469666963652063756c7475656c', NULL, 1827, 1828, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (915, 1, 'T1-479', '\x7365727669747564652061c3a9726f6e61757469717565', NULL, 1829, 1830, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (919, 1, 'T1-1052', '\x6d696e65757220646520666f6e64', NULL, 1837, 1838, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (920, 1, 'T1-642', '\x6d6172696e20646520636f6d6d65726365', NULL, 1839, 1840, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (921, 1, 'T1-963', '\x61727469737465', NULL, 1841, 1842, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (922, 1, 'T1-1307', '\x617373697374616e7465206d617465726e656c6c65', NULL, 1843, 1844, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (924, 1, 'T1-535', '\x70726f7370656374696f6e2070c3a974726f6c69c3a87265', NULL, 1847, 1848, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (925, 1, 'T1-1455', '\x6578706c6f69746174696f6e206d696e69c3a87265', NULL, 1849, 1850, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (926, 1, 'T1-482', '\x6c6f67696369656c20696e666f726d617469717565', NULL, 1851, 1852, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (927, 1, 'T1-903', '\x73797374c3a86d65206427696e666f726d6174696f6e', NULL, 1853, 1854, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (928, 1, 'T1-486', '\x74c3a96cc3a9677261706865', NULL, 1855, 1856, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (929, 1, 'T1-1323', '\x636f6e63696c696174657572', NULL, 1857, 1858, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (930, 1, 'T1-994', '\x61766f636174', NULL, 1859, 1860, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (931, 1, 'T1-882', '\x6772656666696572', NULL, 1861, 1862, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (932, 1, 'T1-534', '\x657870657274', NULL, 1863, 1864, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (933, 1, 'T1-488', '\x7472616e73706f72742070617220616e696d616c', NULL, 1865, 1866, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (934, 1, 'T1-573', '\x616e696d616c2064652062c3a274', NULL, 1867, 1868, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (1451, 1, 'T2-44', '\x66696e616e63656d656e74', NULL, 2901, 2902, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (935, 1, 'T1-955', '\x76c3a9686963756c6520c3a0207472616374696f6e20616e696d616c65', NULL, 1869, 1870, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (936, 1, 'T1-1284', '\x656e66616e74206e61747572656c', NULL, 1871, 1872, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (937, 1, 'T1-934', '\x72c3a967696d65206d617472696d6f6e69616c', NULL, 1873, 1874, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (938, 1, 'T1-490', '\x7075626c69636974c3a9', NULL, 1875, 1876, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (939, 1, 'T1-491', '\x636f6f70c3a97261746976652061677269636f6c65', NULL, 1877, 1878, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (940, 1, 'T1-1099', '\x6578706c6f69746174696f6e2061677269636f6c65', NULL, 1879, 1880, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (941, 1, 'T1-817', '\x736f6369c3a974c3a920636f6f70c3a9726174697665', NULL, 1881, 1882, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (942, 1, 'T1-1348', '\x64c3a963c3a873', NULL, 1883, 1884, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (943, 1, 'T1-494', '\xc3a97075726174696f6e', NULL, 1885, 1886, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (944, 1, 'T1-497', '\x696e6475737472696520647520626f6973', NULL, 1887, 1888, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (945, 1, 'T1-498', '\x706172656e74206427c3a96cc3a87665', NULL, 1889, 1890, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (946, 1, 'T1-954', '\x656e736569676e656d656e74207075626c6963', NULL, 1891, 1892, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (947, 1, 'T1-1306', '\x656e736569676e656d656e74207365636f6e6461697265', NULL, 1893, 1894, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (948, 1, 'T1-1077', '\x6578616d656e', NULL, 1895, 1896, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1109, 1, 'T1-655', '\x70726f64756974206c616974696572', NULL, 2217, 2218, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (952, 1, 'T1-574', '\x656e736569676e656d656e7420c3a02064697374616e6365', NULL, 1903, 1904, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (953, 1, 'T1-501', '\xc3a9706964c3a96d6965', NULL, 1905, 1906, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (955, 1, 'T1-1257', '\x7665686963756c6520c3a0206465757820726f756573', NULL, 1909, 1910, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (956, 1, 'T1-504', '\x6173736f63696174696f6e206465206d616c6661697465757273', NULL, 1911, 1912, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (957, 1, 'T1-505', '\x64c3a96cc3a96775c3a920636f6e73756c61697265', NULL, 1913, 1914, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (958, 1, 'T1-507', '\x636f6e7365696c206465206469736369706c696e65', NULL, 1915, 1916, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (960, 1, 'T1-1216', '\x706c616365207075626c69717565', NULL, 1919, 1920, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (961, 1, 'T1-960', '\x68616c6c65', NULL, 1921, 1922, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1216, 1, 'T1-918', '\x706170617574c3a9', NULL, 2431, 2432, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1217, 1, 'T1-1291', '\x6d697373696f6e73', NULL, 2433, 2434, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (962, 1, 'T1-884', '\x7075697473', NULL, 1923, 1924, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (963, 1, 'T1-1320', '\x6c61766f6972', NULL, 1925, 1926, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (964, 1, 'T1-570', '\x686f726c6f6765207075626c69717565', NULL, 1927, 1928, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (965, 1, 'T1-835', '\x666f6e7461696e65', NULL, 1929, 1930, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (966, 1, 'T1-921', '\x636f6e737472756374696f6e2073636f6c61697265', NULL, 1931, 1932, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (967, 1, 'T1-509', '\x666f726d6174696f6e207175616c696669616e7465', NULL, 1933, 1934, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (968, 1, 'T1-510', '\x6368617566666167652075726261696e', NULL, 1935, 1936, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (969, 1, 'T1-512', '\xc3a963686576696e', NULL, 1937, 1938, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1229, 1, 'T1-1204', '\x656e74656e7465', NULL, 2457, 2458, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1033, 1, 'T1-1377', '\x61707072656e7469', NULL, 2065, 2066, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (902, 1, 'T1-1172', '\x6167656e74206465206c6120666f726365207075626c69717565', NULL, 1803, 1804, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (903, 1, 'T1-522', '\x706f6c696365206465732065617578', NULL, 1805, 1806, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (904, 1, 'T1-472', '\x72656d656d6272656d656e7420727572616c', NULL, 1807, 1808, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (905, 1, 'T1-473', '\x706572736f6e6e616c6974c3a9', NULL, 1809, 1810, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (906, 1, 'T1-474', '\x70726f6a657420c3a964756361746966', NULL, 1811, 1812, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (971, 1, 'T1-515', '\x636f7374756d65206f6666696369656c', NULL, 1941, 1942, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (972, 1, 'T1-516', '\x6167656e74206465206368616e6765', NULL, 1943, 1944, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (973, 1, 'T1-519', '\x6761726465206e6174696f6e616c65', NULL, 1945, 1946, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (974, 1, 'T1-520', '\x63756d756c206427656d706c6f69', NULL, 1947, 1948, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (975, 1, 'T1-809', '\x696e6672616374696f6e20c3a9636f6e6f6d69717565', NULL, 1949, 1950, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (976, 1, 'T1-521', '\x636f6d6d6572c3a7616e7420c3a97472616e676572', NULL, 1951, 1952, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (977, 1, 'T1-1038', '\x63686566206427656e7472657072697365', NULL, 1953, 1954, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (978, 1, 'T1-523', '\x6861626974617420696e73616c75627265', NULL, 1955, 1956, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (980, 1, 'T1-1014', '\x76696374696d65206427696e6672616374696f6e', NULL, 1959, 1960, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (981, 1, 'T1-1458', '\x64c3a96c696e7175616e6365', NULL, 1961, 1962, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1067, 1, 'T1-897', '\x6172747320706c6173746971756573', NULL, 2133, 2134, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (982, 1, 'T1-1084', '\x7175616c696669636174696f6e206372696d696e656c6c65', NULL, 1963, 1964, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1310, 1, 'T1-1049', '\x7461626163', NULL, 2619, 2620, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (984, 1, 'T1-1116', '\x696e64757374726965207370617469616c65', NULL, 1967, 1968, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (985, 1, 'T1-873', '\x696e64757374726965206475207665727265', NULL, 1969, 1970, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (986, 1, 'T1-1267', '\x696e6475737472696520647520666575', NULL, 1971, 1972, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (987, 1, 'T1-913', '\x6dc3a974616c6c7572676965', NULL, 1973, 1974, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (988, 1, 'T1-528', '\x63656e73757265', NULL, 1975, 1976, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (989, 1, 'T1-529', '\x696e737469747574696f6e206d6f6e61737469717565', NULL, 1977, 1978, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (990, 1, 'T1-530', '\x6d797374696369736d65', NULL, 1979, 1980, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (991, 1, 'T1-531', '\x706973636963756c74757265', NULL, 1981, 1982, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1406, 1, 'T2-65', '\x726563727574656d656e74', NULL, 2811, 2812, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (916, 1, 'T1-1008', '\x73657276697475646520726164696f20c3a96c6563747269717565', NULL, 1831, 1832, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (918, 1, 'T1-907', '\x7472617661696c6c65757220c3a020646f6d6963696c65', NULL, 1835, 1836, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (1182, 1, 'T1-753', '\x6a756d656c616765', NULL, 2363, 2364, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1002, 1, 'T1-1171', '\x636f6c6c65637465207075626c69717565', NULL, 2003, 2004, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1003, 1, 'T1-547', '\x646973747269627574696f6e20706f7374616c65', NULL, 2005, 2006, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1004, 1, 'T1-883', '\x7472616e73706f727420726f7574696572', NULL, 2007, 2008, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1005, 1, 'T1-617', '\x6368617566666575722064652074617869', NULL, 2009, 2010, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1009, 1, 'T1-1427', '\x63697263756c6174696f6e2075726261696e65', NULL, 2017, 2018, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1010, 1, 'T1-1428', '\x67617265', NULL, 2019, 2020, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1011, 1, 'T1-697', '\x6167656e636520646520766f7961676573', NULL, 2021, 2022, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1012, 1, 'T1-550', '\x656e747265707269736520696e647573747269656c6c65', NULL, 2023, 2024, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1013, 1, 'T1-977', '\x61726dc3a965206465207465727265', NULL, 2025, 2026, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1014, 1, 'T1-1352', '\x61726dc3a965206465206c27616972', NULL, 2027, 2028, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1015, 1, 'T1-552', '\x73706972697475616c6974c3a9', NULL, 2029, 2030, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1016, 1, 'T1-553', '\x7365726d656e7420646520666964c3a96c6974c3a9', NULL, 2031, 2032, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1017, 1, 'T1-1446', '\x76696c6c65206e6f7576656c6c65', NULL, 2033, 2034, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1018, 1, 'T1-961', '\x7a6f6e6520642761637469766974c3a973', NULL, 2035, 2036, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1698, 1, 'T3-154', '\x7065726d6973', NULL, 3395, 3396, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1119, 1, 'T1-1361', '\x657870726f7072696174696f6e', NULL, 2237, 2238, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1040, 1, 'T1-571', '\x6d696c6963657320626f757267656f69736573', NULL, 2079, 2080, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1043, 1, 'T1-582', '\x616dc3a96c696f726174696f6e2064657320736f6c73', NULL, 2085, 2086, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1044, 1, 'T1-587', '\x636f696666657572', NULL, 2087, 2088, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1046, 1, 'T1-589', '\x7472616974656d656e74206465732065617578207573c3a96573', NULL, 2091, 2092, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1047, 1, 'T1-590', '\x63c3a972c3a9616c65', NULL, 2093, 2094, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1048, 1, 'T1-592', '\x726164696f646966667573696f6e', NULL, 2095, 2096, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1049, 1, 'T1-593', '\x747261666963206465206d61696e2064276f6575767265', NULL, 2097, 2098, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1050, 1, 'T1-596', '\xc3a97461626c697373656d656e74206172746973616e616c', NULL, 2099, 2100, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1051, 1, 'T1-598', '\x70726f737469747574696f6e', NULL, 2101, 2102, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1052, 1, 'T1-599', '\x72c3a97365727665206e61747572656c6c65', NULL, 2103, 2104, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1832, 1, 'T3-235', '\x64c3a9636973696f6e', NULL, 3663, 3664, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1489, 1, 'T4-95', '\x636f6e66c3a972656e63652064652059616c746120283139343529', NULL, 2977, 2978, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1019, 1, 'T1-1192', '\x706f6c697469717565206465206c612076696c6c65', NULL, 2037, 2038, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1020, 1, 'T1-1431', '\x7a6f6e6520696e647573747269656c6c65', NULL, 2039, 2040, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1021, 1, 'T1-555', '\x617373656d626cc3a9652067c3a96ec3a972616c65', NULL, 2041, 2042, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1022, 1, 'T1-1133', '\x686162697461742075726261696e', NULL, 2043, 2044, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1023, 1, 'T1-1045', '\x73796e64696320646520636f70726f707269c3a974c3a9', NULL, 2045, 2046, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1024, 1, 'T1-671', '\x64c3a96d6f6c6974696f6e', NULL, 2047, 2048, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1025, 1, 'T1-1001', '\x6c6f67656d656e7420736f6369616c', NULL, 2049, 2050, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1026, 1, 'T1-1448', '\x6c6f67656d656e7420646520666f6e6374696f6e', NULL, 2051, 2052, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1392, 1, 'T1-1308', '\x73696461', NULL, 2783, 2784, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1185, 1, 'T1-757', '\x64726170656175', NULL, 2369, 2370, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (899, 1, 'T1-470', '\x6d616e69666573746174696f6e20636f6d6d65726369616c65', NULL, 1797, 1798, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (900, 1, 'T1-735', '\x706f6c696365206465206c612070c3aa636865', NULL, 1799, 1800, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (901, 1, 'T1-863', '\x706f6c696365206465732063756c746573', NULL, 1801, 1802, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (1054, 1, 'T1-1064', '\x6172726f6e64697373656d656e74', NULL, 2107, 2108, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1055, 1, 'T1-1328', '\x64697374726963742072c3a9766f6c7574696f6e6e61697265', NULL, 2109, 2110, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1056, 1, 'T1-1043', '\x646f6d61696e6520726f79616c', NULL, 2111, 2112, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1057, 1, 'T1-1035', '\x6167676c6f6dc3a9726174696f6e2075726261696e65', NULL, 2113, 2114, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1058, 1, 'T1-812', '\x73c3a96ec3a9636861757373c3a965', NULL, 2115, 2116, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1059, 1, 'T1-1372', '\x70617973', NULL, 2117, 2118, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1060, 1, 'T1-638', '\x67c3a96ec3a972616c6974c3a9', NULL, 2119, 2120, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1062, 1, 'T1-999', '\x70726f76696e6365', NULL, 2123, 2124, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1063, 1, 'T1-602', '\x616273656e74c3a969736d652073636f6c61697265', NULL, 2125, 2126, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1064, 1, 'T1-604', '\x666c657572', NULL, 2127, 2128, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1065, 1, 'T1-607', '\x636f757273206427656175', NULL, 2129, 2130, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1066, 1, 'T1-731', '\x6d757369717565', NULL, 2131, 2132, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1068, 1, 'T1-609', '\x656e736569676e656d656e742070c3a96e6974656e746961697265', NULL, 2135, 2136, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1069, 1, 'T1-610', '\x70c3a96461676f676965', NULL, 2137, 2138, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1070, 1, 'T1-787', '\x6269626c696f7468c3a87175652073636f6c61697265', NULL, 2139, 2140, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1071, 1, 'T1-1203', '\x6d6174c3a97269656c2070c3a96461676f6769717565', NULL, 2141, 2142, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1073, 1, 'T1-839', '\x61666661697265207072756427686f6d616c65', NULL, 2145, 2146, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1074, 1, 'T1-612', '\x6f7267616e69736174696f6e20696e7465726e6174696f6e616c65', NULL, 2147, 2148, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1075, 1, 'T1-613', '\x67616cc3a8726573', NULL, 2149, 2150, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1076, 1, 'T1-810', '\x6172746973616e', NULL, 2151, 2152, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1077, 1, 'T1-758', '\x7472617661757820666f7263c3a973', NULL, 2153, 2154, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1078, 1, 'T1-746', '\x7065696e6520646520737562737469747574696f6e', NULL, 2155, 2156, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1079, 1, 'T1-1009', '\x7065696e65206361706974616c65', NULL, 2157, 2158, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1080, 1, 'T1-1447', '\x7065696e6520636f72706f72656c6c65', NULL, 2159, 2160, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1081, 1, 'T1-1432', '\x61737369676e6174696f6e20c3a02072c3a9736964656e6365', NULL, 2161, 2162, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1082, 1, 'T1-619', '\x6372696d6520646520677565727265', NULL, 2163, 2164, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1083, 1, 'T1-620', '\x62c3a97461696c', NULL, 2165, 2166, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1085, 1, 'T1-815', '\x696d6d6967726174696f6e', NULL, 2169, 2170, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1086, 1, 'T1-1220', '\x70726f64756974206d617261c3ae63686572', NULL, 2171, 2172, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1087, 1, 'T1-1409', '\x6a617264696e2066616d696c69616c', NULL, 2173, 2174, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1088, 1, 'T1-1068', '\x677569646520696e7465727072c3a87465', NULL, 2175, 2176, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1089, 1, 'T1-1290', '\x63616d70696e67206361726176616e696e67', NULL, 2177, 2178, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1090, 1, 'T1-1144', '\x746f757269736d6520727572616c', NULL, 2179, 2180, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1550, 1, 'T4-89', '\x42656c6c6520c389706f717565', NULL, 3099, 3100, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1091, 1, 'T1-827', '\x746f757269736d652075726261696e', NULL, 2181, 2182, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1092, 1, 'T1-1102', '\x6369726375697420746f757269737469717565', NULL, 2183, 2184, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1093, 1, 'T1-1378', '\x6f7267616e69736d65206c6f63616c20646520746f757269736d65', NULL, 2185, 2186, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1094, 1, 'T1-1272', '\x6379636c6f746f757269736d65', NULL, 2187, 2188, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1095, 1, 'T1-635', '\x7472616e73706f72742073616e697461697265', NULL, 2189, 2190, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1096, 1, 'T1-636', '\x69736c616d', NULL, 2191, 2192, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1097, 1, 'T1-850', '\x68616e646973706f7274', NULL, 2193, 2194, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1098, 1, 'T1-1368', '\x6173736f63696174696f6e2073706f7274697665', NULL, 2195, 2196, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1099, 1, 'T1-798', '\x656475636174696f6e2073706f7274697665', NULL, 2197, 2198, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1100, 1, 'T1-640', '\x746178652070726f66657373696f6e6e656c6c65', NULL, 2199, 2200, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1161, 1, 'T1-726', '\x6d6f6e6e616965', NULL, 2321, 2322, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1592, 1, 'T4-15', '\x3765207369c3a8636c65', NULL, 3183, 3184, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1130, 1, 'T1-680', '\x64c3a9706f7274c3a9', NULL, 2259, 2260, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (917, 1, 'T1-517', '\x636f6e636965726765', NULL, 1833, 1834, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (1129, 1, 'T1-674', '\xc3a97461626c697373656d656e742070c3a96e6974656e746961697265', NULL, 2257, 2258, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1038, 1, 'T1-1199', '\x726573746175726174696f6e2073636f6c61697265', NULL, 2075, 2076, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1039, 1, 'T1-569', '\x6a75737469636520726f79616c65', NULL, 2077, 2078, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1053, 1, 'T1-600', '\x72c3a9736964656e636520646520746f757269736d65', NULL, 2105, 2106, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1311, 1, 'T1-978', '\x6163636f756368656d656e74', NULL, 2621, 2622, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1394, 1, 'T1-1418', '\x7375637265', NULL, 2787, 2788, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1102, 1, 'T1-644', '\x626f75636865726965', NULL, 2203, 2204, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1103, 1, 'T1-1410', '\x7669616e6465', NULL, 2205, 2206, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1104, 1, 'T1-646', '\x61696465206dc3a9646963616c6520757267656e7465', NULL, 2207, 2208, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1105, 1, 'T1-664', '\x61726368c3a96f6c6f67696520737562617175617469717565', NULL, 2209, 2210, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1106, 1, 'T1-1017', '\x61726368c3a96f6c6f67696520736f75732d6d6172696e65', NULL, 2211, 2212, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1107, 1, 'T1-650', '\x64c3a96cc3a96775c3a920647520706572736f6e6e656c', NULL, 2213, 2214, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1108, 1, 'T1-1067', '\x706f6973736f6e', NULL, 2215, 2216, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1112, 1, 'T1-660', '\x68796d6e65206e6174696f6e616c', NULL, 2223, 2224, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1113, 1, 'T1-754', '\x70726f6d656e61646520706c616e74c3a965', NULL, 2225, 2226, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1116, 1, 'T1-715', '\x636f6c6cc3a86765', NULL, 2231, 2232, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1117, 1, 'T1-1287', '\xc3a9636f6c652063656e7472616c65', NULL, 2233, 2234, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1118, 1, 'T1-665', '\x666f75726e69747572652073636f6c61697265', NULL, 2235, 2236, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1120, 1, 'T1-844', '\x6261757820636f6d6d65726369617578', NULL, 2239, 2240, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1121, 1, 'T1-1235', '\x6261757820727572617578', NULL, 2241, 2242, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1123, 1, 'T1-668', '\x706f6c696365206d756e69636970616c65', NULL, 2245, 2246, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1124, 1, 'T1-1136', '\x696d70c3b47420737572206c657320736f6369c3a974c3a973', NULL, 2247, 2248, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1125, 1, 'T1-1031', '\x7461786520642761707072656e74697373616765', NULL, 2249, 2250, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1153, 1, 'T1-932', '\x66726175646520636f6d6d65726369616c65', NULL, 2305, 2306, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1154, 1, 'T1-712', '\x68c3b474656c2064652076696c6c65', NULL, 2307, 2308, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1155, 1, 'T1-716', '\x6368616d616e69736d65', NULL, 2309, 2310, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1156, 1, 'T1-717', '\x747574656c6c65206175782070726573746174696f6e7320736f6369616c6573', NULL, 2311, 2312, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1157, 1, 'T1-1176', '\x747574656c6c65206a756469636961697265', NULL, 2313, 2314, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1158, 1, 'T1-1313', '\x64c3a96e6f6d6272656d656e7420736569676e65757269616c', NULL, 2315, 2316, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1159, 1, 'T1-719', '\x696e76616c696465206475207472617661696c', NULL, 2317, 2318, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1160, 1, 'T1-725', '\x6269656e2073706f6c69c3a9', NULL, 2319, 2320, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1162, 1, 'T1-729', '\xc3a96372697420737562766572736966', NULL, 2323, 2324, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1470, 1, 'T4-1', '\x5072c3a9686973746f697265', NULL, 2939, 2940, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (38, 1, 'T1-7', '\x636f75706520646520626f6973', NULL, 75, 76, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (1027, 1, 'T1-557', '\x626967616d6965', NULL, 2053, 2054, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1028, 1, 'T1-709', '\x6d6f7576656d656e74206465206a65756e65737365', NULL, 2055, 2056, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1031, 1, 'T1-699', '\x6368616e74696572206465206c61206a65756e65737365', NULL, 2061, 2062, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1032, 1, 'T1-833', '\xc3a974756469616e74', NULL, 2063, 2064, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1034, 1, 'T1-1244', '\x696e7465726e65', NULL, 2067, 2068, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1036, 1, 'T1-563', '\x707269736f6e6e69657220646520677565727265', NULL, 2071, 2072, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1037, 1, 'T1-564', '\x66616d696e65', NULL, 2073, 2074, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1218, 1, 'T1-1183', '\x636f6e63696c65', NULL, 2435, 2436, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1132, 1, 'T1-682', '\x6163636964656e74206475207472617661696c', NULL, 2263, 2264, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1133, 1, 'T1-685', '\x696e74657276656e74696f6e206427c3a96c75', NULL, 2265, 2266, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1134, 1, 'T1-688', '\x616c6c6575', NULL, 2267, 2268, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1135, 1, 'T1-689', '\x636f6d7061676e69652063686f72c3a9677261706869717565', NULL, 2269, 2270, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1136, 1, 'T1-691', '\x656e7472657072697365207075626c69717565', NULL, 2271, 2272, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1137, 1, 'T1-692', '\x617070617265696c20726164696f20c3a96c6563747269717565', NULL, 2273, 2274, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1138, 1, 'T1-945', '\x7365727669636520726164696f20c3a96c6563747269717565', NULL, 2275, 2276, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1139, 1, 'T1-694', '\x676172646520666f72657374696572', NULL, 2277, 2278, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1605, 1, 'T3-56', '\x6368616e736f6e', NULL, 3209, 3210, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1163, 1, 'T1-730', '\x696e7374616c6c6174696f6e20726164696f6c6f6769717565', NULL, 2325, 2326, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1164, 1, 'T1-732', '\x6d616c616465206d656e74616c', NULL, 2327, 2328, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1165, 1, 'T1-733', '\x64c3a9746563746976652070726976c3a9', NULL, 2329, 2330, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1166, 1, 'T1-734', '\x726f75746520666f7265737469c3a87265', NULL, 2331, 2332, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1167, 1, 'T1-985', '\x766f69652066657272c3a965206427696e74c3a972c3aa74206e6174696f6e616c', NULL, 2333, 2334, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1168, 1, 'T1-737', '\x706f70756c6174696f6e2070c3a96e6974656e746961697265', NULL, 2335, 2336, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1169, 1, 'T1-855', '\x696e63617263c3a9726174696f6e', NULL, 2337, 2338, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1170, 1, 'T1-1275', '\xc3a9766173696f6e', NULL, 2339, 2340, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1101, 1, 'T1-1112', '\x63686172626f6e20646520626f6973', NULL, 2201, 2202, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1223, 1, 'T1-802', '\x736572766974756465', NULL, 2445, 2446, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1224, 1, 'T1-864', '\x74656d706f72656c206563636cc3a9736961737469717565', NULL, 2447, 2448, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1225, 1, 'T1-1369', '\x68c3a972c3a9736965', NULL, 2449, 2450, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1226, 1, 'T1-1071', '\x636f6e636f7572732061646d696e6973747261746966', NULL, 2451, 2452, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1227, 1, 'T1-807', '\x7461786520666f6e6369c3a87265', NULL, 2453, 2454, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1228, 1, 'T1-808', '\x696e64c3a970656e64616e6365', NULL, 2455, 2456, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1230, 1, 'T1-814', '\x6a65752d636f6e636f757273', NULL, 2459, 2460, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1145, 1, 'T1-929', '\x72657472616974c3a9', NULL, 2289, 2290, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1146, 1, 'T1-713', '\x706f70756c6174696f6e20616374697665', NULL, 2291, 2292, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1147, 1, 'T1-783', '\x6174656c69657220646520636861726974c3a9', NULL, 2293, 2294, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1148, 1, 'T1-700', '\x747562657263756c6f7365', NULL, 2295, 2296, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1150, 1, 'T1-703', '\x636f75727365206869707069717565', NULL, 2299, 2300, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1151, 1, 'T1-705', '\x6c696169736f6e20726f757469c3a8726520696e7465726e6174696f6e616c65', NULL, 2301, 2302, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1152, 1, 'T1-711', '\x7075626c69636974c3a9206d656e736f6e67c3a87265', NULL, 2303, 2304, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1459, 1, 'T2-3', '\x6163717569736974696f6e', NULL, 2917, 2918, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1239, 1, 'T1-935', '\x72657374617572616e7420646520746f757269736d65', NULL, 2477, 2478, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1325, 1, 'T1-1011', '\x6f657566', NULL, 2649, 2650, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1577, 1, 'T4-18', '\x313065207369c3a8636c65', NULL, 3153, 3154, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1240, 1, 'T1-1347', '\xc3a97461626c697373656d656e7420696e74657264697420617578206d696e65757273', NULL, 2479, 2480, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1241, 1, 'T1-1270', '\x616c636f6f6c69736d65', NULL, 2481, 2482, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1242, 1, 'T1-828', '\x617070656c', NULL, 2483, 2484, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1176, 1, 'T1-993', '\x6465747465207075626c69717565', NULL, 2351, 2352, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1177, 1, 'T1-1245', '\x726563657474652066697363616c65', NULL, 2353, 2354, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1179, 1, 'T1-749', '\x676172646520706172746963756c696572', NULL, 2357, 2358, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1180, 1, 'T1-1036', '\x616dc3a96e6167656d656e7420666f72657374696572', NULL, 2359, 2360, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1181, 1, 'T1-751', '\x6f75747261676520617578206d6f65757273', NULL, 2361, 2362, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1183, 1, 'T1-856', '\x63616e616c69736174696f6e', NULL, 2365, 2366, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1184, 1, 'T1-1295', '\x646973747269627574696f6e2064652067617a', NULL, 2367, 2368, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1186, 1, 'T1-759', '\x66617578206d6f6e6e6179616765', NULL, 2371, 2372, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1187, 1, 'T1-997', '\x6173736f63696174696f6e207265636f6e6e75652064277574696c6974c3a9207075626c69717565', NULL, 2373, 2374, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1192, 1, 'T1-1454', '\x6d6967726174696f6e20727572616c65', NULL, 2383, 2384, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1194, 1, 'T1-795', '\x6a7567652064652070616978', NULL, 2387, 2388, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1196, 1, 'T1-768', '\x74c3a96c6578', NULL, 2391, 2392, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1213, 1, 'T1-811', '\x6661627269717565206427c3a9676c697365', NULL, 2425, 2426, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1214, 1, 'T1-1053', '\x61646d696e697374726174696f6e2064696f63c3a97361696e65', NULL, 2427, 2428, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1215, 1, 'T1-967', '\x73c3a96d696e616972652072656c696769657578', NULL, 2429, 2430, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1220, 1, 'T1-797', '\x6d616c6164696520c3a02064c3a9636c61726174696f6e206f626c696761746f697265', NULL, 2439, 2440, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1221, 1, 'T1-1363', '\x6167656e742064276173737572616e636573', NULL, 2441, 2442, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1222, 1, 'T1-949', '\x736f6369c3a974c3a9206465207365727669636573', NULL, 2443, 2444, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1246, 1, 'T1-831', '\x72657072c3a973656e746174696f6e206469706c6f6d617469717565', NULL, 2491, 2492, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1247, 1, 'T1-836', '\x736f7263656c6c65726965', NULL, 2493, 2494, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1248, 1, 'T1-1364', '\x6d61676965', NULL, 2495, 2496, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1249, 1, 'T1-840', '\x636f6d6d6973736169726520656e7175c3aa74657572', NULL, 2497, 2498, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1250, 1, 'T1-843', '\x636f6e7365696c2064652066616d696c6c65', NULL, 2499, 2500, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1794, 1, 'T3-40', '\x63617269636174757265', NULL, 3587, 3588, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (36, 1, 'T1-376', '\x66696e616e636573206c6f63616c6573', NULL, 71, 72, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (1251, 1, 'T1-846', '\x636f6d6d6572c3a7616e74', NULL, 2501, 2502, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1252, 1, 'T1-1208', '\x636f6d6974c3a9206427656e7472657072697365', NULL, 2503, 2504, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1253, 1, 'T1-987', '\x696e73657274696f6e20736f6369616c65', NULL, 2505, 2506, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1254, 1, 'T1-925', '\x6d6174c3a97269656c2061677269636f6c65', NULL, 2507, 2508, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1255, 1, 'T1-1217', '\x6f727068656c696e6174', NULL, 2509, 2510, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1256, 1, 'T1-1281', '\x61646f7074696f6e', NULL, 2511, 2512, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1257, 1, 'T1-858', '\x7a6f6e65206427616dc3a96e6167656d656e74', NULL, 2513, 2514, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1258, 1, 'T1-859', '\x6e617669676174696f6e2061c3a97269656e6e65', NULL, 2515, 2516, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1259, 1, 'T1-1240', '\x6e61747572616c69736174696f6e', NULL, 2517, 2518, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1260, 1, 'T1-1140', '\x7472617661696c206465206e756974', NULL, 2519, 2520, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1261, 1, 'T1-898', '\x686f7261697265206465207472617661696c', NULL, 2521, 2522, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1262, 1, 'T1-1100', '\x656d706c6f692064657320656e66616e7473', NULL, 2523, 2524, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1263, 1, 'T1-1194', '\x64656d616e64657572206427656d706c6f69', NULL, 2525, 2526, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1264, 1, 'T1-865', '\x7265736964656e636520756e6976657273697461697265', NULL, 2527, 2528, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1265, 1, 'T1-867', '\x7265636f6e647569746520c3a0206c612066726f6e7469c3a87265', NULL, 2529, 2530, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1267, 1, 'T1-1382', '\x636f6e7365696c6c65722072c3a967696f6e616c', NULL, 2533, 2534, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1271, 1, 'T1-1408', '\x636f6e736372697074696f6e', NULL, 2541, 2542, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1272, 1, 'T1-1039', '\x656e72c3b46c656d656e74', NULL, 2543, 2544, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1357, 1, 'T1-1124', '\x62617074c3aa6d6520636976696c', NULL, 2713, 2714, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1273, 1, 'T1-875', '\x67726f7570656d656e742064652070726f6475637465757273', NULL, 2545, 2546, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1274, 1, 'T1-876', '\x6173736f63696174696f6e2064652070c3aa636865', NULL, 2547, 2548, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1275, 1, 'T1-877', '\x61747465696e746520c3a0206c61206469676e6974c3a92064657320706572736f6e6e6573', NULL, 2549, 2550, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1238, 1, 'T1-1351', '\x62726f63616e74657572', NULL, 2475, 2476, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1280, 1, 'T1-880', '\x6e6f6e2d6c696575', NULL, 2559, 2560, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1281, 1, 'T1-1003', '\x706f696473206c6f757264', NULL, 2561, 2562, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1282, 1, 'T1-938', '\x6372c3a96174696f6e206427656e7472657072697365', NULL, 2563, 2564, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1283, 1, 'T1-890', '\x636f6e6368796c6963756c74757265', NULL, 2565, 2566, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1284, 1, 'T1-1153', '\x6d6f7576656d656e7420706f70756c61697265', NULL, 2567, 2568, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1285, 1, 'T1-895', '\x6163637565696c2070c3a9726973636f6c61697265', NULL, 2569, 2570, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1286, 1, 'T1-1426', '\x6d6174c3a97269656c20696e666f726d617469717565', NULL, 2571, 2572, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1287, 1, 'T1-904', '\x7365727669636520c3a964756361746966', NULL, 2573, 2574, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1243, 1, 'T1-1387', '\x74696d6272652066697363616c', NULL, 2485, 2486, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1245, 1, 'T1-1123', '\x636f6e747269627561626c65', NULL, 2489, 2490, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1276, 1, 'T1-969', '\x766575766520646520677565727265', NULL, 2551, 2552, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1277, 1, 'T1-1424', '\x676974616e', NULL, 2553, 2554, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1278, 1, 'T1-1104', '\x696e76616c69646520646520677565727265', NULL, 2555, 2556, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1279, 1, 'T1-1436', '\x70726973652064276f74616765', NULL, 2557, 2558, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1304, 1, 'T1-958', '\x61707072656e74697373616765', NULL, 2607, 2608, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1305, 1, 'T1-1384', '\x64c3a9626974206465207461626163', NULL, 2609, 2610, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1306, 1, 'T1-965', '\x64726f697420636f6d6d756e61757461697265', NULL, 2611, 2612, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1307, 1, 'T1-970', '\x6a75737469636520736569676e65757269616c65', NULL, 2613, 2614, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1308, 1, 'T1-972', '\x636f6e74726163657074696f6e', NULL, 2615, 2616, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1309, 1, 'T1-973', '\xc3a96e6572676965206465206c61206d6572', NULL, 2617, 2618, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1312, 1, 'T1-979', '\x73657276696365206475207472617661696c206f626c696761746f697265', NULL, 2623, 2624, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1313, 1, 'T1-980', '\x72656c6967696f6e206f7274686f646f7865', NULL, 2625, 2626, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1314, 1, 'T1-984', '\x746162616769736d65', NULL, 2627, 2628, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1315, 1, 'T1-989', '\x64c3a966656e73652070617373697665', NULL, 2629, 2630, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1316, 1, 'T1-990', '\x6c7574746520636f6e747265206c27c3a9726f73696f6e', NULL, 2631, 2632, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1317, 1, 'T1-1015', '\x63726f79616e63657320706f70756c6169726573', NULL, 2633, 2634, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1424, 1, 'T2-10', '\x6173736965747465', NULL, 2847, 2848, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1142, 1, 'T1-1419', '\x7472617661696c6c6575722066726f6e74616c696572', NULL, 2283, 2284, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1143, 1, 'T1-1344', '\x7472617661696c20636c616e64657374696e', NULL, 2285, 2286, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1144, 1, 'T1-1146', '\x6361727269c3a872652070726f66657373696f6e6e656c6c65', NULL, 2287, 2288, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1332, 1, 'T1-1098', '\x636f72706f726174696f6e', NULL, 2663, 2664, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1333, 1, 'T1-1037', '\x636c617373656d656e742073616e73207375697465', NULL, 2665, 2666, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1334, 1, 'T1-1041', '\x656e736569676e656d656e7420746563686e69717565', NULL, 2667, 2668, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1335, 1, 'T1-1048', '\x61747465696e746520c3a02070617069657273207075626c696373', NULL, 2669, 2670, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1336, 1, 'T1-1076', '\x7369676e616c69736174696f6e20726f757469c3a87265', NULL, 2671, 2672, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1337, 1, 'T1-1059', '\x68c3b474656c2064652072c3a967696f6e', NULL, 2673, 2674, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1338, 1, 'T1-1304', '\x6a657520646520636172746573', NULL, 2675, 2676, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1339, 1, 'T1-1309', '\x617070617265696c206465206a6575', NULL, 2677, 2678, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1340, 1, 'T1-1271', '\x706f70756c6174696f6e2075726261696e65', NULL, 2679, 2680, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1341, 1, 'T1-1314', '\x64c3a96d6f67726170686965', NULL, 2681, 2682, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1342, 1, 'T1-1191', '\x72617061747269c3a9', NULL, 2683, 2684, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1173, 1, 'T1-741', '\x666f72c3aa742070726976c3a965', NULL, 2345, 2346, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1174, 1, 'T1-742', '\x6169646520736f6369616c65206cc3a967616c65', NULL, 2347, 2348, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1175, 1, 'T1-1075', '\x7472616e7368756d616e6365', NULL, 2349, 2350, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1318, 1, 'T1-992', '\x6465677261646174696f6e206465206269656e73', NULL, 2635, 2636, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1319, 1, 'T1-1353', '\x70737963686f70c3a96461676f676965', NULL, 2637, 2638, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1320, 1, 'T1-1033', '\x73617576657461676520656e206d6572', NULL, 2639, 2640, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1321, 1, 'T1-1430', '\x73617065757220706f6d70696572', NULL, 2641, 2642, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1322, 1, 'T1-1065', '\x68616269746174696f6e20c3a0206c6f796572206d6f64c3a972c3a9', NULL, 2643, 2644, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (160, 1, 'T1-420', '\x6d656e6469636974c3a9', NULL, 319, 320, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (1291, 1, 'T1-1397', '\x626173652061c3a97269656e6e65', NULL, 2581, 2582, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1293, 1, 'T1-924', '\x736169736965', NULL, 2585, 2586, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1294, 1, 'T1-927', '\x6f7267616e69736d6520646520666f726d6174696f6e', NULL, 2587, 2588, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1295, 1, 'T1-930', '\x696d70c3b47420736f6369616c', NULL, 2589, 2590, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1296, 1, 'T1-1024', '\x636f6e63656e74726174696f6e', NULL, 2591, 2592, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1297, 1, 'T1-1398', '\x656e736569676e656d656e74206dc3a96e61676572', NULL, 2593, 2594, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1298, 1, 'T1-1260', '\x7472617661696c206d616e75656c', NULL, 2595, 2596, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1299, 1, 'T1-1429', '\x72c3a9666f726d6174696f6e', NULL, 2597, 2598, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1344, 1, 'T1-1080', '\x6d657572747265', NULL, 2687, 2688, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1345, 1, 'T1-1085', '\x76616363696e6174696f6e', NULL, 2689, 2690, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1346, 1, 'T1-1087', '\x74c3a96cc3a97068c3a97269717565', NULL, 2691, 2692, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1350, 1, 'T1-1286', '\x67656e7320646520677565727265', NULL, 2699, 2700, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1351, 1, 'T1-1148', '\x7472617661696c2070c3a96e6974656e746961697265', NULL, 2701, 2702, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1352, 1, 'T1-1118', '\x696e63617061626c65206d616a657572', NULL, 2703, 2704, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1353, 1, 'T1-1282', '\x72656e7472c3a9652073636f6c61697265', NULL, 2705, 2706, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1378, 1, 'T1-1416', '\x74726920706f7374616c', NULL, 2755, 2756, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1379, 1, 'T1-1187', '\x74c3a96cc3a970686f6e65', NULL, 2757, 2758, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1386, 1, 'T1-1229', '\x64c3a96d6f637261746965', NULL, 2771, 2772, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1387, 1, 'T1-1236', '\x6c69676e652061c3a97269656e6e65', NULL, 2773, 2774, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1642, 1, 'T3-202', '\x72c3a8676c656d656e74', NULL, 3283, 3284, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1388, 1, 'T1-1285', '\x76656e7465206a756469636961697265', NULL, 2775, 2776, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1389, 1, 'T1-1255', '\x6269656e2073696e69737472c3a9', NULL, 2777, 2778, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1390, 1, 'T1-1265', '\x6c61626f7261746f697265206427616e616c797365', NULL, 2779, 2780, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1391, 1, 'T1-1303', '\x62696f6469766572736974c3a9', NULL, 2781, 2782, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1393, 1, 'T1-1355', '\xc3a96e657267696520c3a96f6c69656e6e65', NULL, 2785, 2786, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1395, 1, 'T1-1375', '\x696d70c3b47420737572206c6520726576656e75', NULL, 2789, 2790, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1396, 1, 'T1-1392', '\x636f75707320657420626c65737375726573', NULL, 2791, 2792, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1397, 1, 'T1-1400', '\x6361726e6176616c', NULL, 2793, 2794, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1398, 1, 'T1-1421', '\x61766f7274656d656e74', NULL, 2795, 2796, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1399, 1, 'T2-38', '\x656e7472657469656e', NULL, 2797, 2798, 2, 1, 1, '2011-05-25 10:06:25', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1400, 1, 'T2-17', '\x636f6d6d697373696f6e6e656d656e74', NULL, 2799, 2800, 2, 1, 1, '2011-05-25 10:06:25', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1401, 1, 'T2-42', '\x6578c3a9637574696f6e2062756467c3a97461697265', NULL, 2801, 2802, 2, 1, 1, '2011-05-25 10:06:25', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1402, 1, 'T2-40', '\xc3a9766163756174696f6e', NULL, 2803, 2804, 2, 1, 1, '2011-05-25 10:06:25', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1403, 1, 'T2-62', '\x726563656e73656d656e74', NULL, 2805, 2806, 2, 1, 1, '2011-05-25 10:06:25', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1404, 1, 'T2-20', '\x636f6e63696c696174696f6e', NULL, 2807, 2808, 2, 1, 1, '2011-05-25 10:06:25', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1405, 1, 'T2-53', '\x6c6f636174696f6e', NULL, 2809, 2810, 2, 1, 1, '2011-05-25 10:06:25', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1407, 1, 'T2-51', '\x696e666f726d617469736174696f6e', NULL, 2813, 2814, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1408, 1, 'T2-31', '\x64c3a96cc3a9676174696f6e', NULL, 2815, 2816, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1409, 1, 'T2-59', '\x70726f6772616d6d6174696f6e', NULL, 2817, 2818, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1410, 1, 'T2-28', '\x636f6e7472c3b46c652073616e697461697265', NULL, 2819, 2820, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1411, 1, 'T2-43', '\x6578706c6f69746174696f6e20636f6d6d65726369616c65', NULL, 2821, 2822, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1412, 1, 'T2-68', '\x726573746175726174696f6e', NULL, 2823, 2824, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1414, 1, 'T2-16', '\x636c617373656d656e74', NULL, 2827, 2828, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1415, 1, 'T2-39', '\xc3a971756970656d656e74206d6174c3a97269656c', NULL, 2829, 2830, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1420, 1, 'T2-6', '\x616772c3a96d656e74', NULL, 2839, 2840, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1421, 1, 'T2-19', '\x636f6e63657373696f6e', NULL, 2841, 2842, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1422, 1, 'T2-4', '\x616468c3a973696f6e', NULL, 2843, 2844, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1423, 1, 'T2-67', '\x72c3a9717569736974696f6e', NULL, 2845, 2846, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1425, 1, 'T2-63', '\x7265636f757273206869c3a9726172636869717565', NULL, 2849, 2850, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1426, 1, 'T2-26', '\x636f6e7472c3b46c652064652073c3a96375726974c3a9', NULL, 2851, 2852, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1427, 1, 'T2-33', '\x64c3a97369676e6174696f6e', NULL, 2853, 2854, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1430, 1, 'T2-48', '\x67657374696f6e20647520706572736f6e6e656c', NULL, 2859, 2860, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1431, 1, 'T2-2', '\x61627374656e74696f6e', NULL, 2861, 2862, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1432, 1, 'T2-64', '\x7265636f757672656d656e74', NULL, 2863, 2864, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1433, 1, 'T2-9', '\x617270656e74616765', NULL, 2865, 2866, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1434, 1, 'T2-69', '\x746172696669636174696f6e', NULL, 2867, 2868, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1435, 1, 'T2-27', '\x636f6e7472c3b46c652066697363616c', NULL, 2869, 2870, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1436, 1, 'T2-35', '\x646973736f6c7574696f6e', NULL, 2871, 2872, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1437, 1, 'T2-24', '\x636f6e7472c3b46c652064652067657374696f6e', NULL, 2873, 2874, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1419, 1, 'T2-45', '\x666f6e6374696f6e6e656d656e74', NULL, 2837, 2838, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1446, 1, 'T2-56', '\x706c6163656d656e74', NULL, 2891, 2892, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1447, 1, 'T2-32', '\x64c3a96e6f6d696e6174696f6e', NULL, 2893, 2894, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1448, 1, 'T2-66', '\x72c3a9676c656d656e746174696f6e', NULL, 2895, 2896, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1452, 1, 'T2-22', '\x636f6e7472c3b46c65', NULL, 2903, 2904, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1453, 1, 'T2-11', '\x61737369676e6174696f6e', NULL, 2905, 2906, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1454, 1, 'T2-60', '\x70726f6d6f74696f6e', NULL, 2907, 2908, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1455, 1, 'T2-7', '\x616dc3a96e6167656d656e74', NULL, 2909, 2910, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1456, 1, 'T2-37', '\x656e7175c3aa7465207075626c69717565', NULL, 2911, 2912, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1458, 1, 'T2-58', '\x7072c3a976656e74696f6e', NULL, 2915, 2916, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1460, 1, 'T2-14', '\x63617574696f6e', NULL, 2919, 2920, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1461, 1, 'T2-41', '\xc3a976616c756174696f6e', NULL, 2921, 2922, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1462, 1, 'T2-52', '\x696e7370656374696f6e', NULL, 2923, 2924, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1463, 1, 'T2-70', '\x747574656c6c652061646d696e697374726174697665', NULL, 2925, 2926, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1464, 1, 'T2-30', '\x636f6f7264696e6174696f6e', NULL, 2927, 2928, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1465, 1, 'T2-13', '\x6175746f7269736174696f6e', NULL, 2929, 2930, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1467, 1, 'T2-8', '\x617070656c2064276f6666726573', NULL, 2933, 2934, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1468, 1, 'T2-25', '\x636f6e7472c3b46c65206465206cc3a967616c6974c3a9', NULL, 2935, 2936, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1469, 1, 'T2-36', '\x656e7175c3aa7465', NULL, 2937, 2938, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1471, 1, 'T4-8', '\xc3a9706f71756520636f6e74656d706f7261696e65', NULL, 2941, 2942, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1472, 1, 'T4-3', '\x4d6f79656e20c3826765', NULL, 2943, 2944, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1473, 1, 'T4-2', '\x416e746971756974c3a9', NULL, 2945, 2946, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1474, 1, 'T4-5', '\x52c3a9766f6c7574696f6e2064652031373839', NULL, 2947, 2948, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1475, 1, 'T4-7', '\x5072656d69657220456d706972652028313830342d3138313529', NULL, 2949, 2950, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1476, 1, 'T4-4', '\xc3a9706f717565206d6f6465726e65', NULL, 2951, 2952, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1477, 1, 'T4-6', '\x436f6e73756c61742028313739392d3138303429', NULL, 2953, 2954, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1478, 1, 'T4-26', '\x6775657272652064652043656e7420416e732028313333372d3134353329', NULL, 2955, 2956, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1479, 1, 'T4-25', '\x657870616e73696f6e206465206c2769736c616d20283632322d3134393229', NULL, 2957, 2958, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1480, 1, 'T4-58', '\x46726f6e64652028313634382d3136353229', NULL, 2959, 2960, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1481, 1, 'T4-29', '\x313765207369c3a8636c65', NULL, 2961, 2962, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1482, 1, 'T4-80', '\x526573746175726174696f6e2028313831352d3138333029', NULL, 2963, 2964, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1483, 1, 'T4-44', '\x313965207369c3a8636c65', NULL, 2965, 2966, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1484, 1, 'T4-36', '\x436f6e76656e74696f6e206e6174696f6e616c652028313739322d3137393529', NULL, 2967, 2968, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1485, 1, 'T4-47', '\x43696e717569c3a86d652052c3a97075626c697175652028313935382d2029', NULL, 2969, 2970, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1486, 1, 'T4-72', '\x61626f6c6974696f6e206465206c276573636c617661676520283138343829', NULL, 2971, 2972, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1487, 1, 'T4-84', '\x5365636f6e6420456d706972652028313835322d3138373029', NULL, 2973, 2974, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1661, 1, 'T3-206', '\x72c3a9706572746f69726520636976696c', NULL, 3321, 3322, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1440, 1, 'T2-12', '\x6173737572616e6365', NULL, 2879, 2880, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1586, 1, 'T4-14', '\x3665207369c3a8636c65', NULL, 3171, 3172, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1490, 1, 'T4-45', '\x323065207369c3a8636c65', NULL, 2979, 2980, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1382, 1, 'T1-1305', '\x6c69767265207361696e74', NULL, 2763, 2764, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1383, 1, 'T1-1211', '\x666c6f72652073617576616765', NULL, 2765, 2766, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1384, 1, 'T1-1224', '\x6162757320646520666f6e6374696f6e', NULL, 2767, 2768, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1439, 1, 'T2-5', '\x61646a756469636174696f6e', NULL, 2877, 2878, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1441, 1, 'T2-46', '\x676172616e746965206427656d7072756e74', NULL, 2881, 2882, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1442, 1, 'T2-49', '\x696d6d6174726963756c6174696f6e', NULL, 2883, 2884, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1443, 1, 'T2-34', '\x6465737472756374696f6e', NULL, 2885, 2886, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1444, 1, 'T2-54', '\x6e756dc3a97269736174696f6e', NULL, 2887, 2888, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1445, 1, 'T2-61', '\x70726f74656374696f6e', NULL, 2889, 2890, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1491, 1, 'T4-24', '\x63726f6973616465732028313039352d3132393129', NULL, 2981, 2982, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1492, 1, 'T4-35', '\x63616d7061676e652064274974616c69652028313739362d3137393729', NULL, 2983, 2984, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1493, 1, 'T4-57', '\x52656e61697373616e6365', NULL, 2985, 2986, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1494, 1, 'T4-28', '\x313665207369c3a8636c65', NULL, 2987, 2988, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1495, 1, 'T4-27', '\x696e766173696f6e73207363616e64696e6176657320283739302d3130333029', NULL, 2989, 2990, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1496, 1, 'T4-94', '\x636f6e66c3a972656e6365206465204d756e69636820283139333829', NULL, 2991, 2992, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1497, 1, 'T4-76', '\x677565727265206465204372696dc3a9652028313835332d3138353629', NULL, 2993, 2994, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1498, 1, 'T4-74', '\x636f7570206427c38974617420647520322064c3a963656d6272652031383531', NULL, 2995, 2996, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1499, 1, 'T4-73', '\x436f6d6d756e6520646520506172697320283138373129', NULL, 2997, 2998, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1500, 1, 'T4-82', '\x52c3a9766f6c7574696f6e2064652031383438', NULL, 2999, 3000, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1501, 1, 'T4-81', '\x52c3a9766f6c7574696f6e2064652031383330', NULL, 3001, 3002, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1502, 1, 'T4-85', '\x756e6974c3a920616c6c656d616e646520283138373129', NULL, 3003, 3004, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1503, 1, 'T4-86', '\x756e6974c3a9206974616c69656e6e652028313965207369c3a8636c6529', NULL, 3005, 3006, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1504, 1, 'T4-77', '\x6775657272652064652053c3a963657373696f6e2028313836312d3138363529', NULL, 3007, 3008, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1505, 1, 'T4-78', '\x6775657272652064652031383730', NULL, 3009, 3010, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1821, 1, 'T3-189', '\x726170706f7274206465207374616765', NULL, 3641, 3642, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1518, 1, 'T4-114', '\x67756572726520313933392d31393435', NULL, 3035, 3036, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1519, 1, 'T4-125', '\x52c3a9766f6c7574696f6e20727573736520283139313729', NULL, 3037, 3038, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1520, 1, 'T4-99', '\xc3a976c3a96e656d656e7473206465206d61692031393638', NULL, 3039, 3040, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1521, 1, 'T4-105', '\x677565727265206427496e646f6368696e652028313934362d3139353429', NULL, 3041, 3042, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1699, 1, 'T3-58', '\x6368726f6e69717565', NULL, 3397, 3398, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1522, 1, 'T4-127', '\x7472616974c3a9206465206c2741746c616e7469717565204e6f726420283139343929', NULL, 3043, 3044, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1523, 1, 'T4-110', '\x6775657272652066726f6964652028313934352d3139383929', NULL, 3045, 3046, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1524, 1, 'T4-88', '\x61666661697265206465205375657a20283139353629', NULL, 3047, 3048, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1525, 1, 'T4-121', '\x7072696e74656d70732064652050726167756520283139363829', NULL, 3049, 3050, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1526, 1, 'T4-116', '\x4c6962c3a9726174696f6e2028313934342d3139343529', NULL, 3051, 3052, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1528, 1, 'T4-107', '\x67756572726520646573204d616c6f75696e657320283139383229', NULL, 3055, 3056, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1622, 1, 'T3-74', '\x636f757075726520646520707265737365', NULL, 3243, 3244, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1532, 1, 'T4-109', '\x67756572726520647520566965746e616d2028313936322d3139373329', NULL, 3063, 3064, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1533, 1, 'T4-92', '\x636f6e63696c65205661746963616e20322028313936322d3139363529', NULL, 3065, 3066, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1534, 1, 'T4-108', '\x67756572726520647520476f6c66652028313939302d3139393129', NULL, 3067, 3068, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1535, 1, 'T4-119', '\x7061637465206765726d616e6f2d736f7669c3a9746971756520283139333929', NULL, 3069, 3070, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1536, 1, 'T4-122', '\x517561747269c3a86d652052c3a97075626c697175652028313934362d3139353829', NULL, 3071, 3072, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1537, 1, 'T4-101', '\x676f757665726e656d656e742064652056696368792028313934302d3139343429', NULL, 3073, 3074, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1538, 1, 'T4-112', '\x67756572726520727573736f2d6a61706f6e616973652028313930342d3139303529', NULL, 3075, 3076, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1380, 1, 'T1-1292', '\x737461676961697265', NULL, 2759, 2760, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1381, 1, 'T1-1197', '\x7468c3a96f6c6f676965', NULL, 2761, 2762, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1542, 1, 'T4-120', '\x706c616e204d61727368616c6c20283139343729', NULL, 3083, 3084, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1543, 1, 'T4-102', '\x476f757665726e656d656e742070726f7669736f6972652028313934342d3139343629', NULL, 3085, 3086, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1728, 1, 'T3-141', '\x6d6f74696f6e', NULL, 3455, 3456, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1544, 1, 'T4-123', '\x52c3a97075626c69717565206465205765696d61722028313931392d3139333329', NULL, 3087, 3088, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1545, 1, 'T4-113', '\x67756572726520313931342d31393138', NULL, 3089, 3090, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1546, 1, 'T4-104', '\x67756572726520642745737061676e652028313933362d3139333929', NULL, 3091, 3092, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1719, 1, 'T3-41', '\x6361726e65742064652073616e74c3a9', NULL, 3437, 3438, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1547, 1, 'T4-115', '\x696e73757272656374696f6e20686f6e67726f69736520283139353629', NULL, 3093, 3094, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1632, 1, 'T3-125', '\x6d61696e20636f7572616e7465', NULL, 3263, 3264, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1548, 1, 'T4-126', '\x73c3a97061726174696f6e2064657320c389676c69736573206574206465206c27c38974617420283139303529', NULL, 3095, 3096, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1549, 1, 'T4-124', '\x72c3a9756e696669636174696f6e20616c6c656d616e64652028313938392d3139393029', NULL, 3097, 3098, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1551, 1, 'T4-38', '\x67756572726573206465206c612052c3a9766f6c7574696f6e2028313739322d3137393929', NULL, 3101, 3102, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1552, 1, 'T4-39', '\x677565727265732064652056656e64c3a9652028313739332d3137393929', NULL, 3103, 3104, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1553, 1, 'T4-34', '\x63616d7061676e65206427c38967797074652028313739382d3138303129', NULL, 3105, 3106, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1737, 1, 'T3-124', '\x6c6976726574206f757672696572', NULL, 3473, 3474, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1555, 1, 'T4-32', '\x417373656d626cc3a96520636f6e7374697475616e74652028313738392d3137393129', NULL, 3109, 3110, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1556, 1, 'T4-40', '\x546572726575722028313739332d3137393429', NULL, 3111, 3112, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1557, 1, 'T4-49', '\x54726f697369c3a86d652052c3a97075626c697175652028313837302d3139343029', NULL, 3113, 3114, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1558, 1, 'T4-11', '\x4261732d456d70697265', NULL, 3115, 3116, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1559, 1, 'T4-48', '\x636f6e737472756374696f6e206575726f70c3a9656e6e65', NULL, 3117, 3118, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1800, 1, 'T3-11', '\x616e6e7561697265', NULL, 3599, 3600, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1506, 1, 'T4-75', '\x4465757869c3a86d652052c3a97075626c697175652028313834382d3138353229', NULL, 3011, 3012, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1507, 1, 'T4-79', '\x6d6f6e617263686965206465204a75696c6c65742028313833302d3138343829', NULL, 3013, 3014, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1510, 1, 'T4-59', '\x6775657272652064652044c3a9766f6c7574696f6e2028313636372d3136363829', NULL, 3019, 3020, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1512, 1, 'T4-91', '\x436f6c6c61626f726174696f6e2028313934302d3139343429', NULL, 3023, 3024, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1513, 1, 'T4-98', '\x656e7472652d646575782d67756572726573', NULL, 3025, 3026, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1514, 1, 'T4-90', '\x626c6f637573206465204265726c696e2028313934382d3139343929', NULL, 3027, 3028, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1515, 1, 'T4-103', '\x677565727265206427416c67c3a97269652028313935342d3139363229', NULL, 3029, 3030, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1516, 1, 'T4-117', '\x6d7572206465204265726c696e2028313936312d3139383929', NULL, 3031, 3032, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1517, 1, 'T4-106', '\x67756572726520646520436f72c3a9652028313935302d3139353329', NULL, 3033, 3034, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1673, 1, 'T3-148', '\x6f72647265206465206d697373696f6e', NULL, 3345, 3346, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1576, 1, 'T4-42', '\x636f6e6772c3a873206465205669656e6e6520283138313529', NULL, 3151, 3152, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1578, 1, 'T4-54', '\x677565727265732064274974616c69652028313439342d3135353929', NULL, 3155, 3156, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1579, 1, 'T4-55', '\x677565727265732064652072656c6967696f6e2028313536322d3135393829', NULL, 3157, 3158, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1583, 1, 'T4-51', '\x436f6e7472652d52c3a9666f726d65', NULL, 3165, 3166, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1584, 1, 'T4-13', '\x3565207369c3a8636c65', NULL, 3167, 3168, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1585, 1, 'T4-19', '\x313165207369c3a8636c65', NULL, 3169, 3170, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1587, 1, 'T4-9', '\x416e746971756974c3a92067616c6c6f2d726f6d61696e65', NULL, 3173, 3174, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1705, 1, 'T3-30', '\x62726f6368757265', NULL, 3409, 3410, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1571, 1, 'T4-65', '\x677565727265206465205365707420416e732028313735362d3137363329', NULL, 3141, 3142, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1572, 1, 'T4-62', '\x6775657272652064652053756363657373696f6e20642745737061676e652028313730312d3137313429', NULL, 3143, 3144, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1573, 1, 'T4-69', '\x52c3a967656e63652028313731352d3137323329', NULL, 3145, 3146, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1574, 1, 'T4-41', '\x43656e74206a6f75727320283138313529', NULL, 3147, 3148, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1575, 1, 'T4-43', '\x67756572726573206e61706f6cc3a96f6e69656e6e65732028313830302d3138313529', NULL, 3149, 3150, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1596, 1, 'T4-21', '\x313365207369c3a8636c65', NULL, 3191, 3192, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1597, 1, 'T4-16', '\x3865207369c3a8636c65', NULL, 3193, 3194, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1598, 1, 'T4-23', '\x313565207369c3a8636c65', NULL, 3195, 3196, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1599, 1, 'T4-50', '\x63726f697361646520636f6e747265206c657320416c626967656f69732028313230382d3132323929', NULL, 3197, 3198, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1623, 1, 'T3-26', '\x62696c616e20636f6d707461626c65', NULL, 3245, 3246, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1624, 1, 'T3-96', '\x646f73736965722064652070656e73696f6e', NULL, 3247, 3248, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1625, 1, 'T3-197', '\x726567697374726520647520636f6d6d657263652065742064657320736f6369c3a974c3a973', NULL, 3249, 3250, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1626, 1, 'T3-94', '\x646f7373696572206465206361727269c3a87265', NULL, 3251, 3252, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1627, 1, 'T3-168', '\x706f727472616974', NULL, 3253, 3254, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1628, 1, 'T3-180', '\x70726f74c3aa74', NULL, 3255, 3256, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1629, 1, 'T3-146', '\x6e6f7469636520696e646976696475656c6c65', NULL, 3257, 3258, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1631, 1, 'T3-13', '\x617272c3aa74c3a9206427616c69676e656d656e74', NULL, 3261, 3262, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1633, 1, 'T3-158', '\x70c3a9746974696f6e', NULL, 3265, 3266, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1634, 1, 'T3-112', '\x696e76656e7461697265', NULL, 3267, 3268, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1540, 1, 'T4-100', '\x46726f6e7420706f70756c616972652028313933362d3139333829', NULL, 3079, 3080, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1541, 1, 'T4-93', '\x636f6e66c3a972656e63652064652042616e64756e6720283139353529', NULL, 3081, 3082, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1610, 1, 'T3-199', '\x7265676973747265206d6174726963756c65', NULL, 3219, 3220, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1611, 1, 'T3-196', '\x72656769737472652064657320656e7472c3a965732065742064657320736f7274696573', NULL, 3221, 3222, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1612, 1, 'T3-76', '\x64c3a9636973696f6e2062756467c3a97461697265206d6f64696669636174697665', NULL, 3223, 3224, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1710, 1, 'T3-128', '\x6d61717565747465', NULL, 3419, 3420, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1712, 1, 'T3-142', '\x6e6f6d656e636c6174757265', NULL, 3423, 3424, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1614, 1, 'T3-217', '\x736368c3a96d6120646972656374657572206427616dc3a96e6167656d656e74206574206427757262616e69736d65', NULL, 3227, 3228, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1711, 1, 'T3-191', '\x7265676973747265', NULL, 3421, 3422, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1615, 1, 'T3-81', '\x64c3a9636c61726174696f6e2064652073756363657373696f6e', NULL, 3229, 3230, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1617, 1, 'T3-77', '\x64c3a9636973696f6e20646520706169656d656e74', NULL, 3233, 3234, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1302, 1, 'T1-953', '\x636f6d6dc3a96d6f726174696f6e', NULL, 2603, 2604, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1303, 1, 'T1-957', '\x72656c6174696f6e73206575726f70c3a9656e6e6573', NULL, 2605, 2606, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1560, 1, 'T4-130', '\x7472616974c3a920737572206c27756e696f6e206575726f70c3a9656e6e6520283139393229', NULL, 3119, 3120, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1561, 1, 'T4-129', '\x7472616974c3a920646520526f6d6520283139353729', NULL, 3121, 3122, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1562, 1, 'T4-31', '\x6d6f6e617263686965206427416e6369656e2052c3a967696d65', NULL, 3123, 3124, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1564, 1, 'T4-12', '\x67756572726520646573204761756c6573', NULL, 3127, 3128, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1565, 1, 'T4-64', '\x72c3a9766f636174696f6e206465206c27c3a9646974206465204e616e74657320283136383529', NULL, 3129, 3130, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1618, 1, 'T3-64', '\x636f6d7074652064276578706c6f69746174696f6e', NULL, 3235, 3236, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1619, 1, 'T3-99', '\x646f737369657220646573206f75767261676573206578c3a9637574c3a973', NULL, 3237, 3238, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1620, 1, 'T3-4', '\x6163746520646520636f6e6772c3a873', NULL, 3239, 3240, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1621, 1, 'T3-116', '\x6c6973746520c3a96c6563746f72616c65', NULL, 3241, 3242, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1649, 1, 'T3-19', '\x61746c61732074657272696572', NULL, 3297, 3298, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1650, 1, 'T3-111', '\x696e736372697074696f6e206879706f7468c3a96361697265', NULL, 3299, 3300, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1651, 1, 'T3-25', '\x6269626c696f67726170686965', NULL, 3301, 3302, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1652, 1, 'T3-122', '\x6c69767265206c697475726769717565', NULL, 3303, 3304, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1653, 1, 'T3-127', '\x6d616e696665737465206465206e6176697265', NULL, 3305, 3306, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1663, 1, 'T3-9', '\x616c62756d', NULL, 3325, 3326, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1636, 1, 'T3-151', '\x70616e6e6561752064276578706f736974696f6e', NULL, 3271, 3272, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1637, 1, 'T3-183', '\x7175657374696f6e6e61697265206427656e7175c3aa7465', NULL, 3273, 3274, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1638, 1, 'T3-159', '\x70686f746f67726170686965', NULL, 3275, 3276, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1639, 1, 'T3-60', '\x63697263756c61697265', NULL, 3277, 3278, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1640, 1, 'T3-33', '\x62756467657420737570706cc3a96d656e7461697265', NULL, 3279, 3280, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1641, 1, 'T3-115', '\x6c69737465206427c3a96d617267656d656e74', NULL, 3281, 3282, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1643, 1, 'T3-204', '\x72c3a8676c656d656e7420696e74657269657572', NULL, 3285, 3286, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1644, 1, 'T3-57', '\x636861727465', NULL, 3287, 3288, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1600, 1, 'T3-34', '\x62756c6c6520706f6e7469666963616c65', NULL, 3199, 3200, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1602, 1, 'T3-198', '\x726567697374726520647520636f757272696572', NULL, 3203, 3204, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1603, 1, 'T3-83', '\x64c3a9636c61726174696f6e206427696e74656e74696f6e206427616c69c3a96e6572', NULL, 3205, 3206, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1604, 1, 'T3-157', '\x7065726d697320646520726563686572636865', NULL, 3207, 3208, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1606, 1, 'T3-118', '\x6c6976726520636f6d707461626c65', NULL, 3211, 3212, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1607, 1, 'T3-185', '\x726170706f727420642761637469766974c3a9', NULL, 3213, 3214, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1609, 1, 'T3-32', '\x627564676574207072696d69746966', NULL, 3217, 3218, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1684, 1, 'T3-145', '\x6e6f74652064652073657276696365', NULL, 3367, 3368, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1685, 1, 'T3-22', '\x61766973', NULL, 3369, 3370, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1724, 1, 'T3-161', '\x706c616e', NULL, 3447, 3448, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1595, 1, 'T4-20', '\x313265207369c3a8636c65', NULL, 3189, 3190, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1570, 1, 'T4-67', '\x696e64c3a970656e64616e63652064657320c389746174732d556e69732028313737352d3137383229', NULL, 3139, 3140, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1675, 1, 'T3-140', '\x6d6f64c3a86c65206465206661627269717565', NULL, 3349, 3350, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1677, 1, 'T3-61', '\x636f6d6d756e697175c3a920646520707265737365', NULL, 3353, 3354, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1680, 1, 'T3-1', '\x6163636f7264206465207472617661696c', NULL, 3359, 3360, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1670, 1, 'T3-17', '\x617272c3aa74c3a9207072c3a9666563746f72616c', NULL, 3339, 3340, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1703, 1, 'T3-164', '\x706c616e206465207365636f757273', NULL, 3405, 3406, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1704, 1, 'T3-153', '\x7061737365706f7274', NULL, 3407, 3408, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1706, 1, 'T3-232', '\x7472616e736372697074696f6e206879706f7468c3a96361697265', NULL, 3411, 3412, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1707, 1, 'T3-205', '\x72c3a9706572746f697265', NULL, 3413, 3414, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1708, 1, 'T3-233', '\x77617272616e74', NULL, 3415, 3416, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1709, 1, 'T3-5', '\x6163746520646520736f6369c3a974c3a9', NULL, 3417, 3418, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1713, 1, 'T3-174', '\x70726f63c3a8732d76657262616c2064652072c3a9756e696f6e', NULL, 3425, 3426, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1714, 1, 'T3-38', '\x636168696572206465732063686172676573', NULL, 3427, 3428, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1715, 1, 'T3-139', '\x6d696e757465206e6f74617269616c65', NULL, 3429, 3430, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1716, 1, 'T3-222', '\x73756a65742064276578616d656e', NULL, 3431, 3432, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1717, 1, 'T3-20', '\x6175746f7269736174696f6e206427757262616e69736d65', NULL, 3433, 3434, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1718, 1, 'T3-113', '\x6c61697373657a2d706173736572', NULL, 3435, 3436, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1720, 1, 'T3-227', '\x74696d627265', NULL, 3439, 3440, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1721, 1, 'T3-39', '\x63616c656e6472696572', NULL, 3441, 3442, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1722, 1, 'T3-28', '\x62696c616e20736f6369616c', NULL, 3443, 3444, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1723, 1, 'T3-182', '\x7075626c69636174696f6e2070c3a972696f6469717565', NULL, 3445, 3446, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1725, 1, 'T3-62', '\x636f6d7074652061646d696e6973747261746966', NULL, 3449, 3450, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1726, 1, 'T3-186', '\x726170706f72742064652067656e6461726d65726965', NULL, 3451, 3452, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1727, 1, 'T3-63', '\x636f6d7074652064652067657374696f6e', NULL, 3453, 3454, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1729, 1, 'T3-143', '\x6e6f726d65', NULL, 3457, 3458, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1730, 1, 'T3-171', '\x70726f63c3a8732d76657262616c206427616dc3a96e6167656d656e7420666f72657374696572', NULL, 3459, 3460, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1731, 1, 'T3-225', '\x74c3a96cc3a96772616d6d65', NULL, 3461, 3462, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1732, 1, 'T3-69', '\x636f6e76656e74696f6e', NULL, 3463, 3464, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1733, 1, 'T3-121', '\x6c6976726520666f6e63696572', NULL, 3465, 3466, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1734, 1, 'T3-48', '\x63617274652064276964656e746974c3a9', NULL, 3467, 3468, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1735, 1, 'T3-165', '\x706c616e2064276f636375706174696f6e2064657320736f6c73', NULL, 3469, 3470, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1736, 1, 'T3-85', '\x64c3a9636c61726174696f6e2066697363616c65', NULL, 3471, 3472, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1738, 1, 'T3-72', '\x636f7069652064276578616d656e', NULL, 3475, 3476, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1743, 1, 'T3-169', '\x70726573736520696e737469747574696f6e6e656c6c65', NULL, 3485, 3486, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1744, 1, 'T3-230', '\x74697472652064652073c3a96a6f7572', NULL, 3487, 3488, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1747, 1, 'T3-226', '\x7465787465206f6666696369656c', NULL, 3493, 3494, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1748, 1, 'T3-35', '\x62756c6c6574696e2064652073616c61697265', NULL, 3495, 3496, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1749, 1, 'T3-114', '\x6c69737465', NULL, 3497, 3498, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1750, 1, 'T3-73', '\x636f72726573706f6e64616e6365', NULL, 3499, 3500, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1751, 1, 'T3-47', '\x636172746520646520766f6575', NULL, 3501, 3502, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1753, 1, 'T3-166', '\x706c616e206427757262616e69736d65', NULL, 3505, 3506, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1754, 1, 'T3-10', '\x616e6e6f6e6365206f6666696369656c6c65206574206cc3a967616c65', NULL, 3507, 3508, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1755, 1, 'T3-218', '\x736f6d6d69657220646573206d61726368616e6469736573', NULL, 3509, 3510, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1646, 1, 'T3-84', '\x64c3a9636c61726174696f6e2064277574696c6974c3a9207075626c69717565', NULL, 3291, 3292, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1647, 1, 'T3-3', '\x6163746520636976696c207075626c6963', NULL, 3293, 3294, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1648, 1, 'T3-89', '\x646f63756d656e7420617564696f76697375656c', NULL, 3295, 3296, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1671, 1, 'T3-162', '\x706c616e2063616461737472616c', NULL, 3341, 3342, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1674, 1, 'T3-126', '\x6d616e69666573746520642761c3a9726f6e6566', NULL, 3347, 3348, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1681, 1, 'T3-103', '\xc3a9746174206475206d6f6e74616e742064657320726f6c6573', NULL, 3361, 3362, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1682, 1, 'T3-131', '\x6d6174726963652063616461737472616c65', NULL, 3363, 3364, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1683, 1, 'T3-45', '\x6361727465206465206d656d627265', NULL, 3365, 3366, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1686, 1, 'T3-136', '\x6d657263757269616c65', NULL, 3371, 3372, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1687, 1, 'T3-29', '\x627265766574206427696e76656e74696f6e', NULL, 3373, 3374, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1776, 1, 'T3-31', '\x627564676574', NULL, 3551, 3552, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1688, 1, 'T3-55', '\x63657274696669636174206427757262616e69736d65', NULL, 3375, 3376, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1689, 1, 'T3-2', '\x616374652061757468656e7469717565', NULL, 3377, 3378, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1690, 1, 'T3-137', '\x6d6963726f666f726d65', NULL, 3379, 3380, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1693, 1, 'T3-167', '\x706fc3a86d65', NULL, 3385, 3386, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1694, 1, 'T3-14', '\x617272c3aa74c3a9206475206d61697265', NULL, 3387, 3388, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1696, 1, 'T3-207', '\x72c3a9706572746f69726520646573206dc3a97469657273', NULL, 3391, 3392, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1697, 1, 'T3-71', '\x636f6e76656e74696f6e20646520666f726d6174696f6e', NULL, 3393, 3394, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1700, 1, 'T3-194', '\x726567697374726520646520636174686f6c69636974c3a9', NULL, 3399, 3400, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1701, 1, 'T3-130', '\x6d6172717565206465206661627269717565', NULL, 3401, 3402, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1702, 1, 'T3-43', '\x636172746520c3a0206a6f756572', NULL, 3403, 3404, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1756, 1, 'T3-54', '\x63657274696669636174', NULL, 3511, 3512, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1757, 1, 'T3-88', '\x646973636f757273', NULL, 3513, 3514, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1758, 1, 'T3-210', '\x72c3a9706572746f6972652067c3a96ec3a972616c2064657320616666616972657320636976696c6573', NULL, 3515, 3516, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1759, 1, 'T3-97', '\x646f737369657220646520707265737365', NULL, 3517, 3518, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1760, 1, 'T3-15', '\x617272c3aa74c3a9206475207072c3a9736964656e7420647520636f6e7365696c2067c3a96ec3a972616c', NULL, 3519, 3520, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1761, 1, 'T3-149', '\x6f7267616e696772616d6d65', NULL, 3521, 3522, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1762, 1, 'T3-181', '\x7075626c69636174696f6e206f6666696369656c6c65', NULL, 3523, 3524, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1763, 1, 'T3-229', '\x74697472652064652070726f707269c3a974c3a9', NULL, 3525, 3526, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1764, 1, 'T3-80', '\x64c3a9636c61726174696f6e20646520726576656e7573', NULL, 3527, 3528, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1765, 1, 'T3-91', '\x646f63756d656e74206669677572c3a9', NULL, 3529, 3530, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1766, 1, 'T3-120', '\x6c697672652064276f72', NULL, 3531, 3532, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1767, 1, 'T3-184', '\x726170706f7274', NULL, 3533, 3534, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1768, 1, 'T3-107', '\x66616972652d70617274', NULL, 3535, 3536, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1769, 1, 'T3-119', '\x6c6976726520646520726169736f6e', NULL, 3537, 3538, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1770, 1, 'T3-223', '\x7461626c65', NULL, 3539, 3540, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1771, 1, 'T3-100', '\x646f737369657220696e646976696475656c', NULL, 3541, 3542, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1772, 1, 'T3-6', '\x6163746520736f7573207365696e672070726976c3a9', NULL, 3543, 3544, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1773, 1, 'T3-104', '\xc3a97461742070617263656c6c61697265', NULL, 3545, 3546, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1774, 1, 'T3-219', '\x7374617469737469717565', NULL, 3547, 3548, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1778, 1, 'T3-172', '\x70726f63c3a8732d76657262616c206427c3a96c656374696f6e', NULL, 3555, 3556, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1779, 1, 'T3-187', '\x726170706f7274206465206d6572', NULL, 3557, 3558, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1780, 1, 'T3-98', '\x646f73736965722064652070726f63c3a964757265', NULL, 3559, 3560, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1781, 1, 'T3-79', '\x64c3a9636c61726174696f6e20646520646f75616e65', NULL, 3561, 3562, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1782, 1, 'T3-82', '\x64c3a9636c61726174696f6e2064652074726176617578', NULL, 3563, 3564, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1783, 1, 'T3-193', '\x726567697374726520642761756469656e6365', NULL, 3565, 3566, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1785, 1, 'T3-109', '\x6669636869657220696d6d6f62696c696572', NULL, 3569, 3570, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1786, 1, 'T3-70', '\x636f6e76656e74696f6e20636f6c6c656374697665', NULL, 3571, 3572, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1787, 1, 'T3-160', '\x7069c3a8636520636f6d707461626c65', NULL, 3573, 3574, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1790, 1, 'T3-200', '\x7265676973747265207061726f69737369616c', NULL, 3579, 3580, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1178, 1, 'T1-956', '\x64c3a973696e66656374696f6e', NULL, 2355, 2356, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1791, 1, 'T3-163', '\x706c616e206427616c69676e656d656e74', NULL, 3581, 3582, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1792, 1, 'T3-86', '\x64c3a96c6962c3a9726174696f6e', NULL, 3583, 3584, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1793, 1, 'T3-190', '\x726170706f7274206427657870657274697365', NULL, 3585, 3586, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1795, 1, 'T3-65', '\x636f6d7074652066696e616e63696572', NULL, 3589, 3590, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1796, 1, 'T3-175', '\x70726f63c3a8732d76657262616c206465207363656c6cc3a973', NULL, 3591, 3592, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1797, 1, 'T3-201', '\x72656769737472652070726f74657374616e74', NULL, 3593, 3594, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1798, 1, 'T3-188', '\x726170706f727420646520706f6c696365', NULL, 3595, 3596, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1802, 1, 'T3-24', '\x6261736520646520646f6e6ec3a96573', NULL, 3603, 3604, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1830, 1, 'T3-106', '\x6578706c6f69742064276875697373696572', NULL, 3659, 3660, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1831, 1, 'T3-117', '\x6c69737465206e6f6d696e6174697665', NULL, 3661, 3662, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1, 1, 'T1-503', '\x636f6d6d756e69636174696f6e73', NULL, 1, 2, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (60, 1, 'T1-495', '\x6269656e732064c3a970617274656d656e74617578', NULL, 119, 120, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (81, 1, 'T1-891', '\x72c3a9756e696f6e207075626c69717565', NULL, 161, 162, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (92, 1, 'T1-1310', '\x707269c3a87265207075626c69717565', NULL, 183, 184, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (114, 1, 'T1-1300', '\x7265636865726368652064616e73206c27696e74c3a972c3aa74206465732066616d696c6c6573', NULL, 227, 228, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (148, 1, 'T1-41', '\x61696465207075626c69717565206175206c6f67656d656e74', NULL, 295, 296, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (1803, 1, 'T3-123', '\x6c6976726574206d696c697461697265', NULL, 3605, 3606, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1804, 1, 'T3-173', '\x70726f63c3a8732d76657262616c206427656e7175c3aa7465', NULL, 3607, 3608, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1805, 1, 'T3-78', '\x64c3a9636c61726174696f6e', NULL, 3609, 3610, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1806, 1, 'T3-95', '\x646f737369657220646520636f6e73756c746174696f6e2064657320656e747265707269736573', NULL, 3611, 3612, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1807, 1, 'T3-195', '\x7265676973747265206427c3a963726f75', NULL, 3613, 3614, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1808, 1, 'T3-87', '\x6469706cc3b46d65', NULL, 3615, 3616, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1809, 1, 'T3-46', '\x6361727465206465207472617661696c', NULL, 3617, 3618, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1810, 1, 'T3-215', '\x7363656175', NULL, 3619, 3620, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1811, 1, 'T3-213', '\x72c3b46c65206427c3a971756970616765', NULL, 3621, 3622, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (703, 1, 'T1-1256', '\x626174656175206465206e617669676174696f6e20696e74c3a9726965757265', NULL, 1405, 1406, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (724, 1, 'T1-577', '\x616e6e6578696f6e206465207465727269746f697265', NULL, 1447, 1448, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (748, 1, 'T1-355', '\xc3a97461626c697373656d656e74206dc3a96469636f20736f6369616c', NULL, 1495, 1496, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (781, 1, 'T1-1138', '\x66696e616e6365732064c3a970617274656d656e74616c6573', NULL, 1561, 1562, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (793, 1, 'T1-1371', '\x6173736f63696174696f6e20646520636861737365', NULL, 1585, 1586, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (404, 1, 'T1-138', '\x6173736f63696174696f6e20666f6e6369c3a872652075726261696e65', NULL, 807, 808, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (435, 1, 'T1-1319', '\x616374696f6e2073616e697461697265', NULL, 869, 870, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (458, 1, 'T1-457', '\x70726f64756374696f6e20c3a96c6563747269717565', NULL, 915, 916, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (803, 1, 'T1-914', '\x70617274696369706174696f6e2064657320656d706c6f7965757273', NULL, 1605, 1606, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (833, 1, 'T1-707', '\x61747465696e746520c3a0206c276f72647265207075626c6963', NULL, 1665, 1666, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (859, 1, 'T1-1435', '\x61646d696e697374726174696f6e20636f6d6d756e616c65', NULL, 1717, 1718, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (871, 1, 'T1-911', '\x64726f697473206369766971756573', NULL, 1741, 1742, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (872, 1, 'T1-675', '\x636f6c6cc3a8676520c3a96c6563746f72616c', NULL, 1743, 1744, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (895, 1, 'T1-1226', '\x6f6666696369657220646520706f6c696365206a756469636961697265', NULL, 1789, 1790, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (923, 1, 'T1-851', '\x766f7961676575722072657072c3a973656e74616e7420706c6163696572', NULL, 1845, 1846, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (1324, 1, 'T1-1007', '\x6f7267616e69736d652067c3a96ec3a974697175656d656e74206d6f64696669c3a9', NULL, 2647, 2648, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1347, 1, 'T1-1332', '\x61726d656d656e74206d61726974696d65', NULL, 2693, 2694, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1349, 1, 'T1-1261', '\x736f6369c3a974c3a9206427696e74c3a972c3aa7420636f6c6c65637469662061677269636f6c65', NULL, 2697, 2698, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1385, 1, 'T1-1289', '\x68c3b474656c206465206c61207072c3a966656374757265', NULL, 2769, 2770, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1413, 1, 'T2-23', '\x636f6e7472c3b46c652062756467c3a97461697265', NULL, 2825, 2826, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1428, 1, 'T2-71', '\x747574656c6c652066696e616e6369c3a87265', NULL, 2855, 2856, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1438, 1, 'T2-57', '\x7072c3a97061726174696f6e2062756467c3a97461697265', NULL, 2875, 2876, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1801, 1, 'T3-68', '\x636f6e7472617420642761707072656e74697373616765', NULL, 3601, 3602, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (6, 1, 'T1-1359', '\x74656d7073206c6962726520657420736f63696162696c6974c3a9', NULL, 11, 12, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (32, 1, 'T1-861', '\x636f6e646974696f6e73206475207472617661696c', NULL, 63, 64, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (1817, 1, 'T3-177', '\x70726f63c3a8732d76657262616c206427696e6672616374696f6e', NULL, 3633, 3634, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1818, 1, 'T3-133', '\x6dc3a96461696c6c65', NULL, 3635, 3636, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1819, 1, 'T3-101', '\x646f7373696572206dc3a9646963616c', NULL, 3637, 3638, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1820, 1, 'T3-110', '\x6879706f7468c3a8717565206d61726974696d65', NULL, 3639, 3640, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1823, 1, 'T3-66', '\x636f6d7074652072656e6475', NULL, 3645, 3646, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1824, 1, 'T3-90', '\x646f63756d656e742064652073c3a9616e6365', NULL, 3647, 3648, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1825, 1, 'T3-37', '\x63616869657220646520646f6cc3a9616e6365', NULL, 3649, 3650, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (319, 1, 'T1-878', '\x677265666669657220646520636f6d6d65726365', NULL, 637, 638, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (338, 1, 'T1-870', '\xc3a96c656374696f6e206575726f70c3a9656e6e65', NULL, 675, 676, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (362, 1, 'T1-478', '\xc3a97461626c697373656d656e74207265636576616e74206475207075626c6963', NULL, 723, 724, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (393, 1, 'T1-131', '\x6f7267616e69736d6520636f6e73756c7461746966', NULL, 785, 786, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (398, 1, 'T1-1094', '\x70726f74656374696f6e206d617465726e656c6c6520657420696e66616e74696c65', NULL, 795, 796, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (1815, 1, 'T3-93', '\x646f73736965722064652063616e6469646174757265', NULL, 3629, 3630, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (161, 1, 'T1-46', '\x636f6e74656e7469657578206465206c612073c3a96375726974c3a920736f6369616c65', NULL, 321, 322, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (190, 1, 'T1-1167', '\x70726f74656374696f6e2073616e697461697265206475206368657074656c', NULL, 379, 380, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (220, 1, 'T1-513', '\x636f6d7061676e69652064276173737572616e636573', NULL, 439, 440, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (238, 1, 'T1-917', '\x74696572732d6f72647265', NULL, 475, 476, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (239, 1, 'T1-633', '\x636f6e6772c3a9676174696f6e', NULL, 477, 478, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');


--
-- Name: keywords_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('keywords_id_seq', 1833, false);


--
-- Name: keywordtypes_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('keywordtypes_id_seq', 10, true);


--
-- Data for Name: rights; Type: TABLE DATA; Schema: public; Owner: webgfc
--
INSERT INTO rights VALUES (1, true, true, true, false, false, true, true, true, false, false, true, false, false, false, false, '2013-04-02 11:37:43.523222', '2013-04-02 11:37:43.523222', false, 7, false, false, true, true, true, true, true, true, true, false, true, true, true, 'Group', 7, true, true, true);
INSERT INTO rights VALUES (2, true, true, true, true, false, false, true, true, true, false, false, false, false, false, true, '2013-04-02 11:37:45.624083', '2013-04-02 11:37:45.624083', false, 6, true, true, true, true, true, true, true, true, true, true, true, true, true, 'Group', 6, true, true, true);
INSERT INTO rights VALUES (3, true, true, true, true, false, true, true, true, false, false, false, false, true, true, false, '2013-04-02 11:37:48.130788', '2013-04-02 11:37:48.130788', false, 5, true, true, true, true, true, true, true, true, true, true, true, true, true, 'Group', 5, true, true, true);
INSERT INTO rights VALUES (4, true, true, true, true, false, false, true, true, false, false, false, false, true, false, false, '2013-04-02 11:37:50.466315', '2013-04-02 11:37:50.466315', false, 4, true, true, true, true, true, true, true, true, true, true, true, true, true, 'Group', 4, true, true, true);
INSERT INTO rights VALUES (5, true, true, true, false, true, true, true, true, false, false, false, true, false, false, false, '2013-04-02 11:37:52.88356', '2013-04-02 11:37:52.88356', false, 3, true, false, true, true, true, true, true, true, true, false, true, true, true, 'Group', 3, true, true, true);
INSERT INTO rights VALUES (6, true, true, true, true, true, true, true, true, true, true, false, false, false, false, false, '2013-04-02 11:37:55.024301', '2013-04-02 11:37:55.024301', false, 2, true, false, true, true, true, true, true, true, true, false, true, false, true, 'Group', 2, true, true, true);


--
-- Name: rights_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('rights_id_seq', 7, true);


--
-- Data for Name: typemetadonnees; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO typemetadonnees VALUES (1, 'date', '2012-02-17 14:55:14.779645', '2012-02-17 14:55:14.779645');
INSERT INTO typemetadonnees VALUES (2, 'texte', '2012-02-17 14:55:14.811863', '2012-02-17 14:55:14.811863');
INSERT INTO typemetadonnees VALUES (3, 'booléen', '2012-02-17 14:55:14.825279', '2012-02-17 14:55:14.825279');
INSERT INTO typemetadonnees VALUES (4, 'select', '2014-05-16 10:06:15.491712', '2014-05-16 10:06:15.491712');


--
-- Name: typemetadonnees_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('typemetadonnees_id_seq', 5, true);


--
-- Data for Name: typenotifs; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO typenotifs VALUES (1, 'wkf_new');
INSERT INTO typenotifs VALUES (2, 'wkf_wait');
INSERT INTO typenotifs VALUES (3, 'wkf_done');
INSERT INTO typenotifs VALUES (4, 'ban_xnew');
INSERT INTO typenotifs VALUES (5, 'ban_cancel');
INSERT INTO typenotifs VALUES (6, 'ban_response');
INSERT INTO typenotifs VALUES (7, 'ban_info');
INSERT INTO typenotifs VALUES (8, 'sys_rights');
INSERT INTO typenotifs VALUES (9, 'dkt_delegto');
INSERT INTO typenotifs VALUES (10, 'wkf_jmp');
INSERT INTO typenotifs VALUES (11, 'wkf_jbk');
INSERT INTO typenotifs VALUES (12, 'ban_copy');


--
-- Name: typenotifs_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('typenotifs_id_seq', 13, false);


--
-- PostgreSQL database dump complete
--

INSERT INTO addressbooks VALUES (1, 'Carnet principal');

SELECT pg_catalog.setval('addressbooks_id_seq', 2, true);

INSERT INTO organismes VALUES (1, 'Sans organisme', 1);

SELECT pg_catalog.setval('organismes_id_seq', 2, true);

INSERT INTO contacts (id, nom, name, organisme_id, addressbook_id) VALUES (1, 'Sans contact', 'Sans contact', 1, 1);

SELECT pg_catalog.setval('contacts_id_seq', 2, true);



INSERT INTO desktops (name, group_id, created, modified, id, active) VALUES ( 'Parapheur', '5', NOW(), NOW(), '-1',TRUE);
SELECT pg_catalog.setval('desktops_id_seq', 2, true);

INSERT INTO desktopsmanagers (name, created, modified, id, active) VALUES ( 'Bureau Parapheur', NOW(), NOW(), '-1',TRUE);
SELECT pg_catalog.setval('desktopsmanagers_id_seq', 2, true);

INSERT INTO desktops_desktopsmanagers (id, desktop_id, desktopmanager_id, created, modified) VALUES ( '1', '-1', '-1', NOW(), NOW());
SELECT pg_catalog.setval('desktops_desktopsmanagers_id_seq', 2, true);
--

-- PostgreSQL database dump complete
--



INSERT INTO templatenotifications VALUES (1, 'aiguillage', 'Un flux vient de vous être aiguillé', 'Bonjour #PRENOM# #NOM#,

  Un flux vient de vous être aiguillé.

  Objet : #OBJET_FLUX#
  Référence : #REFERENCE_FLUX#
  Commentaire: #COMMENTAIRE_FLUX#

  Le flux peut être consulté à cette adresse : #ADRESSE_A_MODIFIER#

  Très cordialement.

  web-GFC

  Ce mail de notification a été envoyé par web-GFC pour votre information. Merci de ne pas répondre.', true, NOW(), NOW() );


INSERT INTO templatenotifications VALUES (2, 'copy', 'Une copie vient de vous être adressée', 'Bonjour #PRENOM# #NOM#,

  Un flux vous a été envoyé pour copie.

  Objet : #OBJET_FLUX#
  Référence : #REFERENCE_FLUX#
  Circuit : #LIBELLE_CIRCUIT#
  Commentaire: #COMMENTAIRE_FLUX#
  Affaire suivie par: #AFFAIRESUIVIE_PAR#

  Le flux peut être consulté à cette adresse : #ADRESSE_A_MODIFIER#

  Très cordialement.

  web-GFC

  Ce mail de notification a été envoyé par web-GFC pour votre information. Merci de ne pas répondre.', true, NOW(), NOW() );

INSERT INTO templatenotifications VALUES (3, 'delegation', 'Un bureau vien de vous être délégué', 'Bonjour #PRENOM# #NOM#,

  Un bureau vient de vous être délégué.

  Bureau : #NOM_BUREAU#
  Date de début: #DEBUT_DELEGATION#
  Date de fin: #FIN_DELEGATION#

  Très cordialement.

  web-GFC

  Ce mail de notification a été envoyé par web-GFC pour votre information. Merci de ne pas répondre.', true, NOW(), NOW() );

INSERT INTO templatenotifications VALUES (4, 'insertion', 'Vous allez recevoir un nouveau flux', 'Bonjour #PRENOM# #NOM#,

  Un flux vient d''être inséré dans un circuit dont vous êtes valideur.

  Id : #IDENTIFIANT_FLUX#
  Référence : #REFERENCE_FLUX#
  Objet : #OBJET_FLUX#
  Circuit : #LIBELLE_CIRCUIT#

  Vous pouvez le consulter à l''adresse : #ADRESSE_A_VISUALISER#

  Très cordialement.

  web-GFC

  Ce mail de notification a été envoyé par web-GFC pour votre information. Merci de ne pas répondre.', true, NOW(), NOW() );

INSERT INTO templatenotifications VALUES (5, 'insertion_reponse', 'Réception d''un flux réponse', 'Bonjour #PRENOM# #NOM#,

  Un flux en réponse au flux #FLUX_ORIGINE# vient d''être ajouté dans votre bannette "Mes flux réponse".

  Id : #IDENTIFIANT_FLUX#
  Référence : #REFERENCE_FLUX#
  Objet : #OBJET_FLUX#
  Circuit : #LIBELLE_CIRCUIT#

  Vous pouvez le consulter à l''adresse : #ADRESSE_A_VISUALISER#

  Très cordialement.

  web-GFC

  Ce mail de notification a été envoyé par web-GFC pour votre information. Merci de ne pas répondre.', true, NOW(), NOW() );


INSERT INTO templatenotifications VALUES (6, 'refus', 'Un flux a été refusé', 'Bonjour #PRENOM# #NOM#,

  Un flux a été refusé.

  Objet : #OBJET_FLUX#
  Référence : #REFERENCE_FLUX#
  Nouvel identifiant : #IDENTIFIANT_FLUX#

  Le flux peut être consulté à cette adresse : #ADRESSE_A_MODIFIER#

  Très cordialement.

  web-GFC

  Ce mail de notification a été envoyé par web-GFC pour votre information. Merci de ne pas répondre.', true, NOW(), NOW() );

INSERT INTO templatenotifications VALUES (7, 'retard_validation', 'Un flux est en retard', 'Bonjour #PRENOM# #NOM#,

  Un flux dans votre bannette "Mes flux à traiter" est en retard et attend votre validation.

  Id : #IDENTIFIANT_FLUX#
  Référence : #REFERENCE_FLUX#
  Objet : #OBJET_FLUX#
  Circuit : #LIBELLE_CIRCUIT#

  Vous pouvez consulter ce flux à l''adresse : #ADRESSE_A_TRAITER#

  Très cordialement.

  web-GFC

  Ce mail de notification a été envoyé par web-GFC pour votre information. Merci de ne pas répondre.', true, NOW(), NOW() );

INSERT INTO templatenotifications VALUES (8, 'traitement', 'Vous avez un flux à traiter', 'Bonjour #PRENOM# #NOM#,

  Un flux est arrivé dans votre bannette "Mes flux à traiter".

  Id : #IDENTIFIANT_FLUX#
  Référence : #REFERENCE_FLUX#
  Objet : #OBJET_FLUX#
  Circuit : #LIBELLE_CIRCUIT#

  Vous pouvez le consulter à l''adresse : #ADRESSE_A_TRAITER#

  Très cordialement.

  web-GFC

  Ce mail de notification a été envoyé par web-GFC pour votre information. Merci de ne pas répondre.', true, NOW(), NOW() );

INSERT INTO templatenotifications VALUES (9, 'quotidien', '[web-GFC]  Récapitulatif journalier de vos flux', 'Bonjour #PRENOM# #NOM#,

  Voici la liste des flux vous concernant pour la date du #DATERECEPTION_FLUX#.

  Flux : 
  #REFERENCE_FLUX#

  Très cordialement.

  web-GFC

  Ce mail de notification a été envoyé par web-GFC pour votre information. Merci de ne pas répondre.', true, NOW(), NOW() );


INSERT INTO templatenotifications VALUES (10, 'clos', '[web-GFC]  Un flux dont vous êtes l''initiateur a été clos', 'Bonjour #PRENOM# #NOM#,

Le flux #REFERENCE_FLUX#, dont vous êtes l''initiateur vient d''être clos

Très cordialement.

web-GFC

Ce mail de notification a été envoyé par web-GFC pour votre information. Merci de ne pas répondre.', true, NOW(), NOW() );


SELECT pg_catalog.setval('templatenotifications_id_seq', 11, true);
COMMIT;
